SystemOrganization addCategory: #SVO2!
SystemOrganization addCategory: 'SVO2-App'!
SystemOrganization addCategory: 'SVO2-Commands'!
SystemOrganization addCategory: 'SVO2-Parser'!
SystemOrganization addCategory: 'SVO2-ProgramVO'!
SystemOrganization addCategory: 'SVO2-Tests'!
SystemOrganization addCategory: 'SVO2-Things'!
SystemOrganization addCategory: 'SVO2-UI-VO'!
SystemOrganization addCategory: 'SVO2-UI-context'!
SystemOrganization addCategory: 'SVO2-Utilities'!
SystemOrganization addCategory: 'SVO2-core'!

Trait named: #TSVOUserNameAccess
	uses: {}
	category: 'SVO2-core'!

!TSVOUserNameAccess commentStamp: 'hk 4/28/2015 11:06' prior: 0!
I provide the userName functionality for DomainVO & ContextVO!

!TSVOUserNameAccess classSide methodsFor: 'accessing' stamp: 'hk 5/10/2015 13:39'!
named: aName ifAbsent: absentBlock
	"answer the vo with distinctName aName and the highest version"
	| res |
	res := self asvos values select: [ :k | k userName = aName ] . 
	^res 
		ifEmpty: [absentBlock value] 
		ifNotEmpty: [ (res sorted: [:a :b | a longName <= b longName] ) first] .

	! !

!TSVOUserNameAccess classSide methodsFor: 'instance creation' stamp: 'hk 6/9/2015 10:27'!
new
	 ^self error: 'no instantiation without userName'! !

!TSVOUserNameAccess classSide methodsFor: 'authoring' stamp: 'hk 5/27/2015 11:40'!
newOfType: aString
	| vo |
	vo := self named: aString ifAbsent: [ ] .
	^vo 
		ifNil: [ self addASVO: (self basicNew userName: aString; id: self nextID; initialize) ] 
		ifNotNil: [ self inform: 'existing VO used' . vo ]


! !

!TSVOUserNameAccess methodsFor: 'accessing' stamp: 'hk 4/28/2015 13:06'!
longName
	^self userName, self autoName ! !

!TSVOUserNameAccess methodsFor: 'printing' stamp: 'hk 5/8/2015 13:57'!
printOn: aStream
	aStream nextPutAll: self longName "substrings first"! !

DynamicComposableModel subclass: #GcSlotWidget
	instanceVariableNames: 'key thingVO'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcSlotWidget commentStamp: 'hk 6/11/2015 14:47' prior: 0!
I am a widget for any slot VO
I initialize labelKey

Instance Variables
	thingVO: 	<ValueVO>‚	the vo I display
	key: 		<String>		the slotName I am in
!

GcSlotWidget subclass: #GcChoiceWidget
	instanceVariableNames: 'sel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcChoiceWidget commentStamp: 'hk 4/23/2015 19:04' prior: 0!
I am a dropList widget for a ChoiceVO.

Instance Variables
	sel:		<String>		the list selection


========== Test ===========

| ch wl   |
(ch := EnumVO new) list: #( 'supplier' 'customer' 'endCustomer' 'bizContact' )  .
wl := self on: ch key: #bla  .
wl openWithSpec .

| vo wvo |
(vo := DomainVO new: #top)  
	enum: #( 'supplier' 'customer' 'endCustomer' )  at: #bizContactRoles .
wvo := GcVoWidget on: vo key: #bla .
wvo openWithSpec!

!GcChoiceWidget methodsFor: 'initialization' stamp: 'hk 5/22/2015 21:28'!
initializePresenter
	
	self dropList whenSelectedItemChanged: [ :item | thingVO selection: item ] .! !

!GcChoiceWidget methodsFor: 'initialization' stamp: 'hk 5/22/2015 21:28'!
initializeWidgets

	self initializeSuperWidgets .
	self instantiateModels: #(
		dropList	DropListModel ) .
	
	self dropList 
		items: thingVO list;
		displayBlock: [ :item | item printString ] .

	self layout: (SpecLayout composed newColumn: [ :c |
					c newRow: [:row | row
						add: #labelKey right: 0.66 ; 
						add: #dropList left: 0.34 ] height: self singleHeight.
					self addEmptyLine1To: c ] ;
					yourself) .
	self needFullRebuild: false .! !

!GcChoiceWidget methodsFor: 'accessing' stamp: 'hk 5/8/2015 11:21'!
widgetHeight
	^ self singleHeight +6! !

GcSlotWidget subclass: #GcDateWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcDateWidget commentStamp: 'hk 4/10/2015 18:45' prior: 0!
I am a leaf widget for a date


========== Tests ==========

(self on: (DateVO new) key: #bla ) openWithSpec
!

!GcDateWidget methodsFor: 'initialization' stamp: 'hk 5/22/2015 21:28'!
initializePresenter

	self date whenDateChanged: [ thingVO date: self date date ]
! !

!GcDateWidget methodsFor: 'initialization' stamp: 'hk 5/22/2015 21:28'!
initializeWidgets

	self initializeSuperWidgets .
	self instantiateModels: #( date DateModel ) .
	
	self date date: thingVO date.

	self layout: (SpecLayout composed newColumn: [ :c | c
					newRow: [:row | row
						add: #labelKey right: 0.66 ;
						add: #date ] height: self singleHeight .
					self addEmptyLine1To: c ] ;
					yourself) .
	self needFullRebuild: false .! !

!GcDateWidget methodsFor: 'accessing' stamp: 'hk 5/8/2015 11:22'!
widgetHeight
	^ self singleHeight +6! !

!GcSlotWidget class methodsFor: 'instance creation' stamp: 'hk 5/8/2015 12:01'!
on: aVO 
	"for use instead of GcVOEditor"
	^self on: aVO key: aVO userName! !

!GcSlotWidget class methodsFor: 'instance creation' stamp: 'hk 5/22/2015 21:27'!
on: aVO key: aString
	^self basicNew thingVO: aVO; key: aString; initialize! !

!GcSlotWidget methodsFor: 'initialization' stamp: 'hk 4/11/2015 16:15'!
addEmptyLine1To: c
		 c newRow: [ :r | r add: #emptyLabel ] height: 1 ! !

!GcSlotWidget methodsFor: 'constants' stamp: 'hk 4/11/2015 13:17'!
buttonWidth
	^ self class buttonWidth ! !

!GcSlotWidget methodsFor: 'constants' stamp: 'hk 4/12/2015 12:33'!
editWidth
	^ 50! !

!GcSlotWidget methodsFor: 'constants' stamp: 'hk 4/11/2015 15:03'!
fieldWidth
	^ self class buttonWidth  * 2! !

!GcSlotWidget methodsFor: 'initialization' stamp: 'hk 6/24/2015 16:26'!
initializeSuperWidgets
	self instantiateModels: #( 
		labelKey		LabelModel 
		emptyLabel 	LabelModel
		marker			ButtonModel
		) .
	self labelKey label:(key asText addAttribute: TextAlignment rightFlush).
"	self labelKey label: key .
"	self marker borderColor: Color black 
! !

!GcSlotWidget methodsFor: 'accessing' stamp: 'hk 4/9/2015 14:50'!
key: aString
	key := aString asSymbol ! !

!GcSlotWidget methodsFor: 'constants' stamp: 'hk 4/11/2015 16:11'!
singleHeight
	^ StandardFonts defaultFont height + 12! !

!GcSlotWidget methodsFor: 'accessing' stamp: 'hk 5/22/2015 21:28'!
thingVO: aVO
	thingVO := aVO! !

!GcSlotWidget methodsFor: 'accessing' stamp: 'hk 5/8/2015 11:20'!
widgetHeight
	"for single line widgets"
	^ self singleHeight ! !

GcSlotWidget subclass: #GcUListWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcUListWidget commentStamp: 'hk 6/22/2015 16:11' prior: 0!
I am a widget for aLSlistVO
========== Tests ==========

| lvo wl   |
lvo := SlistVO new asvo: (ThingVO newOfType: #bla) .
wl := self on: lvo key: #bla  .
wl openWithSpec .

| asvo wvo |
asvo := (ThingVO newOfType: #top).
asvo listOf: (ThingVO newOfType: #bla) at: #bla . 
wvo := GcVOUI on: asvo  .
wvo openWithSpec!

!GcUListWidget methodsFor: 'initialization' stamp: 'hk 4/11/2015 17:07'!
buttonWidth
	"shall be smaller"
	^50! !

!GcUListWidget methodsFor: 'initialization' stamp: 'hk 4/29/2015 15:04'!
editMenu: m 
	| item |
	item := m add: #edit target: self fList selectedItem selector:  #edit .
	item enablementSelector: [ self fList selectedItem isNotNil ] .
	^m! !

!GcUListWidget methodsFor: 'initialization' stamp: 'hk 6/25/2015 14:47'!
initializeWidgets

	self initializeSuperWidgets .
	self instantiateModels: #(
		fButton	ButtonModel 
		fList	ListModel ) .
	
	self fList 
		items: thingVO list;
		displayBlock: [ :item | thingVO displayStringFor: item ] ;
		menu: [ :m | self editMenu: m ] .
	self fButton
		icon: (GcIcons at: #plus) ;
		enabled: true;
		action: [ thingVO newExemplar. self fList items: thingVO list ].

	self layout: (SpecLayout composed newColumn: [ :c | c
			newRow: [:row | row
				add: #labelKey right: 0.66 ; 
				add: #fList ;
				add: #fButton width: 24 ] height: self listHeight . 
			self addEmptyLine1To: c ] ;
			yourself) .
	self needFullRebuild: false .! !

!GcUListWidget methodsFor: 'initialization' stamp: 'hk 5/22/2015 21:28'!
listHeight
	^ self singleHeight  * ( (thingVO listSize /2 max: 2) min: 4 ). ! !

!GcUListWidget methodsFor: 'initialization' stamp: 'hk 4/11/2015 13:23'!
widgetHeight
	^ self listHeight ! !

GcSlotWidget subclass: #GcValueWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcValueWidget commentStamp: 'hk 4/29/2015 11:30' prior: 0!
I am a widget for a valueVO



========== Tests ==========

(self on: (ValueVO new val: #bla ) key: #bla ) openWithSpec!

GcValueWidget subclass: #GcRangeWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcRangeWidget commentStamp: 'hk 4/29/2015 11:59' prior: 0!
A GcRangeWidget class is xxxxxxxxx.



(self on: (RangeVO new min: 1; max: 7; val: 5 ) key: #bla ) openWithSpec!

!GcValueWidget methodsFor: 'initialization' stamp: 'hk 6/25/2015 18:49'!
initializeLayout
	self inputValue text: thingVO val asString.
	self layout:  (SpecLayout composed newColumn: [ :c | c
			newRow: [:row | row
				add: #labelKey right: 0.66 ;
				add: #inputValue] height: self singleHeight .
			self addEmptyLine1To: c ] ;
			yourself)! !

!GcValueWidget methodsFor: 'initialization' stamp: 'hk 6/11/2015 15:13'!
initializePresenter

	self inputValue whenTextChanged: [ thingVO val: self inputValue getText  ]
! !

!GcValueWidget methodsFor: 'initialization' stamp: 'hk 6/24/2015 16:31'!
initializeWidgets

	self initializeSuperWidgets .
	self instantiateModels: #(
		inputValue	TextInputFieldModel ) .
	self inputValue 
		autoAccept: true ;
		ghostText: self labelKey label asString .
	self initializeLayout .
	self needFullRebuild: false .! !

GcSlotWidget subclass: #GcVoTextWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcVoTextWidget methodsFor: 'initialization' stamp: 'hk 6/16/2015 10:15'!
defineButtons
	self editButton
		label: #edit;
		state: true ;
		action: [ self edit ] .
! !

!GcVoTextWidget methodsFor: 'ui actions' stamp: 'hk 6/16/2015 10:16'!
edit   
	thingVO edit! !

!GcVoTextWidget methodsFor: 'initialization' stamp: 'hk 6/16/2015 10:03'!
initializeLayout
	self layout: 
	(SpecLayout composed newColumn: 
		[:c| c newRow: [ :r | r 
					add:  #labelKey ;
					add: #editButton width: 50 ] height: self singleHeight .
			c newRow:  [ :r | r add: #text ]
		] ;  yourself  )
		! !

!GcVoTextWidget methodsFor: 'initialization' stamp: 'hk 6/16/2015 11:51'!
initializeWidgets

	self initializeSuperWidgets. 		" #labelKey "
	self instantiateModels:  #(
		editButton 		ButtonModel	
		text 			TextModel
		) .
	self  text text: thingVO asText .
	self defineButtons .
	self initializeLayout .
	self needFullRebuild:  false.
	! !

GcSlotWidget subclass: #GcVoWidget
	instanceVariableNames: 'mode subWidgets builtAction subValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcVoWidget commentStamp: 'hk 6/15/2015 09:53' prior: 0!
I am a widget for a node in a structured VO.
My mode is either #collapsed or #expanded.

I dynamically assign: 
 	expandButton:		<ButtonModel>
	collapseButton:		<ButtonModel>
	plusButton:			<ButtonModel>


Instance Variables
	mode:				<#expanded | #collapsed>
	subWidgets: 		<OrderedCollection of Symbol>		the widget keys
	subValues: 			<OrderedCollection of Symbol>		the keys of value widgets 
			
		
========== Tests ==========

1	
(GcVoWidget on: ((ThingVO newOfType: #bla) val: #bla1 at: #bla2; yourself ) key: #bla ) 
	openWithSpec
2
GcExample initSVO .	
(GcVoWidget 
	on: ((ThingVO newOfType: #bla) 
			val: #foo1 at: #bla2; 
			thing: ( (ThingVO newOfType: #bla0) val: #foo11 at: #bla12; yourself ) at:  #bla3 ; 
			enum: #( 'supplier' 'customer' 'endCustomer' )  at: #bizContactRoles ;
			yourself ) 
	key: #bla )  openWithSpec
3
GcExample initSVO .	
(GcVoWidget on: (GcExample contact  ) key: #bla ) openWithSpec

4
GcExample initSVO .
(GcVOEditor on: (GcExample contact   ) ) openWithSpec!

!GcVoWidget methodsFor: 'ui actions' stamp: 'hk 6/11/2015 15:47'!
collapse
	mode := #collapsed .		
	self reBuild! !

!GcVoWidget methodsFor: 'initialization' stamp: 'hk 6/15/2015 09:55'!
defineButtons
	self expandButton 
		icon: (GcIcons at: #right);
		action: [ self expand ] .
	self collapseButton
		icon: (GcIcons at: #down);
		action: [ self collapse ] .
	self editButton
		label: #edit;
		state: true ;
		action: [ self edit ] .
	self plusButton 
		icon: (GcIcons at: #plus);
		action: [  ]! !

!GcVoWidget methodsFor: 'ui actions' stamp: 'hk 6/14/2015 21:28'!
edit   
	thingVO edit! !

!GcVoWidget methodsFor: 'ui actions' stamp: 'hk 6/11/2015 15:48'!
expand   
	mode := #expanded .		
	self  reBuild! !

!GcVoWidget methodsFor: 'initialization' stamp: 'hk 6/16/2015 09:20'!
initialExtent
	^ 600 @ (self widgetHeight + self class toolbarHeight)
! !

!GcVoWidget methodsFor: 'initialization' stamp: 'hk 6/15/2015 09:56'!
initializeLayout
	self layout: 
	(SpecLayout composed newColumn: 
		[ :c |  c 
			newRow: [ :r | r 
				add:  #modeButton width: 24 ;
				add: #labelKey ;
				add: #editButton width: 50 ] height: self singleHeight .
			(mode = #expanded) ifTrue: 
				[self subValuesBy3 do:  
					[ :swarr | c newRow: 
						[ :r | 
						r add: #marker width: 2 .
						1 to: swarr size do: 
						[ :i |
						r add: #emptyLabel width: 24 .
						r add: (swarr at: i)]
						] height: self singleHeight ] .  
				subWidgets do: [  :sw | c newRow: 
					[ :r | 
					r add: #marker width: 2 .
					r add: #emptyLabel width: 24 .
					r add: sw] height: (widgets at: sw) widgetHeight  ]
				].   
			 self addEmptyLine1To: c] ; 
		yourself  )
		! !

!GcVoWidget methodsFor: 'initialization' stamp: 'hk 6/11/2015 17:55'!
initializeSubWidgets

	thingVO slots keysAndValuesDo: 
		[ :k :v |  | w |
		 w := ( 'label_', k ) asSymbol .
		self assign: (v widgetLabelled: k) to: w.
		v isValue 
			ifTrue:  [ subValues add: w ]
			ifFalse: 	[ subWidgets add: w ]	]! !

!GcVoWidget methodsFor: 'initialization' stamp: 'hk 6/16/2015 09:17'!
initializeWidgets

	self initializeSuperWidgets. 		" #labelKey "
	mode := #expanded .
	self instantiateModels:  #(
		expandButton 	ButtonModel 
		collapseButton 	ButtonModel 
		plusButton 		ButtonModel 	
		editButton 		ButtonModel	
		) .
	subWidgets := OrderedCollection new .
	subValues := OrderedCollection new .
	self initializeSubWidgets .
	self defineButtons .
	self initializeLayout .
	self needFullRebuild:  false.
	! !

!GcVoWidget methodsFor: 'accessing' stamp: 'hk 6/11/2015 15:38'!
modeButton   
	^(mode = #expanded) 
		ifTrue: [ self collapseButton ] ifFalse: [ self expandButton ]! !

!GcVoWidget methodsFor: 'private' stamp: 'hk 6/11/2015 15:47'!
reBuild
	self initializeLayout   .
	self needFullRebuild:  false.
	self buildWithSpec .
	(owner ifNil: [ ^self ] ) reBuild
! !

!GcVoWidget methodsFor: 'accessing' stamp: 'hk 6/12/2015 10:50'!
subValuesBy3
	| res size |
	size := subValues size .
	res := OrderedCollection new .
	1 to:  size by: 3 do: 
		[ :a |  | o arr | 
		o := (a+3) min: (size+1) .
		arr := Array new: (o - a) .
		a to: o-1 do: [ :i | arr at: (i-1 \\ 3 +1) put: (subValues at: i) ] .
		res add: arr ] .
	^res! !

!GcVoWidget methodsFor: 'accessing' stamp: 'hk 6/12/2015 11:13'!
widgetHeight
	| h |
	h := self singleHeight + 6.
	(mode = #expanded) ifTrue: 
		[ h := h + (self singleHeight * ((subValues size /3) ceiling)) .
		subWidgets do: 
			[ :sw | h := h + (widgets at: sw) widgetHeight ]  ] .
	^ h! !

DynamicComposableModel subclass: #GcUI
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcUI commentStamp: 'hk 4/13/2015 10:54' prior: 0!
I am an abstract GloConn UI on a context.

Instance Variables
	context:			<CF>						preset context

!

GcUI subclass: #GcAspectUI
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcAspectUI commentStamp: 'hk 5/11/2015 19:08' prior: 0!
I am a Gui for a fixed aspect of an item.
I use my context's view (or a FullView) to display the aspect

Instance Variables
	target:		<NewValueHolder>		rebuild on target change 


(GcAspectUI  on:  (GcExample makeCustustContext))  openWithSpec


!

!GcAspectUI methodsFor: 'initialization' stamp: 'hk 6/16/2015 11:50'!
assignSubWidget

	self assign: (GcVoTextWidget on: self target ) to: #subWidget .
	self subWidget owner: self .
	 
! !

!GcAspectUI methodsFor: 'testing' stamp: 'hk 6/23/2015 12:23'!
hasTarget  
	^ target value isNode
! !

!GcAspectUI methodsFor: 'initialization' stamp: 'hk 5/7/2015 18:46'!
initializeLayout
	self hasTarget 
		ifTrue: [ self initializeTargetLayout ] 
		ifFalse: [ self initializeNoTargetLayout ] 
! !

!GcAspectUI methodsFor: 'initialization' stamp: 'hk 5/10/2015 19:41'!
initializeNoTargetLayout

	self identityLabel label:  'none'  .
 
	self layout: (SpecLayout composed newColumn: [ :c | 
		c newRow: [ :r | 
				r  add: #chooseButton width: 24 .
				r add: #identityLabel .
				r  add: #inspectButton width: self class buttonWidth ] height: self class toolbarHeight. 
		c add: #emptyLabel height: 1] ) .
	self needFullRebuild: false .
! !

!GcAspectUI methodsFor: 'initialization' stamp: 'hk 4/27/2015 18:25'!
initializePresenter

	target whenChangedDo: 
		[ self initializeLayout .
		self buildWithSpec ]
! !

!GcAspectUI methodsFor: 'initialization' stamp: 'hk 5/13/2015 18:07'!
initializeTargetLayout
	self assignSubWidget .	 
	self layout: (SpecLayout composed newColumn: [ :c | 
		c newRow: [ :r | 
			r add: #emptyLabel. 
			r  add: #inspectButton width: self class buttonWidth] height: self class toolbarHeight .
		c newRow: [ :r | r add: #subWidget] .
		c newRow: [ :r | r add: #emptyLabel ] height: 1 ] ).
	self needFullRebuild: false 
! !

!GcAspectUI methodsFor: 'initialization' stamp: 'hk 6/3/2015 19:08'!
initializeWidgets
	self instantiateModels: #( 
		emptyLabel 	LabelModel 
		identityLabel 	LabelModel
		inspectButton 	ButtonModel 	
		removeButton 	ButtonModel 	
		chooseButton 	ButtonModel 	) .				
	self inspectButton
		label: 'inspect';
		action: [ self context inspect ];
		state: true.
	self removeButton 
		icon: (GcIcons iconAt: #delete) ;
		action: [ self remove ];
		state: true.
	self chooseButton 
		icon:  (GcIcons iconAt: #add);
		action: [ self choose ];
		state: true.
	target := nil asValueHolder .
	self setTarget .
	self initializeLayout! !

!GcAspectUI methodsFor: 'ui actions' stamp: 'hk 5/7/2015 19:03'!
reBuild
	"recursion stop"
! !

!GcAspectUI methodsFor: 'private' stamp: 'hk 6/23/2015 12:27'!
setTarget
	"set the target to the defined DomainVO aspect "
	| t |   
	t := (context container vo ifNil: [^self]) slotAt: (context targetSlotname). 
	t ifNil: [self inform: (context targetSlotname, ' does not exist ')].
	target value: t .
	! !

!GcAspectUI methodsFor: 'accessing' stamp: 'hk 5/7/2015 15:39'!
target
	^ target value! !

!GcAspectUI methodsFor: 'accessing' stamp: 'hk 5/10/2015 19:49'!
templates
	^ context templates ! !

!GcAspectUI methodsFor: 'accessing' stamp: 'hk 6/4/2015 18:59'!
view
	^ context view ifNil: [ FullViewVO new ]
! !

GcAspectUI subclass: #GcReferenceAspectUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcReferenceAspectUI commentStamp: 'hk 5/11/2015 19:09' prior: 0!
I am a GUI for a "reference" aspect of an item such as a customer


(GcCustAspectUI  on:  (GcExample makeCustustContext))  openWithSpec!

!GcReferenceAspectUI methodsFor: 'ui actions' stamp: 'hk 6/24/2015 15:38'!
addAspect: anAspect
	context container vo define: context targetSlotname thing: anAspect! !

!GcReferenceAspectUI methodsFor: 'ui actions' stamp: 'hk 6/11/2015 16:19'!
addTargetFrom: selection
	selection ifNil: [ ^self ].
	self addAspect: (self view copyFrom: selection) .
	self setTarget	
! !

!GcReferenceAspectUI methodsFor: 'ui actions' stamp: 'hk 6/10/2015 09:38'!
choose
	| dialogWindow ui |  
	ui := context choiceContext newUI .
	ui extent: 1000@400 .				"where to find this constant ?"
	dialogWindow := ui openDialogWithSpec .
	dialogWindow 
		okAction: [ self addTargetFrom: dialogWindow model selection] ;
		cancelAction: [ ] ;
		extent: 800@400 ;
		title: context choiceContext displayName  ;
		centeredRelativeTo: self 

! !

!GcReferenceAspectUI methodsFor: 'initialization' stamp: 'hk 5/13/2015 18:07'!
initializeTargetLayout
	self assignSubWidget .	 	
	self identityLabel label: (self target userName, ' id: ', self target id asString) .
 
	self layout: (SpecLayout composed newColumn: [ :c | 
		c newRow: [ :r | 
			r  add: #removeButton width: 24.
			r add: #identityLabel. 
			r  add: #inspectButton width: self class buttonWidth] height: self class toolbarHeight .
		c newRow: [ :r | r add: #subWidget] .
		c newRow: [ :r | r add: #emptyLabel ] height: 1 ] ).
	self needFullRebuild: false 
! !

!GcReferenceAspectUI methodsFor: 'ui actions' stamp: 'hk 6/23/2015 12:13'!
remove   
	 (context container vo ifNil: [^nil]) removeNode: (context targetSlotname) .
	self setTarget ! !

GcReferenceAspectUI subclass: #GcTemplateAspectUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcTemplateAspectUI commentStamp: 'hk 5/11/2015 19:14' prior: 0!
I am a GUI for a "template" aspect of an item such as a request, offer or order. 
The aspect may be created from a template or via bizNet.


(GcTemplateAspectUI  on:  (GcExample makeAnfrageContext))  openWithSpec!

!GcTemplateAspectUI methodsFor: 'ui actions' stamp: 'hk 6/11/2015 12:28'!
choose
	| index |  
	(self templates size = 1)
		ifTrue: [ ^self addTargetFrom: self templates first ] .   
	index := UIManager default chooseFrom: self templates lines: #() message: 'choose a template' .
	index = 0 ifTrue: [ ^nil ].
	self addTargetFrom:  (self templates at: index)! !

GcUI subclass: #GcCalcUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcCalcUI commentStamp: 'hk 4/13/2015 12:33' prior: 0!
A GcCalcUI is xxxxxxxxx.

====== Tests =====

GcCalcUI new  openWithSpec!

!GcCalcUI methodsFor: 'as yet unclassified' stamp: 'hk 4/13/2015 10:58'!
initializeLayout
	
	self layout: (SpecLayout composed newColumn: [ :c | 
		c 	newRow: [ :r | r
				add:  #pIcon width: 24 ;
				add:  #pLabel  ] height: self class toolbarHeight .
		c 	newRow: [ :r | r  
				add: #emptyLabel ;
				add: #calculator  ;
				add: #emptyLabel ] height: 6.333 * self class toolbarHeight .
		c	add: #emptyLabel  ] ).

! !

!GcCalcUI methodsFor: 'as yet unclassified' stamp: 'hk 4/28/2015 14:45'!
initializeWidgets

	self initializeSuperWidgets .
	self pIcon icon: (GcIcons at: context uiKey) .
	self pLabel label: context displayName  .
	self assign: SvoCalc new to: #calculator .	
	self assign: (LabelModel new) to: #emptyLabel . 
	self initializeLayout! !

GcUI subclass: #GcDashUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcDashUI commentStamp: 'hk 4/13/2015 12:34' prior: 0!
A GcDashUI is xxxxxxxxx.


====== Tests ======

GcDashUI new  openWithSpec
!

!GcDashUI methodsFor: 'initialization' stamp: 'hk 4/13/2015 10:48'!
initializeLayout
 	self layout: (SpecLayout composed newColumn: [ :c | 
		c 	newRow: [ :r | r
				add:  #pIcon width: 24 ;
				add:  #pLabel  ] height: self class toolbarHeight .
		c add: #dashButton ] ).	
	self needFullRebuild: false.
! !

!GcDashUI methodsFor: 'initialization' stamp: 'hk 5/11/2015 12:15'!
initializeWidgets
	self initializeSuperWidgets .
	self pIcon icon: (GcIcons at: context uiKey ) .
	self pLabel label: context displayName  .
	self assign: (ButtonModel new
				state: false;
				icon: (GcIcons default dashboardIcon)) to: #dashButton.
	self initializeLayout! !

GcUI subclass: #GcItemUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcItemUI commentStamp: 'hk 4/24/2015 17:41' prior: 0!
I am a general UI for editing one thing of a list of things
!

!GcItemUI methodsFor: 'initialization' stamp: 'hk 5/8/2015 12:16'!
addPaneHeaderTo: aColumn
	aColumn add: #paneMenu height: self singleHeight .
	aColumn add: #list height: 2 * self singleHeight  .				


! !

!GcItemUI methodsFor: 'ui actions' stamp: 'hk 5/22/2015 14:49'!
backToList
	owner clearSubUI.	
	owner pLabel label: (owner context displayName) .
	owner resetList .
	owner reBuild
! !

!GcItemUI methodsFor: 'initialization' stamp: 'hk 6/7/2015 13:29'!
initializeAuthoredMenu
	| environment |
	environment :=  { context . self }.	
	self paneMenu addGroup: [ :g | 
		context menuBehaviors do:  [:r | r 
			hasMenu ifTrue: [g addMenuItem: (r 
				menuFor: environment 
				wrapWith: [ :s | self notify: s ] ) ] ] ] 
! !

!GcItemUI methodsFor: 'initialization' stamp: 'hk 6/9/2015 15:29'!
initializeHeader
	self instantiateModels: #(
		paneMenu		MenuModel
		list 				TableListUI
		) .
	self list setView: context view .
	self list items: { context vo} .
	self initializeMenu .
	self initializeAuthoredMenu .
! !

!GcItemUI methodsFor: 'initialization' stamp: 'hk 5/8/2015 13:21'!
initializeLayout
	self layout: (SpecLayout composed newColumn: [ :c | 
		self addPaneHeaderTo: c  .				
		c add: self mainWidget ] ).
	self needFullRebuild: false .

! !

!GcItemUI methodsFor: 'initialization' stamp: 'hk 6/3/2015 13:43'!
initializeMenu
	self paneMenu addGroup: [ :g | g 
		addItem: [ :i |  i
			icon: Smalltalk ui icons backIcon ;
			action: [ self backToList ] ] ] 
! !

!GcItemUI methodsFor: 'initialization' stamp: 'hk 6/16/2015 11:50'!
initializeWidgets
	self initializeHeader .  
	self assign: (GcVoTextWidget on: context vo ) to: #item .
	self initializeLayout! !

!GcItemUI methodsFor: 'initialization' stamp: 'hk 5/8/2015 13:21'!
mainWidget
	^ #item ! !

GcItemUI subclass: #GcItemWithTabsUI
	instanceVariableNames: 'tabManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcItemWithTabsUI commentStamp: 'hk 5/8/2015 13:35' prior: 0!
I am a UI for editing a process with aspects of a list of processes

Instance Variables
	tabManager: 	<TabManager>		does not work with DynamicSpec

====== Tests ======

(GcItemWithTabsUI on: (( CfPlainVO new: #bla) vo: (( DomainVO new: #pro) copy; yourself); yourself)) openWithSpec!

!GcItemWithTabsUI methodsFor: 'initialization' stamp: 'hk 5/8/2015 13:39'!
InitializeTabs  

	context subcontexts do: [ :sc |  |key icon newTab  |
		key := sc displayName asSymbol.
		icon := GcIcons  iconAt: key .
		(newTab := self newTab) 
				label: key ;
				icon: icon ;
				model: (self newModelFor: sc )  .
		tabManager addTab: newTab ]
! !

!GcItemWithTabsUI methodsFor: 'initialization' stamp: 'hk 5/8/2015 13:19'!
initializeWidgets
	self initializeHeader.
	tabManager := self newTabManager .  	
	"can't set owner in TabManager"
	self InitializeTabs  .
	self initializeLayout! !

!GcItemWithTabsUI methodsFor: 'initialization' stamp: 'hk 5/8/2015 13:21'!
mainWidget
	^#tabManager ! !

!GcItemWithTabsUI methodsFor: 'initialization' stamp: 'hk 5/22/2015 14:49'!
newModelFor: aViewContext 
	"create the UImodel for the view, fall back, if there is no UIclass"
	
	| uiClass  |    
	uiClass := Smalltalk at: aViewContext uiSpec 
				   ifAbsent: [^LabelModel new label: aViewContext displayName]  . 
	^aViewContext newUI ! !

!GcItemWithTabsUI methodsFor: 'accessing' stamp: 'hk 5/8/2015 13:38'!
tabManager
	^tabManager! !

GcUI subclass: #GcListUI
	instanceVariableNames: 'hasSubUI selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcListUI commentStamp: 'hk 4/28/2015 15:40' prior: 0!
I am a universal ListUI

Instance Variables
	selection:  		<VO | nil>
	hasSubUI:  		<Boolean>			
					
#subUI is the dynamic widget for the list element


======== Tests ========

(GcListUI on: (( CfListVO new: #bla) asvo: (( DomainVO new: #pro) copy; yourself ) ; yourself )) openWithSpec
!

!GcListUI methodsFor: 'ui actions' stamp: 'hk 5/11/2015 14:55'!
activateSubContextForNew
	| new |
	self initializeSubContextUIon: (new := context asvo copy) .
	selection := new .
	self reBuild



! !

!GcListUI methodsFor: 'initialization' stamp: 'hk 4/13/2015 12:41'!
clearSubUI
	hasSubUI := false! !

!GcListUI methodsFor: 'initialization' stamp: 'hk 5/31/2015 18:09'!
contextIcon
	^context hasDataPending 
		ifTrue:  [GcIcons at: (context uiKey, 'Dirty') asSymbol]
		ifFalse: 	[GcIcons at: context uiKey]
		
	
! !

!GcListUI methodsFor: 'ui actions' stamp: 'hk 6/3/2015 14:05'!
deleteSelection

	selection  ifNil: [ ^self notify: 'no selection' ] .
	context asvo removeCopy: selection .	
	self resetList .
	context checkDirty .
"	self reBuild"

! !

!GcListUI methodsFor: 'ui actions' stamp: 'hk 6/3/2015 14:03'!
editSelection
	selection  ifNil: [ ^self notify: 'no selection' ] .
	self initializeSubContextUIon: selection . 
	selection setDataNotPending.
	context checkDirty .
"	self reBuild
"! !

!GcListUI methodsFor: 'testing' stamp: 'hk 4/13/2015 12:51'!
hasSubUI   
	^ hasSubUI =  true! !

!GcListUI methodsFor: 'initialization' stamp: 'hk 6/7/2015 13:30'!
initializeAuthoredMenu
	| environment |
	environment := { context . self }.	
	self menuRow addGroup: 
		[ :g | context menuBehaviors do:  
			[:r |  g addMenuItem: (r 
				menuFor: environment 
				wrapWith: [ :s | self notify: s ] )  ]] 
! !

!GcListUI methodsFor: 'initialization' stamp: 'hk 5/12/2015 00:18'!
initializeLayout
	self pIcon icon: self contextIcon  .
	self layout: (SpecLayout composed newColumn: [ :c | 
		c 	newRow: [ :r | r
				add:  #pIcon width: 24 ;
				add:  #pLabel  ] height: self singleHeight .  
		self hasSubUI ifTrue: [ c add: #subUI ] 
			 ifFalse: [c add: #menuRow height: self singleHeight .
					c add: #list] 
		] )  .
	self needFullRebuild: false .
! !

!GcListUI methodsFor: 'initialization' stamp: 'hk 6/3/2015 16:44'!
initializeMenu
	self menuRow addGroup: 
		[ :g |  g 
			addItem: [ :i | i
				icon: (GcIcons iconAt: #edit) ;
				description: 'edit selection';
				action: [ self editSelection  ] ];
			addItem: [ :i | i 
				icon: (GcIcons iconAt: #delete) ;
				description: 'delete selection';
				action: [ self deleteSelection ] ] ;
			addItem: [ :i | i 
				icon: (GcIcons iconAt: #add) ;
				description: 'add copy';
				action: [ self activateSubContextForNew ] ] ]
 ! !

!GcListUI methodsFor: 'initialization' stamp: 'hk 5/31/2015 18:08'!
initializePresenter
	self list whenSelectedItemChanged: [ :item | selection :=  item ] .
	context dataPending whenChangedDo: 
		[ self resetList .
		self initializeLayout .
		self reBuild ]
! !

!GcListUI methodsFor: 'ui actions' stamp: 'hk 5/9/2015 14:00'!
initializeSubContextUIon: aVO
	self subcontext vo: aVO .
	self assign: (self subcontext newUI) to: #subUI .
	hasSubUI  := true .
	self pLabel label: (self subcontext displayName) 



! !

!GcListUI methodsFor: 'initialization' stamp: 'hk 6/10/2015 11:34'!
initializeWidgets
	self initializeSuperWidgets .
	self instantiateModels: #(
		list			TableListUI 
		menuRow	MenuModel
		) .
	self pLabel label: context displayName  .
	self list setView: context view .
	self resetList .
	self initializeMenu .
	self initializeAuthoredMenu .
	self initializeLayout.
	
! !

!GcListUI methodsFor: 'private' stamp: 'hk 4/13/2015 17:17'!
reBuild
	self initializeLayout.
	self needFullRebuild: false .
	self buildWithSpec
! !

!GcListUI methodsFor: 'protocol' stamp: 'hk 4/23/2015 14:43'!
resetList
	self list items: context list ; resetSelection .
! !

!GcListUI methodsFor: 'accessing' stamp: 'hk 4/27/2015 14:13'!
selection 
 	^selection! !

!GcListUI methodsFor: 'accessing' stamp: 'hk 4/13/2015 12:22'!
subcontext 
 	^context subcontext! !

GcUI subclass: #GcMainUI
	instanceVariableNames: 'menu chosenWidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!GcMainUI commentStamp: 'hk 6/3/2015 12:26' prior: 0!
I am the app main window model.

My subcontext related widgets must be addressed in action methods with the uiKey authored in  the subContext.
the menu is hand written, NOT generated from DSL text 

Instance Variables
	chosenWidget:		<Symbol | nil>		the widget to be displayed next 
	menu:				<MenuModel>		main menu

======= Tests =======

GcExample exampleUC1 !

!GcMainUI methodsFor: 'private' stamp: 'hk 4/12/2015 14:10'!
ableMainButtonMenu: aBoolean 	
	menu menuGroups do: [ :g |
		g menuItems do: [ :m | m enabled: aBoolean ] ]  ! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 5/12/2015 00:09'!
addBizNetButtonGroup:g  	
			g addItem: [  :i | i
				icon: (GcIcons default logisticsDataIcon) ;
				description: 'show logistics data' ;
				action: [  ] ] .
			g addItem: [  :i | i
				icon: (GcIcons default bizProcessIcon) ;
				description: 'show supplier processes' ;
				action: [  ] ] .
			g addItem: [  :i | i
				icon: (GcIcons default bizNetIcon) ;
				description: 'show biz net' ;
				action: [  ] ] ! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 5/11/2015 12:05'!
addCommunicationButtonGroup:g	
			g addItem: [  :i | i
				icon: (GcIcons default contactsPaneIcon) ;
				description: 'show contacts' ;
				action: [ self contactsAction ] ] .
			g addItem: [  :i | i
				icon: (GcIcons default routeIcon) ;
				description: 'show route planning' ;
				action: [  ] ] .
			g addItem: [  :i | i
				icon: (GcIcons default mailIcon );
				description: 'show mail' ;
				action: [  ] ]  ! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 5/11/2015 12:04'!
addControllingButtonGroup:g 	
			g addItem: [  :i | i
				icon: (GcIcons default controllingIcon) ;
				description: 'show controlling' ;
				action: [  ] ] ! !

!GcMainUI methodsFor: 'initialization' stamp: 'hk 4/12/2015 15:06'!
addLogin: aRow
	aRow	add: #loginLabel width: 120;
			add: #loginNameLabel width: 160;
			add: #logoutButton width: 80! !

!GcMainUI methodsFor: 'initialization' stamp: 'hk 4/12/2015 15:06'!
addLogout: aRow
	aRow	add: #loginName width: 120;
			add: #password width: 120;
			add: #loginButton width: 80! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 5/6/2015 12:32'!
addMenuGap: g	
	 g addItem: [ :i | 
		i icon: (Smalltalk ui icons blankIconOfWidth: 130) ] ! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 5/11/2015 12:05'!
addMiscellaneousButtonGroup: g
	g	addItem: [ :i | i
				icon: (GcIcons default calcPaneIcon) ;
				description: 'calculator';
				action: [ self calcAction] ].
	g	addItem: [ :i | i
				icon: (GcIcons default settingsIcon) ;
				description: 'settings';
				action: [ self inspect ] ].
	g	addItem: [ :i | i
				icon: (GcIcons  default userIcon) ;
				description: 'show user data';
				action: [  ] ]! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 5/11/2015 20:47'!
addProcessButtonGroup: g 	
			g addItem: [  :i | i
				icon: (GcIcons default dashPaneIcon) ;
				description: 'show dash board' ;
				action: [ self dashboardAction] ] .
			g addItem: [  :i | i
				icon: self bizProcessesPaneIcon ;
				description: 'show biz processes' ;
				action: [ self processesAction ] ] ! !

!GcMainUI methodsFor: 'error handling' stamp: 'hk 5/25/2015 15:41'!
addStatus: aString
	"may be enhanced to keep a stack"
	aString isEmpty ifFalse:  [self status text: (aString, ' | ', self status text)] ! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 6/18/2015 14:40'!
addTourButtonGroup: g 	
			g addItem: [  :i | i
				icon: (GcIcons default taskOrdersPaneIcon) ;
				description: 'show task orders' ;
				action: [ self taskOrdersAction ] ] .
			g addItem: [  :i | i
				icon: (GcIcons default tourPlanIcon) ;
				description: 'show transport planning' ;
				action: [  ] ] .
			g addItem: [  :i | i
				icon: (GcIcons default tourIcon) ;
				description: 'show tour planning' ;
				action: [  ] ] .
			g addItem: [  :i | i
				icon: (GcIcons default tourSubIcon) ;
				description: 'show supplier processes' ;
				action: [ self supplierProcessesAction ] ] ! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 5/31/2015 18:09'!
bizProcessesPaneIcon
	^context hasDataPending 
		ifTrue:  [GcIcons default bizProcessesPaneDirtyIcon]
		ifFalse: 	[GcIcons default bizProcessesPaneIcon]
! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 4/28/2015 15:43'!
calcAction
	chosenWidget := #calcPane .
	self reBuild! !

!GcMainUI methodsFor: 'initialization' stamp: 'hk 4/12/2015 14:31'!
chosenWidget
	^ chosenWidget ifNil: [ chosenWidget := #emptyPane ]
! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 4/27/2015 22:02'!
contactsAction
	chosenWidget := #contactsPane .
	self contactsPane resetList .
	self reBuild! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 4/28/2015 15:43'!
dashboardAction
	chosenWidget := #dashPane .
	self reBuild! !

!GcMainUI methodsFor: 'constants' stamp: 'hk 4/12/2015 14:38'!
initialExtent
	^1204 @ 932 ! !

!GcMainUI methodsFor: 'initialization' stamp: 'hk 6/2/2015 17:34'!
initializeHeader
	self instantiateModels: #(
		gloConnButton		ButtonModel
		loginButton			ButtonModel
		logoutButton		ButtonModel
		loginLabel			LabelModel
		loginName			TextInputFieldModel 
		password			TextInputFieldModel
		loginNameLabel	LabelModel
		emptyPane			ButtonModel
		companyLabel		LabelModel
		legal				TextModel
		status				TextModel
		) .
	self gloConnButton icon: (GcIcons default headerIcon) .
	self loginButton 
			action: [ self loginAction ];
			state: true;
			label: #login. 
	self logoutButton
			action: [ self logoutAction ];
			state: true;
			label: #logout .
	self loginLabel 	label: 'Eingeloggt als ' .
	self emptyPane label: 'please login' .
	self loginName 
			autoAccept: true ;
			ghostText: 'login name'  .
	self password
			autoAccept: true ;
			ghostText: 'password' .
	self legal 
			enabled: false; 
			text: 'GloConn International © 2015'  .
	self status enabled: false ! !

!GcMainUI methodsFor: 'initialization' stamp: 'hk 6/2/2015 17:34'!
initializeLayout 
	self layout: (SpecLayout composed newColumn: [ :c | c
			newRow: [ :r | r add: #gloConnButton] height: self logoHeight ;		"logo"
			newRow: [ :r | 										"login"
				r add: #companyLabel.
				self loginName text isEmpty
					ifTrue: [ self addLogout: r ]
					ifFalse: [ self addLogin: r ] ] height: self singleHeight ;
			newRow: [ :r | r add: #menu ] height: self singleHeight ;				"main menu buttons"
			newRow: [ :r | r add: #status ] height: self reducedHeight ;			"status line"
			add: self chosenWidget ;							"the widget chosen via main menu"
			newRow: [ :r | r add: #legal ] height: self reducedHeight ] ).
	self needFullRebuild: false.! !

!GcMainUI methodsFor: 'initialization - menu' stamp: 'hk 4/12/2015 14:09'!
initializeMainButtonMenu 	
	(menu := MenuModel new) addGroup: [ :g |
		self addProcessButtonGroup: g .
		self addMenuGap: g .
		self addTourButtonGroup: g .
		self addMenuGap: g .
		self addControllingButtonGroup: g .
		self addMenuGap: g .
		self addCommunicationButtonGroup: g .
		self addMenuGap: g .
		self addBizNetButtonGroup: g .
		self addMenuGap: g .
		self addMiscellaneousButtonGroup: g ].
	self ableMainButtonMenu: false .! !

!GcMainUI methodsFor: 'initialization' stamp: 'hk 5/31/2015 18:08'!
initializePresenter
	context dataPending whenChangedDo: 
		[ self initializeMainButtonMenu.
		self loginName text isEmpty ifFalse: [self ableMainButtonMenu: true] .
		self initializeLayout .
		self reBuild ]
! !

!GcMainUI methodsFor: 'initialization' stamp: 'hk 6/2/2015 19:43'!
initializeWidgets
	self title: 'GloConn-App version ', self packageVersion .
	self initializeHeader .
	self initializeMainButtonMenu .	
	context subcontexts do: [ :sc | self assign: sc newUI to: sc uiKey] .	
	self initializeLayout ! !

!GcMainUI methodsFor: 'private' stamp: 'hk 4/12/2015 15:17'!
login:  aString  password: anotherString 
	 ^aString isNotEmpty ! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 5/22/2015 14:51'!
loginAction
	self loginNameLabel label:  self loginName text.
	(self login: self loginName text password: self password text)
		ifFalse: [ ^self logoutAction  ].
	self ableMainButtonMenu: true .
	self dashboardAction  ! !

!GcMainUI methodsFor: 'constants' stamp: 'hk 4/12/2015 14:21'!
logoHeight
	^ 150! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 4/12/2015 15:15'!
logoutAction
	self  loginName text: '' .
	self password text: '' .
	chosenWidget  := #emptyPane .
	self ableMainButtonMenu: false .
	self reBuild ! !

!GcMainUI methodsFor: 'accessing' stamp: 'hk 4/12/2015 15:08'!
menu
	^ menu! !

!GcMainUI methodsFor: 'error handling' stamp: 'hk 4/13/2015 15:31'!
notify: aString
	"write the status line   -   tbd"
	self addStatus: aString! !

!GcMainUI methodsFor: 'private' stamp: 'hk 6/2/2015 19:43'!
packageVersion
	^self class package  mcPackage workingCopy versionInfo name! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 4/28/2015 15:50'!
processesAction
	chosenWidget := #bizProcessesPane .
	self bizProcessesPane resetList .
	self reBuild! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 6/18/2015 14:40'!
supplierProcessesAction
	chosenWidget := #tourSub .
	self reBuild! !

!GcMainUI methodsFor: 'ui actions' stamp: 'hk 4/28/2015 15:57'!
taskOrdersAction
	chosenWidget := #taskOrdersPane .
	self reBuild! !

!GcUI class methodsFor: 'instance creation' stamp: 'hk 4/13/2015 10:51'!
on: aContext
	^ self basicNew
		context: aContext;
		initialize! !

!GcUI methodsFor: 'accessing' stamp: 'hk 4/13/2015 10:52'!
context
	^context ! !

!GcUI methodsFor: 'accessing' stamp: 'hk 4/13/2015 10:51'!
context: aCF
	context := aCF .! !

!GcUI methodsFor: 'initialization' stamp: 'hk 4/13/2015 13:03'!
initializeSuperWidgets
	self instantiateModels: #(
		pIcon		ButtonModel
		pLabel		LabelModel
		) .	
	self pLabel label: 'undefined'.
! !

!GcUI methodsFor: 'error handling' stamp: 'hk 4/27/2015 12:43'!
notify: aString
	aString ifNotEmpty: [owner notify: aString] 
	
	! !

!GcUI methodsFor: 'private' stamp: 'hk 4/27/2015 13:43'!
reBuild
	self initializeLayout.
	self needFullRebuild: false .
	self buildWithSpec
! !

!GcUI methodsFor: 'constants' stamp: 'hk 4/13/2015 11:41'!
reducedHeight
	^self class toolbarHeight *0.8 ! !

!GcUI methodsFor: 'constants' stamp: 'hk 4/13/2015 11:41'!
singleHeight
	^self class toolbarHeight ! !

DynamicComposableModel subclass: #GcVOEditor
	instanceVariableNames: 'vo emptyLabel doneButton identityLabel subWidget inspectButton inspectVoButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-VO'!

!GcVOEditor commentStamp: 'hk 6/15/2015 11:06' prior: 0!
I am the VO editor, wrapping a GcVoWidget.
I add a done and inspect button to the GcVoWidget.

Instance Variables
	vo:					<VO>
	emptyLabel:		<LabelModel>
	identityLabel:		<LabelModel>
	doneButton:		<ButtonModel>
	inspectButton:		<ButtonModel>
	inspectVOButton:	<ButtonModel>
	subWidget:			<GloConnVoWidget>

========== Tests ==========

(GcVOEditor on: ((ThingVO newOfType: #bla) 
	val: #bla1 at: #value; 
	date: Date today at: #Datum;
	yourself )  ) 
	openWithSpec
	
GcExample initSVO .	
(GcVOEditor on: (GcExample contact   ) ) openWithSpec
!

!GcVOEditor class methodsFor: 'instance creation' stamp: 'hk 5/22/2015 21:24'!
on: aVO

	(aVO isKindOf: ThingVO) ifTrue: [^self basicNew vo: aVO; initialize] .
	(aVO isKindOf: CfPlainVO ) 
		ifTrue: [ ^self basicNew vo: aVO vo; initialize] 
		ifFalse: [ ^self error: 'GcVoUI wants a DomainVO or a CfPlainVO' ].
	! !

!GcVOEditor class methodsFor: 'instance creation' stamp: 'hk 4/8/2015 16:57'!
title
	^ 'VO editor'! !

!GcVOEditor methodsFor: 'ui actions' stamp: 'hk 4/12/2015 13:15'!
done
	self window close! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 4/12/2015 13:13'!
doneButton
	^doneButton! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 3/21/2015 20:18'!
emptyLabel
	^emptyLabel! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 3/21/2015 20:18'!
identityLabel
	^ identityLabel ! !

!GcVOEditor methodsFor: 'initialization' stamp: 'hk 6/16/2015 09:26'!
initialExtent
	
		^600 @  (self widgetHeight + self class toolbarHeight )
! !

!GcVOEditor methodsFor: 'initialization' stamp: 'hk 6/15/2015 12:10'!
initializeLayout

	| h |
	h := self class toolbarHeight .
	self layout: (SpecLayout composed newColumn: [ :c | 
		c newRow: [ :r | 
				r add: #identityLabel.
				r  add: #inspectButton width: self class buttonWidth .
				r  add: #inspectVoButton width: self class buttonWidth .
				r  add: #doneButton width: 50 ] height: h .
		c newRow: [ :r | r add: #subWidget ] .
		c newRow: [ :r | r add: #emptyLabel ] height: 1 ] ).
	self needRebuild: false ! !

!GcVOEditor methodsFor: 'initialization' stamp: 'hk 6/16/2015 09:22'!
initializeWidgets
	emptyLabel := self newLabel .
	(identityLabel := self newLabel)
		label: (vo userName, ' id: ', vo id asString) .
	(inspectButton := self newButton ) 
		label: 'inspect UI';
		action: [ self inspect ];
		state: true.
	(inspectVoButton := self newButton ) 
		label: 'inspect VO';
		action: [ vo inspect ];
		state: true.
	(doneButton := self newButton ) 
		label: 'done';
		action: [ self done ];
		state: true.
	(subWidget := GcVoWidget on: vo key: vo userName ) owner: self.
	self initializeLayout! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 4/12/2015 13:17'!
inspectButton
	^inspectButton! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 6/15/2015 10:59'!
inspectVoButton
	^inspectVoButton! !

!GcVOEditor methodsFor: 'ui actions' stamp: 'hk 4/11/2015 18:52'!
reBuild
	"recursion stop"
! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 4/7/2015 12:37'!
subWidget
	^subWidget! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 3/21/2015 20:18'!
vo: aVO
	vo := aVO! !

!GcVOEditor methodsFor: 'accessing' stamp: 'hk 6/15/2015 10:54'!
widgetHeight
	^ 1 + self class toolbarHeight + self subWidget widgetHeight! !

DynamicComposableModel subclass: #SvoCalc
	instanceVariableNames: 'display button0 button1 button2 button3 button4 button5 button6 button7 button8 button9 buttonPlus buttonMinus buttonMult buttonDiv buttonEq buttonDot buttonSqr buttonLog buttonChange buttonClear previousNumber currentOperation operationPending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Utilities'!

!SvoCalc commentStamp: 'hk 4/13/2015 17:37' prior: 0!
SvoCalc new openWithSpec

A SpoCalc is the Spec Demo Calculator enhanced.

Instance Variables
	display:				<TextInput>	the calculators input & display
	currentOperation:	<String>		the operation to be performed:  +  -  * / 
	operationPending:	<Bolean>		true after pressing +  -  * /  =
										means: new input expected
	previousNumber:	<Number | nil>	stores the first value entered
										no operation pending ifNil
			
	button0:			<Button>
	button1:			<Button>
	button2:			<Button>
	button3:			<Button>
	button4:			<Button>
	button5:			<Button>
	button6:			<Button>
	button7:			<Button>
	button8:			<Button>
	button9:			<Button>
	buttonChange:		<Button>
	buttonClear:		<Button>
	buttonDiv:			<Button>
	buttonDot:			<Button>
	buttonEq:			<Button>
	buttonLog:			<Button>
	buttonMinus:		<Button>
	buttonMult:			<Button>
	buttonPlus:			<Button>
	buttonSqr:			<Button>

!

!SvoCalc class methodsFor: 'specs' stamp: 'hk 1/5/2015 14:43'!
title 
	^ 'SVO Calculator'! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/5/2014 17:01'!
addition
	buttonPlus
		label: '+';
		action: [ self operation: $+ ];
		state: true! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:37'!
button0
	^ button0! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:35'!
button1
	^ button1! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button2
	^ button2! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button3
	^ button3! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button4
	^ button4! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button5
	^ button5! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button6
	^ button6! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button7
	^ button7! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button8
	^ button8! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
button9
	^ button9! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:35'!
buttonChange
	^ buttonChange! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:35'!
buttonClear
	^ buttonClear! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
buttonDiv
	^ buttonDiv! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:37'!
buttonDot
	^ buttonDot! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:37'!
buttonEq
	^ buttonEq! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:35'!
buttonLog
	^ buttonLog! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:37'!
buttonMinus
	^ buttonMinus! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:36'!
buttonMult
	^ buttonMult! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:37'!
buttonPlus
	^ buttonPlus! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:35'!
buttonSqr
	^ buttonSqr! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/5/2014 18:02'!
changeSign
	buttonChange
		label: '+/ -';
		action: [ display label: display label asNumber negated asString ];
		state: true! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/6/2014 16:13'!
clearDisplay
	buttonClear
		label: 'C';
		action: [ display label: '0' ];
		state: true! !

!SvoCalc methodsFor: 'private' stamp: 'hk 11/6/2014 14:56'!
defineDigitButton: aButton label: aString
	aButton
		label: aString;
		action: [ display label: (self newDisplayLabelFor: aString) asNumber asString  ];
		state: true! !

!SvoCalc methodsFor: 'button definition' stamp: 'hk 11/6/2014 14:57'!
defineDigitButtons
	button0
		label: '0' ;
		action: [ display label: (self newDisplayLabelFor: '0' )  ];
		state: true.
	self defineDigitButton: button1 label: '1'.
	self defineDigitButton: button2 label: '2'.
	self defineDigitButton: button3 label: '3'.
	self defineDigitButton: button4 label: '4'.
	self defineDigitButton: button5 label: '5'.
	self defineDigitButton: button6 label: '6'.
	self defineDigitButton: button7 label: '7'.
	self defineDigitButton: button8 label: '8'.
	self defineDigitButton: button9 label: '9'.
! !

!SvoCalc methodsFor: 'button definition' stamp: 'hk 11/5/2014 14:45'!
defineOtherButtons
	"basic arithmetic operations"
	self addition.
	self subtraction.
	self multiplication.
	self division.	
		
	"other arithmetic opertions"
	self squareRoot.
	self logarithm.
	self changeSign.	
		
	"remaining buttons"
	self clearDisplay.
	self enterDot.
	self equals! !

!SvoCalc methodsFor: 'accessing' stamp: 'hk 11/5/2014 17:34'!
display
	^ display! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/5/2014 17:06'!
division
	buttonDiv
		label: '/';
		action: [ self operation: $/ ];
		state: true! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/6/2014 15:56'!
enterDot
	buttonDot
		label: '.';
		action: [display label: (self newDisplayLabelForDot )  ];
		state: true! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/6/2014 16:12'!
equals
	buttonEq
		label: '=';
		action: [ operationPending := true. 
				previousNumber ifNotNil: 
					[display label: (previousNumber perform: currentOperation asSymbol with:  display label asNumber ) asString].
				previousNumber := nil ];
		state: true! !

!SvoCalc methodsFor: 'initialization' stamp: 'hk 4/13/2015 17:47'!
initialExtent
	"(4@4) is the size of the border"

	^ 251 @ 191 ! !

!SvoCalc methodsFor: 'initialization' stamp: 'hk 11/6/2014 15:36'!
initialize
	operationPending  := false.
	super initialize! !

!SvoCalc methodsFor: 'initialization' stamp: 'hk 11/5/2014 14:33'!
initializeDigitButtons
	button0 := self newButton.
	button1 := self newButton.
	button2 := self newButton.
	button3 := self newButton.
	button4 := self newButton.
	button5 := self newButton.
	button6 := self newButton.
	button7 := self newButton.
	button8 := self newButton.
	button9 := self newButton.
	self defineDigitButtons! !

!SvoCalc methodsFor: 'initialization' stamp: 'hk 1/25/2015 20:49'!
initializeLayout
	self layout: (SpecLayout composed
		newColumn: [ :column | column
						add: #display ;
						newRow: [ :row1 | row1
										add: #buttonClear;
										add: #buttonLog;
										add: #buttonChange;
										add: #buttonSqr ] height: 26;
						newRow: [ :row2 | row2
										add: #button1;
										add: #button2;
										add: #button3;
										add: #buttonDiv ] height: 26;
						newRow: [ :row3 | row3
										add: #button4;
										add: #button5;
										add: #button6;
										add: #buttonMult ] height: 26;
						newRow: [ :row4 | row4
										add: #button7;
										add: #button8;
										add: #button9;
										add: #buttonMinus ] height: 26;
						newRow: [ :row5 | row5
										add: #buttonDot;
										add: #button0;
										add: #buttonEq;
										add: #buttonPlus ] height: 26 ];
		yourself)! !

!SvoCalc methodsFor: 'initialization' stamp: 'hk 11/5/2014 14:46'!
initializeOtherButtons
	"instantiate the basic arithmetic buttons"

	buttonPlus := self newButton.
	buttonMinus := self newButton.
	buttonMult := self newButton.
	buttonDiv := self newButton.
	buttonEq := self newButton.	
		
	"instantiate the remaining buttons"
	buttonDot := self newButton.
	buttonSqr := self newButton.
	buttonLog := self newButton.
	buttonChange := self newButton.
	buttonClear := self newButton.
	
	self defineOtherButtons! !

!SvoCalc methodsFor: 'initialization' stamp: 'hk 1/25/2015 19:58'!
initializeWidgets 
	self initializeDigitButtons. 
	self initializeOtherButtons. 
	display := self newLabel.
	self initializeLayout ! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/5/2014 17:17'!
logarithm
	buttonLog
		label: 'Log';
		action: [ display label: display label asNumber log asString ];
		state: true! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/5/2014 18:26'!
multiplication
	buttonMult
		label: 'x';
		action: [ self operation: $* ];
		state: true! !

!SvoCalc methodsFor: 'private' stamp: 'hk 11/6/2014 15:31'!
newDisplayLabelFor: aString
	^operationPending ifTrue: [operationPending  := false. aString ]  ifFalse: [ display label , aString ] ! !

!SvoCalc methodsFor: 'private' stamp: 'hk 11/6/2014 16:06'!
newDisplayLabelForDot
	^ (operationPending or: [display label isEmpty])
		ifTrue: [ operationPending := false.  '0.' ]
		ifFalse: [  (display label includes: $.) 
						ifTrue: [ display label ] 
						ifFalse:  [ display label,  '.' ] ]! !

!SvoCalc methodsFor: 'private' stamp: 'hk 11/6/2014 16:16'!
operation: aCharacter
	"Defines which operation will be performed: it can be a + / - x"

	previousNumber := display label asNumber.	
	currentOperation := aCharacter.				
	operationPending  := true.! !

!SvoCalc methodsFor: 'private' stamp: 'hk 11/5/2014 17:08'!
sqrCheck
	"checks whether we are attempting to calculate the square root of a negative
number"

	display label asNumber < 0
		ifTrue: [ display label: 'Error' ]
		ifFalse: [ display label: display label asNumber sqrt asString ]! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/5/2014 17:07'!
squareRoot
	buttonSqr
		label: 'Sqr';
		action: [ self sqrCheck ];
		state: true! !

!SvoCalc methodsFor: 'operations' stamp: 'hk 11/5/2014 17:05'!
subtraction
	buttonMinus
		label: '-';
		action: [ self operation: $- ];
		state: true! !

DynamicComposableModel subclass: #TableListUI
	instanceVariableNames: 'list emptyLabel header view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-UI-context'!

!TableListUI commentStamp: 'hk 6/9/2015 13:05' prior: 0!
A TableListUI is a list widget with row titles 
(may be replaced by a more sophisticated widget).

Hence the list items must respond to: 			This dependence can be weakened
-  displayArray and 								by introducing two inst vars:
-  displayLabelArray 							- one for the displayBlock of the list 
and it must carry an automatic id !!!!!!				- one for the header text (asReactiveVar))

Instance Variables
	emptyLabel:		<LabelModel> 	needed for fill in the blanks
	list:					<MultiColumnListModel>	
	header: 			<LabelModel>
	view: 				<ViewVO>				for list display & filtering


======= Tests ======

| ct |
(ct := GcExample contact) copy .
(TableListUI new items: ct copies asOrderedCollection ) openWithSpec!

!TableListUI methodsFor: 'accessing' stamp: 'hk 2/14/2015 18:08'!
emptyLabel
	^ emptyLabel! !

!TableListUI methodsFor: 'accessing' stamp: 'hk 2/15/2015 17:00'!
header
	^ header ! !

!TableListUI methodsFor: 'accessing' stamp: 'hk 3/23/2015 12:21'!
height
	^(self list listSize max: 3)* self class toolbarHeight! !

!TableListUI methodsFor: 'initialization' stamp: 'hk 4/13/2015 11:49'!
initializeLayout

	self layout: (SpecLayout composed
		newColumn: [ :c | 
			c add: #header height: self class toolbarHeight .  
			c  add: #list  ] )! !

!TableListUI methodsFor: 'initialization' stamp: 'hk 6/9/2015 14:22'!
initializePresenter
	list whenListChanged: [ header label: (list listSize = 0
			ifTrue: ['']
			ifFalse: [self wrapItemString: ((self view target: list listItems first) displayLabelArray ) ] ) ]! !

!TableListUI methodsFor: 'initialization' stamp: 'hk 6/10/2015 11:39'!
initializeWidgets
	emptyLabel := self newLabel  .
	list := self newMultiColumnList  .
	header := self newLabel .
	list displayBlock: [ :item | self wrapItem: ((self view target: item) displayArray) ] .
	list menu: [ :m | self listMenu: m ] .
	self needFullRebuild: false . 
	self initializeLayout! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 2/14/2015 17:15'!
items: aCollection
	list items: aCollection! !

!TableListUI methodsFor: 'accessing' stamp: 'hk 2/14/2015 18:08'!
list
	^ list! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 2/21/2015 14:38'!
listItems
	^list listItems! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 6/10/2015 11:41'!
listMenu: m
	| item |
	item := m add: #inspect target: self list selectedItem selector:  #inspect .
	item enablementSelector: [ self list selectedItem isNotNil ] .
	^m! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 2/14/2015 18:02'!
menu:  aBlock
	" [ :menu :shifted | nil ] "
	list menu: aBlock  ! !

!TableListUI methodsFor: 'private' stamp: 'hk 6/9/2015 15:19'!
paddingFor: aString
	"shall mimick equal distance"
	^30 - ((aString size /3 ) rounded)
	
	! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 3/27/2015 16:46'!
resetSelection
	list resetSelection! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 6/10/2015 10:18'!
selectedItem
	^list selectedItem! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 2/21/2015 14:42'!
setSelectedItem: anItem
	list setSelectedItem: anItem! !

!TableListUI methodsFor: 'accessing' stamp: 'hk 6/9/2015 13:04'!
setView: aView
	view := aView! !

!TableListUI methodsFor: 'accessing' stamp: 'hk 3/23/2015 12:09'!
size
	^self list listSize ! !

!TableListUI methodsFor: 'protocol' stamp: 'hk 2/15/2015 17:09'!
title

	^ 'TableList'! !

!TableListUI methodsFor: 'accessing' stamp: 'hk 6/9/2015 14:23'!
view
	^view ifNil: [ FullViewVO new ]! !

!TableListUI methodsFor: 'protocol-events' stamp: 'hk 2/14/2015 17:18'!
whenSelectedItemChanged: aBlock
	list whenSelectedItemChanged: aBlock! !

!TableListUI methodsFor: 'private' stamp: 'hk 4/23/2015 19:21'!
wrapItem: anArray
	"construct the displayString by adjusting list spacings"
	
	^anArray collect: [ :s | s padRightTo: 22 ] 
	
	
	! !

!TableListUI methodsFor: 'private' stamp: 'hk 4/23/2015 19:20'!
wrapItemString: anArray
	"construct the displayString by adjusting list spacings"
	
	| str |
	str := WriteStream on: '' .
	anArray do: [ :s | str nextPutAll: (s padRightTo: (self paddingFor: s)) ].
	^str contents

	
	! !

RTUMLClassBuilder subclass: #RTSVOBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Utilities'!

!RTSVOBuilder commentStamp: 'hk 5/26/2015 22:41' prior: 0!
I extend the RTUMLClassBuilder in order to show SVO classes with just the 'protocol protocol'!

!RTSVOBuilder methodsFor: 'accessing' stamp: 'hk 5/27/2015 11:22'!
attributesOf: object	
	^ object class methods select: [ :m | m protocol =  'authoring'] thenCollect: [ :m | m selector ]! !

!RTSVOBuilder methodsFor: 'accessing' stamp: 'hk 5/31/2015 14:32'!
methodsForClass: obj
	^ (obj methodsInProtocol: 'authoring') 
			asSortedCollection: [ :a :b | a selector < b selector ]! !

RTUMLClassBuilder subclass: #RTViewBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Utilities'!

!RTViewBuilder commentStamp: 'hk 6/2/2015 20:04' prior: 0!
I extend the RTUMLClassBuilder in order to show the relation of Views to their targets!

!RTHTML5Exporter class methodsFor: '*SVO2' stamp: 'hk 5/27/2015 15:55'!
export: aView  
	self new export: aView ! !

ThemeIcons subclass: #GcIcons
	instanceVariableNames: ''
	classVariableNames: 'Icons'
	poolDictionaries: ''
	category: 'SVO2-Utilities'!

GcIcons class
	instanceVariableNames: 'default'!

!GcIcons commentStamp: 'hk 5/11/2015 13:06' prior: 0!
A GcIcons is a container for the GloConn icons.

GcIcons default createIconMethodsFromDirectory: 'Icons'

don't forget to rename the protocol for own package!

GcIcons class
	instanceVariableNames: 'default'!

!GcIcons class methodsFor: 'accessing' stamp: 'hk 5/11/2015 12:16'!
at: anIconSymbol
		^self at: anIconSymbol ifAbsent: [  ]! !

!GcIcons class methodsFor: 'accessing' stamp: 'hk 5/1/2015 19:38'!
at: anIconSymbol ifAbsent: aBlock
	| sel |
	sel :=  (anIconSymbol, #Icon) asSymbol .
	(self default respondsTo: sel) ifFalse: [ ^aBlock value ].
	^self default perform: sel! !

!GcIcons class methodsFor: 'accessing' stamp: 'hk 5/1/2015 18:52'!
default
		^default ifNil: [ default := self new ]! !

!GcIcons class methodsFor: 'icons' stamp: 'hk 6/3/2015 16:44'!
iconAt: aKey
	(aKey = 'Customer' ) ifTrue: [ ^Smalltalk ui icons userIcon ].
	(aKey = 'Service' ) ifTrue: [ ^Smalltalk ui icons forwardIcon ].
	(aKey = 'Shared Information' ) ifTrue: [ ^Smalltalk ui icons smallCopyIcon ].
	(aKey = #add) ifTrue: [ ^Smalltalk ui icons smallAddIcon ].
	(aKey = #back) ifTrue: [ ^Smalltalk ui icons backIcon ].
	(aKey = #blank144) ifTrue: [ ^Smalltalk ui icons blankIconOfWidth: 144 ].
	(aKey = #edit) ifTrue: [ ^Smalltalk ui icons editIcon].
	(aKey = #delete) ifTrue: [ ^Smalltalk ui icons deleteIcon ].
	(aKey = #add) ifTrue: [ ^Smalltalk ui icons addIcon ].
	^nil
	! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
bizNetIcon
	"Private - Generated method"
	^icons 
			at: #bizNetIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self bizNetIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
bizNetIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/bizNet.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAYFJREFU
eJykk7tLw1AUxvP3uDgILjq6ODm56ObglIKgUKEKEQULBqRQRYWKFqz4yKCRYqstUqKI1AdU
UhVCDSqiKDi6HfsduLG3SShi4VC4ud/vfOdxFaX+U1WVTNP8U0CjCLFlWWTbNjmO4wu3VuNo
PMNdaBgCWpi4+nBPoxspGljUKXt+Rk+uK0EMwyDlKJ8PFONybCtN/Qtz/N81G6PKXVW60xIw
tDRPyewelW8r1DEdpdObK6mcUADEECFr2+QIi1FK8z0JAJEI1Ns5M84igErXl4EuPQBE26Ui
RTfX2DIy6/s7fB40BR9gJXfAIm03w5nRtPfnF19ZzSAPgE4D8v3xSZmTAvXoGluHQPQD31Fa
YBOREbMulC9oOJWk7vgEOwJ49fiQgb36FLVrY1yi2AepiZH1ZRYBgHkja7qYY8hgfZxfr2/c
J8CFC6mJOMTmNVpEHwABGPaRBECfg7BVFiE2sS8R55EC4K1yq8ckJvA7iUf5Mf33Of8AAAD/
/wMAW+YqMXKK/dkAAAAASUVORK5CYII='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
bizProcessIcon
	"Private - Generated method"
	^icons 
			at: #bizProcessIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self bizProcessIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
bizProcessIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/bizProcess.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAfNJREFU
eJyMU0FLW0EQjuDFi4ee66lX0R6EelDooSA1SKEKXpTqEyGQEJTGthhExCRgjIqRJK+RJLZJ
FWMwapBW0RaLwRoRESJBLVTtQf/FV7+R9wiGxC58b9/Oznzzzc6uwXA7vF4vPB4PVDVQGgE/
6EsYjUYw1sCfpfgiTs/OBNnsiY7c6W/dng/uxWLROwJmpuHv1SVmExmMBHYwHknL3Nq/hIY3
n/G8+5PM7yY2hJS+B5kMnE4ndILLiz/wLfzCROwQ4ZWszI3KAh6/8OFR4zTKa8fQbF1GNncu
vowpIKACf/wIsfUTHYsbOZicm6io8+BVf1JXUEBA48D4V8n45KWKll4VjqkvcE3P4+3wR7R0
DqGnzyUHmEqtFZZAAosjhbJqF8qfutFu8ePnj02k07sS4B5zwW63w2w2S8eKErDWyvpJWIcj
0h0Ga+DaZrOJipIKqpru5LPvwWBQAhhIUEFRArappi0sp+2dWxfZWkZNflECdsGhfsdoaF/g
DqxKdu7z0jxIQAUfJr9hJn4s6HkfkkCS/DcBz4A37llHFK9NKm6ur/Sea/UXEPDDBR3DiT10
DSah2FekhO2tLb1l9xVwTwi090BDZn9PwAC2TXtUvAv5CuhPQkP+UBRFGLmhgY5Evp1E+ku8
Hf8AAAD//wMAj0i5auNCpVEAAAAASUVORK5CYII='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/11/2015 14:42'!
bizProcessesPaneDirtyIcon
	"Private - Generated method"
	^icons 
			at: #bizProcessesPaneDirtyIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self bizProcessesPaneDirtyIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/12/2015 13:06'!
bizProcessesPaneDirtyIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/bizProcessesPaneDirty.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAARdJREFU
eJxi+P//PwMIz5k9+z8hzMAAUvofhQZrXr9h4//Xb97/X3ngDV4M1wTFYP606TP+P332Eq4A
HUh4LPsv7roUQzMcg5wGs90ldw9ejNUFyAYQAljDAN0FIOdyW839L+q0EI4F7BeAxdRC1qJY
BA5YdANAGtjNZ/9nMZ71P7z2CFghSDO6ASA9OA0AaQZhEAAZQrQB/uWHwAaAFIM03rv/EK6Z
KANAAGSzQ9YusGZk24n2AswQZL+TZADIGyBgm7aDPAP0ojeCDQFpAMUGDLMYz/yvGLgatwHE
5AUQfvHu+/8/f/8iDIBlJlB+AAl+/vQRLwapAakF6QEnxv/Q7NzXPwFsIihz4cMwNTB9AAAA
AP//AwDYOOC1RfYNAAAAAABJRU5ErkJggg=='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 19:21'!
bizProcessesPaneIcon
	"Private - Generated method"
	^icons 
			at: #bizProcessesPaneIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self bizProcessesPaneIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 19:21'!
bizProcessesPaneIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/bizProcessesPane.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAASlJREFU
eJxiYICCObNn/ycFMyCD9Rs2/n/95v3/lQfe4MUnLzz8//TZSzC+dOnK/+MnTv1nmDZ9BlgA
pAAbkPBY9l/cdSkY60VvhKsD6bl3/+F/sNNhtrvk7sGKzRO3/ue2mvtfOWg13ACQHgwD8AEF
n5X/1ULW4jcAZBvIqSDbRJ0WwrGA/QKwGFEGgDSwm8/+z2I863947RGwYpBmkgwAaQZhEAAZ
QrQB/uWHwAaAFIM0ghTANBNlAAiAbHbI2gWWRLadaC/ADEH2O0kGgLwBArZpO8gzAJTaQIaA
NIBiA4ZZjGf+VwzEk5CIyQsg/OLd9/9//v4FqwclZ3BmWrlqNZgDEvz86SNeDNMIyoBu7u6I
XNnXPwGcTUGZCx+GqYHpAwAAAP//AwAZPOXhpyPN0gAAAABJRU5ErkJggg=='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 19:21'!
calcPaneIcon
	"Private - Generated method"
	^icons 
			at: #calcPaneIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self calcPaneIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 19:21'!
calcPaneIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/calcPane.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAALhJREFU
eJxiYACCPavW/CcHM8A0R03r/c9QlkISzp49BWIImCBRMwzjNICxLA3CLknGqZm7KhPTAGSA
SyNYDmgwVgNQFGFhX79+HYzxGoDsAnQXIdNkuQAZEA4DUAAiYzQL8LqA7GgERR8Ig0yHYVwx
g+IC5HjHpYE4A3BgePSR6wJYoCEbIFaTgzCgZO507LZDQx/ZABjduXLp//aKSkSOBOE1Gzdi
xUvWrIazYWphmgEAAAD//wMAYv9oVAHLa7MAAAAASUVORK5CYII='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 19:21'!
contactsPaneIcon
	"Private - Generated method"
	^icons 
			at: #contactsPaneIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self contactsPaneIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 19:21'!
contactsPaneIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/contactsPane.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAblJREFU
eJyMU10rg2EY3h+gVg4cScmBIweccMhaqTemOJDPEsWBlEhhrezE12T2ae9SW2HkWylFiEVo
w6mP7Zdcuu551ti72lNXz/Pe7/1c93V/PCbT77Lb7XA6nUXDlLvcbjfu4vd4f3svCk9Pz9B1
HZqmQSLzcjqdxufHBy5urhA92xMnZfsP2knkcrlgohx+0HhyeY4Gbx8q11sxGJ5FIpk0JFCg
8ixB6juF6egSzD4LSkMWISEhiVNf33JB7QUJKL3a0w7zhhXNniHEHx8yUvdDGN10yP+CBIz0
+pJAf2QOtaEehI+3xIl7xZomqpge68VgeQQ0OKKrqA/2oirUiQ7/uBR0PuaRtKiKaR1enBoT
0LnG0yGRysItKPE2YTK6IHbrxgjKA5oUVgX7Q8CW0VlFIkhU5+vGztE+pmIrklbwIGJMcHV7
jS73hOTY6BsQ8MxCzuiLqAl2iioWmCR5KVCBEVjAZn1YLhNU1hYYy6rI60LupHFnzqyFSotg
R1SHhICjXGhsl7f9UhsOGMEzwXnIjjIXH4aSVcxjSiaSiO3GMo9JLbL9ByUagf9sNptc/gEA
AP//AwDkxNBN/fahIAAAAABJRU5ErkJggg=='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
controllingIcon
	"Private - Generated method"
	^icons 
			at: #controllingIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self controllingIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
controllingIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/controlling.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAaJJREFU
eJyUk98rQ2EYx0dCiSxcKEKupBZFuKDIPyB/gVxIWu0GwyLa8qMO6pA2tvaDlaPYlskmRWNm
Uxqrlcawm7lws0s3X563zmml9uPUt3N63uf7Oc/7vM8rk/09KpUKHMflJfKQV6ZUKiEIAqLR
KGKxWE6iXPIwCNEyJb/F3xFPfEpKXyOvjOd5KUAJYY8Xl9Ma3OpWcTG3gIO+QRgUHTC0dcLW
P4CwwyWByPsP8Oh0wb+oxdOuEcE1DvbuXuzUN2GlqBhcWTlCFmt2wNWkmpmpigd+m30bWlqx
XiHPDriz7LPE46FheCeU2KprgLagkGlTXp0d4DdZcT4yhq9gCL4ZDfia2vwBL0cn+EmlELXZ
WTVkXi4pxUZlVW4A6vyrw4nAkg5mRTtu5hdZY6manHpAf6Sm0bFRH76fI/CMjucHELXT2Mx6
ETGaGfTeZM6+hXSApauHzQSJ5iGwZ8oMiFz7cDY7h9MpNRNNI03fw6HA4rQuAtgo02Vyu90s
8JFIMCWSSUliTIzTm3LJQ17pRlIler0+J1GuaP4FAAD//wMAMDbkfc8FbOIAAAAASUVORK5C
YII='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 19:21'!
dashPaneIcon
	"Private - Generated method"
	^icons 
			at: #dashPaneIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self dashPaneIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/12/2015 13:06'!
dashPaneIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/dashPane.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAAAAXNSR0IArs4c6QAAAJJJREFU
eJxi+PvnF0mIAUJxh3hwh7iDkQt3qAuIhCAoGySF0ABWTRTCruHnFq6/WzmBJBDxhrng0wBR
hIm4Q1zBClywa4AYD7cErAGbDViVwrnYNeBB9NEQisPTuIwnKVhxRVyoK5o2vjBXgjHtzhPi
zhvmzhvqxh2KI2kQmZygie/7j8+vPrwnPnkDAAAA//8DABFxjsk2lKh5AAAAAElFTkSuQmCC'! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
dashboardIcon
	"Private - Generated method"
	^icons 
			at: #dashboardIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self dashboardIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
dashboardIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/dashboard.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAA+gAAAH0CAIAAAAhSpB6AAAAAXNSR0IArs4c6QAA6ZZJREFU
eJzsXUGLHccR9s/KySEYhA5JSAgrDI4PIckeBBHCIpEJOSkEgkl82s0hRiwJDma9GwLGizEB
X3LQRW+ET9HaN28W4pve/IHON1Nv6tVUVc8+7bzV2vHXFEVPzdfV1dX95n3Tr1d6qcwrTTlZ
LMvi6eL8WVc/e3SqdVSkLhXcglEubV0r0krrahG3aVvtTv1YXevIxVbTnfPx0GoxaN2Owo3F
jfRCPdGLz8DSt23LWtccapA2NmvUIacJVFh0IvEkmGWJrmp1F56OZTV9QxdakVBt126K1ZXF
u4S4BKYpSoP0fsyMuJT6dWgG4pZNmqtOlsWGapMcfdYCi0PQOcpnZOnBNmzXKpnE5Wj6aqvI
xWlHd+EsMB7Gw3gYz5XG474OnENXj0b7NI7P9mixePlKqvmBJd5ymKgjoDHfjJKEGvGIDV1W
/Rxl2atlQ524HlFvy1rbjrRu79rUqdEFbLUlTmkMLubVcIZFmA7TBqZGyEzi/dLM9iiSEReo
nRv/YRt/NjQ1bkZHt5Z+EbhLm1nvJyS0FuGEtjGPKulljMGshs11mjGXuufyFqcJ6bUL2i9K
6cg8HKOrRXgaJmvazcvSfwacTzuVbkVNP8QnVmDKjNNBpUvUfm4tL49rI0Y7qocl5P3EfAbP
+eIMz5rR+h+vmfQpUxvpwn1Ixw6np8OlN4GNF0m6JOJXmrtrYYyH8TAexnMV8SwMm48OrRNr
TJ7G4ydqbO5pj9E1uxu19ZnqUVTuqyTQKue/1tB9l+mlG106C+p87SRsMykhsXdHfVmAS+z4
a1TpeJyCuHhq1MsNLR3mWgYGAmlns+4tEPcycHcbqH13WXPW5ejSfgDS7VibMrcIHDJOgNrd
R91W7N1Y13fcNDCLSS9rsW3Yu3tqxNWWZqAWpyZ5Mbx66kpaV8aL0q7CNNUXPL8G57WYI1if
Vi5dtU97reI+8+7lIT6SYi9xCyeuzwk/cTi2noYdl5Adgj5kU6S91Lb20R9z6O5OzE78eLpn
4vRn1j2d4yKx8UQPduDpInQwxsN4GA/judJ41JX1vzDMbDH+uVsd2of5tL321eYu3VeAe25r
ADWMY8axlQ7WXsZd8JHP7KstmcoxFbZcK204XY/G0aSPf6NeW8ZHFdKJnpiIxlALtz7tQoqw
rVDu7RB3FhYWFhYWFhYWFpYrLSTuLCwsLCwsLCwsLF+DMou4txQKhUKhUCiUr4bIeYwX1rbp
/x70SqUYPTOwdIDReLk8PFeTOWULxL0Z/6kvNTU1NTU1NTX1C9D2ciYxK6HthZftNqhgzX8N
WSbxE4E5Y4qMlzG9E/FvnurrJO4UCoVCoVAolBcsmxDQzV2VwP5L8FljzJfmhJHyxqjSu47K
O/sEX1d8xNQsE+OtZWk6IddJ3L8Kr5vU1NTU1NTU1N8oXTJm2db567R2bWt02d2d02McQvTT
ZmO0l+V5WtUGpZclM06PdCJLE/o6ifvEFFIoFAqFQqFQrkKUAtZ47eUcRnLsADXk5XosZgiR
T9acT5Dm6XiasN3uLmvDlMo04908IXPK1o7KXPurJzU1NTU1NTX1N0Gn9LEMty7ntjWM7v0P
9yB33zk4/vwkxbSzGaBtm3pTqtmUJIYmOyGThuQAMXtNthlvuW4zpuxxLkrFT03PKds5KkOh
UCgUCoVCeTHiiGlz0WbwJg7L2KFU5P9Iiv2WCudOY0i5sus3OkwZprpKoy2hXkIArlVkszV7
elmynCyWF2R7TuG/KkNNTU1NTU1N/bXRbaCSx1+eHL79QO/Ocage7t+9A2kzzir1w4e79z49
cMw1+q8ZmzEFf//DPXR3897rqAgGI8Ll7Y/uS6dH/3rw1m927BglQmuBAINWkg24Qv21V3bg
5PjzEze6W7dfhk+FQdAWmdQIcVf8CxHXS5sN7VF+o0AFGKRlOufXTNxtIgottNBCCy200EIL
LVdmUa2W88/2d3/9FvhlM2y9l37ft7v13z/hlmurxiflyNqlbTPQaMify6ntSJ03PbHWgzQS
JHosQ70MwVuAG5G9K/Yn5S/gvvBz/OkuIoQFtBgxnD3eB/n+2fcOUBckaDSClFFrhGh1952D
puflf/3PKhgM9uc/vi/Z6Ls4gjcLk7aSQx0vMJIB8Q8nGK8mWSNHj6/unzc9fYer82fl7JN9
dNfU34jKvLK1f1WmhPUUjcQQQwwxxBBDDDHEzMG0wQiWefPOgfBmUEzZugY3BUmVfWswzt9+
fEPsgIFlglzCCPCt2y/DDrAwVOlCtpaFmoMiAymwjpv2pP+1V3Zg2Xv3cdPTX+kCdQG/8fHO
4cOOQ7f9xvzvywIEV3amhfs+Kbuy541L2eSWgua/+sWetHrjH+foC61+9+9y9ugUddyyNFqI
u82YdFp6uo9WMhYY9dVFkOK87WFH/zyXHGrACpNMokd5W1g8XSjRxwvGKkUPd5EE6wqBqZ90
NnWwlytb+1dl3Hqyt2qXxBBDDDHEEEMMMcRsjkmNIO5KYcF9waohP/r7Lmg3bgnfLf2tjqw/
W/FjGME7QaZB4jtC3x9nb/ptdTkkI3+fCsxP/va4DGz4iz+uGTkYLTSaozsJ4IsP/vDDhwfo
QniwvCSA2qJH2eSWAzbvvfkARtBfeNY97PPPVnvVsBx99H1w62IouGycy4447h6+e4LRdV08
XUDjHQAWodFNz56luZBveTfAXcAwKMu24VyG/N5P7whBR/BSJA+Sui7Jj07lUvLc9r8wwBX6
bfoddxB3IfT2RwAF28s5Zcv/AVNj3i3SRdZUXj7SJsQQQwwxxBBDDDHEKCYiy3DMQyyyFV2G
ozKyT4w6MGDn0kR2zZV3dpZn6z+pBMkGwYUIHQcGXLwZNqoP976t5BiV7lx4f3xcuhOCjsuz
T/aFuAPf0f2e0JeBK+N1Av5BxIVAl4FhSwxC3NEcMeuhF3scCF3fuv0tNNdd/zLQaHk9kIMr
ZXhXWZ0a6seosBXJfrqQkzm6na/Jl971h4gn5UjfHDT5sp2POprL+SJx2Az/Eo6bL5nTOWWb
/wFTXFLOUsaWdL1GIYYYYoghhhhiiCFGdTF2oZgg5Te+uyPHY+wW+NmjU/mLSTnQAg2mK5QU
lwDLoRdlqM3AyKUOJyDBwsVlU1n6Wv1xZ7fHfCQEWjbL5aBI058zkY18aSVEvOk3tjse//ZN
NMFddCQ7/bJzL02Ov1ydwkcv8hNBdxbo3uu//M6O/DIgo8YLg/7OoNmAB/Sodtnvd/mU2GTU
YtGDLoIpZdUjUtpt2PfnhfRsDwJY/zjQvxfBLj9NAA8AGuqLUDNmv+r/2oh7uqQ2sehIJgCb
rF1iiCGGGGKIIYaY/3uMWmo7p8Xc1VspWyvBIlvR1nnEtP3GvL103FROlcRQxa5nTtqeKPcn
3Y9u/GAv+nedliwPYMxpXzqcMrTVcdVS0fSb8c5/TEKcETt87VGNFuzin1P+BwAA///snT1u
GzEQRm+YY7hwFSCFex/AlasEhmvZLh34AOkFpAxSp0gX6ALMGoyJCYcz3Ggpcbl5DwQx4n4k
hxRXO/sjqf3vuCdfQ2mQzuLT01Rc08GeQTRo0KBBgwYNmk1qispUmFVJJVNImjbt337ZMOul
2II0sng92fL7rOFvV2XL0o4vH1/exWfrH55vP4ZvoTTYUJqQ5KruLjP0jDnuWa3J8iTOjGyA
sn3LMSe0rtL+d9yDclRrsiHp6tKwlNU5QoMGDRo0aNCg2YwmlMRJkNXKrqCnRrL2teagXvqa
+JuSUpa+wRmUn3sR7FqaTK/9cVIm00GpnOTUe3FCsjalRjYbhCBrzfJ5CQ2uuAd7yiyP9QIt
Lo6gxP77hAbNcZo5srX5jAbNHFlYgT9o0CzRSNn/vJ73KjrSSm37fTkl2rAKj0jO2K3uguFD
1tTeOLcprhzLAT/4dCbBmhbt6hLaPOOup8aaO39I1j5pjdxfnWjQzNFUd3jZQqj1hQZNXw3r
Gc2WNPPX88HYuhlNsGvJEkcwX2l1V30v5uTO8J2+jvPQmeRqeTHKtSZBG/6Ql8Teja+4J0eL
W603MrhbrZEXqwQ0aP5FI5WWzFl+qQoaNGvQaLuodNpBg2Y9mjnKtR1TTqHxa1ma+a1Zs23V
SrblefXzp9i71ZF8WSxPPfpbrb6qM1CtW50WPd4lLArcAQAAAADgPBC4AwAAAAAMAIE7AAAA
AMAAELgDAAAAAAzAosB9Hz4/fX9ND8+30SCRRkxpAcc/UiYn30ZuffuKRBouAUBk6RX3uEfF
I8SPX3/+oCsZh7d/25IvsbHXZsc1HEumiEemp5+vJTGm1wbllK+zPNrxpDSelz5+uSb38zRR
pzPiO9J9pCPmcSW//3Q/2eTk58/18tvdXRy3JqdaPQN3AAAAAAA4AwTuAAAAAAAD0D5wj7cC
q/caMDBWbpCTk28y391cX75cTfmHr/fTAav5QRAA4HS0D9ynuIfvkQAAwPpZ+LDp8t55dJuc
fIg87qe7uwtZ0uUDpH3gPg2meZsAAADNuXy56th7vM7FpS6AETl0OvNvH7hPJyLN2wQAAGjO
7qbnAYsb1ABD0+XM/ySPyjRvEwAAoDl9bxFzgxpgaDZyxZ1LCAAAMAR9bxFzgxpgaLpEvL8B
AAD//+ydT2gbRxjFr4VeemghkECbiwixEzcibU0LMm1dWRRj3INMINUhkApjN8K2WgTBlxIK
QbeC3UOLESnpwchtTiaYQm+BHqy6PdVNbk0E9S3WvWzf7LcabdbWn11rtLvW+yGW0Wp3duZp
tPPm02eZOe6EEEKGFEbcCSGBCeUjzBx3QgghQwoj7oSQwITyEWaqDCGEkCEl3Jg3p0tCYg1T
ZQghhJDBEW7Mm9MlIbGGqTKEEELI4AjXOnO6JCTWMFWGEEIIGRzhWmdOl4TEGqbKEGIEfK5+
t7bxqB2ywAILw1hoN7nyV2UIIYFhqgwhRsDMHXYTThtwQsQEXcdqw7UNwNDGd9sJy1+VIYQE
hqkyhBiBxr3vUFJDmDbuQ0s7YfmrMoSQwDBVhhAj0GX2HUpqCApriHaTK39VhhASGKbKEGIE
mqG+Q0kNQWENEc2IO6dLQmINU2UIMQLNUN+hpIagsIaIZo47p0tCYg1TZbzgVgtRSkuFTDHn
3pZXs9i6C+5XccrWASc/0oJmqO9QUkNEQdhTaSWjGXFnqgwhsYapMi8A/w1FsK3t12qHVmvr
eerZuV/berItj1M5/ZAAnMQMhTuKIjuG/Uoa2Y4I0WleL8I2Oj7tkc5nNY4U4k40c9yZKkNI
rGGqTAvMXnDtcOGoautgGwVUu3lf7cS2+Kfa6gc8OrYq0P5E/V5v/bklx6OSUzPrkJMQ2Lhj
RM0tpI4dRagzPZtIT41evjaeKeb8jjR1+nS20dEYYQy3u3ro+JJUdzbYhYZK567CVh9NZAqV
Ozvp/O2y1eygr1/nrD6cgZhyOjqOm2p5NbvyoF5/vC5V1f/9NrGsXsWcFB1lTkg0I+5MlSEk
1jBV5oVK9qyqjqPDP6Vn1z7ZqGCy+eCrXWyv3tp8+WLlpQslbKVw5mIFB+Bg9e827NB7vwTt
vZI+vn+8m/eRk0fcUcNGfmLxt9Z+mBu4nB+e2YWp0a//8lennP6d9XeHi7rL+QV/lzCNL0m1
VgEIUeejrw4Av2NVOth5ZeLhy/ezGMnHhe0z74w6xt0tmucHKGN6a4pmjjtTZQiJNUyVaVFa
Kqjw+X6t/lzdcFFeeVDH5K18/KGFAjz6q1dKr7yehWWXLYx7plARoy/eXaXZmPk3Mb6mya5V
+Try6CTaSw1HJ+mYzr7B6GyGYFMyxdzcQgpmBYWVnSwMIhaK8hIK4tpHLoxfvjaOceic9Xj9
o7GkmBttKMurWYkNy1MxQLKn+ksdT98r5nEhrDyf/TEzOVZAGS/hEjJQN9cm3Kdj7SqeDG5S
9svVN+aTqtrZBA7bs/KffVqSP/AY5HvaWVJ0RPVrOotltmU5nhuNVLFzW1hptnQfBWn5yk+q
+x53Ljp/Yx3iitc/zMraybTOuBfjHUdrUcbdRkwtTh+Azl3HqgTFcdjcYhLtf7q7Lt9FoOVQ
UjqIEfLmFznsx51T6pQRjm3+xnnoeXYmi3Pt4a36KJqLVmLcdQFKSkE+IOlUAjXH8e4RzYg7
U2UIiTVMlWmBCUZcu7hw3FvVrFmo6Me7tzbxuGo/zhyJuOtsGXMTTI81975y6JpyehK3HceJ
to/0YoYkpgtPY7vh6s23lC3WyQPYA1cHU+h6Q1V4UpmeVAIuB6fD98DlQGp4KZ0ZIls4RVhY
HCAXajS9LGwoav58RDnI6qMJcZa46GiygLPU8bNrsja4e8Vx7T//lxOnixpwGK6FZshfdETE
uKP9aJJjdqezIuPkiIrgos0w33DhYg2lgD146emvzTyNpuZNWjrf/Dg/AJ1xMPYkp9TdQ7W8
aWFRz9r9XdM692jcUYAaGJBKhFQCY0MWNtJB3CFxsEdSvRy6/vY5zxcXWAhBN+ms/cVIxum1
vQfKQBBZg0HhmAaJo5njzlQZQmINU2ValJYK2n+riPuBE3FXoXQ76K5eOlT3YuzM3y57Iu5y
4sLDUl+y7WEaXlsu92jBj53Xpamew3yBifPsVOquVb+zk5bGdP4HivrVIY+1C13NkDZnsD4y
zCSdwB2DhKWD3dECihmS9I+N+UuwNfA0YjHh/uEIW3FKO4qcnqzsWfmmMXIu2rDbBgeG4Y3T
8RbjeDm9YYelJ8+v2cuGezqFA4clxlTsMzn+BsylbqGhL5fa0UFStFCMtXxTga798/2MFqQZ
NXesIfw09kj30S9RD/3S3XHrDBFEZ1EAqhrSWd5KdRVU6KzcnGNMf3y6pcpk3F12D1Ewt5i8
8aPSB13TI0RWUPoYCL4xn2gmfWUyxRwOxjI1sVyWhZAn4t58m+5h2aMFjyPRjLjHdBVECBFC
+Qj/DwAA///sXV+IHEUe9jkaODC5mIyajZyDJKu7N1EcI86iXrJ7CssGnCAk+yBJJsveZs/b
VVcOH9Tj4IieoIw+nAxLjgRZdlUUJCcBMRziHS63t09HFORIMuA92u9S91V93b+u+dfT05OZ
ndmtj6Gp6a6uqv51z9RXv/7qVz0qleEsNCpesC1eWKWLXT7id8e21uNO7o6WtCREQbd3+0wO
n11H0+jan/nyCk8fOjmwY+rVON3V3IeaE4x8UpY96PCGZ3J3vbMY/9rrcuuV9UJqPA/ijl4Z
F0vvYEscYmtSdqIJcbfm5JG4g9yQpouDkx53Enf7LJtQgp6ePrHAPeIAxqjSMx5NPKs4Zd/9
MkdTe6PBI0mJQKRw+pGJoj3i8tQYKSzaz/Yo5TuMw8Ybv3sXCGUVIkwK/iekHM0+/k9FnYlY
2FzIGHkw9yxfLi9fGsdvuVZIXTlAyoid31Y/qo7ZGXk49pAq5EnYYI+7sQbfXfDtkN5jhC4y
KELjA74eDjJlJMNr5OsI2I2+eSQePLuEMUDAy/XdMRbWCVSHu4MhQSevu+PoTY27k8o4OPQ1
nFQmBIg7+iHRuIO4/2xvfpvRsuOzLZC2M2FPTgUDoKse26W3Xm/JOXRkIv3zFyfRXeEssG2k
QZRbaja6NxB30HTp3ekpP/hNmKdNxUvthLlGk8aaTibbOjw+vscdjJCDolJhBExFpALKkML0
UA6PopylPbITaWxPPVUg7UOeB57VUnjQIKV8UoWvLz6eB5daUwUODPxDE0WKqsE1eS9ATDPZ
geEXJskspWFsD7XXSpzuE2mciBEda+8d4k5HOzXuh4wdQs3GD+9heKzJ5adv4hopT8IeqoBo
vSO5NM8ikOHhQe08xiFYg/Wy/A7ZGT9YrXGfytDdjir0X0qgmdlY4q4CKx2bzh2/V49DMEyi
fh083p4tAKPph/N35zxr1KFM4089lJbXERS4w1wwDm+KTE6lGIljAIxtkIevRPqUwfemx91J
ZRwc+hpOKhOCUpnrX1+l6GV+XU9FtQPI8COsPfS4S1j31jXu6OdQDrs09I67nl4AceckMKpT
1tT5O18epEseW+NKrMCaWibjDya0aV8svsr4Af1rajyP0/GRUcGvX77zkfVF9Iis8aOf9Kwy
qWVn0PWiau7Ze1kLcz1T2pG/6YmJPDTySdnzm3GeObmVBuBCdswOYGd6biFZiI8+RVMyVPuc
JPsptjpRuNV6IwRXvUPciVopVxy1WLQALPrciDzJ7m9ECzuHRoatezkRjYm2dm0JjcR1ROmM
z9eXL4XvK/oLvalxd1IZB4e+hpPKhAAZlaiOSPzxP4rEnY72Q2eX0ItkRovig7c17vbk1JZi
q4FAgzpzwhw4bupwnq/jQeirNO5g56nRHI+qyt4OHdtt+9Pg9JSWUidD2Q/KpEefFwgGz2yo
F3yac7+Q7e7pyYOlRWV4OTk33wCQamvNzOH8I2s689DJATQju1pmgUHhY+D9B77SlWInroii
bZS2Y+pV6kBQBVu1RdCJ1Sjbt54Xj4F1rgHtIL5Jk1HkdixTW1SC0jZKWhaHuNfdH3PME11a
I+Bvh+80qDtq8eyeQG963J1UxsGhr+GkMiEWnp8F+RaZe/HCKqg5uDsJOr6CuIvHXfZT4+7L
3H9s7R8ZdFY07mDDu46m914oy35b4/7RT5N7crmAmmt4VjcP0q9LeOOGZ3q7PY/5AeywX7u6
jfydHnSQdXxdU+ePP5m3BgZj4OU85ca/x1FvlVxnTRXQPLSNqms5camodT44ESVzSABce3+c
bN6uXQWOfxl4bHrcLOLe5oO9mUZKN8Wk7dtzM5mUSGDYdqR3jb526G3DBt6v3tS4O6mMg0Nf
w0llQozNT17/+qqwcDB1snPS9+XL5bFZn6+DuEsc91qNe3xvn/FJZ1EO6Cw5N5juXe8siq9a
iPsDz2b5ldlSM7nb9qfvnp60pS/0lM99OCInskChzuVv3+VX1ivZ8HV4JkfRi/a4/9ZX5f7h
8yMg+qDselTw+K6D3/jhOIS4L381AsZPbQ897soIfphBjwFmB0Q8Y+tntgLaZ5nd/GX2RUfe
pkmdPRuhJcMm00rVff/QvrKox9GbHncnlXFw6Gs4qUwI0bhji8/cx2XQ9G1787caPcxfb6jM
aFFEMhUe90DjzqgyLXHTUw+lxcMtUSBtjTtwY/0+qlO8xu+ml4ojYMYHS4t7cjkSaBV43EWG
DuLO+DNV5XO6GLk1ThfX++0zfrAanIh07oJ/IhrsBZWmxvOMV40SUpVC/DWlff9o0ibogBMg
DhmKLxe+iUf7FwmmDcQ51ObRTYCOPqvJit0cNu9NjbuTyjg49DWcVCaEjipjdOooilIZMPVd
T2t2/tzFVfD4wcysEHeZsao17sZDT8bfdCRkt5OebyrCVcCVyXRB6EmswaqHg9midVm7kHJx
bMtCJ/Tcc96qdsZ/nsdRevdZvl2vvei6Z4lzlHG9g4LTVV8plRmh+mVlvYASMPyoaluVUn9z
dMYx0ZQMLX81wjkGRJxplMlw7pX8/PpigpLj5LSDh3Qa0SZ19kyM5s/q319nDEcbnaDysG3e
sm1tae1M8GXkn27atjc97k4q4+DQ13BSmRCUytDdTuL+YDAhFaSWOhnydbrhqzzuVMtES2Vq
Dw2dHCDb3jE7AKJ83xu+651SGS9g1anDecZm4azQugVqn/do7u7pCjkKeMC++/OMS3PH/jSD
DLJekcSgP8P4QZohSnRGiMeevZdnj4wO5ozGXTQ8nsmAUQHIvdmf3f1wjroaDg94IcMvTFIw
g0O/ON/Zldt7Ck3JEIY9OvijWR3z4cE87ghjius5eZXUjfMTMAqSoOOlM9na+EJ1Ya8bimIZ
iQ8JNI9UBlU3pTJchb7uFXHdKC7v2umbG21SZ8/EaPqs+oFKjdH8CaMmzAsXlK26Xr7BY5hI
/kvE/H/m3BvftsaqnJmKf1dlBg+0s4qcFCthJetmkDXFuoPe1Lg7qYyDQ1/DSWVCgLgrpWSa
KYg7FTKg5tjuenrBnpbaKKpMS1IZ+2+9tqepSseZFdcojxd5dKlUkMmp7HQl/Evttqphvtj9
0jjoPiPHswSMCmqr3pQT+xqhaRx3hv0mz5CEsIpaMgRGiFETVVWlqbS95BZB21ZRE1CrMHD4
Ab2MjmfuOOqyIxSpyFvDoONyRRxYyiAQrO5BKwh65xBhUmfPdhD9rIZ8OliXylqMVqPqSml5
ZOAruNJU5tl367gbGHrfti2HWJ6quHf4R5UA+arxP6R8tQPkSy0SlAZpe8mtTqM3Pe5OKuPg
0NdwUpkQDAdJ/k2P+6GzS3p5xcvl5y7qiaqk7PbWjipDxh9TKoPt97fcIzu/v2W7nbbybFdh
emfdTivxhcuJoCa4lmBx8sKeXK6KuDcFSrg9iB2plfTj+arIjxHq/M2KmAswaSJokSFwnXAR
+G/fzWQHkMaHi8xrZmNYIz2gKGfu87z2TY4OrnynPb7nXsnzFNFyICdpE72YQoBwy7jYU7BK
awGEBjsXntetQjnYnxrN0dnJbJ6ZB/LofAHZjt+bHczMspal4ogsM9RRRBN3Z8/EaOZxHwuo
8BhGO3Vtq93hxm74CqviWmABrqNUmrqftvVXcQqWvsJlDu6/U66aGXhIbiLbNnMg+8yXV6wV
hQv4rz59YoH/2P4SWhNpEn2OrLAftk3PLZTOZPlWhG3gcq1dU8v0psbdSWUcHPoaTioTQnex
hnxLYEd0AG+rK0gjQXkMFTLidAdxf+K1VdHJXP/6Kv6R40tlwMVVwM6/v2W3lW6+vbkYtkJS
pt64IesdNoXk4UJLjCED1l71on8L9hPRZEhWetd0JJcmt/DXUg38mgR4CR5FElPsxFc8kx8c
zY98Uib79ExdogYBCwT7Ee+jeCtJhkTChBJAhq59+iZVv2wGqI/oH+j4RCNNUWMH7ilqUhvU
cmw6y8zAtdIDXCyz04gwqbNnO4h+Vq+9Pw4KLsRdj3wm0mTw2EOzK/Mb56iGC76izJfOFi+q
z0pTGZBmESxprj9R/Iu6uqbOw2i/+YeybUt78m2G3DIUC9vy7tC2GE0tXy6LDkqZ9VZloVxk
E9uuqeUXH8/LHUQhHE50B73pcXdSGQeHvoaTyoQQjTuJOKPKHJq6ga6IOpmqmam3moVUGQ7S
C6K/x1yASTzQ5O6qTnp303QnsAUZdocQ/RgIjxFaSd/kwdIi9ohOWlzCttu48Ptz1HNjSx3w
gfuypFZUSIv8QHQdnqrgr5yTQA8x67IFBiiWbubDQ5nhFya9wLuMiigzwNc//TIrSgkQu+7I
DyJM6uzZDuIPMpEgt35nat8Tr63CjNgjIxkYTRLal/HDe8iDy0c3Q9viqmEEHlVmirwMV2xK
zWGPeNxPPVWAbZmB4wfbtnTz45bBtrpSc19gcNxWZW6NbVsU0k2Pe29q3J1UxsGhr+GkMiG4
AJO/eKrRuO95LA/ufkfgZZcPv27bq4+iT1r532fidI8/EhKhZ9sNd+hFNJXKgPGQ3NC5+C/j
mwTRAfMApaOzsDA9SA5KIkgZ9Etni6SJYCHgNPaTT6L5trrCr8emc1QIGIylh3w50/UvdBVI
k4mSDInrFH8K+C3YkxOoCL+olikO0VzqoTQbpkhMu+LFjJbKOHsmRpz5GOJxJ3EvTaXBgM10
1SL2wLby/kF7xA3Dxs4PTs4Oz+TmPi7/97WF0ycWRMfPqyOlZtpWonNgI2MtvuXAICE1Hirg
/VC51i3zSb8ZO+GKcBZHBWiq1KKJvn+DuoHe9Lg7qYyDQ1/DSWVCTF9aQAdMN5vI3NGXwEba
Y3RB69exh1sewpbxH+SUle+SE3GvUrneKO3QF2hKhkA4JDjJ4aEMtqHYwwigQTKwPzWaA9XQ
OoFAPHDtz4VMdgAMkiJsZNYRVOYnZTIlSCQeTpQD0slIoIo+ZuObpPeXD6ooRsjAgnkOOnYQ
W+W7hL8NpcxUQYBmpQ7nRZMNfrzhxN3ZMzEaGVaGGcG7BV8qA4MgQRUQOf3yJW1PGAQ2p6aF
ZcK2uOqjJsQtRi90uofBfKYyKApXDYPIZFYVEHcUxSA2fKVpa9wD17sGbMvgM4/OF5SxLZ4E
jBAYhQYJ2FY07qUz2aNdXFmiNzXuTirj4NDXcFKZEOh6wdE5KY0yd6HjEqkdCVld1V93qTJb
HOK+xcXfWwTNQ+zVRNtIhtpHSLQHrUYEr1tU9CPKuYOygm9HEW1SZ8/EqGtYu53gu4+srdbm
aZS/0TVWvc0QT3lT6bntU29Ufu3RqmwSanOLa9ydVMbBoa/hpDIV0J71t14Xpi6hIbkNabq1
VOqaWmYCbdDnxujXm3beDpsATYn7ynrh9IkFCSKEhExgMOntVWmvev/OBvv9NIjs8Xuz3FnF
e1S9r8lQ/uG9sfnJ+LOZ20HTaQPOnsnQ9Fm99umbsC3TXjA/Xuws2SrTOxvs3y7VyX7Y9lf7
0o1qv1l2EJ/9hhvWRZVxcHBIDCeVqcbKdzqO2LHpHDoqvjdHGl3p8Sf9BD5M4Ciz8StM2eX1
Dh16GfGXka9LE1tNexb1VBZt7cIosWs6rqYmdfZMhtiGtaNgNdrW5mm0Z3fV0e6gFzRILqqM
g4NDYjipjINDR7BR046jeUlfe9o2xKSb2J6CCMN61usFrzrCVSNfe6vp0Dd/07Gxrzd7U+Pu
uksHh77GhvyE/w8AAP//7F1faBzHHX4tFATFf1Jqxy4Yn00tx6nsYlWBE3WV0+FUQqGRMbH1
YFDOQrGkWMZWCX6wCQEjG0qD5IcUIUwkw3FSRAKuMQ4mIRi19BTFeUmcvFkStt9SPSdMv9lv
d25u9/6spFtp7jQfxzE3Nzu797vZ3W++/fa35lplLCwqhZAss1LjtrTrt+Kr2xCEv4ixdmyG
eCqUJu4Vqd+cMFNxt6dLC4uqhrXKWFhEAuubqjgs24gINrARwUyPezWeLld65WTp2c2lb7Ys
fVMnRDL8KoxFiW1L9o+rZyaWXXCdf+O8yLRNd+6Zad99v7/svSXzIrVvtv+aWPr45y60Z25Z
Yfb/slGwVhkLi0hgM/RXHDakEcEGNiKYqbhX4+lSPqLBIeKLj/aXbexQvaTD2k1B6t3h7QNp
vBQfDaJgXqayl7ZA3NFtsfbBxS/1jXAzyhLighcYB6eb934UB7FWDzYugaciuevTBiakwrLv
3UtgQT1llr4K8HU0xoYNfnYQzT4Uj4MB0W17Bbdwk8BaZSwsIsHcd3PMK6oSE9nCSgtM68Qw
Lv0oVKpWQzavNgrLzh35jPCGb0xtFNS4RaHgwcFmlVk5kg5xl68wrUNS/JD4ypuArTpusfPD
TT2L21vSp/5TAfuZXpPoGNlxfVGn0aWvTmDSyM0ou6Igb156dvPAg3q8wLB9PRRcY+ahZPmq
JRg5qbmvWdmr08v5LwtrlbGwiARfWRWz0rAhjQig7BZRoNiItVllVo6k5O40wDy7qSr5ETTd
ebmc3vtYR6vMvMg4X8nGHpvnNKBO7205Vynr54XzPC+3c04YkqXXWwyYGP/m7ez730r6DpLt
rU6K5fGJpTOT2a09V154bUhxehDZbeeH8do+kMY7lqXPBO1R3uZ0whq8mvqkkI9OUI9ODmvP
F5NifEua/bz9b1lzqW9EfmxJbzm9+OIH42TM1OxRiZoSFwSEo6Dvvt9/W9wAd3/5p4xi820z
zajnR+rl/xBf/HU6tWsyBqa+Z6Ydjcna8cK3+2aloI4a1LdNd4Lc8+nLnBJgKdQL7+lv7IHL
7nkoCStaDt7rZA26Ws8Er4bAWmUsLCKBZZkVhw1pRLCBjQg2q0xFMC9uSX4MouxwZUWg5WWi
fL6uVHZdm/eaKedMMp+C13kF2Z5zAzZwGL9anUPWvx9V9QXXWxBgxiDKy45hBgWljpNYN38i
nwhGIZy8ualncf91WSltLV7lsatZ0GuSVFTu+dJ9fll3m+wcneCrkYksylS4pcA/kAY/Fk4/
qKc9hmU1STji8H7OAWIDaU4SFHwDODFTT+o8ON2sbOiU4VHPy034irK6vJQ3uw+cG3MG9gmm
jo9ctWpJjo6/GAyekaFnpjErH1aNSjSQjT87CI6OblHmPGHuf+6q2cOmgrXKWFhEAkuGKg67
j0cEO1YjgpmKexWeLnXDeq7sEmiHTAs/cc+Vfc2K9JzUhPOkLsyrP1H2+XW78GYCC9kC6y2I
beeHD/9XFrAIODfKijSDQ5OLS+Lu6OIgo2hDgXzuuzksC7a6MPsYlfEJ96G/mAC88NoQ2XCi
Y0RJ8lM/3Pnl78ZJedE+9onbHv2gc358fUzOEK6JJbJnNuN2npnMlnDhL4vkrskYDeuIA3V0
kU/cRU5xl8SdZWVqf3Mm91Hx70KG9SQ6d1YkC+xqXqS8qUKSdJ/bT998GMN9LcFaZSwsIoEl
QxWHDWlEKOsxtVgdio1Ym1VmRci3tid9BJplESDuOUWcBhuNuOd8Mjkl3mXwuuLOyoV//Uq1
LLveIMCxQKapglPVJlkXHufmIElP3FF8HW1Avl2S7ajsYOT0t9D3ggIVdOGwf3RC5rrwYJRE
nESfswXhEXTwfsHbZD2rvST0Trdbe66gW2wnXsVIMCgyaPQ1MaekbpL1eXGLSjx/1NTsPuWA
V/ebuhK75nGXErsnnJO7U4+nVYbUHGs5fD/F6QEJOhk8/TZ4522yvJO11ACqOVirjIVFJLAs
s+KwIY0INrARoZgqZrPKhIduJVfvJMolFfecJd1nsAlOA7yC27/GwgunpnGtO4XWGwTYOfPJ
gBODduNdJXVp6suR+JGJrKqPOY5z+s7JgKWCPpCmf4ZQQjWJO+kviDgnCSDuSuYnUM+PtOuw
W8nvB9K7J5ZUn76CAg4Rb84cVIyZ3Jryue5dEZ6srqwyqsyv9s32c9UyU+RMM403wss/4xl4
ZBlbyymBWp3XT1KJ/ZsW1ipTFGvpsNouRFpUHuaToRKj1MwBbEJIw0fGzBgWhAmBLYYqCmMQ
Ziru1WaVSQbu/pR8WsreDiPXlO8txRX3ukK+dqEr7kHyjQWdSn8m+IDinpsJDF/u3NpzRXeb
6Mq38NwvpOD0vpPvUnEHc5167krvvgwqYPmx88NKDlffYmJAjk6/DcqU7RMtrml+mR4Yxx6j
r8hn11HdEoPTzb/YP6RWNy9SPrr81LOsyFtFHcs7+kcz3mDK7X/vXkLPPyPvKJ2MXRNz7pZr
Ov3HP3dhwT/OZ5c1j/tTzSqjQzL+mfbNprLrsFaZoqjJM7TFusFMMlRQSsHJ5sKj8YINjBrb
Gx7S8NHQQ7qKxdcZGx7YYigYsRKxNS3CZnrcq8sqUyhtS1K7VzWXFiZPZdcldmr2qpn3bCYv
1UydVO5zun6dTspFfqoZdpJvp8lNGHjDpc53MVZ9uReXndSQqATvBOf2edzJ+BtapYWGIr2y
wQgu6Ij3eL34gZs9pqln8djV7GHHuY72b3z+heL06Ied4J2mdoJyvsoqA+7OSwExR31nsxO9
8e0Xu0iO8bumHqXok1GdIAioYVKXeXFr59/qmePl0k/jez+Kc/IACq7KwrlSAXbOrDK3xQ3w
eBB3Jk4Vjh7PRzXhHZwexB1fuYq7l46GG4x+Dt9PuXlmZtrRSYhxVFOwVhk/aKJKxGOYDZc4
DWAcJ/7SiVdwqmphIQwjQ+mR5pdONiZa6wdnpOiij1Vs56k/d+q+xszddgzs2KG4aZKGOSGl
mISjRKIj9v63/m9xSkMAddUNW57qrcd52syjhDmBJdJjKQ7X4PmJw1WPrfwjWusR8OAfseEw
U3GvLqtMGHC3CjOMi+2AumoeSDoZFlhkx7l456MccyiYelyv0b/KSe9e9kYAxB0Mu8SCxX5R
cL1lgQ3gNuCHYAfkfaj6t8EOy66iRANF2Ysh87BZOW2KJZAx84gaNaxVpgDGUs04DRytzzs9
+LjOtd83vj42ri+1OQeQRTGYQ4bII5d+dAu+4+9CdjR5oUvXUTL3peiCvaCpL23UqDYhpAwI
ojT1g9yYsZ6GQlFKJlpyJ+95cWvonX60TPaPm3kbqAmBVeAolafq70d1ZYR48s92XVLBlmO4
onD26q+728ZNy+hspse92qwy5bH2n5PvU3dl+JWuKHO3fVdvV4l0imEAXqFuPGVyGKWCrxvS
I7HtF7tK701r3KQw8wrerurdzJpSyWQsrFXGD542JsWdsbONJ0eXVM3gPamvn+iNTz2/g/fu
P8TwEacQTH3kt9PNDa0jt8WN7uMpKf8c3aloPfqR6lFHDDOBM5NyCivV+qM7Xz4XT3Rsuks8
mwfmkKH0SDO1XjBIqpX6Pg9CiXGu1yy79QePXc0adZQ0J6QKmN4cySfu2EhUYk/3hQ7/Ag4R
RsVTwajAgjRwuPLAq6RHgrEVAcVRqi0drl/WnCCbqbhXl1VmfaD5Z+p0X81KsQqdO9j+rdND
fEzS9oH0juuLITssKO2XXlHp7VnFfhRmkeXQaayw+6inOKkkNsFlzdnf1w3WKuPH4tftPAGQ
7jDzEbg4zxapXrcA4q5oPXg5GLzwDtPL2vVc9IaztXDIOpU51ChnwrkDjb7TkkXNwBwylLnb
HnNyioG4Y9zqQ47cSPfJzIsMOFBD42+DYueGw5yQEowejqHL+ZWY2AdTqqnpk4EwKrBP/u7O
cBBJ3zwT29l9PKXyxylg0J7obQxOljYcZnrca88qsw4ISUnX3rPOa9U4DworUWPVa1kp7y/d
zPfDTdvBNwrWKuPHWM9BsnBeqGXC/6P1UvihloaTCq0yKk9TIp7zuQ5flubgREcscXQnFlEX
dqXK7slIYEVSbnd8mVb8qFWYQ4YUZSQT0q3AapqqoA6O6bGUj9NvOMwJqfBmONy79YMPQxo8
HOFfsFaZMEiPxJr60k6gklTcVTAR21cupHyxxcZf/FMnD9qmwUzFvfasMmaiIkFei/5tAtay
2dFNlqod1irjQxLngB2vdkpWHY+9ekhmMlp4MPrSSVcaH+tpONKXxuH43IHGNz7/QjjnEtB6
kiF5zqY701Hd5FPQwP5b67H4qb2NtGrxzj97S2vNwwQyxNGFYVnfwPvxky2H+nXBEtRzcGap
4LkBRBNLGTU+TQgpgS0ZeqdfkcWg9Si4iFXcQ4JXKfXhqjB2tpEXPHXgSJt6d1gYeSw10+Nu
1aJVI6IxFl51jnqN4dcV8iLAem785oG1yuThyac3SFY8m2/DKxdSIO4g36TmKjUECt7pOamI
+9SjVMsBScqlvu5k8EiPpXQpyPEZZ/DVyERW1VjUJIwiQ8OX5VxUDVR1h2X38dSH4rE+CJ1L
QzFaZXiXqjkwJ6TyzpbWekzs5Zzccb4xUNi7GVK9MTb7RG8jG+Nl2g2UwqTAEhiuCJQarkTQ
1iWcW9YQWIoj+HbquVk/xEzF3VplLCyqGtYqkwecMN46PaQ0HvD42KE4Old59NhAV9zdW1ed
b3nRlonMcNYB76EGz49YkGcd5uY7sL/xRG+cC1rUHkwjQ0RBU8c66yWrhpkhJRiozN327rZS
aWSFYSElTA6sAoarsV6jYjDT426tMhYWVY0N2YX/DwAA///sndFLHFcUxv+MQgPFPGRfNAks
FpcWVqyptUVEHwYCxodCMlkkSlOxQvGl9CUsLYWy9qFhKIFAWBQKfQml0PeCxeSx6dsmC+1b
2X/g9syc9XZ2ZlxX4+p37PcjBJ3MxMnJnXu/PfPdc5CtMkdvgBjcc9ZJhL66YONWAmCOYTJU
wMVQJ/kACbifrw/IIe1uDGhM5r0c+CAH1uPryRgCM+NOqwwhpqFVJsvp6pj2i21NwMe/UB2u
ZBiYEEO2BqSJkFrEYmBNDF1MjzutMoSYhlaZM8XETZJTwYoYMjQmGdIhYSWw5sDMuFtZLgkh
hdAqQ8hQsCWGTCzksCEtjJ6JkCqwge2DifBiety5XBJiGlplCBkKhsRQfStYf/7Ded/F0ZgL
qQlx6YADmw+g3KrENl3DFBnMjDuXS0JMQ6vMyTFXO4KcJVBiSCsSzq4v54flvnvkW/l64nKH
C6VMT9BzByqkWhHy2s1KvjDUvtu5N1rxNSKddrNKGq49cL+f8X0OAlRgXSpc+d2oEtuNKe2L
1yUZ29U7tzahxqqC6XGnVYYQ09Aqcyh9+gsM3n/49dscEKPgiCGR5p+tNqJa+aAhZQ+vns3L
8d4G8o9EM4kYQivigRNS52a1p5XWe033tHK5kO67UOuRR7WrmDvUkQIrzGoDJolbuledkh+u
m5+sReFk6X7dUGBZVYYQcmJolSlAe9CEn9cHrBmcXzC67eWvVB64dp9zyAUGQQxlOqGKaswP
6ehuxXf67NY0jEKZF0RlprvNI4AQUpdEqfXNpIhLfcxlrkj3CXKJQNcmD+lL9Ph4r+gEASSw
igxUrdceT6HTQWYQFhaFlEv0v8OBzauYHndaZQgxDa0yBTxZjNONE2ODll0vrOwer+hJv9X+
y4n8IENeWDI4UGLIJb2BvLjxxH18kw6p6YPhx5dl5Gt7eaiRiRPSWFyuL6u4lEdYxKVHj+jU
0fseI/Sd2tDACazQiq75D0VLVyqZ2BZ2w5CxjdmbCTPjTqsMIaahVSaLTLVL03Hr7I2pYOm3
+IisuLc/Cleebmp7c52LtSm3tj3X0149m9cj6iRW4S5fxx2557qmTC3rrt25RTNFtfKN0eDS
+8Hg2X1iBSgx5A4y7pmnPW88aDZKooFckomvPsZSmTghjYM5F3TF5XSQzrj7frQuJdzlzp8s
BuOrzTO/04HACaxLDVT9CJSJrfazy1/y7noIKEYxPe60yhBiGlplsrQezt+5tSkyOr1+zFRL
aieIwsn3vtjza3Niqom/EHF/4/qaim9ZyD/9sS1XlSsj7yRLtdqL5WQR8boOiTySI7Je3n67
hJbXJKcClBhy2sQ3tzk17ZPpHgkn5cPkG4sl+dg5MYa1mRInpK2fvlL7teaA05+6Jc4ZL4fc
tppkzvouBwYnsC6ZVGUEHtiQGrp/QMkPV6UZhTppo02kmBl3WmUIMQ2tMllEZGtSp/1iW1/L
yvohglvf2IqsKX/Q2Hk6rwme1tehnixHtMSE/Lr04Ujj8d7LX7dF7nuZrleJEtI1Xr4WoS/T
+sZUrPKZbr944IghLbuh41NGoL+xfbfjTR15ZKij+TpwQur8O7fkGff5dU3AZ0Iqs4TMJP5V
G+DDDhVYl4pt+iOQlj9K15NRZhZKcuab1ap8Lt39G+sfgulxp1WGENPQKtODrLv3RitvrSyL
yilXRuR3WTm0cIQsz3HmLBHu+y4cK6/JQTlHF2k5x2fclZd723K5umiajTiLKWJdViM9R/5U
JZT8uMXodG6eQIEmhjz+gW89LDYe5M8EAS2k+e0r+i4OUJr3By2whRzmk0EGM+NOqwwhpqFV
pgfNsuts20mSjuOrzViUjwbdjHutXLpfj+5W1APT6b0wvY/KuVkR7uoSlqtkORexLsJdz/m2
dlmtMhtTNnrfkOOCL4ai2tWb24c2sgFUSBZCWk7XkwGMYSHIgf3PJ1Mr9RmumGB63GmVIcQ0
tMr0UN8K0hv4dp+HE2OB/OWl61V9RSuSfXZ9WbPpuhXVv/uOr5VvF0pyfOeX9u6f8SbX+OBc
fFwT89pYJP3GXI7It5jdQ8jrgCyG3CGlkMABD6ldMAN75MjEH7qYGXdaZQgxDa0yWTq59aDT
e7yj8j2pvNH+x4kE/979cWQbpj4tnPLfkgsAphgyDUM6JBjYIYHpcadVhhDT0CpzEtp/fddN
tycZ90GaK1Ga/9+gGDp1zjekx32EDT3y5saqldhiZtxplSHENLTKvC75bDohzpQYqm/Z2GgB
GNLDnn0JaaZ5qgOeKwADqxSGq9no1jvqHHXmuYPpcadVhhDT0CpzbDjlkUGAEkNf/jyjey3y
ddnzFQy1/JHfmIEDVEg7SYvZwr4/6eapnrjnw1y2xDsIUIF1Bzt/tKhX5o+0cH66KKTcfLgy
htmACTPjTqsMIaY5l0f4XwAAAP//7J3PaxtHFMf/kOQQH7qUWrEakR/CAYkG/8K4xj0oBJIc
AspGqLFIY1pDyCWEQjBpC0W5NAiTkh4WOYcGQgg9F3pI6vbUH7kp6SFX/QPT7+yTxuOZlWIH
x3pj3gcT5NVaWd6OZr779rvvBWOV2clKwHC1EDjAQQzR4ITWaf/Sax826bUB2lSx3emTdqYH
OfydRwuHkBI4ktVrDSoO688Adjs2gqT82XqRW2l8gk9gVd+I2FUZza3U9sa0yjoRc411hlU4
eXrcxSojCEEjVpmBZH6gmNeFHcJBDDmDM67nztxyU5jU6dMvSR5fGvN3Hi0cQmqTNLP14vVH
ZUeZde4vxjfWoC8zG3+OHFaB7Xw7R5dDdLVj+lsRiCF5kOyN1OWaTgSrCZlnxl2sMoIQNGKV
yQCz6sxsbnizD33j26oFKQgOfMQQeQkwXJ00sEqbp/rdKLHzdL5AT12zWuA5hNQOCOR49MWa
EyLKE9s+GWz58hNdewqis/yzCPe3ABU+uZz0Mu5LvebTfeYuX1j1HVzmTxSzEcvT4y5WGUEI
GrHKbLHV6SMuz61cnBrPaK9twEpM/stuKnR43gEXRggrMUTcfjbjaHfHeKCsb0HS0u2BWV2X
cgupfnIgjZ7fPNXe0n66uHqt0U1vbmDCZaUsCVaB7bQmTMadWlabiCG2eEt5ghgnAoJ+n49z
J/DMuItVRhCCRqwyLvSs3sabJ0aa249DJc1orrH+fW1sOl84tFiBvsdu1FMJ22nNTpplvPVT
2hWVei191lqnD9eJz4VeSyaej6kJewUrMURgZEL32Foc16jGJ+MAlYnhLcJ9CFDkvnCnkNJr
2o5vfaF4BN/63EeH8S/mFm7anVVg6SleDDxMvM4ItGNrg7EK4c5qrBI8Pe5ilRGEoBGrjAuJ
b6wB5JukxA9kuumcOrmcYDq+c6xolpDqiYisq9hz/MOikekUWeNG6HxXPnNLiyR9h/06x/yQ
sIfwEUPa+rVQ0dpxKWo+3JLpGK7V+dhpH6Zl01KEAX+2XrJ35gCfkOJIEB+S47g4N5KRLvv9
O3UURpNxZxVVxSmwBAJFVWXsiR0Hidjaw5VeYLjiJ8qXcEZwIljFlmfGXawyghA0YpVxadUK
va6o/US7WYx1qY3aUYhvW7hDl8f1nBHuWGw+/63/UVeKWFGmxiJ8DjaaS4LO47vGkSkcVLiJ
IR8MyCHOYNGXu4JiZYc0IDgH1tD5Jh7+3BHDsPP0uItVRhCCRqwy26A6BoemK1DqM6Xo/MnD
6cNkc6dyWrhjeTPCHWLdZNzxJzQRa2meanR6jc+hG77YARtfPb9XKB6ZOFd08kbCgYS5GOpu
98kEMRo5h5QCiJCaG24BwTmwhiG2LrbwzLiLVUYQgkasMtvo3F80hWLS/HphcjmhPPqPr3sJ
daqRVz0R9TPuD+4cK/arvGmJT8K98/gu1eXQ3oNShI1JK45vrNF/xDCdKewtQYghQxCjMayQ
BgTzwA4anPwHLU+Pu1hlBCFoxCqzjbWbFdOBr5sW1qDKbkmzHOVLM0vN1hVtYd9UbfPoqkqf
UTs0W6p+uk45daowgyn76niR7MJQ81//pTPu2o6Zuo2r8zE3O6awt7AVQ+GOOrYhdQguwqEE
1hBKhHlm3MUqIwhBI1aZXdDdza/+WxD99EwqZvOZ2dz2+sTCQSMgMdSqHQ2inimHkO6wBRu+
7Ct/rr/vg9krOAR25yTNXg8m/vD0uItVRhCCRqwy+wc5bahtk19FTjhgsBJD7aeL508epqrY
Dk4hFPwa13V5U/xI59QhtH8tT+cLiFJmSJ0eTJuqfbZecuq784FVYFW/ZRjd/PT7W1XnYzu2
t5/NILZRvuQ3Zho5PDPuQSyXgiAMQqwyo4TnKi7sCRzEEA0waB3K/rbist/p0xQ7MkDHG7cY
qyHKIaQEQrrxUh8MQkrdgmwQUqp/0i8I+2D1WoMqybKKp4FPYFV6MF8tN3UXvMa6U5q96/W3
whVR70TUCseXE26l3Hl63ANdLgVBIMQqIwjvBVZiiCDh7m+0m9pQAp4ewAhFBu0/5sKmVTvq
J4Yz659Q9yue0xSfwBqojYY/AnH94/dg0vW+4uz9RwvPjLssl4IQNGKV2W+4ZTGF9wQHMWSP
NCpL6ogekum2xwBbqOMvfrgZ3zmE1IYsMc7kQ14OvwcTlOgkv5QwwS2wKg2XqdputwzL7G/1
e9qmmmEhTp4e94CWS0EQfMQqk807y2tay39Q/ziflvlaOMCwEkM6JanbgTWd7Y7xwIb6rbIS
mtxCCrEY33i79YjotCaOi1VmxyTNyA8jeZCcPdmqdsU14y5WGUEIGrHKZICgTJwrXr6wSssG
pSqHtH938prUb9XZAdud59j0nkuR1JY5qLASQ/GlMT9/qTyfjE3Sik+vxCLcB4GQklh0pgVc
IGXWPBGrzK6AcPf7pLZqBeqYYW/HibAvn1hFmKfHXawyghA0YpXJQD8XtXLRJM43VTw1Xtl4
+aSblTu3Nw56pK/bLynjCvfZHHVr2hUy5wYBHzH0+o9FXDTqRgQLFfyYA9tUD5ziJ4r6hfVr
H3Er08EqpKdylV6FKOsOm289UlT+dUF3YsZZoPhz+wrzCazqh6t6IsKI/fhqyVw6bqq275PB
5EyNMkxH6hEc8WB4ZtzFKiMIQSNWmS1MCY64noNMxzJAycheGccFvTZM5Rsbb55A3Jxeiekt
SHzIcUzQ2Hj5wioWEux8R+m0kNZAacclLCevnt87/0ERMt0k2v/79x5J+Rd/v8AWWp/Wbuqb
77R04c9ppccOeOv6o7I5jOQhu7VfcGAlhmzMyBnik/Fz8xzgE1LnAt4Oqd3BLRT4BHYIgzxI
nOHpcRerjCAEzUi+wv8DAAD//+ydT2gc5xnGdRYmDk1pi2OIU+rFVGurTBO8VWBFW1m7qImQ
ISsKrk9RR8vG2aYSRVB6Cb0Yxy00aHtIGIxDilnWrVtDECFgkkMPBRtFt0a9lLUWmmP3XqbP
N8/u59mdkSwps553lPeHMLuz3/zxOzPfPPPOM+8n2irTfq93eWh755yS+WAuGKU8a1p71bP4
tbUxn3fq/OnCpMN3+Dx3evpvHQ6eagT6dgM/Uamvf3CfC1nduoH7AbTv9pN2bICWzCS9U33+
hTea1PR8KIwp5foNNGYhv/bvXD5wz5As+GoiXwzxiE17Kw6A5JDyfMSNNzRZ5s5NyYG1ILa7
2bqGkBN/mRn30T1nP+hi9zmcmaIoYdQqMwxz3sbcAvUcuAUorOlpaa5PQ7KjO4b+xpTmem7t
zTqkPH3w4Tw6FHbYbIAGfJ5L6zyXT9c7RPnMZJ0tvaozFQh3rJG+GtwqQMrbGgu3t9xoeWNF
IJkQQ9m6TEoOKSMZu4XyE/CSAxtGcgxjkelxH8XaH7trvnwDRVGIWmUGoEU1N1k0HpVS3imc
WrnTsalxPygNwVw79PSrn3x662LlT34LWhx6GtODrqfMxg/vNWYmKqGOu4yvtY01q8h5P2C0
frBSumu85YLJuAfmnEv/MM0o3Nt3r9OjjFmo+7WbE05WxFCG0JCOiIwGVn4fKDPjPrq1H3qP
RN8Kk79zFSUt1CozgDG0vFyxi+LofQ/vN2YXcn2rjMOabiYFvnp56RWTBf/VDysrf55mA2bc
Ifcpvt/fMe9UNT/4EAth2t4MLNKrsmekPK0yVyZM6n3Tv4kPF71gjUHavhs8AUB7zIXlh19+
1X5NONkSQ5k4nLIVUj8jUfUzG9g9Xs8QgkyPe7LP2e2+aN48Mf7co6vnHjaYyE/lqepO+Cdc
E3Mra8lto6IcKdQqMwBUslN6VOu66bnn85XWxjwmrnxkBDT0NAsXoGeZfO0UhDs+o4GtGGP0
ejFH3ws197mfFvDV+GEmKlf9B3Tg2EQ+l4YVnSgWl+Zct2bM9FT/aOwHdwW4Q2jfvc61915X
VauMeGSKodizHQfq6taN/bdPC5khjSVbZnexgd3tcI01uwu0JMnMuI/otgHXL6dw6geDPcn+
HDLlb3136GQp537RTHLjFOUIoVaZL8WhiwdHZ4xdlP3M4Ui6QU00U3DmX1pVRjqixBCOlsXX
HdwBRg8bjjwQrmBohl4KfGLSBhkQFVLcbLNuZjRKPElpabMg/j//2ZrM01ZUYMFvP5o1eYqF
HJMXYVgUcqjgJgcLiy2rny4yPe6JXy75ANkpre98Ng8J7gelZr93pTgz9vTx8bFn3zZ7C5cw
fMbf089V8OvE18ePjx87fuyZ4PQpY/ex8fc9o+BNjQepgx4oSuqoVUYuYSnP8ZtYCnppLnv1
5r6CyBFDOHjW3qy7tfzSKzGvNXM0ym6ocevjDj1asUI/ReSEFPIRN88+n6pFSmq2715H6Gyo
sdmMf3REISHICawfHIEP/vmgG7zeQ/9hGByudmg82x5SHvdFrO4lKsIyM+6juFz+9fS3r/od
jmL7B/+/2CnPjjnv+p9jyszXCrzRosX0zNsdhiX4muMTZjTGPmVjzIXZ+ehb1N5UFCGoVWaA
fcbiECEbktoHWoJ2XllElBjyaeiKK4P9TvV5Fh71B4+0xVpx6g1ZT6ulhdQPosSXXsJwPNqh
iXTcyTyXpQWWUaJvcOgnb7lAD5KNZOvv05iCmEcHVU0dmR73xNe+6d9kvtwk0cfHgrIKZWs6
vTLRE+47W2e+MWO6FOhy2/7E6/eNo/2X19j40mk2Loc9q4qihFGrTAJoMVoligQxFD4OWVF0
SLgzW8kxBMKNcSW2A5DJQUJIw9BlNNSHhq1Hj42/EEQFFhsDyT4zUYnajaK2Lk65/cWHNL5L
63hlZtwTXzsUOWsn0LAOzd3ZbtiUOYQ7vTS5ySLbtL1zvMva+WzeCPd+Yybs0dhIefW4K8ou
qFUmGaRdMJTUESWG/EA4RjO+9MkMqclNv3XrYgXTfWEHtrSQLtaKHPMhjPHJxAn02PgLQVpg
iXlGNKjdzbjUg0+N2u/NcxfgiH31k0/T2My9kOlxT/xy6VXPslwy+c5TJ5ue+82fmLIwiMDS
nPuu/3n79+Xjx57B31Njhb/877L5HFjep4LyxxyJ3A8y7rTK5J16otuoKEcHtcooykiQJobs
GF7hE96rOkNpdWy2V81Rtfsq3Hdh07/p1vKQjHbKoxqyVYdvSQ4RG38hyAlsmGu/qcxM1sPC
3XOnh9Q59gIHtrsw6eDf1seyRgKWmXF/MgMwxU7p7v71sbMrikLUKrMX++k7tH9RYpEghuyb
FYu14uz5k1A2+LMJy85//rg053LkL9ueYwj0xiB7uSLhf2GRszG3t9zZYo6SMZwVpvXIejli
4y9Qu8sJrM/YlvIspBuu2ENXDG1dQ/Bd6tj7pXSR6XF/ArcN+790ijr2FCUTqFVmgE2/5dbM
NcMpnMK/uxW3HsJcURZyHC9JUYjYC5I9Ss1wY5HXVSUfwwJD2g2pc78f0r1jKDDCAgMbBbHN
3KA8MjPuo3jO3t3zq6IoCaJWmQEg3Jde7Enw5vp03qnv52UyCPcLk86Bil5rLv/II18M7TaQ
jVhSD+ljFTnryTyhrUmO1AO7HxDbvXPqAjtMmR73xNee4OVM4E5UFGmoVWYAYx7oC3fImpdW
XQr31sZ8efXybClvK1vf3nLNU/JS/v2d3lxofD5vBg3hlMVaEcE1j8j7bVgPodu/AvFFq9rG
Gp8Lc0VYKeu1M9+vvVh2yYQYij3AxB512Q2pcDIR2CwiM+Oe7tqjZPGUUZQUUavMABTcfNXJ
1h2Dwl6ac9kFYyLWhSlMsXeDW5/OdgNSm5k2KG++Jo8GnIIF/uit++YZen+B3nLBtNluQOhD
uFspj5sBrAgK3noYtEfLLnLE0JE5iuSE9EDIj7/YwHYjn+UHM4xMj3sqz9kFvtqhKBlFrTJD
lCGdV+50qLxZ4urhvQZfRGPhAijsf7+1ZnPkPj3uxRwbe9WzLwSifLaUD8ahMFUmXlp1zTt/
/TerINPL9Z7c783lmlJxTc+FoMdicTuFWbSbyzS6+xJHrL7MOhrYESEz4y6hCJt2j4pyaNQq
M8Cm37r044oV09Do3WBkPkxk/tu+gsY6ZYTCnR53yHRI8E3f2GawHFNcz502wv1eY+JMbwA5
tJnqi3vMZdosFyD3Ie5pv8EdQniQESWLqBhKHL3YjwgN7IiQ6XGXYJU53CDiiqL4apUZwnrc
/f7wH8aevt24dLpw1X/QDTWDLqe2xqpNNv38yZ7cXy7MLjzy0nAKpDx68MWaqW7GwjXl+g0z
14LN05us/K3X6u6vr9n7Ae3RMo0K98TRkI4IDeyIkJlxT328QrByR1bFfUXJEKmcwv8HAAD/
/+zdP4sTQRjH8Xdgk/cQLA5tLMTCQy3cwsImINinOFBORAQ7LUWsDgtFgmATYmVj4VtwCZfq
bMUF7by8gXFmx+zlvOQ2f2b2mZ35fliWTS4km4Xs/va5Z2fDbZXRqfrJjd7BR3P3Zr3PvX+r
91Z9txeqmktIb+9cunfV7nF0rDfV8bvd0dfiVMW9v6uDuxmK+E7PvlI/Y0cx02/SvXzd3G1x
b+fm8292wOyqTq9f8+PzK/spetJZP8y7o2NFhCHn8qNcehXixIb1ZNk+PLJRZTZQ3eINwLpo
lVns7D/yFl5bc/baqWV/XeX9hwe71x4MVXn+oLO7PWdAS3He5RznQp6wYT0Js+IeQqtM99HL
+hfBtbH6oNOF9FpgW7TKrMFhW97ClG9r8HbKHg5Ifq1GGHKO81hP2LCeMKrMMrZEhSYVv94o
lUmvBRygVSYsjJkVDdttlR/lZn5sllnYcuHTb7anrw2rF4o/oaxP2xfsfBpqxT2EwyUV98Zl
pPZo0Crjy1qdMwCARDCqDBX3xpS9MaT2qETSKhNCCQEAgFqMKnOF4N6IWXsMqT0qkbTKhFBC
AACgFqPKENxdOTfAmci+rF8L7RVJq0wIJQQAAGrpA5bgp4dQ5yK4u7Iw+ZRhnUJ7tCJplSG4
AwBagVYZgrtnpPaYRdIqs/flqf4m++8Gw9cv7LJ9qPePzJkzZ86cufh8+L5vj039Z5JjquzT
KhMzE9kZrD1uIj/hJiru4hUFAADm2QF/R8nfgIng7geF9iSI/IQZVQYAkCjZAxatMvGhqT0p
kbTKhFBCAACgluwBK4Q6F8FdqX837bLTdu9Ee0xaom2V2ZjsvUvFCyEAAK9ko3MIda7kg3v2
8/Di/FRMOuV0YTaZh/p5+8pZNf3UVI3RXi4gIbTKhIXgDgBxk43OtMoIWqunZaxGOpTbqUrq
c/dUysaqrx9Sa08NrTLnEd+7AQAiI1tpCqHOlWpwp0AOB2iVgRtsfwBYBaPKpBbcTdV80uHi
UThBq8yJJrvbibkAkCZGlUknuJd9LIz3Apdolamx2daZ1p0G5Mcnf/1vQXyXur3q281/TQCA
kq550yrToKyYdILqQZ+Wx+XHhwPpFcHmRH7CfwEAAP//7F1faFtVHH4VHXtpddpM7XDEMqeW
DbFueINTmwylWFiGoH0oaixtTVnmyBAfNkQYnQgO9UEJobIKIRuyhzGHPmzIUDFdrS/+edQF
9E3vowjX75zv3tO7Jm2XJre5ufl9hHJzev7f3Hu+3+9+53fDK5XxV4KLLXV07K5jY41WsujM
DU5b8Vwel0fdJg5PDt03Oeb/73Xn4uFJt8jmM91aE2LDNQDnljIY3SmnmjufwMFnN5rqm0Ag
EEQMElWmC4i7u3m07v/a5c9Cu1PfOfuPZuqSE0GnQKQyLnKX09tG46DpHzjuLxpkGsT9ziOz
jUpoQNxjSat34sRq18YjL64k7sDwC3EW2ZxLGuPtmbZgdpvmMF70audctsma8ZPC8HF3eOfy
cOzZtJlPgUAgEDjtps4ilQkUIABk7U1HZ28xwC5GC8VDV662N+a1oHmIVEah+ufHsZE0aGtv
tn/vD24irjqQ6XvPFNf1GdedwTXc2Kj2rpkSiTsvIe1xH4KRsBrXX01XU5tttdO5Ir18aUQZ
KjPLd88bP46Ayr/0/ToF6zZk1/y39sBuhWtfIBAIOh0SVSa6xD0VTkX7KaeKOZcH4NGASGUU
zi1lQNzP/XXxkReHwNRNOsh078SJ3OV07GA/SC1MVZX52wdxTEeyYvzT1hOLFR5TWkPdCw7I
wum5B0tmDYtOeXDaImNW7U5baBF5MpO76d1HPeiG8fQjncoT5Hnp6fTwl/nZt9Osbeo7t5/s
BlKQTjqOpsvXErBDenT9zOafIpoKGJ25kl8/efdtA3nzFZ2M6QpjvoYwroeuFfE3lrRO/Ftm
u2wFH/TZUf6GzKBORNkHvrmoU8oosregCm4/vttvHiAzZh6ZUSFroBkzqCvEx0yjQCAQRAMS
VSZ6xF2rYsJI2bGkgnikskVxtEcGIpVx9eVk0uCy257LG1J+eGoPuCwYp/rX1B6QSFDY8qUR
sEw65umlpqcc1y0IN+phhUwkP0ZBEuLXXs7Pa1YKxvy5856Rxzg+N/yiM/foK/3G+547n7j9
fiU4MfbA+HzF1v2h/YB+okJ0UqvnLap9kNhnWfELVVufY6TU7nwtfZhAZnJoUnBD8Q8e3w4+
zVKsE5YD0tExNLpP33PRH4wXPJ6dxETpzpf3H8184vzKdo11gdGhQrYFU4QGAxpFhYkLVRQ0
PTengzYMhgb2L7cbgUAQGUhUmYgR99CGesQ6G58pGdebIBoQqYzmrEnlNbc9Ij5UUVSSxN0Q
aOUd19lArEFYY6dvIH/h9SHwVHB9zctT5KbawTxEFo5Kto3GQaD9LWbGd4CkoqDx3CtZzvNp
+ptRnB53tos+oFp63JFuiD5oN9rFBWk7KcOPS4UMjQS0axzYtbIWfoWlcc/jLlnHwGmWaKgK
B05XzfxwCHScGyc99UXk4r9/OmKME//EGqc7bRUQet3zOK0CU4PtdUBnSGFcnDEMuZBJmJrb
vtgIBAJB85CoMhEi7qGLG2OAtZUePUHEIFIZvYcyaZE+ksT73b3kvo5H3EkoCxOK0FPRceJf
JX1BcdRDqQmYPQkuGDb4KD3fjsc7lT0waW3ZpbQuZu+mcdLbnifbL5UhL6dUxqSbrZ/MT7mO
Ck1jLI1vH0TToM7oybxzkVReNTqSJkHX/VRPGPDf3PnEHbuKLAgWjmz01jueMYOvaGj6oSEz
IbhVDb8Qh5GDr7Nvp42mBRZFj6fbwTDRMRR89TG1W4Bbb3NfP4yJAkenm581YJg0df6ofMTJ
UfqZkbSZJbn7CASCaECiyjRK3PVGz5S9rCBX7x/V20DbiNSNpYEQOtoZ8BGU3f8zkwU0Suh2
qYxizFpzQqX19uO7KR8nnx5+fpmPgrgbcXb5WgL5s/8V4zlFQEE6B05XUQ+9136pTPW3j1Aq
4ZFgAtXunMsq6XzSMkWGky5z5bFR2pATexoS1xNv+4QuJO4uKU9ap5wFo4qxzYOCpfpmN0n2
aKGIUYNAMxF9ponCgZsnEv4nCY7m/UoadGyMPNs4zlFVQhsqyIB06yxd9cvPEDB7WnuzQNOI
FJ/GCWswD0AM5KYjEAgiA4kqszGPO5YMUnZyd/Dm6k891Z+26k8Pnd9ODblveXQX04EN13CL
MSQ2UJWt96HGj8zW7kNt+0kXtArdLpVZdDJ9liKXJsZL6X0VzRDWKvl3j7f39MzEDt+uSqWK
MXtMyaHptne8vZU3idc99p95a/Zz571HX+lnQXrTWQrZPK38HI8pJgGpvW0gT6kMdfA8YeTc
SFdM91lXsmLOJRJpLdDRDrui7tudKIVHhr4nl73abOi+STcyJiyHZUvGF7OSnvh3f76pQpo3
rm5HS2LYNBm/J5VJsMN8HIEhrOibX/pP54FAIBBEBhJVJgipDJYz/8fQ95v5/dbmjrd6XvaN
sHbz8qN4Lg9uHZ8pHbpyNfdFFSujWeaa2ULKgI84v/5FU+K5RQ/dLpUBHfdpuxVMUBe1eTS5
m7FQ6JJPeFJ1W5NLnxA8RYc3WSyI77Gn0qCeRhtDv3Jvtp/02kjnkSF2sB8dmNcve/KrUFQ4
l6QF9jx5KU+Nu5KmaB08CxYm9pieHzy+3dWWJC0yfqOxYZCZ1e4FHMiWXSrPCgOdgXT8se39
eh7be0qAvtFfTtm647XLgDDU8NASYMwZSmW2PZfns4JB/ZTA5GcfqPLv8f61cy4rtxuBQBAZ
SFSZCGncbxW2F5NxfL6CRRwfrHfgDODuB05WsATj0zdVAfNGBuRs6KUurBx0BWZAgGMQhAPd
LpUJM27RSn7n8vA9j7tObm73vPfMZuxHQbt9T6aX9waMpBsN/0KZPj3uqAFWyv1fua9/Ei+7
QCCIKiSqTFcRd8w2CHp8pgRSvvbSxpwg36DyyH+n9soPnK4mLlQ/cK6Wv6rWnjh68cn+Zd3s
EnS7VKbTcV3vK6XQxfFe2sptoE6Qz8UovkdbK9qtK8hZDei5CTpJ0Y55b2vb1xWBQCAICBJV
pnuIOxa4eC4/WijWRnhbG7ZHyj9xfv3wbMU6WwWDV475Z0pg86Dy4PeoHEQ/lS0eXSoGNwRB
2NDtUpkIgC94AuvtzfbjA+67OWY32wVf33C7Sirz5hiDz6AGEw+no08lBkUdET4Lvyyov/94
X+Wg8YNzf6mD6t9uijloe8c67mDFHMqPM6AD2w2BshYkqkw3EHcQblBtUPbWvq/UsPnx+cqB
k5W6XvyOXkMF60KkMlGAseNXbOUM+k1pG2t3xV4Z8zUaj/nkjtlayHwGBJnY4LDu3EpUmWgT
d7BqDHBvqyl7XQS9ygtCCJHKCMKFTt//vq6zTdAQZD4DgkxscAi5xz0Mfq6oEveFf5x9b5QO
nKzIu0cEwUGkMoKQokNPqPCh1kLmMyDIxAaHdedWospEj7hzh2h8pnTKkbgugmAhUpkIotFN
MJ3ebqiw8MtCu7sQKch8BgSZ2OAQco97GPxckSHulImOz1cYinG1sMsCQQshUpnooF13h0bb
rRvQyul8kQwhjszWQuYzIMjEBod1t+tIVJloEHcsVYeuXAVlZ1hkgWBzIFIZgaCVaIYPNfQb
7pIf/Lrzaa9yvHbO4NApe6wb+qE2s338Ft/uHqXfc8g97mHwcwVH3DftMgdZjx+ZHZ+vROA5
c6f3v9vQlkv4fwAAAP//7F1faFTZHX7tQykFrVKztYrtVGqM7riLo6UztLXJsHVDpDtBqgFl
dRoSd7SJ7pQixbYIIRVLyiQPyhAsWggTN7ggrgjL+iBamhjztKv7Fh2I0Aczj+Jy+p373Xs8
M5NMYrw3OXc8H5fLnXPPn9/53TP3fue7v3uvDZWxqFssjbjjf9i8NzWhffVaoTgzhF1N+2PN
bZFkZoHvd4gqIiuLt+VqWzUp0nu2Gfp52lf1J/t7QTwUi6OMFa6Wfm5pxBquXpBZVqUkm/fI
A1Qj5+hUWj/QtRHoEVk4DvtG6y//Mj6SS+zLDzsJSQySOV/YOp+dqAGd3f3RCNrKpxOgOPnO
KOjOkwetdALHNjYwAqvdYuaAXAwMj3E3QedafsUdo2tAzPaMFTEOu+/J4JaKjQ++uK321t7G
GucHrMMyS58POE9mXizH5xot/IUNlbGw8BNLIO4VMULprsazX+rpycZo5l9PXJbDXdw7Z1sV
/+fio8FFMkXmSR/eqDex4ljQn1UZJL9c/FvYKudILY0uoWxpJPufL3M1eIAW2TQyqwNdXf8r
fchsaXhVxR0DCY5lqVLV5GfOuQo6mLs8rn4qTIoC5wCKuOuVmDDqXhOGK+4m6FzLQNwrhih4
NqagpOlckKL/XPwC7h5Syq6fYQ6J3I5b6Ss2ZC6EsKEyFhZ+osY1GzQl2duBpb0rBr4ykk83
7Y8deS9NXfxnvWkwxXxn9NfbokjH5cEt9WgQKaCDqPnIuxFSvfzvY9SG8bPkSObUiZmCIkcP
Ztu74snM8ONxSdzbu6NYown+TXquJtgKa1MqdT6daI5H9mxJ9U5JGWYkl2CdoF9oIv2nftSJ
SgJ3oobFCMPSyL0pdBY/nzxojbbk4OTmnW9RNadvYfmBH8eU8fAA+kV3KaipERo98KsUHzXr
P52iE8jIlSqPnKNfX+cxReXZExm4GlMsHFwp2Pd20NVwqTougkfKcTV8C/KK9K4bWRBc1Nbc
FuFUAXlY5+4gyc1Cjk3CTnJruKJPTDz+XA4kGLbtww3YxUwYrrwXBOforsYancIYg0OcjieR
uGWzzCnH58wQJ1eKuKs5Dzg9crqD2TEgjDBccTdB51pOxR2jaEd+GCex4GJajKUfc86oQdZB
2T9+MVwHQT5vJmyojIWFn6hN3CXbe3odDG9noyTHZEWgMipYBcSFPEkvhcykR9QpC3cS4OXk
PUjRm2BgA9KjsQ0u0Xwk2+q+h5ovgbmiLRmo0JYraWK8rGdLSinuyAyQqJU8s3GmAA8rvxWw
HKjNgUjTaSQYM8yDb9FNMGCkYFI0IGbZTbkxM0S/vfQAHNWiz/mT8BUJPRhnScwR1CG87oPQ
YybDowOKLxzCDRexaU4AZKyIR38pV9MY3i059QtZUDeGjcri8UiFGO87ajuWBtNpYNIwZvrT
c3ALxgZSYDmGKHoHD8jeOWJ8ybtloSyvuKXAqSBmU3QaC2KDrTAF3uAEDNPa8J7VDVfcTdC5
lo2494ki2pozgO0NBPzQ/FkWixXaQw0bKmNh4SdqXrOTZCcUdEmsFXsDaXOCfQtIoThEKDES
e0EWGXYs+WVbhEqzS1sdTR20CUwU5FWFI8vQEY+U970dS1wronhkWxyZj7wb2X6yw9mVRCnV
OlX86fPJhpY464RtsJB1LvP5ojYHQl/oAeHchdg/WCTjBHtWqrmyXE2T8p1RarqUhJWsS1bN
4wJXR46PTF9slb5qiyB949aX0xhXdG/LqQPnBpN4rkZDh6+MT+ebtmyO6ZMuPQ+jkng4WKFn
jIyVrw7U8RcLEndOaTiTdO7/JPdszHH0YiDFLxcxQuATeANu5PjR47JQQz6d4JQGReRdJk9H
510gPbSdcyon5RJzKq+GEYYr7ibMiJaHuH/wxW2cMK2uTOhCu0WoYUNlLCz8RI1rtuIrZJAD
QhIcsje1Szj0uk8U1cWGvJM0Lt8ZwXUIbFI9pcpshTsJV6cECzw+wiJKcUfNfWJC0VaqyCqe
WIUvs8I/v7OaKilaUYEltJnkzCjiXrjReuS9tKuIO2EYiqaDAjKyiL2jrEsNeCSfBk3UPUAg
Azgo/dBzVbqUrtZzUmYWjqtBPnDg4BYVRQPGybslNEZvSLgNJdXRxIHuvif9rCJPqg9HcFiQ
uDNACwZj7iGH6OeDoN2FW0U1kKp7p8/uwPiPbXFjsTCB0ccnR3tZqIw3ZUIG/ZoU0rO64Yq7
CTpX0MQd//fdH40s/x1CM4EBSaHdvgS2PmBDZSws/MSCoTLkKyTu2AB70+mmcAIMwJCU6E46
SEVW0XGpGVNx96IsqB+jWiqauuKOFKqY2092qDAPht+oKHDyJyeYIYJ69g9KoVRF0uMSqE8G
lhPV/iyV73Vj0NvklEZ4MR66vs5wGlJ5FcjBUuip/qIestWG1hQD/Zmuh1yr4Bk+nIBqVSyN
A7o6Lqt1Ytz1EHkGrHP+w07lO7ciXb5rxYuDZx5l8Mo+nOrGr+9N8aYQBwl6R1cwj4zOd8bh
bicaQX88F8QdHWTYFYN/sAtH5B1nLLGDfPCaG96ITbqBYXtT6jGP0MFwxd0EnStQ4o7hui8/
zOcuLM48L+yaGu4TK3/QLfyCDZWxWEnU36kk7JKGaUck7P40FmY6lm+NFM4pXT1LHToYrrib
oHMFRNwxYHbkQVInTHuseUVG8gXxkEK7ad6weE3YUBkLCz9hJh8KL6w/A4KZjgVf5/MbfJdR
SM/qhivuJuhcQRD3ATGLavk4kFFYkWHcJ3K/vZoeELdD+ieyqAEbKjMHcNpFhdkTGb68T23I
9+tp66MHs9jge/f6T6eQbVIUfDRDwf7xQgQz+VB4Yf0ZEKxjg4PhirsJOpfvxL37nqwzRM+h
8lHR9Z9Gf3I3c0jk8HMxn0eYDxUBhKj5kI2NqV/YUJlKwCMj5/+KtXxJ8zP5H5j4Sj5tVpwZ
gqce332IRKTIgE5nQ77i7ZkYfXodRbCYcBcydKin84vlQ/7C+jMgWMcGB8MVdxMuUj4Sd76m
nRFWYbmUMIiF4wTrj18Mg2pvGmvFgnTy+CVUq39WKURzGItXhQ2VKQP+QtkTGfJyvviM1Bxr
SdPLU0jZ3Z/eLjJ+P2yxCCUsH/IX1p8BwTo2OBiuuNdTqMyAmI0cH+EbusKCAXF7Pjnc+TpS
geRb8XjQ+sXw+Cua0G4pe33DhsqUgbRbEndq6jNDWJDuppDQe0y9Yq+b+GiQb4uzeDNh+ZC/
sP4MCNaxwcFwxb0+QmXQhcS1Il/THiKAtYOL619pqGE/BhJZPnm8F1RTuO8I6qoshfZdU4vi
9xZ1ABsqU4b2rrgS1/lC5enzyemz3/3fh9/Bmgu33ZSB9dMXW13pfdZl8KjEF2MUSq8Y7ray
oKm1Hw9iX0x7hMgXWD7kL6w/A4J1bHAwXHEPe6hMyQuPyV0OxwtD1bX7zPNCamrp3z9iUA0I
OsV46ut9Dq3Hhj+2WoQBNlSmDMnejsd3H1I7H316vTgz9ORBq54BP0HWwdrVAuKObCjFaBks
Rw9m/bBFbD8Wb2iJr90XWXUsjmV9V4f5nxL8tzi3Lh5vOBavce+y52Yq0pMdnUqvOdVB4aGe
GLzlQ/7C+jMghMWxhp/x5oThinvYQ2Vw1eB3A8I1NsCt/X1g9IoogLVX6PcWbwJsqEwZ+k+n
GPRC+l58JqSmnm+S64rFEd2xgMqDuJO1M+TdX8Udcwmdss8pvZfm2V4aVA3oUbqrMXGtuPg6
m9siuyZdFQR+YFn9pt7SLAkRMABK4uWzEOr5B9M21IPXtJZrQ2zTjRz92lDbqjcqjjg9bIht
81lrgiXVG+4/yEvUbZ5vlzkbvArUPkvYt8osjbjDyYevjO/LL120XhHAbFD23tfQ2i0sdNhQ
mTKAa7qExnk4VRL3fNN8mZXiXtKC4O/7p7gToM7f+0M/NelJUZCfdRxrXCUl7dvg9Gv3RbD8
4J/u9y9B8XuuJhochX5AzCKxvTuKDGtOdeDnpEiv/U12JJ9e5Wj5/KjhJ990vPVHWSFqdlKS
yPz+1RRKIc/qzAbsggFoGlVhGylsjjlZCRr90aUMTOUtAqS3d8WQE3U6ZWW7748l8DPzYpj1
YNl0x72797uxrSjCRPL+0al0g2PkD29lwnWyux8SITMsCNfRDxGsY4PDgicB+1aZJRB3jNjU
1HDPWHFO9co06IpV143smecF8222CAtsqEwZXobKeCycgTFU2V2m7mnty6O4gw2DKJO4oyFJ
i4+7pzyGo0yKS037YyDQ2EviW/JKFe4kVneeoVpPZi+p8OWi8zXyKCk1ihz4j6ztbzebUXPx
0eC6eJzUHD1q3ptSqjkXlN1+LH72S1nbup+nWJbNYS8sQf3MCYK++e9Ftvvtn0aYzkkRE7ef
7OgTRXB0GglrYcAF8ZAzEPROGtAW2RUqoSJEpoYCdiIUEKxjg8OCJwH7VpmlKe6GP+5VqtrA
v+yQyIG1r4xBFnUKGypTBqW4uy9r/2qCT6Byrx7druj79D8S953QGvWCSN+JOxg2TwSKZ5e8
XSoC3iHQSfDjHf99Werx+CD3gh+TK2MbRBwZwJK/tTnbczOFNScAkkC/nf3kG6l8o5KSFypD
8o1tqeW3yNrA7JEBFB/bVNzbu2Kb7si5E3h2bLzomCoVepB1x9Ik6yT6T6ekGO/I+TB7+mIr
iDvKTn96Dhu05Ps742tOdbB3O/LDIQqCt3zIX1h/BgTr2OBguOIe3lAZk1l7NaTw9FnWfrvU
wnesyF/4/wAAAP//7F1daBzXFdZTKWlrGuKmiU1DcMmmtIrtpi1dEtDSWpXV2BE27QZTxw8t
zrooZGusxAvBBJwQKEKJSdl9SVmEG6mwSMbBkKrBeagpJX2Q4jp9SKtAMRuL4Mfss2H63fl2
ruZntfLuzmjPrM7HMNydnztnzt6d+e7Z754rVypjI+6WvpOgc+86WfcVSNxJbXluvFKZ6qns
/b+recx1HHyXxP3Wvya+echwbsbRDQO+MrPb4+Ug2WTDhkwfznt0fxycmzHyWjlH+QoOI3Ff
WDIVgjTbGDwj7tSu4HK7f5ZvXs49gAagS+C4XPxbf5jF8ei0kKDz3F/+7RqT84CFc4IM1s9o
+u4JE7Bn74Icndc115rIs7OROigfihfqz4Sgjk0Om/pWs8rEPnPqpuDbBF/NvJdLMbmrsPCD
q4WPkryWYttCpTIBMOLOWDu5Owk694ZEMiGpTDOPZAJSGRtxZ8icIW2jAp8wTNoQ6KMZ8PXr
ToEMHjh1/gHQfXDx5/+5zqH9Spuxy8Og7KzQUnnsQrU4xpJvQ8Srs44bDm/2E1YrX/1uBqew
w4DrorD3WBbE3fENTnXPzVK3A2r+gPdXAKzd+f2SEcYsTeB0bKyVM55o3vHfJnsgTqpCLI7y
obiRrm8/RUjRv1ipg/CIe3qlMl2j4alWxv5aYk50lJN7VjPWrsleFAlBpTIBWKkMY+dYt4y4
++Pu9Qs5fxYRvA6PH8jHYgxx5lLORtxBmi01v+4s4EI7iw+DRpN8k++CoPNG7h+tgcrv9w0n
xekoPzlVoH6mOa70g8cY7bYBdXzkJezeXc8v/9mZ2e/GxTlclQdgL4eQYte3/26yZ1qi77g9
AXQt0D2AGTiL0XfHTXPJSnghUHnag4UJIhsM/3sqoN87Kylib8qH4oV2hBJC6hw7SA8BzSqz
ZcS94c1UymTnlkwz/TkWbIk3Lv6RO4Np6n5fihRBpTIBNCPuXvg8FHGPhtv9EXc7qnV86kQs
xlisfNFi1IsTHLfeZq8FKbKNZDuRg1vmbWxsUH9oi31XRXNTtjmr4fVMnGC3pGXNqUBantel
00UJgbdNkRZ/EvDq5FKcSrnkkC7Hvvb+WG0uNaoD4RH37SOVAWV/+lIeNLp4Zza6t2FSoZvQ
eIxKdE6MGm+CZoUiBJXKBADOTcpuuLhbsBH3685CS6lM/ULOcdbTNsc+ODVGwLxdIyPSssku
3n4PfH33zx9OyyRT7SGHDy0sTew9lh07OGz7QgSHKZ98quCfJAsH40gsr3+y1Xa2hxx/EsZR
I5mxw/moo+BVbP/TrfUtHN795FRB4KA6iY49OIwWy3x/hFcYH91X9DfXWrWAg/e/eELm3JnC
Ne4SeuyJEneqYmxAfdOfHuk7Dj57p/uXY8OdGDVRBY5iUNFpq1OpTAAg7v7JPsDCbVgdS1Qn
w11MKcN1w5FL3B1vsmhpHMJaJZDfdAohT22SSDMXwWoFhZBvP3fGx46U7RYcvHDV/A8Dljle
nO2Tya0hxJ8EHAW+DpOqhRwd5W+ueBRgo92Crv7ZF8o8UuC/RkIca1tgs7l6Bb9j628U/D7E
MZyWq3oq+8QLNYFPDOER98GTytjnG2UqYOFdEGjSfcrfO706ULwzO7lU2uhPZoUiRqhUJgBw
bupkrGadHB2v5Hp1b3upTHP+vLgHp8YFfZRsDYTwofofJx4/WGZw/fgBTq213gbAeH56PhCq
tAlGfz2/LKqpCPFnCNXfPg73hug4ODpTJ/kdWCvn0EdS4t4eaK7sXqK3M/q9vA2u05N0bFTC
x29hi029GwiPuA+kVIZR80c/LM67L+Ku62m4FDw0enXTRyJOKQaj9aKeoooBg0plArARd4bb
HVfXDmrOvc3RqJEJmEys3Z1plSfGrnFXpAhC+BD5Ion7M5NZEneCVD6k6EBv04hqgtslQIg/
/WAapTOXA2NFwDhDOhkCX4QNw4t6l8txbMN4KWObKxqh37fcEqLyjhekZ09JGoRH3AdMKmOH
mfY+z5F/oBfFNiHB+kYG5F0BqqgfuGKAoVKZAEBfrFQGdBzvBjNn6oUc3sprq5Wwut1l8CTu
FLiT7suMuCu2BkL4EPjiD10VgeE3Ixm/IBst9rlnSy1fM0Y67AoV5ECIPy2ok3kiQjvg1ehG
x6OkolxKiHIsvATvuV4aDxH3kACJgPHT5/Jw7NaaebcQHnEfDKkMuTVD4wklXsQlzrrR9zZZ
YsDs7chX5e6KTRFLI1GpTACl00U7bSq5uFXIREemWuIObsSUMiT9Sty3M4TwITTLHw8HRMMW
YDxMzRn95WNXVAHSXwjxp8XJH2WOeglP/Q4Emz9WaZGvCZQ0yjslQIhj6Zn6lZl1jfuRAA+r
nsrSsdaHsPzd4axNOysQwiPu6ZXKNLz0jvkbs4+8MxLLSNC7+W3iigzqo+AfgAHW3nuYX5Eu
SHiYq1QmABB3VGUj7ibJ43LFMf+DX8QbhZye8XW+WtZWK2Zxib5N/Y5K4rBFkUoI4UOOqwA2
WWXc/Cd+gcHovmJI0fG5M878M2NHArF5CU8oOf50vO4QvQp3+YdLRlVGnPgMRD+zz0x0IKo7
5AhzrON2GtkI/YliQjoZAt8CHAuv8osQODml8Ih7eqUy+BGBrP/iksnIvsVfOi6Hlz7p+7xz
seGydvD4rbVC0X+sv0xXKzcP5f839I2OlvqVGTzWerRBpTIBLH76Xu3NV224nRGgZoJ2m6nd
KzBrJPO+25Gp4P0S4hmKfqHvfCgaSvdvMaSnrcBA2r+9ffdnS4S81FLOsdGJQiDEsW0meXC8
5iqtz7MphEfcB0Mq0y9wVtRH3hnRWPt2hkvZ90R4+R5vvcdXDh3z4M3zvc71oVKZMMi8rzsL
zO1o50NlwWrZSe7tXpZxbuHl6bgsUaQRwknG9Ll8ujqWQvhle5ROF9Hh7zsZ6ghpcWy6mish
fOZUCS5NL3EnUvRjT5GpaUH9jYJLwb/mLg+667sPuptTUEMvBqhUJgy8z1AbXhhjh/PPTI6M
T504+VQBheeeLZnMG4fzxw/ksRFlFrDmdpRxYkxWKNIK+XwoXc/xVFibCiNDkN9Q0wvhEff0
SmUUSSONj7Ktx81D+ctDO0DB54fuvTz0Zaw9Er/hgsP8x6OGXgxQqYxCESeUD8UL9WdC0Dd0
chCucVepjELRC6qF3G+G7gH/jq5f+dJXWCZB92/hmoV3h7O9GKBSGYUiTijRjBfqz4Sgjk0O
wiPuEuJcCRH3rtnMYMzb7ccg3Ys0kLhjGR36OgsdLb/aec9LP+kp4q5SGYUiTmg7jBfKLxOC
OjY5CI+4S4hzxUXcV75wCi9PH63OwqVdN+m3nf+iEtSw+GmXGYpw1uufGEtARbqbw67h3gsM
QCVYdzdWipVM3TDeWLi61sW92ISb9IbwIVv9wmfLlfpc5eb5En5KtTdfrV+ZwRpfPQpmPVdB
oVn21jiSh5ldcxXU0AtVUKmMQhEnlA/FC+0IJQR1bHLY1LeaVSaWCZjAUKfP5Rc/fLT+1kO1
i7tKp4udMm8cDNJfK+dw+sI/chy73ylbXfnPCi792vtjqOTMB4+hkvLccqdmwHJUgtNxO2cu
5VAG8+7IDFQyuVTC1WvlDO4FleBeOv2i2RGiV1HDyadnF2/rGy2M3v+c6ZEnqFRGoYgTzCpt
Z+NiwWYX7U/ZzX0UtYd2ricz7Zd5bc3WkE9C0B5mchAecZcQ5+qduL/tmClT1j6+b+3jHZ/9
5V6sr/17B0hnR0+Mt5xrYO1uJaYeLCDfHWWIargsCmfxdCy3bnzHEN/bHXQheC/gyrYSlE1m
6k7uxZhRzth7qVf3opKOuHvDzeNEM+BS16v3cVbKDuzYBqBDuqPvDV8NXaMvP+H/AwAA///s
Xd9rG1cWzj/QpeBml2xKsfdBW7BqZ5UsFi5IbOtaYvGamHSCIfVDaDoNrq2m8raGEsp2oRBM
28WLnIcWIVJcirC9ZQslLJTSPIR2wSTrp6Ut5EGJIX1r57GwzH53zujmaiSN5pcy+nE+huHq
zp0zZ85oZr5z5swZTpVhDCxktVDioHbbmtSfD7NNyjT8bOrsEVVbtlW/otkp6oUGeRrS2VDL
yMauW8sGRfiM+t81dn1aatj8M3atvDSoNLD7VYKryoQn7qXtfYun/kISd7BMsHDvZADDXnph
neipZMzgu6Cq6rd73SXguFv+g6qGYP++qJXIuBDU/4EaaIP9ew+6077AdVHVQCO/tuTdGnA2
intZqQbJgf/QC57eYOBWna+H5KucKsNgRImOMZLueZgukvvXrb3FwZ7uoCO5ZHQPXFVGf3Mj
5PugsGE9wGxTTMyr1x6VMeaOknGtzhcqkqRKIV6IuxQOBxgM2yEEWtEOegGGndnTKc5de+fR
+nykfDHlPfOH0vQbdRDTG6vi+2Uevw3n8IXqLs0IXAKzn28ikUPelaihzv0OCAZOlWEwokTH
EzKWy5/79yl7GUzcuwQ2bIwY5qoyRNYTr21Mr1Zpfmq1Ou1zolUcEXdB3EvZxKv2AMekdmK7
mD/z9j4GOyLuRNxPuWqF1U+WK1KTs8sZSJA6UMR99nRJVbXlLsgJg5ufHhT3srQ0lSu5m4Km
p9f0xpi9EAXOfapuZ8yp0U4I1KB9UYUUv3gK/QvlSqK47vcwDcwE48CSMt1f3j0dBB24be60
W0QNGhDy8supMgxGlOgjPtQXp0wf2bO/wIaNEVxVxmMY2F2C/uZGY3LISL5Q8S6BUtqK/9Ls
pHAr2g0C7TcvfPn6em3zCSX2P1It61TUxaMc7MvGZe3ewZMPMnY2n0APWcldjlGXUN3+HA6D
mieDffFFjQzrnwmTqjF7ytiRanh8mjF4UHe5OY7eMrLuEncPefnlVBkGI0owH4oWbM8ugQ0b
I7iqTCT4wPz2jdVS+WKqWsoWv3hKvJnqMwGMqOr6pQI4LqYAVWUg4aN7JtQQ5Vyu/RqakBBf
agCg3VRVRgrxW1UGalvpQ9mdm1nwfkijV3V9HevdH0Q9SpiiVp6gQj1QzJcawwD1BdOW8+ZG
y3lgcKoMgxEleoEPecxo7Av0gj0HEmzYGMFVZaICiCkILqbip4eBK1BRuBpmAQUPfGGkEuyl
7f2PTd/F4OklV0xYHUJoXzy+A2A0tqmc/ELZrkkfIEZOzgwkBOD9Q4KWNJ1yYNRMGHWuLjVD
X345VYbBiBIuJ+TZ5czs6cRzk6mJxfTsnIYLa1QblX/+alk/sZLBBVftd1y4oQZO+43L2tpB
JSoFugfml10CGzZGcFWZqBDVfT/wa7Ly6mo09nhcMdjSh4DYFehlKBnt1zA/vH9VziWPp5/E
1JvbnCojMEghBEZfw/2ENOq8ubk/ABy3CsxXxtOb5g3qxIZK2/sd1w2pQ7fh6wLn61FDhPvb
m6ZzBxP3GMFVZQYJRhv6Hv4zPQE0CQkOsXvB4Y/W/P5VlaOLUrAKg283j+TlVE6VYTCiRMcT
Ul9Oytv2zvX52dOJicU01Q7DKT07p5EQ0O7d74Xvnl9bWr9UQL9YmkuKaU7DeYtFaOA6i8sB
Bv/l552ynp18cXRmXKO2iOvnksVPD+/9Zz5fqGC7EIXBJAHbBa2HTHTaPXMaZIpt5ZJY12+G
ZZfgbk9pKPrZ0ilSYTlOaVgPq5D1wgOisN2+u9sxcY8Rw1xVhtER4Rl/LEWHhwdqkowXst5u
HhicKsNgRIl2fEi+znLld+nzH4tAOPh0MlX46J7oBFd+57/m4XdbM5MF+pLLuWe1V74xa5+9
Cw6NRaqcaik7vVoVFPx0qU7c0+f+LZZe+H1CDqa2YV0jUulRapv1fEr4DM+8vb/79W8xNy2Z
0Gr3QAePJ5nQBLrFDnd+eXd/C6bbNH8yG23bDnBUYFg4MxFr6Qo1SUk9lPGCiXuM4KoyDEb/
QhL3u1+Lev+4XaLR/NOlzakyAhxCYPQIWp6Qapa5vpwkcln7cP7pNZ0GlPUs2KSgoVYYGEL0
82MgeeghKm9apz06RWh86vHZmQpYPng/keyV8fQV85CYKzF4MFS0iSbCQ8BaxMKFkOXkc5Mp
8N1UrmQH4HNJEgX6PpXUwC8xgHpih4s9ARghMZmhXcPI1/+gwba3TR2GxV7AdckX7Mpu1VKC
9pR8GNPM0w7CUOLdgzktUVyHNeC3wD74KVcUT0WsRxDk4WDMiT8v0esKcJzIL5Lm/WRBo8HS
2hCIgwt9/n5xbDaTgHnXDrx+IKarYOIeI7iqDIPRv9j9/kGgXWXtFIBv2XAw+JBvuHGqDIMR
Jdz50G3zGvj04pYoi7t7oIM7SuYHNgmGPTOufWCKkxwk75VvbM5NBBoMkjJqQK9BGUFPMdgK
NudB7jEYILpPaoCAopPkS/Zf+1uWSh2jMb1aLV9MPf/VDale7T39pRfWKYDdI+jEL8W+y4g7
dhl+izCj9bgApPzCH3U0dm5m4aUYdR8GZhE+kkW7a5+9K2s/o5PcALK/fUTGSsTv6X1iMiaO
EfkJ2DScByyCqH/8b2na+pC7eMSRsx+G0HE0LPXeOvUYHSYCE/ehBVeVYTD6F4c/Cs5598st
0/VjTLJBuU8y1g7aHfLD1Zwqw2BEiY6pMivjacmVP3lRJJTPTj0u47sbl7WJxfS5ZzVKTydS
SC8MgVZOJcVSkFGQTgwuv5zGz7PLGTD4K6ZI/wBzlXkglOaOM1yw1Rm7ODEEUk0bbEJG3Ckb
nrSCAuCa4K8ny5WH/3JVMzo5QoKaE3HHvoBJq0aDm0Q5SPCRYAFi5Nhx9Ai3x2L89AiCAuq0
lKTBpUEPfCRYibZFjhCIPnlBVqpSwhY1Jl45qL2fF48y5rRUetQeU9dESmhOlYnLwkzcYwRX
lWEw+ho4iUSR+/f/CvIpSPz+FhrqXDawVF2EtdAIv/VI9sIXOFWGMbDwy4e8ZE20W6pWMOiY
1tKyoC/IPX0mENQWDPWW/wrE3UazPR0agqzT6S9TVmrv6fUE/TyWgiuDUtddF8GtKStJPoWQ
6+JqSIF2GXGvfTgv85EoOR5e0IU/VSyvQIcPJtKZvrTZOWyYeG1DfZGA8mrI76IIfYQ1QEOC
iXuM4KoyDEa/w7ByZkDEfU2RXHg5VaYB9BQDxs2vLeE+fWZPp7b7HF4X3wUZBO//hJZFxMw2
Pc1LfX1Qw9GQbdDQVHqU2pTFYfQYpXMn7iKz0OLNVCAfxNowne/4UpC7/HL6+LyGs5Xe2SWm
vmn+tHMzS2Hy6dUqiRL567kkJdKYVmLSxGJ6KqktlCumJRxUnqLyIOLPf3WDeiipHY4Q7Inx
lH7jiLhDh8Rkhp5sxP40o6eO8rCBq8owGIzA4FSZBiFUJ86vKNwCq9ufcxiDYbq+THnnyFFq
3znyyJ0jx0y7E+1H6u2jHvrbjVHbxxrHtGs75Rv2ug9kBrZDVOhYDlK2m3MNHf0ekDdbPZcI
48x0o2x8JGDiHiO4qgyDwQgMTpWxgdsY5QwEg2Hx/t0f+F447HDnQ0SdaUx72u1oHyOxnsdH
3I4X7lVlGn0Pp9FatX0ZwUVmuzHO/lZ+2lHTPqC/8W+PyMDEPUZwVRkGgxEYnCpjA6ydvoYV
GLDj+qVCSDW8b4vRm+iFEoqDhI5/9abHCzH4NoFcgmNeTdAd8DUkRnBVmYcMowvPvoK9mORL
eAAhfF4PAzhVxkYknPvMnh5eSDO8n8Mex/C53T1wIDNasD27BDZsjOCqMsFQ3Msen9d+tZCg
r1WY1r1MPz+W+GfbT6rRzW73QH+sMIoVs9ZIwxKFnyMrGS/Vbx3bPbx/dWIxjXVdVt+5Pn88
JwZIVWlFEuIe38G6WBGry+qxZ5cztLmT1ps2ZvubuNwuVaE1rRrEJ6x1f/n6Et/6BwOxnML/
BwAA///snV9oHEUcx30UPexDm6R6sUYsR0ivqaRCTgoJNfRysW2o1CvBeA/FupZLTU1iOSk+
tJaCpNUHSfJQCKVQH44kFgolFkQU6VMTr33qv7c0h/FN9l3W7+5vMtncn71zb687e/19WI65
zc7OZG535zu//c1vVHSVSUykaq/GcF+y9pOUw3m2ovORJSOKMPWA9ZC3VGzPIA5EVagzX6g+
wlFlXJF46w9zBlrOmHv7yxRNB0fHDakaW3JaCxlyOTI2aVjXPBK09sXQdJ7iPr3+Q0UX2cJy
ddt4oGR2eVqUuCOdksuxxX/O0FoTzrVFXsMszgz7S8t6yHjBzlWV5cqqQrXLBZuD+aMzJWBX
GYEnwt1bV5lj6RgNu2mo3TQ2WXIqXkVK6nimTnDbekv149Ug4uO/0ACtF1w4qowL5u707r0r
0pNfmytXrF/DCbm/JKv32+UBtBqDjURF4V5Qrj376r3BYmM/zvaXkehd348sZJXHAAOJC7fj
EO4OJf70b0oqe2gAygt9Qr2/8/OwoFyrqonwpVW+2RsMdpURKOgqQ/8RhsvbTp6r0kWP/kSv
4SoepoLZr/HA4xUbxSKUieWHy5x2kTYD5f6fCd8Ft4myaRVgi7uPcFQZF2SnIjZF2yOFe/7x
tINw1y3FLDOS4pd/Rb9vGwBsZClX7vhCLwl38mbZkU6VdJVBKQVZIPHfzS3pVnhZ52EGDvjw
t99123lyxhz+3+YzKZSIkzjktZdLAwb81i0fRLaNtiEvGfKZBoBdZQQlLe4QDVteCm156YUy
n5SgLUQLXtZYjWI+urHbLtzxg4VP9YQH2vCJJ07OuIb7GfcnOe3hzjS96A5m4jei+MQB8/c1
02y/v6VpbPKK8ShnaDjb+G3TXQ8V3mx7YGoFv4LQnQ+X5UYyVO6XXzldTZrCsxbIetL0lBAD
pIfLck/+n02jJvkVCZndx7S92va6/VlqyOdlomBEpMxSUM8hHFXGBei8pK0aXZit80q0X3Jy
lUEnSFIYChhKnaS2bpnSnX1sisslNWwfjUfGJosd1pFl6voS7YfmRhZ6f05e9djk4tkla/ut
sUxFQJbI2orixjN6eSuALFe3WdzfO79EBx9d0LjHbwzYVUaAq5w6fnMxF2sReDQNurrN6txp
29VuLj7vyb8jQX1mT+5u2ng0JML9PfSsuXA73nw6a+r4A+KloXb8zTeu51duXsbwGgnLopaA
fKcHDR4BGKznH0+/2t1Db/fwKNl1x30ETKYYtmJ6To2xnphy8LXqIxxVxgX5tZnXRpYosW9C
I3lKX9GjObiA233cE6NXaewqjdMV44DZy5ULIUsvnXc+z8qa6LYsZN5GQfaqGpZ9bWi6gkd+
k7UAM1XbnpfOdsV4VO4Vgb3cnddGaQ/VENgXg2MCDbvKCMjiPv9EmO6iHa24ypGoUrWTcPfE
Ub4AyGsp3FfvDZJ9HensrPZyx9W5O73YQ8NoLR3FTfv012kId7IQrHyn0fgeA/1QRwSKX85q
x419eCG57eS5in41TPWwGPKcapo0cFevChXma9VHOKqMOz47v52io0jL8cBXreEB0w+k5WAG
irZcRshl6govPqC7L4F+kJxP8FlRzhaVm6C8L7Zn7I43do6NdIWtnne9RAG6bOfJqWZtf9lN
1aO+Pv/Y7NPD/T3hUz0VTeayXDLY6dbZSANUjGbDBAV2lRGQjzsNANClkfncsrhXK9y3d0Tq
Ecc9OxVpPi1GzGakp0Ex0yVnaHhUkScM3cxHFzRI/JWbl3GL7r1r3rEQ+uEDG6/26IGF48lm
T97zLNw9hMWQ53CT1gluWB/hqDIMw7iGXWUEZCzfEtpqbpbPunbWfF0l9pg7Q5sS8uv6Fu1o
rYvFfaRLWtzzazMYdtN4nWS3uWfQfOWHxJ5P2iDxkaDYsbo1Ug91RKR/Hk7ydGk6PNBGZzAD
2R7MsHD3EBZDnsNNWie4YX2Eo8owDOMadpURkLF8Y9ZpaGt4f8uBPV2dQ7H4oSRU8vwTMdNL
TpWDTMd+OgDZ0ZSe+7gblqtMZDwj33ClFzO0kETzmRS9pKMXYZDv+ybMmLKmlX0wKee+4Hh6
TYYNO3OGaaHvtTwCzTOXmljDuCZAYigoQ7UANWmw4Ib1EY4qwzCMa9hVRiAs7pbHC7R4tKNV
2texZ+crhdZ0KPWuWBv+BNWOg5FGrhPv12vlVL38V6MoUmTxAfYJLvrm/YyHsBjyHG7SOsEN
6yMcVYZhGNewq4yAospAnZuS3bK700RVNBC5sBcId2t/iOakmgeEtkLK18PiXo5qfrZqQr8z
HqK+GArclaB+kwYUblgf4agyDMO4hl1lBFKXQ393DsW6oyIiO05LZvViIzeyQLVjzIADst9/
o3u9cqozzmtGOOxk6oeyYki3JkvgipWXRHZWM728+qPHf3Ra0cN3lG1SCWpILnPYtLOTflen
WtRv2AaGo8owDOMadpUR1KK5pWtKPSanOhRqsDRXDDXFUH5tBpe3lo6eOHxVznKGZEcaCYxR
VfaYUrNJ7aANP/04o3IblkT9hm1gOKoMwzCuYVcZgSea+xkId72SZOcnso+oLIbmFgfjR6ZI
X67MdiKtr1uLLz7wu3LlUblJiZwxN9yXdFjOUE3Ub9gGhqPKMAzjGnaVEXji5fIsLe6Mgqgm
huw3RXaql1YNtNIRSufXZk7tiqm8DrZqTVoM2vDYSFd3NNk5FBu/kQ+Kdle/YRsYjirDMIxr
2FVGkF7M1N7jPksfd0ZBVBZDEO5d/WJ8izRZ3yE6h/uSbHH3hNV7g9Gu0aD4sAWoYRsPjirD
MIxr2FVGMP/kFtqixpPUfgYm0KgshsjiTnf706XpzqEYCff4IfZx9wY8hTAcCkqFg1LPhoSj
yjAM4xp2ldkA/W6NJ2EzxnOOamJIWn+PpXvi3a0U+eSK8Qg7Z7VeSPb4kYji3h2qNWkxOUOL
90e7o0m058UHgXkIqN+wDQxHlWEYxjXsKrMBetz5v29lvhhNTKSq/xzuS6YXM/zykTGCJoYC
EeY/WE1qBGeKauAatpHgqDIMw7jGl1v4PwAAAP//7N0xaxNxGMfxd9OlQaEgZoqLxA5FEElw
cZMbUunSQSlOikvn6lbESWriZiniG3CJoZPFzgG7ad5AfK6nban17vLP8+T+z/H9IOEuDb3r
L4l58vDwvxhHZYD5uSiGylwBIB4uIvWIYCvEqjIAgjEqA6hxVwzF/65xF6kXBFshVpUBEIxR
GUBNnMVQ/NV5jjgjrQGCrRCrygAIxqgMoIaPQ3VEaoRgK8SqMgCCMSoDqKGLqY5IjRBshVhV
BkAwRmUANcOjYdWnUDfUl0YItkKsKgMgGKMygBqKIXVUGEZ4rVaIVWUABGNUBlAzOa2H5N/w
aJje/kp3xz//3snu7LuDk/NdUlXZze6J+XK5tceqMgCCMSoDAMDisKoMgGCMygBqvk73s6b7
WWvz31bxpTvPd4seOeedNThQvf+6RRzo9PbPS/Sqh7Ex/8akxBgSq8oACMaoDKDGy+XuHSFP
IwRrpzBbVpUBEIxRGUBNYactKi5KN1+ROkKwdiLvuNPnAlxjVAZQQzGkjkiNEKydwmxZVQZA
MEZlADUUQ+qI1AjB2om8406fC3CNURlATc4HNl8sw1BfGiFYO5F33OlzAa4xKgOocVQMefnk
dhSpLwRrJ/KOO30uwDVGZQA15Yuh8i9Xoxf25D/bsaG+NBIQLIsmlRR5x50+F+AaozKAmjLF
0N5u0l7rbB6+KXxMsrVtUSfJ79zbuXXtQXN1Iz2HbPvebt75VCg/0tE0ad9pSFZyK3GVuRro
xUi3n3Xuf0jePX3eO3hy8UfvP+6sbj60/s7w4lNbzjxLvn9wV54F+X9sYZXxrH/d+MdrySTs
equjab/bay1db8nTJBuz/pLR9K0ElfO1Qe4fHCZyepF8r4i8406fC3CNURlATZliqLu+IiVm
fqHc7TVN35mPl5svv6UbcsJJr7H+xexIc8uPVKpJKen6n8fysMbKxuCkIP9Jmm1rcLx/tiuy
OvLSj6zJmcvh5Myzc3h0Y0mekUW2tAMK95uNTljhPv7+Skr27PpE8nxlr73ypHC/vZweumQ4
i3werxR5x50+F+BaJW/h3wAAAP//7F1fbBTHGecZUUWqSaAcaR2pOSFsYtdE4ijqWW3Bd6Tk
ZJSchQp+oAmLZTtXwESOKlRBKyTkBCqQ/ZLoZDmCVtaZolRCVlWpSh5QQDUYPxGIlAdjP/Qt
uUeUavqb+e2Ol7N9/3x7Pp+/n06r2d35t9/Mzf6+b76ZFVcZQX2iCDIU7+gcnlbOu284V5Um
QQifODYIpg5mE0+Ngp1c63kF4dABzUdp5jz6a22XBft57UgE5KmjM0yqjQCu4NgWG0bRQ+eS
OEVkPzGifV3bpDvDv7owhSs6/84w8rmk5tM9bSgLYbLG9MmIzjDWjFuabHUOgwMhMmpYHmNb
OQqJNI5H+3ROh8B9KRbIgTZ4ygEyhEz0DMNAN7QU3jp+Y0qT/s7wgZY23PpYPeatcEv0zK15
6gPIlmkRDUkgZJzub0mRgEIyJv/4vgHdgjpO57DyGDlLgS5h5Taedijb3UZng6g7orr5UJwO
e63AdtTTCKb+yA35I0P0EFRAVc5fpWBfnf0kwQqj22Q9UbM/oKO6V7weggBrlXbamYo9itDy
jDWjw6NQqii2mRCZMRmH/ZmCNZlHoOXOPUxA7CwIQuCnSXVylLugBsRxC+2ohblnOySPd5sb
x9cTkInuBsHMZVnUuMVd7FwCwZqGuMqsCKItCPwo+MIGraE7xPu/TIKxKZ8hc1plwEJIbi79
TPO5rGeqBB3x82YQcZA/8CrwD2UYISJnJhOgj0wCBmPjk/Qwf3DQ1PejyB+lg5qzwgxnTUwS
dNJTDA2ozw3zLXdLjquPghZ3MO/Ws93kaqg8n4KcG8wPAVyhoJhbf1OEz2sBAdJrCLcgf5JI
T2gOSR7ioOGQFfKkiFxi/WQEXJ9XwAhZHHMDa4R6wCJsK5DxD1/X16EqsA8oMw9D/sr6Z72O
gQpwJgGnfK5KjTn5BTsx41CeVG9Qt6dTI6gPuiIS4lkgKDw7Auwh6Frusxt/Fd7yddo4mimU
SKKZqLKi95LrU0fK6d5QFVAcSTllwjD6MIVGIbN6qCdLtBMCnERCHLJ825osa+Lr20FroTVu
cRc7l0CwpiGuMksDIy9EM3gqhfcQXsBLHvljHAZQjYKT9YI6RjF+HdqmGA3rXyfpddyyChrL
wU7A760ZEndJmqfVmDY6RmllH2ZuTTsi5D0g7vQhRgRmyD/27F/ayZBQt78dTh69p6gYWFcZ
hGmoBpcKHWykUV8T9zvtfsbP+YHqI79ILaVThq4dGZmH6NyJCM/gDbmBLNKnH7mle3ZBCEq5
lFSbxvdsf/29cY/T33eZotc6ePZQLEohW5a5QGf/PWKv0AFJs9hXzcSFT9sBGSWbZyu8/fkX
Wc89RhkXcLYI8kErtEUakRZHyB8E2raCtRxXQbCsMAlu+mQEgiWTpkZEDQdXvLqNcf4BMdF5
8Oyg6SDN9vG1/ulJ48xNnTMUIT4mYloNAZngdH9Tct+Aw/klVtIqt9QzURn0avRVtp2nIcSp
TrBL69kkY/tH5lwFoTUN/JtKcbkpGzVucRdXGYFgTUNcZZbODXKZ+Pq2ts18dR8DvT1iRLYB
xERg/lt9VB7XR0KxZ6xbLPfCZn8gHeQVchGXTFhzoMfMSOPojWAtl+BStP6CkYD6gCSB/9ki
tIG887mOxzAIPZklmB9ID5mlJe7KRx9BGf0uMagPLff+ilUfxehC1Ja1Op3SRvGfvze+2KRK
7kjmx2eBZLjSQLsSmXZJ97RRh9EGY8PwwC+t1Zx6FP1qKBM61VCzAnmF5oCcZ6/EwSxzSseY
wFagcdqoZ258PiNVNWVawdMZFh7Q47WRqhF3Kgx4UqptnFtwbds+G7ydf0AvgjRwBZzbZuLr
jXH0dhJ3PDWaCd2VEws2JkrkFTQHvZKsoxFLdGdRTFfk/JKVkt+sziJ0Q3hGfetcZIW56sRd
dpURCARlQ1xlckH+DcoORo4weTlfDE+/fIzfApVngD+efnUfr7rxK2J3X6fI/wEmTtTYK129
UXKdd99wDPPLcLUiyeX5Zxn1PB0ha8EPlIXMxvUJjjWTKYJ38pTO0P5yaXSk0VcZxSDHVYbX
XQflQ9rk6XcLsd4d1UdB4q7nGTyHckbGUxxoaQPVC592XYlwt2mHu3NOumcXSGRyZhSSp22Y
z8vFqbiFwcRPl4++qr2uQ4kkB5nBUykuGOCiWOoDWTMf0t8UoYcG51VQMQjNkkuuH0Bub910
qa1dIqyeX5yqvbRR+ZYo9RASaDL+/JsRVVCwrDAfhN3GysTa/ukP88C4yqAXkZejs9GUHj4z
aLOyfVUb1H30GtFCsSgULesVhjj4O7Se7fZNerhKFx6/q69twZne9HbOPvldbqgFQXq9k4N6
+sX0fLYv86zCCuAat7iLaUkgWNMQV5lc0GQOgk7WjuPbn39x9J7C0QbaP5u3V/DDG0tb4o0Z
HklA+sHdZWRchyh1p44829stF7anIC5kohMzjt9Gu0LkKXpVUN6ejHlqW+SD2KWWx29MkfPR
YbpgDuC7NNIzSU672BZcnIn/ShWWAldks8sln6Jgtym+CYppx4LRcmReC8RddpURCARlQ1xl
cjF4KkUTOy3oIOJcEGZ/OL2k7uPdbHk8nVOV8VWlhd5vWBWsH1Tta0F0LdA7zMSa9zcl7a6C
/ggVL3FVUBHiXoyCtOQtkG86bUPOw9enCgqB7cLdaXC0bjbFlFuMVlBBlPcBpoqg+HyKiVmD
JpIat7jXt6vMYsUsIFWtBjueYJ1AXGVyoT8y4tnOaUfvuwumPm+OOoyXMfg6fgjj2P7Z/FX1
HYk7t4FDqhPHBguVI6hDVNA8XJ6pMg9hXSEHWlvEfTHyiCI/Y84uCiyXTxlYxXd/SYJdXQJd
Us5rYppIdpUJCF29kYb+qF0Vjcds7Y/iV/G19eAJ0Oe5lr1ehSmoWYirTC4GT6W4AtV6v2T+
lbvUjMDoQKP7x+oxPYm5SfDTLx9XyuJeRtsEbXkV5MFqbXZexxCRBgQRbHAoKFvZVSYgdHSG
N/ecxxuZp1xN/tLvx/3rmFW5ZhE/Tl7YioLw9l/OlVEgCA7iKpMLMm//stS901PWtR0atvWQ
sW7uFx+p3snBFzY1dBxKKmNuAfsvvsQyLHmlPmwtWKHWA6rmKrN+ICINCCLY4FDjFvc6dpUB
cd/ym0FrX/cTd35CAW/ztNO+OdW45XAYb/DZyw7D0euut+H4cHtDfxQXX742mvU2LGrtjyLO
9g+acRFZIQecItqmnWFcmVZOKKaT8JjD5gWCikNcZXIxdC7p3/kRx767LkEHceeyVHvED3fB
9cHa2yKNSF62j3uplvKyp48FwUHIUMUhXTcgiGCDAzcIzgPZVSYgLCbuof4oKTjQ8k4j2Dbe
2sr4tYJ2t36fyZpwKKE35yUFD5sI4Ebcrrf1bDczzEwmkJy8vKs3giR4+2fNBEt2qdIFgoAg
rjK58Pu4c30qKDu5uzW0k7jv/o/2d9834GzdqfduQ/ysch1sSrK4EyucZcuftl6H6VqDyLni
EF0oIIhgg0ONW9zr21XGT52n1VhHrPnF00PcULWrrw1hjtJg3pt7zjPm3MPEtmgUL3QSd+ta
49+PSG8O+2Rky+Fw2OX9kZfe7/ZzdLQp7iITgSBoiKtMLvw+7lyfOn79Njdr53UMATTAg7Uf
To9u3NjYvHM7+Tp+dLMp1eLuZ+00EtgNnsv2ilm5G5+gVHBPa+pv/mNOj5LTYk7t5qoizEqd
+sVoP1VRI3Wrs9P8kF1lAsJyPu5K8UMZERB3Gsid3mbE5JZxEzMOTenKbLNLVxncvaT0zEnm
TjvYPB1jcIxe18T9zE3tUWO/vQBAK0ASWToiqALEVSYX9HHnRpAPDB0fmBn1bwZs16TuG3Dw
T/7RnihKpzc84yNhSbvKYEDRX2a5555q4h5bdhU8hxjO66nndykW1AKq8F3GdYWCHEhQHh6I
xT0YcOo1P2RXmYBAOu57e8ZD/dFtfe6urKD1JO6MaW3zs5edLYfDkamFXSjmHibI0WmMx9vZ
fHpsDNGa7mg+kO4Jb/tF0r61Qe6tF41AEDTEVSYXrsXdeMvQfE7iTmsKcFVpc/vu9OjWnWH8
nD8MQS/3212Qduhcsnixgqm3vNNohwC1iPxlfdsUQK3fO73wOcb8G9tln0+b9V10czOLdfyp
FvNOUQxKgvChykLkGRDkTx0QiumxsqtMQAATAIHeOzPK097JwVBMO7XzDQ7i/vK1UdrjQdyt
r8vf/9fNtaqg6Xwhzj8ZQT54y2cmE5tTjXSLHx9ut84wXMPad1eHtUOOcdGRd6WgOhBXmVzE
B7rtfjLa++U7PVmvfH/Iif/e1h4yoegLmxoQOH5j6uIjfdFdz2qOJbnKIP5rRyIcGliFkKe7
T8w4oYONGF9+3NuNmvz21i7O1vH0r+oj3MXpT8dS1ByQ8M1bekA5/yyDoQQJMdBgqGIpSPXm
zeT2D5pDiSSUDW2cSDVu/EkSyT+d00MV18Uz8xumVm/ddHDaEMA+uPUKIZqVhcgzIIhgA0Ix
gpVdZYKDfmnGongV6qnpA+76UcJvcU877Za4z152rL385IWt9IoJfTjHhEPnkg1mVxmkxTuR
Pu4g6wjjevj00J//2dFgiuMP1H9VHlywfiCuMrmw20FaIs4vJvI/DHY7fuVPHYeSekHq6SGu
UuUHmGih10tYvlUlEXfuNuWzuMc5iED7tz5zKLHpzugDs6EVBw6ksuNOR6wZgwXiY5yyK+jp
l49oYN7GMBDfFtVpkUn6ZATRmJu3DVacueFi69luXERCWhdw91rPK3YUE+SH8KHKQuQZEGpf
sGt0wKl9i3sdu8oIBOsB4iqTC70dpLcXJF1fzj/L8Nbslfg3G37wx9c3j6edfQMOPd3xu/hI
+c3tpe4qQ+JuHezo4w7GTDMAt4bFb8eHmnN39bVxcczsJwkQa+4mi7tg5HTLs6vaaVBHBObG
Na+8C77+4ukhzdH7NZVHD5j9x0d0nWd8KAnMjUYIZMXF+DLcF0Tt86G1BZFnQBDBBoTat7jX
sauMQLAeIK4yubCuMnal6YBxmEPgmx9Gfrdh48CGrSeODdqd3fkBJu7VYHeQLMniPq3G6ONu
P/ewaWfYWtztIlSiqzcCBu960XhLVBlh/skI/flMno63MW2G5vynUyOhg64nfdppBxFXZurQ
qgGMb1e7Wv1BGcd6Ie5FQvhQZSHyDAgi2IBQ+xb3+naVEQjqHqvyF/4/AAAA///snVFoW2UU
x/McChVXNpk+rKJDSHTSIS2OpYzGNOiE7eHuwboHYYbQ1TBSRmDsQSaCTJkwMh+EUDrcQ2hl
ogwZ+u7E1rInnXvrWtC3rc/C5/nuSb5eb5s0yb6bnu/2/yNcvnuTppeTm9z/Pfd/zifaKmOK
U02l6fl7WrvTcvrHykxq7Mx05nRt7mhtjiTvaX/JxakMl6h2VZzKbjlTkM6KmVZJfLOCN4pZ
F8H4nhmlGh1nx5bWA/8of/C4x1KbhPj+i2dvrimTODfWF9XMoOv2WH6p62YtTjNbz8I9+Hpu
bQvhviPO6SHhn6lz8XQFhwIr/BANIT/jDqsMAE4Dq0wYnoBpwy9CZe1uMus8ARP72j+4tWS6
uS8+/F/n6W6LU5U/oxvpbF0QM5Phunh6Z8VF8TOZ508coC3l29oqQxr6lc8bppryXc94aWiv
WPHzHyo/m84lNbSknWSrDD9b/nac3fPln3W1K2ff6d10TY//bqV/57jEh/P3xioDdkSUHjIf
GR85x2YLZku9VqDLRTpIhN83FxXPVnC5SG4yTSGleDrxTXElsFMT3uszOraFS278BMnPuAv/
ygMA2gOrTJjKhRJJdtOUnRvLGNf7o18ekDT/Wj3gOlQamLmZ+Cke0Jv0PHGSatHDsdUrt27Z
+gjNAxcat9kN0C3S9BB/iHQlWa+O505VzV2d3EmPa5d5IBZp8dyWFbVw7u2Ccqp3qhOBpZ2k
wG7tYysZ+Rl3WGUAcBpYZcJQROgyIDgHE0t2WlWqYWH/vTn7oNH0xldDf0i6v6vi1M7p+bzV
gxwPqXzQIWL1EAn3kcmGcF/9Uo9ZuE9NeNyNWCZi4xmEwsj60hVxqRwJ7IqaDwbWidjKz7jD
KgOA08AqE4YiQg8zeTUJcdMd8hZXoP79FafeeTstzUYW8YsP79SvXektrK2y4G3erc2znWt0
YAuxeoiEe740x8n11dprb35U98d5CPenh29csFWmfHt95z8QgCuBpeNTe7om03HyIKGrDACg
Z2CVCbPhvxvFxSTXuVeMyak3Bk/UZt498Gz9mzuunGBAFIjVQ0GrjB6f9DaaHuLP1PIu71xr
xMZzW1a//2I0Ldp6ZHArsIv3C+mRrv2Hu4L8jLv1ywa6vnpvKDmYHBgc2DdU/JhWs0dKxuB0
vTj86R/6NSl6zcC+wWTiwDsVuzsAwJ4CVpltYO1euVDKz5798P3KmemMGZglyR3aSEt6mdle
uHS1fu2KW2dEYBexn/4nd3N0lLKmXP/rhvG40zEsuV+Q2HhuC/2eUmB5TnXhuBVYEu7HZgux
uSKKWVcZLcpfrJpPh4W7Kaz6Lj12/p7eePGEx/eHz71xWPJdPgCEA6sMADYRqIdol65e9kbG
Do2mteuATqgraoEuOHOTaVKZs/dF3yASGM+tPFq6QYHVTV0nPC6GkY8TgdUXQqcO04H61pGR
hZ/i40GKWVcZvv43KfYVNc8Zd378cPlZnpM7m2rkCILCXfKPDwAygVUGAJs4oYccAvGMCAQ2
IuRn3KOwymQTzwwmE/TgOUNIuKumKL9eHOaMe2oomUweotcg3Q7A0wCrDAA2gR6yC+IZEQhs
RMjPuFvPc+lsejPFrvyMOxvwmFrxVX8O7/mpl/Xtvnp1fH+2bvX/A7C3gFUGAJtAD9kF8YwI
BDYi5Gfco7DKZFMej1lP5EZfmPqVN+RfSrx7c03b87LDVd9IU8g+lyEpb3cfANg7wCoDgE2g
h+yCeEYEAhsR8jPu1i8bSJSbrjIHj7PZPZ9K+FuSA0d/0+vBVjO6HW1xTeEgBKAnYJUBwCY4
FdkF8YwIBDYi5GfcI8pzBWfpVm0nGHGiOxAAYoFVBgCb4JxkF8QzIhDYiJCfcUeeCwCngVUG
AJsgkWkXxDMiENiI6ORMFLOuMgCAfgKrDAA2gR6yCy7IIwKBjQj5GXfkuQBwGlhlALAJnbY3
/OXyE3/55zIt6bH+WPGAt2O1k1UaLP6zGT0OrJzdc27VBDAUWCG7F4NVHu/4KxGzrjIAgH4C
qwwAAADQP2LWVQYA0E9glQEAAAD6Ryy7ygAA+sOufIX/AwAA///s3MFpAlEQBuC+LCgHIeDN
uwWkgRByXuJVLMYuUkGWbANhX5jf2f0+5KEnn4dlxuFnRGUA2ClbZYDVRGUAoI6tMsBqojIA
UMdWGWA1URkAqGOrDLCaqAwA1MkWLHMuaG0jURmNOwAtZAuWcgmtbSQqc7yf519yev+c3i7L
++Xj/L/E6XQ6nc74OX28LLVpLlL/XgT/7iQqA51FHuGKibuJAgBP5fv3NRes4B2+RGWgs8gj
bKsMADuVLViiMtDaRqIyRggAtJAtWPE51/Vxy14AWttsVAYAnlC2dY7PuQ6vU/YC0JqoDADU
ybbO8TmXxh1GiMoAQJ3spCk+59K4wwhRGQCok500xedcGncYISoDAHV2vlVG4w4jRGUAoM7O
t8po3GGEqAwA1LFVJnsBaE1UBgDq2CoT/X7oTVQGAOrYKpO9ALQWeYR/AAAA///snV9sFMcd
x/1UVW1UKYoLhHOpoeWK4kvsmLZchHpWCviOYFkg9RCSY6kpZEG2e1AMPRShKFBFshxClcru
QyvrRIWFTjiFICFapYqaB5RGtUv8FJK8VAZL5a3cc6rtd+Y7O16f8fnf3e5t+H1lncZ7OzO/
+e2e7zO/+e1YUmVEIpFI9JhKdpUJ1wCRKNKSVBmRSCQSiYKT7CoTav8iUbQlqTIikUgkEgUn
2VUmXANEokhLUmVEIpFIJApOsqtMuAaIRJGWpMqIRCKRSBScZFeZUPsXiaItSZWZp3+5N+GR
/IlcZrD3YF8Kryz37Mrilb/yuD2II3238lN3p6pigOgrI9xLJf2Kn6mHUlhNwf5KT+JTVieG
fcUKslwZsGRXmXANEIkiLUmVmdcI3IHXiQc3Z//rghLwanHBogPf4q88AVWKF8+HHsYQ1ZVK
+kdURYlLayT8EQvbhMdLsqtMuAaIRJGWpMoY4avLeW0YRM4Q1B/cz1xfkO/eR5+ZAJX36+x/
fo8TgPg4X9W6O9V3Kz/xRT1+/wXGOhU6Cp23gjegnmGoFt4IwMP17NJISxwbsGRXmVD7F4mi
LUmVMVKB9i9uEsEZRwedE8rhIBZw3B+Dv/fBKCkfB0u6heLF80t6syruXjL0WLvY5NqbXUUL
Jd+IQp8DLFMCQ1VXRF1a/3dsRB0bXcmuMuEaIBJFWpIqY5QZ7J14YPJhGFA/tanpFw3feP1r
38QPCtcavsXCeMOTgw0bULhyOMcIvUm9vTt1sC+1/B6D/zovLVKuVpvLf2vtdaul2nUkMFR1
iUtrJHFswJJdZcI1QCSKtCRVxqhnVxYMB15X1K6z2K+cmeeaKwey5Hi8sjD0fNKf6Q6CXwW4
LxMc18iXMA+2fftXw2jnjnv1YF+SZRxvG1DHYfwabVhLjL/kK5SVJ6adpwZSQ+7s8NnsutO9
77gPl+xrsbcCnikJDFVd4tIaSRwbsGRXmVD7F4miLUmVMcoM9toEdwbdBxs2XN+6ZXxb46lN
TfhBGUeuNTyNH7I7UF6l0+jsGkbcX305X7kXTAyAoesPxPGzTFxeKCAsWbYC75ZJw7oBdNgA
cF93vIjyHfdSZzqxakusnL5EbCDV8/GcDejxuUPJltsFvz2r6OXe5CgG++anbnGkI7YnS3Bf
UrVYW1ipBIaqLnFpjSSODViyq0y4BohEkZakyhgB3NECw+185JQR94XNzty4wLj79UQSEOzf
t64yuN//pBsMGtfEjGav3u544c6k/4TSAuJc2DuOHOxvX3e6Fyy7/NAybOvsyjYee6PklQHu
9l0/T9t1gBX5E0PDVCT21v2S7wgGiwH62ykD99KC10eqAoWvMcxfU6YPBYZWMaj6z8C2ihxf
lhYU6lORc2zUJbvKhGuASBRpSaqM0cG+lIpG+549HW940ubG2Ex3f6oMwB2gb9Pi8dqzK1uh
i+cOJdfvyw+5s/YIv85REdOGWDoF0o0NpE5em7UmHRgr7Bx0GKHHcZx/8m/PNuaan3gmDnbf
1NfLUHrL7QJaQHVMOdAaqqCM09AaQ9S0zabHoAognpMNlNEOy05f4oXpQv5ELra3GS387O8f
0pJx9xIXCvCKn+9fynHuYcWUG5vK4uoYPLug2vTQOBB6gF13/iXfdkqNfdy9yeUI29F3flfg
6kfrYTUQDAdHrP0Y8vDZrJotPMrOpjMJNUc6mec+nnQvjmOky4zZr13LgaHimNO531xZ19UL
IF3ZVc8oUF0tHK1kfPD5Kqp0phO4mXcksrgQ6teurL0fcN3xYVSTw3Qi3prCJJPvVkUVXIoP
AhfNFnMd5vacnAejq7e64SJ8fm0ZbqlbfBdwD1iyq0yo/YtE0ZakyhjhS93sJKOpHTjCJ1BB
5/xWY447H04lvuMIcMGCPqC/Qo47qXTd8WIZlikM7W+PdZskEJXEcroXaIvj+LJHley0+r4H
TFssHnM6cH7/P0x4tU2j6vYxgwVowaaUwB62xvQYpsQoiN+aBAQzVYZAbyEeTXEdgHXRjqLJ
/fGfnpukl9D1n+4/gizBoKjb87HLbBwYv7F/0uPseSYB6Idc9f+qQOQbU6ltb82yd1RJXVYI
q8zYl+f5wHoWQD9Pedk4gELOKwjlZXb6W0B575km9AiboZg366BCj7jj0uNqHhqdpUllELxS
oTpgeqUUvrtlZT3SyKvvz879mk7wlsDHYej55Cvjk7Ofj+Ic+pwzk5WMY1FVcCl6xNjZ44qk
nkqvwS6uziub3/xUFfixGrk8WbfU7gq4By7ZVSZcA0SiSEtSZYxefTnPqLl6RFU/nApwB4Xg
rX+fy1/fuoWbzDAMz1cyvdnH/aFbOeJ+/5PuxlwzWdYvAn38PUM2d1wVLAfLupp41nl5NcWR
DiI4SRRUSnBnaJOnaWVYnUzPDBY2jtb8EXeb1+4/7m+qOBIn+9JCQjDp+ZFBa5wGCufMRA32
2Buw0G+SPY0mEeitGTjO2Qt7sWXXw2uQGSqS7OFnG86HnV/fZuyMpQ3ZwwBvnjOvdwwcLa+a
jFekJWEII8JdN/FA3TZ0KYawp7UdRmIy1rmft3QGBfXPenMqjD18NovpDS4TodDVDzyoI/vj
OHJvchRNqXWMdAK16J+xY+3qhK6svRw7Bx1TqytLCtfdxdGFrpLBEb6re8nASP6rYGO2rsJ/
dMBfE+05EjPAHd0B3GduXMB8AIOCzadfzOJIVRxe0aUZmGGtmvOM9pXx5O4CZ5WwHz/wg9OX
gMNRhoW4Z+g0jALHOXY6H404rw3z4fUjLzmq2VS8PT1i50j4eLIvzMHoBDSLiQQmqGNH1XF4
I/dl4crhHE7DW1x04nIH5uTtyWY4P0SyF3APWLKrTLgGiMpUz2EF0UJJqowRU2XA7sw2Ibi7
roFjFMa3NTLi7k+VsdtBMuhu4WahSORM9vB/SIALTzwT3/5PeyDz9I4UY94W3F3XPJpJKj16
boMNSKPTgZY5/CVY29YIu8nJ2fkPpF61O8z4d5vRTkgyD545OegRtMfwuQ3Dx3Vff/6fye3B
D2kJzYL/UH3cl4oDzbztzE92zwCdPXCfmz+gegyW69QCFez3PQnAXmIvrsfMBxXVWPrbabPa
duajH3D9AR5uGzCuw4WgAWoOkE7583wI9PUA7mNOB+9bYCLA0fWi10DGuYIOJHOBgmTJ00CN
aB9HdreaBYSSx/1gaBIhChPTDrDbPthAbGVkGgW8Vbx4vmVbElcQv+JG8uDemHdgrHDvg9HN
z6r4ur1v2UvbqV5YOHJ5ErMFfzLMkR+p+DrPUXy8o6mKKSKVI+6AYxtxh2eYGKM8ubugxust
AuAtUjitwqhJ9mrxoTVHz9D5GBp8ReeA10HbqtmcumTFMcf+zbFXgR9GQjn8QGM4dYFPlKN4
+fCp3Jfnsh4qvuN+yDn/KpYLqiUB94Alu8qE2r/ICF/WuS/rN4VPtJgkVcYIX/NMcIc7SOEA
d/57VFfvbXJ96xY/uDNVhojPipVTZZgKYoHbimwNHrVUBFxuuV0gXvvBfS5V5miSSSOWtufF
rfc2M+LueuF80DDD22RZppQwNO7qzHvCOnrkDjM8/pu/djLsrdocSG1MaXruzrJfftOXPckK
SsYkJDtdaPOtIbA6ZyzWpNRlFZhksN9aDkswRhI2I8rgTgyn4z0TngSCo1lWtJ45+W4H7YRJ
fkYng7LWdy+XP1cQgCrDEK8vvKoAd0fTzkHH1beZhWBDwBo3OTXCPRBvVdFf0CRPwxGboc6g
cqI9x7kZmtVb8cRxMqO8HuJndm8eGde2oanhs1mL/pg/4OLqOV4K7ff8uCl+vDhz44I1yV5B
5sa43mwBl8NLlbnE+DoxGmbjCtKkWruUZvO24QoVDICdmJbAAFfjNWPkZoDpBBCcuT1cyLJ5
SjwB949aN9DOwRGANRrnQwick1jOnvltx9z6htNBH6JZmyrDMq4FjOG14HKKjfHbRYywvkEF
3AOW7CoTrgGPrfyPy//cHdn+vuNfpRRFRZIqY4SvaqbK2H+oxO0gXZ0qM9CSPLWpif96CUde
/2GjTZWxtSpH3F2Pa793e8R7EtEhkrYebgbQM8Cscjm8dHA/1wJELIzajRH9uS4sq8Dq/rhN
6Waeuk019u3jPrc1pK1eFgJnvgqYRtHznvLElTJZpFNPi2p6tin4OKiy8PflyXZMViFn+wPz
TG6xsxHX7CnpbPxJNjmpvMTQu12LWGhnccwpS7DhYIFcdpaymP21UGUYglcZf3W9RHM1R/p8
FGBnkjR0poqK5rZkrYu8/Bkzipm3DRajhYkHN8n98LN6zljnWM/8sdv/RGaJkelWwr0Dz+RP
5FBlyJ3i0w6YKqjJgE7bQAHd4fwy8lZziXTCj/IgfuZwWwBVpnaZ2xifHbtPaO1cCqu42kBZ
Jp47Yf4DAMzwYTjcJhFxssGZCdea/OA+OF3APca1Eb8YcdddZ4685DCh30u2UWJQHzfqr385
78+U9S2cz3magPtjItlVJlwDHh8t/JNS0oF2IDtj7RJuj6JC+Qj/HwAA///snd9rFFcUx/sn
FNqX1hYi0vWh8Qe2pZHChqJsYiuSIiuCChF1KjEsYlq3lD5ISyEItaSsPhSWRdpQQiJ5Ei19
Cn2wDy2Sp1J9EwL66D5K4PY78529GWdmd2c3M/fuj/NhWWZ37ty5c3Zm9nvPnHtuL4bKQN/4
ErzhQWcgO5O1K29w6m/nSiwMwYS1VWecoTLcEO9tJ2CCud/6ahTq9vXSiFaT+NuG2GUiFKY9
0UEsuZdj3LUPb8dsHpvgnQEqTAtDXKH8xZkdR0boeKZc9geefloOCvd6Y2hs7kpZC3d631Uj
W7znZVyGCNP5XlAgGmrSOBA3s8cbH+aD+WSUjqIpjVDT62cO0N+65cwoj2NkMT5eYJMgx5lv
h6NX6RCNbacujHbqBxTsOaBaHkJu20nrE9JaDOEH1f9eOhAcEvDC6TKOAjJR+2KhBbVKxinH
WGrK8bp3+Oj4oTBDLw5MVCig6SFWehMvpDvoV1besxrX9zxVcXfXCIt3dXk+h03QEtRGl39I
ATOlDPbLOBx2GN7M56E+GRijuxA80vdT+p9u7XH3Q/MnRhlSD8voA9FebUZP4XsU43jr6sU9
KIOOh/KGgGOZ1vB7TQ3hDkPxAQi6B9xLcMStHjPAQBr+oPpq1SKejzKwrf4tGp0o1/EfO+bb
DCLcDSNZZazuf3jB3aZwv4yXXPJ9jYTK+DCrjPLiZCDc8VdK4a4j2vHOHDL4iw0mnAkOTm07
AZPyRr628P6Gvgn6NUOd43ogLXo9bg6m1pUHl0N53EPNc2NmJvzBqZAakNpv/1SLPTRdW+yu
64EDie6a0fnUN6G9RJvUop1bETLHirv+fOne1MwsGdH2ztiiMcFVsfVEt01SVYuSsY9TQide
7PnTrPIsHnFs/88megjNniPFfo/eO3smK+uODjGK1tzsm+iqUHtEuA8JklXGbgOGk3kvNuba
i2Xxsvc7Eirjw1jhjUc3mVImlMddp5Fh1PuTH5xojDsW2nrco+I7WiA5Zi4/urQpxxkME5y8
KdqM7g7BmRnVAwA4kDe0l7Y1h9rpuu0DETLm6UIMdf3rZ6T5ktTZTOt3VElCEpo04d67aBhD
gPaeHDu8M6cjYTqlRRfLFiLcDSNZZew2YNhwY2g9R7tI9sFAQmV8Th0qcoAphfjKs7t6rqWt
NDLe+FTodZTfyirjlW+bVYZ0oYSsw9gbP7XLkREmgG/tT00iTUJlGBXzWmMUbOu+QbN2cuYp
hsro4bB97cXsrmvX6VYpGipTayd5iBHbhoStalvsobrdRbXd7cskItwNI1llrO5/uJhWFUj2
BbXWU/ccYTtIqIyPnoCJvnMsMB5GT7eEF4PdVcPj/uXHbhps393uRbq3njlVGCrMRNIPFWLS
jBDDGkayythtwAAT1HOL6u7B9RqEu1zgA4aEyvhQuEOFQ38z1oLCPTjp0uFXXj3/QQ6F+RHC
XXnOqpVndynfRbgLGvFipo6YNCPEsIaRrDJ2GzCQhB6DQ68fv+MsyqU9iEiojE/5cslNqOd5
3LHAwab4CAXPBap5PdcSV2G5rpR20rcNlRGGBxFDqSMmzQh5hm4YySpjdf/2gf0Prtd2rR7D
q3C/PK8q0ZtbR9kUgsVQ1e4Hpaubtdaj54X+RUJlfCDWl258GxTielm/9CBUvnPtymP/e5gS
r1QORxgA5EaZOmLSjJAekWEkq4zdBlhnWlUW1W0G2UK1cy4kiHi8Y7m0WQvNi5RcvqMbgEo4
laQwqEiojA82n7lXRj0Q4m6Gx+cN+f5cBfPGUK9vrfVEPNZC9Jcvl+RSETToCuqOH08YnkXy
MflHXl/ajLg2rTdp8D662S0fi3A3imSVsdsAi9Q91T63XotdhesRqp06/p1f8tTxlPhta15Q
ayh/dbOmxMcx6EiozEtAuHMGHLywsH82Pzl35vwnTuGoO8cklvEKLp86VOS79YePQq+R5FYr
dASEppAFcq4aRrLKWN2/NegRv/ZiOXZV9JtFtTzv5YTZ/aCk/fGLEW86p1XCWhmEOiRIqEwH
iDQXkiNiKHXEpBkhdzbDSFYZuw0wjE5TC3kdlN1JQtjrgRe2vbpZY1ANg+Oh4zmtEt4zPACh
x5BQGUHIBFGZqSMmzQgxrGEkq4zdBpjnZ/UfVXtaFXo6/jbjauT6HTYkVEYQMmFgbqa90yUe
GJP2GmJYw0hWGav7z4QW0w5CrxfXa5KZUUgLCZURhEwYVDFk8S9/UE1qHTGsYSSrjPmdxgaR
p3s3i61tQa0dv+PIJSakiITKCEIm9N2dOvaPLUkxY5gxaUYH2Ms3qL47V/sdySpjtwFZoy92
Di2VMaNCukiojCBkQo+IoeR6MaFw76jOdLFo0u33YZKMQrNFj5yrw4Nklcmu8haHZvioF9Ta
tKqEVHsvd+CFfkFCZQQhEyyKoaWqs/fkWOFo8ft/3Y8bT2/NvjuGb/D6rFoLFS5fLn005xSm
crr3i833nRspTIxeWd3QxVbW3aSodnvIVkyK+6NzdidMpBp/ute/KcIU2mJo1YmZPL6BeZ2v
r4c2n7lXhiVRmL8FyzszowcmKkzb/N3vBffHmhhd+tXanCki3A0jWWWs7JcJFqPpFLMAO3rv
Dye4o9Qjc4ShRUJlBCETbIkhTlSEherFPfiDrHvC/cLpsk5JFgSrJufO/PNcbTy6CenpFvYW
3G+e3jq8r0R3EZZPHSqeuHTg7OLfpo8ngHmTYo/o2FSd8cKU7zmDffbP5mkfGuqhWoZxmpnX
/y2ccUj/eqBC9gRca09V+BuxNiuIcDeMZJXJehchxYwri7Ma8XX8jpO6fA/WNu1Nhppq9YKw
hYTKCEImWBdDEO5060JZNhOFS5VxqHMtHK+sbuCboJScV67Tfanq4BKrfj528qb7cdgcw7AJ
pAYN+OTH8clSjcL9xMzYpb/8Xk29iUdNO+n5W4QqfHJjEtbmbKawdmiec2NYP1eHDckqk2n9
waPDuQ0NvWv1WGmzps9zTjLK9Oe4DNO1BnZUuF8WT6KQHVYu4f8BAAD//+ydz0sUYRjH/5A6
dFKowYMVGYERhCtBXkLpUGBge9hgD0oJ0SGIQKQOgl0MCaI9xCZBIBJC0MlLRrfyanrwlnuM
YHpmHneY/em6u++88+PzQWRm3Xfm9dl9d77vM9/3WawykH7siiER6yIr75W+VapWGfVy1KXM
RUHmHy+qDM0XHJGhNcI95zz/6W08vDYpl7rV/FVtngXhHv4fd19PiOw+Eu6rQwOz88HERuMj
wl29SY13JPSZU4XRcOjeLw9c8G+G1E2Tlt/ZuaGBcI8YqsqYPoWMqad/y2F13kjJ1/Q31yZF
avflM61Crh0iAasMgBHsiiFRiuPFN26tAN37MeEMFwNTR6NwnP24rzlg/zo3PnMjL8JUxX3F
TxvrTMC1pN1thfTZ5zGNkqsRU09RVbgHiBpremdDpj3yWoQjFoQ92NC7IuXN/UwFNrNQVcbc
wVU9i17v8AuPVtwdTck3LiQ9EXKcwsa8zBa6PwRAZ2CVATCCRY+7qHbR3HUKu1JdYKqiXPSi
XGl2X3qPLLmHgce6vDFxyanxuOcLztDtkanCiHP29JW5vK2ssGsvpBKTQLjvfnoRrAGoM7f8
/vJKAqV/0g1dhzo+d9etneqIXr9/Z17aBq8IHvdMQVUZE4dt6orpcEloxbe4DG4VRe43day1
P4iMXFHtJbdMAhEiAKsMgBFsiSFvvePowFjOU9vXz3la0FOHOccrhJJz3u4dmTdEncu2IMrS
K4qSc8qbRzVkFp9MnsqNzlwcCCeAK379maxVldFZ0PDImSB6rh8H75Gco3MYTbRrzDViGl65
/IuUP2rr+5RUEEw9GJYH5QnyoBxfFLxWlZGjZcGDBK7tTJP1PFcXwr390Ci56yK4z2/mp3v2
rC/45pnLrb/otNKwKz9yal3tinCHCMAqA2CEuImhpqsnK7V/bdokPlgPaZvonahhh60iw3pg
swZVZTp/cvthEhjZl9yvPfYqfMaFagmaY5PoMnbkad+rNWriM6ghxWCVATBCnMVQh/eO43YR
shvSDoPWajduwQwT5/dqKqGqTHcNw1PlaXd5cKt4ay3f47u3VSUoN5TIFx3fNGLbh66q9l46
AHBSsMoAGIFP876TypDGQdCnMrBxhqoyXbctNTOyN6VfI2vF3VH5Xld8puTn2uMwfiFrYJUB
MEKcxVBCB0vKQhqfVyHOgU0lVJXpolVQLqZV5RajNwmDla/yW7Y1GR+fIQyZAqsMgBEQQ33H
Vh13c01iAu/ViKGqjNXzH0+b+Dz690bkO7l2sAhWGQAjIIb6zvavbdtdSCe8VyOGqjJ2OwCQ
aLDKABgBMdR3CKkhCGzEUFXGbgcAEg1WGQAjMJPsO4TUEAQ2YqgqY/X8AMkGqwyAEbSyr/yW
H/0Szf0/7J54N7zx4WA9Dl1K2a73lboHZNwjhaoydjsAkGiwygAAAEQHVWXsdgAg0WCVAQAA
iA6qylg9P0CywSoDAAAQHVSVsdsBgESDVQYAACA6qCpjtwMAiSYlVhmEOwAAJAK5YNk9O1YZ
AOt0PQxTYpWx/kkEAADQCXaFO1YZAIv0LlatDOH/AAAA///sXd9vHNUVzlOFoESgEBBxG1Iq
bS1iSHBcsQTVFoHYLj+ioHQRavADJQyRiQPYDVuhqmpASFGapEq1zgPVamWUPGxtKyVqcCP6
UFQhWtUhmD5QUgkhJ1aTNzz/wPS7882cvZlZT2Z3ppm1934aje7euXPm3jtz7/nOmTN30yfu
wzNFTIXY0J7Jv71l9mZv9mZv9mbfgnvoKevNw6krwfiYNKEyBgbNIhUfcUIhmQxh43E3MDAw
MGhTZEudM1eXhrgbrABcmh3/+snCV6vWNrR9fbCY/NIrJFTmtazf/RkYGBgYGMRBttQ5c3Vp
iLvBcsf8scGvVt1bj5rfG9oHC4Duf+ok+s+7TIawWQ7SwMDAwKBNkS1xz1xdGuJusKwxf9Ry
Kfit7rbW368NpQMbDt3NAvNnjiSpwAoJlcnchWBgYGBgYBAHwzMpvC5vGiZUxsAgCb5+snDa
peCnVt1+etVNsieVP71qtc/pb9VzWIwJSEhSgRUSKpOW/fGpcxY9Unxt/+DYEPeSiNjj6uf/
fT6VChgYGBgYrGyYVWWyrYCBQRKUrb6frboZ/Bt7SWD/q2/dgr2+MV8SUv6PXfkkFVghoTKp
NKN0crZ67K2xucqlT74EgwcXh02zcOUEfiK98I2DPTemscdRJKb+cxYnZj4bGhgYGBi0PrJV
FssrVCbsWUTO1NWz0NfvOl8253e03e29y87o6YUknsvzi847X6h9xIWuewhkAzWJEBINtgVd
AQn29S4aLQcS9GqYBT+WwoFHCz+942ay8CY2nHvowUTEfYWEyiRvBgYPJlNQcNBxsna1X6yl
9YT8FDaPeeQPfyphn0pzDAxaEK08j9uuF7PumrDM4YqxNLDlKMZ7wo+EDAyaQJuvKtOTwOMO
hvpMuVJ8bf/odN/hXxZeODUrhDU+DjnnRcKuaQs2QKMSUH5srmK9eRhCBseGkCDlbUgOTtl2
cBY1qZYtSEC7GqyFAjoE56IhqMYbIyXYAA2BfB0GzOD+CioAUZgbkxgA7YCFi+PUJvNHrfkz
R1TizBFsyAwkZGOOlITqSVKBFRIqk7wZVPnQ4h4XX3TEre5lLtYSLCBU3nZ5P+4ERmA67TFY
nmj0zwR0HlmXWcbf69eFEEgTUYEq8bUSy/CKTETIn5qzoBLqXiv+HsqJNDrhnBUGRx82eSfm
Gd5agu/N5E0aEzwr9foYGEQjW4975i+Hmw6VwcgFQ61OrFv4fPWlD27HhjTUrq0VuC6mrp6F
kI/+pSRAzsLna6qlHKbEhqoBpouz3NOVBGyoRkNvAFASLJltQWWwn/rkB0KaYwJXRFvYFagD
5MAOacgZYbvPg7SFXdpoNdoN0sNxHjy73k+zqoxC8mbAWtVd6Xqipv41sl7j7hqPf3a4N43W
GCxXZO7Kui7qTiINnZ6wgXbaAb5k7Tpf93i5OzAlgZL6KFY8ftF/XeaenmKVDAyika3HPfNQ
mdzrjf3/lMw5pZOz4Kk+XfaY9+h03+SHsSJeWGbbwdnLc50ihHLEZR4Hx53FN0ZKugRs87/t
a4jvgvqDqYsFwsTwTBFtiSmB1N81G2q9gf3g/kp8CWg1TI5AW9Cl2X6G0eJIruXNHzApJG8G
iLvuYmeYu+14pFzPDxeTAoa4tzlanLhnPt0Q6S6pIay99rrMN6TVVyjfOHUtbTTkgjNJNzw2
o6UMbiSy9XlnPk3FZ5Y6UOfCXMV1Ua/RWWa1lGuIMbtkd3WYuOuMOVra1NWzoLY6V6bf/ZEx
K77vX9ztmoTV5b3d8e8OpjII0evAfXwjhAE/sB9oPIgJgZw9T1eaDrtf8dBvkPjO4zjRG3o1
FAETKuMBRiceUyhyqPOFKyca23y6APafTnsMlieyXeUtGlNzVnMxlMQFZ2L3Y4UmwknDSJcl
M/ZGJ+uBIBnk4L5Ap+6athgzY/t0XxzwmQcPGLQVsvV5Z/60g7g3MY34QR19AdoN4h4nSJ0F
MN5xdeG48+/cRiEBshstDRSfjv8A333p+Qbm/20HZ6/l/Qza6cPUFLNzMHEFjBCKQgNjfi8b
jpPx3h4cX2+iZSIQ4Oj6zzCPt/2NOSZUpoZUQmW4Ssx/nUEncsPokv3luc7Ln+1Qa8uYUBmD
xikpHpj+ga4HnsvTbkwXgWm3vLcbxB3Pav9T6jMmXg564oVTcT/Msoa7DjkphJSkTtxr0S++
312879RtYO1jcxXsGRFHj7seWlM9lj6RSh5W1FL4fzSHb+pXUi/FRDuvKoPbnXu1CnLZM1LN
vX4YCQzMrSPVpTYUwL7HTfTvLGHO1D3u4JqYPLsHSiwDgXWF9Lh7HKUcTIbiJhdnuZzOukVU
CcVQEwl08cJLzhVQjR6tGioxWlxKCCSIEVKLchkbQufodZZEzxLbfPkBjforIZC8VBP06rHz
8XPPExZ6QMwPpGGWsK8iOqGtNnSFPlPJx7t1KbsOZuqHLjiTSxWODxMq4wEDRoXBXDlhK1K+
mrw8wNR1yi4DXkXLuU53nG6Ie5ujIX2Mp0U8NGqFk6txHS0xhb/rfMk0VzUF7X7vsnNpdhzE
HdtxZxFzR9nq+8lfP4opE4Wbc5UFkC5rgbSao90PW9e5e8e+XvQtegNbx4DqFj1+hul0X5VA
MoyxzT8fQoevDFaKtmze15suyYZM3A7cnaYX9Vu+aPNVZQIrn8RxltPGc9wwdyhrsNX54+ur
E+tgljf0XSmxpVwR2g0hb4yUGn22+YWr57ZHTUp9qInTiH1ru053+bQU265pq3qy5pQNl6+b
2f/nIirAtiAR7W4Pn87Xj/qXA2iLcbeHEREeo3NxpPmFpHMta9dLOomJuwmV8UCXJ/iN/9nK
Gp27h53u8nULEkIC8PQH3kNJom5+oExd0JG/9vWoqDUyM2jWiGLPvtKNo024zcwAjo/4+vjy
Zzse6iqIApP7Un45Tx/8O1+on+7dz/fvzMHoX7g4jkM4a9OLG3Ai7vgjYxYeOWRCk719rh+H
MGtjasDMizSEjJ6WqM3Bjfcq5QQhODQ1Z6Ekiftz46oM5WDjdemY54jA44dM23XPp0Lc02Ut
/Psz/Qty+YMF5qCXwBHBDqGN0BY9Gl5CZZrzuHNsrtlX23b/wxuM6HwZbnads/LgrPd8WHvN
Mjrdd+er1aUGLyWM/uX+jh3K4qKnB5YYLoGfjtIHFq4eIUGXEwDE3vVkkWLrlrzgTKCZdz5e
DfAt1Bn1OeTE/ZZOB55/6a5oJC/gxJhmbzDafFWZhOD3lJiL0JDmPiu3XQOgf2fppeeLEILZ
oLlqgKzTvQ1pzVUDVgffJASC7JuoBrZnyk0GpoO7sxrskBYZJi2LMBcPeNaXIu4BTt80TKiM
B8/jrgbwoLxBAykPh8f4BVYLcZdPVAMed9uly9DQ64eHZDhR3cZfPAgAb4sm7o7PFai5hSuI
KHIFOaoD2hcalOrfWXHv928w4huQJMG4HWDMeEhwizF1Ts7swLxpu8vEdg8oUcgRR47seS6o
z+Ob9tOJntukuLVi2wPKrY7bfejBPNk2MX/UgkD3ERrEWbwubjo97vO/964LUSimeO2VE3ue
sGge7NvoWRG4Ys9ItTU97ox+kb9OYyA70sedj0AulXP3xxswQKauXrM2FMtj1DdXJRJ3sF72
OYb2nQeGXvl7nZIBb83uxwoYdOt6e/OznqpW515Lu8N0v7y3G20Rsot7LZdGNTq2F3Dp8Mvc
uu4DXbhUWzKlGsyh8YbqBaqEE1EfnljXRIkAnmH0gP6Ihpsckd+QS/K68m882nxVmeRIfisb
fWKjhaQiJ9tqpNWWdgC7SCfusg9nhvm9Y0JliOTNAI/xjaFreHnY7y6rrtZCZdzF3UECAh+n
KoUH4r69QN8SQQ0dGBtLjRZb87hH/LFCHIUn7D9gAKA+qJ6utpcSyEzzpXkE4vM/kOCubs/h
iiFNV7dyt7txLBs7VaAFj+75YQ5Htx2c5eP0UJdyjW8dqYJkQwL9uyDZDIw58GihdHKWxF1n
kLgciTsK0yRADtg/+NNz4wvzxwZhXnbnN4BQslawHPgpKupGjzsAfp8KcU930lEx7q5znWRd
4mRQ+eGZIjqTZJ17JqRkGh73/E2dRY5NdCa6kTYwX3Cxw7/zi667nslh4zzgOrDz4Mq4j2KQ
I+e7v6vI80BHPk6RApigOh69S7nVDwxh88i66/BWnu/thYcv1Hx+MFFQ8tv35dYPe4YEy28p
VyCHkvltA85FMf1a7tXzzEGVbH8eu2Pvr1XvzVkd7ps9yLlj/wa+Z8BPGnub3eohByd6DXml
e+PH6kUHC9PqwEXx8+6HelmSHn3Uja1GB1IU0ijGOjhuNTr2ed2CfHF/sLbM4Y1wLY08rosJ
GXcE5QtzlZZiJG2+qoyBwbJGwH2OCcfRqHk4M5w2oTIKyZvBz/W0UBlvA0cXvzud7tWJ2+RT
dN3jDroQ+AMmKmAoPOhRqj0Ug8ITBYMC4GfU6N//uCRmnHLjuZnQN2R14PpPTxegFJF/yDlP
x7nSdh9aoCZQTqcgytWaIuSRMYtKDsqSqhdCoDLX9XqZrAwvhC03WuSJEAiqsUYLvGFDHp5T
mhXXwtGxFlOELYL4lJRdCvLkODWvNk6XBcX0PQNX1F/0ucuogSCCfF9wLHrccRQkGwnIhBwS
R+uF75FtO5oEetxB/YUL4qlQxP2oJTanfkVSKFgOJH+47tbW87iTuNv+jKn++ttdYWbXtIXO
9Pi69o9ptlPzuMt6UM1NIOJx5wcDHIBMM1RGt3KRc8t9FdpXsKIxGHG7xaTnizifxCs5rDDO
4szA8U6PO7HpxQ249Og5RcRz73uee89Z4EfUUJRnS7i0mJ806Pm4NORQLB4hPAnMn5zxolnY
TFRYTQ4DXZzKKIRufvfECZzI1ilyP+CdiCeTadv30PtGTk73F3DG2+KuesS5i3WYP3MEp2/5
J5KD6N7ekwusMOrAVw3IRNp2a46+osHguIYT5HOqZEiPvFRsBbT5qjIGBssaOnEnEa+7Dx9N
JU7GMaEyAlnHPeRxV9yd9P3SB7drf7fmrSRF4k4nXyBURnHupxRNF03maBqafqaAqsMpVEuk
5rAETrkMD0epIFV07HZPCUFzK0/bXIWFocOggGV9G1HeaNop3xiAEBx9+1y/lIRWu/megnhn
JZ6esTfi7oLCBuMXLY4atpQibBE0xP9IjtHJ9LKT0JStvo2dXlC77XJlFNi+qRuUXX1X6oah
v/S8WqkAup9BLxpxnyi/7Dnakcht6qV9xWuxzMLFcZ7FOBxIBr+nCUfh8m0TcnAUDxVOZKw8
6rY1wX+VC9JlLdVjb33q/w3q+uEhcL7hmSKeebX442Jt6Rj97xfkzxnE6Q4hzRL3PO1eenZB
PW3fKtOJO0accG76ofnajbMBJgHcaPFbQ9TGjys8EXJu6ixyllBnuWzbfxGXpxGOSUALFh9E
ZTp/4/0fDWPfwXfJuSUaB5eTuHYGy+nvZ7xn5uI48vveV6LwEKK8PqvYap2i+6U+jvb5IC9K
Fi5vFfgwozw7Qb2d8DtE1c19Hxj+5AO9AVH52QVUZt2PvPLVsuUT8UEkaLTwGYZ8iR1SE+9V
pSAxi9KR3zqOhnZeVcbAYLlD96kz4XlvfYLOBDKdEJvXve9NI5Mh/D8AAAD//+xdXWhcxxXu
Yx9SKNhOcGSKDGUTbKVV/rDagIyTWrKLK2yaNSGJHkydTZBVJZXiyoS8pCYQXFMoVHkICBFo
A0JyTQrGMYGQUEIeokjRU1oH+lDbEL8197nl9pv57j07O/dn7717tdq7mo9luTs7d2buz8z5
5sw5Z3rRVAaDOyV6lLgHrtbvfh+f6MYNjCpDEhAl7pQfIqEhVJb/VJNFXi8iI1EUJLHozygI
qZ/juwKyBeEEsqVmBYbtuzirkcntnhmEtI41lREJGs4ERkX68sSaFtWK09+ogzTgL4uIKBZy
tB5ry7vDUVgem/p1P9WbOfbELDlB5angTzLTSllCYTpeMFOLXxil27h72scLcwx8q4NXJ8Hd
ZaOl9TBSpOjahb5LOJoObNxHSKy9cPp9/7m1wKXkZ02bddrGNC2jTtQ5OSdNf/rjT+iCQhMm
WpXIUpioxmkSIwRU+bScV1eqTEdCLTjGB/wE0xXie8+B2qOfN1f/pAuDiL/lf+GFmnih4+j1
YMNoAxuAOYAaW87sZ2M4ILBwTORME52VTw/LBIZ8msbxMuxw3KAzK2cy1LgH7gEvL4sugMp+
msRwDoD2//hXg1r1rh159b3FDWcjxRJp98O4KHuWwnqzOMJ2DTs8qoyDQ6UhKnMh4infFq0n
m++QuDtTmQAM8GxFlTHixe4yvsMtjlUMpsDGnTzAMpVB4vTBESHTEGna2qRpzEoZqaSdtl6F
jKTchSyUy7EI+uJLj5Bzo/DzR+omG0DJzEbZL1o33w/Nal9ebhrjHq3LCoCwAZxI+afCtB1X
IhN/UTFvtgFzD1N+OwiKyeNYxpx+bwvc+eXFBj1Q87bED8MWhcY2naL0qDKrd69R0U61Orm7
hHo0KTsTV7+2FfCd2LhTde2HRthUnJsqZPBLTsulqz73VL1pGKP16MNhTBjqmJ98Yy3qqLr4
Uk201DTjpnk9Bg10WLFHD21LCKWTDpTfJ5sqA3ZhcFxPz+hE446bSTs64NZHC1QiqMvUJvt4
hZRP7bmgbXSPkaFj76HA15YRY2jWQqUDVwzMYYe6A5J4GbvYNpyOSzj8flDU3sfrSuOuFwBN
V4HgYo8P4l55kamvrGn4hna/d8YrF1XGwaG6ULJD270II+cBQ4qb357B4JlH5b+5UCwaksCZ
ygSgxh2CHKIifesltemS3ndJDoQZRHdONRXVlHMDoYTe8Bv3HKiJBKKMFA1ZSyGhoxuw+tkD
jMbga9sYywzdVOOJ2t7612/VuD979SFRn5M0kPHHCkJqzhiWTmxtHQR5J5DF2Hlh/pGxwNhs
68aubx2idBt3Bmhv8vK71/BTTGLWjY1UydTFKUXydGIqI6YsJOi0S5GlNnHZFCK+bgR4FfYp
LiWcIFn28UJAhSib5vVeaNJNb5ZhrRc3zeXp/2BWSu04eTOL5QiA0ez+UcX7vTAdBJr2e5yQ
UD3Pvi/hID0dtkipxj9vthOzBTL+Xc/fZvuVpjxc3MPMQYYgbsorlyxt8LW9Fov6998uR33o
ZUZhzSc5cRI9RQ+OVy6qjINDdYEhEXLn1tqCpKxnDgfph35ZnTTAmcoEEBv35hr6N2/j8eDD
A1Pey+zKXHyPatx9vZwtcgXTrH0XhkApmAIxj59ULFHUgTFbDBsU5z3/MgV5U3uktVYkDT+Y
mpT1ZWQzTDxHeAyc0gvNZmDpizfGRFSraBXjzYDKEHgMXmnxGFODyGVuCGwXYcaCU2VlQbmD
DsNBqlCPXwc9l8R9+FVl7/7b/y7Rih29FcMlEvF6i25eCH3hB4d+sSuMfwK6iS7MdE6q/dB+
hgtr+PxkY43+3+amCpg24HTTsxyXw1MY2oVlguAO6EUwjA8sRAK54idqEZfWYb1oxjIlg7k4
oHp9yLlZ3S7t5Rm4x4SxYsbGh2gJQ7McFn5oSNXLcYbtJKenrfzeQyo+/dnHa7xSS18gww61
4GyA3zpOeiFfx7Wr2DsnlBtP0LDjg4xaI06uatqjbxQuVpYd6BQrYyYvtqeIu4sqkwvb1dpy
6y1QWrUe044COlHjNbUIiQNItFsfLSgqr7/J6eWn9Q2W2Pne4c5UJoDYuFs6OZHuZroZCrrp
4vaPLyyNO4VNbXZeCC4FEqiDF4aJYKAYJIJSQD7xLAk1A4ZBCS0snMSdWiuQBimcQlR+KsUb
NyY8Pjh79Q6tSEWC0t5G1vcpldEqruNwSVpCzZDEH7+wj/6sPp3DxkdpIOtgwqmysqDzYcuE
2oBJR35kAM13fBWVdeyDedB3JuI1BoOvG0bwtKsxnVazP7jsyyBRj4VYv4XYf4mkVY4k/wfr
OGP3TPF58DOEf436Y3hxiV9ENgw3d5ywWuJF/sWY890H58WZdXh6VPzmY5shiaID66nBykWV
KYYKMfgOm5rUr7fuRIe88PTGVaCOuT6dh5TxnamMQIWDNAS5F4Z5lr3T11tDyAmPp2xI0rj7
rRLLTxYwsaLdEnWSaO6QEu2oSTwgVr5ax0ktNLNFBbMDUV2J2E2Uqy2QDZjopgma3njtksSN
8bQzEP7SEdY/wTE+IPfIJr0Y39lNZcxs//rOHuP4eynHHKxj83hpx3vS82QpP/bYy9n+dtdl
l5O3fGm/l5zn3vOTcozHF3sfktqwXtK+J2XBRZUpjO0SPVnqTZoDp2frHNF6s2RzqC6cqUwA
cG5Td26q201jmKZdrLHPi+RBIVvdN7a0/BTG7/p8FlRaInYN5RJ3pXEnTf/WF2sZ9krau5Op
D0+Pqn/DKJBiBN9JVBlf8cK9fsgUW7+j6Uk5+b0n9d+k/Hu8SEo0v5dWft706FUk1pszPb2c
oF7MzcIrynJ/9lopPQIXVWaLkHJd23jJ2151vz7uHQtnKhMAAt7cyaWFu7caxpjad9OihrZN
BQaOqLrda033O6DOrsd2E30sEUtEuXfJ3DnV1/p1cd43e/HYB/NPzDX4U+xqZBKeK6qM1Xiy
w/A4h946ojNOKidJb12w3p4/bnNdhe5bM08vwEWVyY4Cqu5cedrK5QJ1yelR3pzLMayYiU7S
qr5D38CZyrQUAhJApV3TUfVb2+TdUsaTx+NExpMugFJ6VBc08Q5tAUbI+RtehpW//859W9/g
x3ObS7EWZYXBPuuF+9iJnRt7bnCsuyq4+z0HAn+Sd/x/SqfmpL2wIWlSipVYTBeYvVXZVWtR
A7zOsRVDRCl0rTfhosoUg4qLoL2Wn/74E3nhT597pL655Ic/Y4GRme7Os1ebu5Qw8LF4SGes
l7Wo3Q/DjcmTaqSPGbfvJVDv7pnBtlugLC8GGxRw+zMv3GwBHwxi0dARsfUy4IQXOryhwGj8
JYeKwpnKtADyG6zi2NzkC8+rbQLx/dxTdfyURPnLypZd8LcV+R2ubVVXmPUBnMY9C8p1TuWe
qS3Wa5qmizZd7N3Fto37MckMnKeX2KTsKItD5z290m9pJ0uRPXLhLqpMLrC1d24u/PDdGdLu
2mwQHwlSeHWzIdGc4k/85m3k90MKy+3SqGhDiuXlnF4vTpcwbkh5z7/MoE9WCSj2gc+UegKn
MEob8csrjdNTI4ffj29tiGPc6UU1+zeXUJ0KEfvBfJZX3awXTWVAC9TISBLVeugOKXCmMiWg
R7TdXbsDrv8noXIScVtQ+l0C8zat2yVmK2WtF7qPr95tepar3TE0j8eJudpToedbWJVgpWe0
GejErqBtIXnZuRc56Cm4qDJ54ekA/6Lenn9lBmQ0/PNYLYEK87njROHKv7gSBBIlNvxGCnG3
6vV0vULcfR3s9eCnS/zL9Hs2a8Qp3H9g5frEm1+puHDpkUnxbkiGS6/XZX6S0gsk3awX56Ic
XqDvHNX6C85UJiuq9dJXq7V9A0fcs2Areiuk5vIfAmscOTC/zYOp6/PyE8S99MY4OKTDRZXJ
C88PKC8HWGG0vlYzp+uwL94YEyp8empU9vz2wglAOnG36pWNCLjXATdfswCObpFvUmpPm8Gk
E3eT2XOeoHZPPxHsbGBtAGy13Kx39sphNPWv/5uU4M4Dv7/txFN/wJnKZIV74x3aAoxwu5tQ
AWxFb3Xd06FCcFFl8oIaa5LmkHAHZB2ceGTtjp88CKxuNoTvkrgzHYT+zF/WYusy12qi9Yrp
C+YMjy4uRQ3Hb320wJ3LvFDzrfY51gR694zaJpm28rGggp8qfFRnsfxaqp26WS+vVNnb6FkN
SzOXCxyqC2cq4+BQGsq13t46ZDFy2DpUcaXewaFEuKgyuRAOUIHxt2nA7Yca9xRTENq4e3qn
evBsZnv26kP0cM1ggtJSr+zv69PaXm+QbGHDb7AiZhB3Ukt/H9/amwvcdFmqk8xI+emvl615
RWy9Hrem0YbyT8w1gsuYm3TEvT/gTGUcHEpDLxD3Aiy8y8S9iiv1Dg4lwkWVyQsvVHjvnhkc
mKjTwB0pw9MqgoqyA5moJ41j4LunFpcGjtx37/lJGsaAzg6Mj7KogenRWHOXlHpxOmrcd2EI
BSbZvYAl33eqhg+j1kjb2hJ3YPZGHa1FdbTquf3lBO1kpLQUoF7cEOTEucFO59qqB+1v5xTr
UBk4U5m+RcqC2ob/7tiJupWhreNL+r9uxcPv7srP6mbjhefn5SHi54+eGRkbH7LCkuJZPzHX
oOoF4od58I2UDX/l9LlHVj5sHw2tXPTHNJtxP13oz1775kPZ7rejDVxUmcJI15GnnJVSWvYS
csm7YiLVyum1Vtq2wV5Ctuo+cYconKlMJmDKfun1usWToty3bSGNM/txliJPJ+r0ituiZnOB
jKoFkLOBo/XTU6NgbHObS75ejCN1sxoAJteYGkI2rkKqKFQZrhHUEIWnKC12Drpm444bvrzY
GDsZvPZ4BHg5vTCqsRk2YfHFEa6u4mk+91RdHhNPXGwcjrXy3FL0xzSbY44Th70GrzcWvtLh
osqUi05o9Ba1QYh+J7qtAtcVa0Lj0GdwpjJZgaGWLFYYz7GZfJWCuJ99vPbmVy2JUdl/emqE
9Do7rKGB0wxatoGLv/XwCCtFmx/THI6BqyziznRwu7M/b3A/i3TiDoq//Odr4vEjxzsZ3WQM
eDp4A3nPb3858cxC01VLjvFA8Trh6dO+c/rgCIm7vHUXb4wJ++8a+oM3VFpt2d/o/UfjosoU
Q5QE5+Kphd8KS+edpfDO38BiJXitxz3eERyKwZnKZIXaeXF8aPaqokTgTELBler9ZO3ggyMg
yipQtCa7U9fnqVm/85+WEhpn9pvEnWp7cCmqwHE6uDVq2f9Qff4VteODmi2crOEv6lMZFgqk
HCkQTvIt9FqI+/kjdTrdm01dfHHk0cUlX3M7nPJH/9uV6xPD06NjozX8XPnwDsoHcUeZJP1o
lZSsWqJNLI7NTfraUx4/cdWcvaAcev/scHSTMeDJNl4LIgzggYriHMdPvrEmefBAqYDnW0FT
Gergff3gQNy7vFpSXd5goj+mH32J3g/u5KLKdA09eKUZrU+7X7VDVbAtXfj/AAAA///snc9r
HGUYx/sP9GYPoZf2YC67aeqgdFHYoCbZRSQkhw2FkkOhjMtaQzGVQhFBvUjQipB4aFhKoaWE
rQhCqVIQPAgKoanHeuhh24LeZP6B8fvOd+bJZLObzE42M282z4cQZt/5se/svjvzfZ/5vs97
+KwyBMKXbplN3y06i/QNT407zBUFLQsVC9kEbcSQZ7M+JgrJj1QU1orehZb66rXSXPPWvbna
9M/mZoONUQIRFuxVnRxfjCY/K6MQBz9XrFF842hOKRxu0qw7otW4CrXi+HHTkQikPyQ43pcK
HoUoYScEb9T+6Wu8EU8H2g4yDmd64U8j3KMKVCdPr9AVTXMF9kJ/IB7l5RH8o31pyF64i/7m
t+kH3a1YY6iiocadM9wR2p1jqrjcscFBMxyS93B1P/b5yH7Pg1uF/VYZzSqjKEpq1CrTB5Cn
kEFG10ZCmYKVuodrn/+6Kvq4q3DnKi92zNHxMlQ1y+lsYfT0+cYqZLrR3LOjEOIoRAlkFp0P
0GduI3wCAJ3tVLYctzjmpfdcDkjHm14uhPVpN8+cnDERdFPtgkwgV5WQPCPunC1ienYFgp4R
d/NkoFLkH2qLnoZR8HWHSbhYVYbweUTbbuSZkWWoD98sn8P4va0y2AZNqMPstOnfRveMXT4+
e8lYuB8uydsLa9VPV/PAwIN8Oy289mB/RFmzyiiKkhq1yvSBycIRmLmhbpkWqvVwm3CHeqbG
hQ5+7BvfC/W07M5VRCQXCqGuqHq5F2dngM7G7qK6POrjSpEBfuNjKW95YNBDkHpSrskAU9SZ
Uy5Do5+slFuPXmJfeVCwvjKBffkuFO7ccfnTGj5VnB11PJ8wxGsuHQw/qBg6GJojNmOPu0Tc
AfR3NN6ghK84Kq7u1OV0c3EcRcc3mw3WSt6+sFD9oEtWXVpgB1vGP+AahR/7XPCYzjijAqNU
X4fFcaZnR2VovhcdR6yDtmG/cNesMoqipEatMv3hNkzIWWwh1EDf+b9x1fTsCl0oK3eMouUI
UakYRPnlQklUVJjo41UzFPWTt2u8s8Yj7lBd0N8yPxy3l3A+bTMU3836WDziTm+6WGUkzI/b
LXsC2MApnUIhlToPwnfE7Rk19KJYrFhocOeWzbzoo5DILror2FLHs2fWDk2OoNlRPpCheGIP
EC8jn1UIviY+ogkfm7xfw45Q/FzL1DQZC3cLJW8KbOt+MKzAWtGDR/cdn6d5UaFI7YRfOg57
7coifYBedBWSeAEKM248SVCP++7Y1nQVRekLtcr0B4eTxvPJrK8YB/nUuMOQ56bvThZMqkcv
UFdiZvAj4c7RgQx5QklzAxwW/YFQXv8+EdhjTOyW41C5Pe64FNAyQBaFlPXNuiNRcwK51nr0
kpO98U3RQ5B4P434EOjttTByhkOxtxCz0JiDQBp64WlOSCJLHgQnjrW8nbdvVGUqZhXuudDL
tMD0R11XUcntMvn2ATEcusG27ofpPBe2bFG8UuESRCcerjZmQHkwDJ1PC+XaInPKVJcW8Iee
PP53NBjz89/RwcP1wU7hrh733bGt6SqK0hdqlRkYXYd/7SLjBhKf7uVk5T21rx33rGov+JTA
/ihXBtjwIXjbm5YZxFzpHlNn9s+MDe6+Hb/W/WNb9yM+QBygT47vneMZ2NNmWiFG39vfuDLP
w9ZTtcBVxWd0Hba3ncIdR770xmj2kwAkwX4riGaVURQlNWqVGQw7VXgu9fFico1TQD87doKr
sPDs2PHY8ki0fDxW3mv5RI/ykeD41clC7cfogEcZ+0N9XdE87imwrfvRIdwptSGv0almBiE+
YKFwbz00Rjt6qzh82Sj4Qriw3T5naNbH4sF19NXF3Wchud8O9kSzyiiKkhq1yuyLwc77MPAZ
HEgyad5reWR7eS+5PyJ1tvyWeaDYrBh6ta7sK2yb5E2HbeqHIXMxPrmN4sW7GxDrn73+ikTc
Rbi312big029aGAMI+5ih5O20f52W8R9vlGyVrX7djz42h3NKqMoSmrUKqMoA8Nm4S7k/lzI
NsmbDguvOUZ8B0NWOGUbC69dWZQHQUyKJcvTs8by7lTM2k3fjUbp3Ea5GKiCyXfLU+MOx7fc
9J/e/8vFSym5/++DrM9zL+x/8KVZZRRFSY1aZRRlYNgQ6tszlJ77L8VCyZuCw3LN8Xb871jb
9SwSFtr5CdjfM9SsMoqipEatMooyMGwI9dmppeIMx691OM5iKLG/Z6hZZRRFSY1aZfpj/6rI
fl2lpCavKKzbMO4I/HVkfqRfmdZk/EbmGyb9X3VpwY8sEPQ5ZFzn4dANw3EWQ4n9fSrNKqMo
SmrUKpMUCB3OZdOBXgEVIa874oV3a/GBhkKz7rzzuUnY9+LJDJruy/9MHsCzVxfoYEZ7zmXm
S/t1VRKG4yyGEvv7VJpVRlGU1KhVJinMW9xLuM83yvGUxqrmjya5eNyZn3unlZldTUjzjhi8
TMfbdVadDLBfVyVhOM5iKLFfmGpWGUVRUqNWmaRs+i3IoA//CPOmQaJxwtG7fqvpTmD59FhN
pkE9c77klE6515e9yK7w8Q9mPtT1pltdWsB9hZkfsp/7RjlQ8vK40yqDZhaXLEzsjTYWF+4Q
65Ltu702U3QWsxfu9uuqJOC7xh/OBRefHP+v3/iiVzWwCpcpqacsSzm3xEss4CUKuSO36XrY
jn2t+h8/37xbxx5oVhlFUVKjVpmkQH9fLpSMcP/n+6lxZ/3OAy8ItM81bz32H2AVXQeb/m36
FjiF+MqdDTN1ZekU5zGF6Me+jM0vf1p786P1A661kin53hG9wDPDGex9f2suzBhVNj+v5wZZ
MDQBv0Otfnav/KE+NfvRrDKKoqRGrTJJkYj7843VM+dLnFmw+YEZDhj3JLTXZqCNzl5dYEwd
FcP2KKHHhomWw33dibeW3OzjncrBkXso68tfpiVRN3X5Tf8pX6IBQ9azAxnfoGNy+wzI/VNS
lHzRrDKKoqRGrTJJkYj7iyczMrNgsz7mVIxL2L14WoS7RDEjdVKdLIRTiJt9A+EOrQ/hHp9F
XBkCcg9lzTfKaFFbDS+wypBm3aGVy49aJlqjWmUUJXs0q4yiKKlRq0xSINyhikxqjr9Xp8vh
KFWJuNMqQ0e7xNf9UCFVzxUlrmmWw33dfIYGKgdHLoNTIQI4BSb6hK1HW1liNv3Wvbkap7jn
bJcnp2rGCl8puteXzY4PZ9ACs//9a8BPOeJoVhlFUVKTy0/4fwAAAP//7N27TQNBEAbgViiA
ABEhAhICKiAidwQNICJCB2SEZCQWMV3QBR1QAWMtOiGwERx7+/B9XzBCOoNO+HH/WbOzXbbK
RCK/OD2PdP76cn94tDcE9xR9IgNFIk8Tspc3H/Ho4PIk9bhHqEp97Wlha/xuZP142Jep2/Su
tVVxcXN4fLXatuFlxP20MKPwi1BuYOZMlQFG0yrzX593mP9lABLWd9Xq7rapJzduFOMecuMp
re8nz/arzDWSG5g5U2WA0bTKFNL46ZFFg6/DBnuxGvwvQUmmygCjaZWBbBqMpNv6ZLYdLaC1
hiIozFQZYDStMpBN3X1nvh9a76TzsIijq8fn4WHDJjXxw3Co2AmnvXtqP1FQk6kywGhaZWDu
vHegJD3uwGhaZQCgnLrfuFeZWgvksiOtMj6JAOhC3WUevnGHrlX5AJmkVSb73wSA7OpG57TE
pcqCHFVVR9f0zq21Tix/cG9tfjYAbFTlujtwrYQevf1lv6DsLE4FYKaejEEDuiK4AzBTLlhA
XyaZKlO9/UhVVVVVf66R2hfXy+wXQYDp5A/uAABAdoI7AAB0QHAHAIAOCO4AANABwR0AADog
uAMAQAfeAQAA///s1gEJAAAAgKD/r9sR6ArFHQAABsQdAAAGxB0AAAbEHQAABsQdAAAGxB0A
AAbEHQAABsQdAAAGxB0AAAbEHQAABsQdAAAGxB0AAAbEHQAABsQdAAAGxB0AAAbEHQAABsQd
AAAGxB0AAAbEHQAABgIAAP//7NYBCQAAAICg/6/bEegKxR0AAAbEHQAABsQdAAAGxB0AAAbE
HQAABsQdAAAGAgAA//8DAEIBSIUHHWosAAAAAElFTkSuQmCC'! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
downIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/down.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAAAAXNSR0IArs4c6QAAAAZ0Uk5T
AP8A/wD/N1gbfQAAAEJJREFUeJxi+E8iYBg+GhjWMBGD6KmBGD1YPE1QNZZQwq8ae7DiUY1d
A7IeLFJYNUD0YBfHpQGnQaRqAAAAAP//AwDrSk/69sF0oQAAAABJRU5ErkJggg=='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
headerIcon
	"Private - Generated method"
	^icons 
			at: #headerIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self headerIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/6/2015 12:35'!
headerIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/header.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAABLAAAACQCAYAAAAYwpLqAAAAAXNSR0IArs4c6QAAOlFJREFU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'! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/4/2015 13:17'!
logisticsDataIcon
	"Private - Generated method"
	^icons 
			at: #logisticsDataIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self logisticsDataIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/4/2015 13:24'!
logisticsDataIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/logisticsData.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAD72lDQ1BJQ0NQcm9maWxlAAB4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'! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
mailIcon
	"Private - Generated method"
	^icons 
			at: #mailIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self mailIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
mailIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/mail.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAALFJREFU
eJxiYACC4uLi/62trSRhkB4GmOYFCxb8v3z5MkkYpAdsSHd39/+7d++ShadOnfqfAUQMDgMe
3Nrz//46rv//v9jixSA196/s+3/n3iNUA57c3gyRxGMITP7xlW34DcBmCLIcUQYgG4IuTrQB
uDBtDHhw6/T/V2eb/r++2IiBQeJgfLwZjO/fOIM9HYBMJQajpANykvLDhw8RBlCcmWA5EuQS
UjBMMwAAAP//AwCfOhaMxMvdvgAAAABJRU5ErkJggg=='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/6/2015 17:44'!
minusIcon
	"Private - Generated method"
	^icons 
			at: #minusIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self minusIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/6/2015 17:44'!
minusIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/minus.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAAA8AAAAPCAIAAAC0tAIdAAAAAXNSR0IArs4c6QAAAAZ0Uk5T
AP8A/wD/N1gbfQAAADVJREFUeJxi+A8HDAw4EVwJAXWoekhVTaRSMKK5aiIBHVQPkjChZcwj
eRmPOggAAAAA//8DADPQVbk+AZ3EAAAAAElFTkSuQmCC'! !

!GcIcons methodsFor: 'accessing' stamp: 'hk 5/11/2015 14:42'!
normalSizeNames
	^#( #bizNet #bizProcess #bizProcessesPane #bizProcessesPaneDirty
	#calcPane #contactsPane #controlling 
	#dashPane 	#dashboard #down
	#header #logisticsData #mail 
	#minus #plus #right  #route 
	#settings #taskOrdersPane #tour 
	#tourPlan #tourSub #user 
	 )! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
plusIcon
	"Private - Generated method"
	^icons 
			at: #plusIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self plusIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/6/2015 17:44'!
plusIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/plus.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAAAAXNSR0IArs4c6QAAAAZ0Uk5T
AP8A/wD/N1gbfQAAAGVJREFUeJxi+E8iYICzmA4w4EcoGtDkfoPB199fMfUwYDX7NwxgSlFJ
w48fP4Cqv//+TpoN+DRAvAhEcO/CXYUsiKKBGECBBiLjgXrBSm0NmHp+/v4JVA0ksaclklIr
AAAA//8DAAPZuC+nylbiAAAAAElFTkSuQmCC'! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
rightIcon
	"Private - Generated method"
	^icons 
			at: #rightIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self rightIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
rightIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/right.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAAAAXNSR0IArs4c6QAAAEpJREFU
eJxi+E8iYKBUA8MaJggiWQMuPfg0YNVDQAOmHsIa0PQQpQFZD7Ea4HpoYwNpfkBXgF/DfwxA
vZjGqhqLBoIAAAAA//8DAHZeT/qK33wjAAAAAElFTkSuQmCC'! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
routeIcon
	"Private - Generated method"
	^icons 
			at: #routeIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self routeIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
routeIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/route.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAPpJREFU
eJzEkz0KwkAQhXM7wcJDeIV0gpVY2Ghh0kUQJP6cwCI2wqJNLMRCAkbEwiCKBKLBiDL6VhIi
Khti4YOBZWfet6/YkaSHZFkmTdMSl6IoBI8Umhlj5Ps+JRVmDcN4QkBMI0CQRNJ1/b0Z3KKz
afuUV9eUKS14xYXH3wCF9oYGMy+CAIDCfSLAchtQZ+RyyN67RsYwiRAAWc6ZemOXil2HcpUl
tdmBAwBMBICQIFu2qd7ffewLAcw6cQBeTgUYzo+/AcIE01VKAIz/BYgUfWUsxMQ0yT1ehKb4
P8ACvmxkTW2Q1tIJib5VVWnyGUQPzXcAAAD//wMAbvWGYsI4g+kAAAAASUVORK5CYII='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
settingsIcon
	"Private - Generated method"
	^icons 
			at: #settingsIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self settingsIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
settingsIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/settings.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAaZJREFU
eJyUU01LQlEQNfEfiNHOhQR9UEaLSoICUQgDScKFgUhQG0kod27exk3RQrRNgaJJixamZCal
+VWGC+0tEoQgK3/Jqbn6zO9y4PC4w8x558ydKxI1wul0YhiIWsPj8eA5n0e5XP4XeJ6H1+uF
TqcD+zM112o1vFerXfj4qjXwyb6Uo1oicrlcdQI6dDZSjoiTvjOkDnYZKlf+thpSzghIUi+C
mxM37pbnUBwVo6AQI6EeB395+jcBSaRceMeCtFSMlHYJJeMCStMSpPfNzM5AgtefcyGbwbXF
iFuZBBGdGvmNFfCKEeRsW/0VvFUqyLqPETPpETWuIWZcRWJWxlSQjUflGPjTo+YwmwTCEMlz
ckrOikludEaKuEmDpF6Np00NsoccU9eXIGqte2YN2+uM5N5mRjGXYzWksOctCAQRu5V5Di8q
UdDO40UuQcqx19yHzptqmwEVxAPniKommp4fVJPIBH09F6xLgbBdMX8AcYedIXkRZJ4HEnAc
xxT0W+VeaFtlCnoYwhxa13rQgwqFQjAYDL+vktiGgdD8DQAA//8DAFho6hka/5odAAAAAElF
TkSuQmCC'! !

!GcIcons methodsFor: 'accessing' stamp: 'hk 5/1/2015 18:41'!
smallSizeNames
	^#()! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 19:21'!
taskOrdersPaneIcon
	"Private - Generated method"
	^icons 
			at: #taskOrdersPaneIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self taskOrdersPaneIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 19:21'!
taskOrdersPaneIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/taskOrdersPane.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAAAAXNSR0IArs4c6QAAAGlJREFU
eJxi+E8iYIBQh86dbZg6EQ8CKkBoAHIiy/N5E+zxIKACiB6QBqABQKHQpnJczgBKARUAlaFo
IIjQNQyEDRCDcWog2YbwujIa20DLUCI5aUD01E3HmfJqp/agJD6SAAAAAP//AwAJ6cw/EMeF
ZQAAAABJRU5ErkJggg=='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
tourIcon
	"Private - Generated method"
	^icons 
			at: #tourIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self tourIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
tourIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/tour.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAQ9JREFU
eJy0ky2ug0AUhWcTKBQeiUXiMCyAHbQGS9I2waOQWAQSj2IJKCwayQJu+W4zTfvy0tC+15Oc
hLncOfd3jNmQZZlUVSV1Xe8ivnmei7GXh2GQdV1lL/Dtuk6FDIqfAJGyLMU0TfORANAMviKw
LMsuPgnQyL7vZRxHCcNQXNcVz/N+Jf8u55NORAVQo6tpmqoRp21AL3k4HG8CbdsKjONYfN8X
BMnAcRyN9EhslvhrCUTWcWyqRA6CQBlFkYrN86zEGRtMkkQzUAGiUzuXyIAzpRCRbws2D5vt
T1EUz02cpkkbCYiI6M/J2Cbbzf2fPfjLKqsAj4m03gUl3V8kDbHj3EuNvuEKAAD//wMAqSAU
yhArfBgAAAAASUVORK5CYII='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
tourPlanIcon
	"Private - Generated method"
	^icons 
			at: #tourPlanIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self tourPlanIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
tourPlanIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/tourPlan.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAVBJREFU
eJxiYACCvLy8/319fSRhkB6QXobCwsL/y5Yt+3/t2rX/d+/eIRqvW7cOYgjINJDmBw8egCUe
XL/w/8HlUwgM4kPx/dtXUQyZNm0axABkwU9bG/7/2dny//fenv+/F8X8/96kDsY/mjT+f12V
hWLIzJkzMQ34vqXi//9DPf//H530//+m4v//Zjj//9ej//9PHsv/H/P8UQzA6gKQLf/WFUE0
A+l/m6vALvlTwPH/9/wQIgzosoR7AWQrDP8tYPv/Y0EQcQb8P7vy/79Lm+AaQbaTZsCZ5RQa
AAzEf8dmQpwP1EyyAf+mO4BDnywDvk1xgbhgXS7cCzBMvAHAKARFGXIsgNMBmgFYE9LjU9v/
Pzm4DIIPr0LBj8/twUzKoAwByhjgfADMD/cfP8OLYXlm586d/0EZkQGWnUGmgZxEDAaphWVn
AAAAAP//AwAZqRAUbTZNcgAAAABJRU5ErkJggg=='! !

!GcIcons methodsFor: 'private - icons' stamp: 'hk 5/1/2015 18:55'!
tourSubIcon
	"Private - Generated method"
	^icons 
			at: #tourSubIcon
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self tourSubIconContents readStream) ].! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
tourSubIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/tourSub.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAeJJREFU
eJyUk01LG1EUhuc/iL9CdOF/iKTRUk2FBFEoaLswWg3YTZFCCyUYd0IW0ljbKbHZqAtJjeBI
1GqatERDPhSZj8xkSv2oFt2U0sWr58CNRJKAFx5muLznvfe8M0eSbpbb7UYwGLwXDocDkiiO
RqPI53I1OSzcIvYymQxkWYbT6YQUCoWgaSosy4Jp6FCKJiJZm/mwZ2MmZVaIZss40kuspRqq
rRgIYkUbS+olYsYV5gsX8CllDK2Z8MQMvNr5CdWy8S2ZhKIo9Q2oeN28ZVn9w0ZvUidIJNMY
HRnhFsLhcLUBtfAuXULfFxNPVkt8smAgbmH48za6ut3w+8drt0AGclrFhKJjMmHwsz+msWHP
+y20u3r5dAqybgYbP7LcJwW1e1TGs7iOx/JXtHV64Rp4ir18sXGIiUyeBWdnp9g/vkR/JIlW
l4eZjOdwoFsVbU2Dzf0Crv7+w/9fx0gvrsAzPIqWjkd4OKtwiEXNaGzAN/h9Dt03ho/NTXj5
fJBboBxe71hsQFk1DJGEb6cj8D/wwju1gK5P35kXawecD7VXP8S8jrmUxkZ3WckaVdoqA5Es
3cI2DYb2xLtAaOlT0lBJNBD0V9GA1Buou5CW/kIeJrECgQDug6i7BgAA//8DAISED9c5nqtE
AAAAAElFTkSuQmCC'! !

!GcIcons methodsFor: 'resources icons' stamp: 'hk 5/1/2015 18:55'!
userIconContents
	"Private - Method generated with the content of the file /Users/hartmut/Smalltalk/Pharo/Pharo3.0.app/Contents/Resources/Icons/user.png"
	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAQtJREFU
eJxiYICClubm/52dnURhkFoGZFBbW/vfxcUFVRAPAKkF6QFzQCYSqxEZmJiYQFwCNwkL8PHx
+e/q6YlTHqwXlwHrdm75H7yg6L/v3Lz/VVNbsarBa0DsnIr/fLNcwBhkyOJFizDU4TUgdV7d
f8FZbmAcMDufdAN27d3132Z2Chi3T+8j3Qvt8yfBDWhdOPH/xIkTiXdB1+yJ/9WmBsG9AGLP
X7+ceAMylzaDNQrP9QBjUECmLKgjzgCHaF+ws5VnBaFgq8kJ/6saUQ3B6YKyzob/JW3VGBg9
HMB6MTIGCQCcDUJCQv7n5uaSbEhZWRlCD8gQkEtATiIGg9SC8glILwAAAP//AwCfZwYA9vZe
RwAAAABJRU5ErkJggg=='! !

Association subclass: #SlotAssociation
	instanceVariableNames: 'viewTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!SlotAssociation commentStamp: 'hk 6/23/2015 09:06' prior: 0!
I represent a vlaue in a slot dictionary.

Instance Variables
	viewTag:		<Symbol | nil>
!

!SlotAssociation methodsFor: 'copying' stamp: 'hk 6/23/2015 09:06'!
postCopy
	viewTag := viewTag copy .
	value := value copy
! !

!SlotAssociation methodsFor: 'accessing' stamp: 'hk 5/31/2015 19:04'!
viewTag
	^ viewTag! !

!SlotAssociation methodsFor: 'accessing' stamp: 'hk 5/31/2015 19:04'!
viewTag: aSymbolOrNil
 	viewTag := aSymbolOrNil! !

PackageManifest subclass: #ManifestSVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2'!

!ManifestSVO commentStamp: '<historical>' prior: 0!
I store metadata for this package. These meta data are used by other tools such as the SmalllintManifestChecker and the critics Browser!

TestCase subclass: #SVOTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Tests'!

!SVOTests commentStamp: 'hk 4/3/2015 17:04' prior: 0!
The SpoTests class

for manual experience evaluate ...!

SVOTests subclass: #SVOTestSuiteCopying
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Tests'!

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 6/24/2015 15:31'!
testCopyBasics
	| bla blabla |
	bla := (ThingVO newOfType: #bla)
		define: #zip val: #private;
		yourself.
	blabla := bla copy.
	self shouldnt: [ bla == blabla ].
	self should: [ bla id + 1 = blabla id ].
	blabla zip: #new.
	self should: [ blabla zip = #new ].
	self should: [ bla zip = #private ].
	self should: [ bla isAsvo ].
	self should: [ blabla isAsvo not ]! !

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 6/24/2015 15:38'!
testCopyMaster
	| bla sub subsub |
	bla := ThingVO newOfType: #bla.
	sub := bla copy.
	subsub := sub copy.
	bla define: #sub thing: sub.
	sub define: #subsub thing: subsub.
	self should: [ sub master = bla longName ].
	self should: [ subsub master = sub longName ].
	self should: [ sub id = 2 ].
	self should: [ subsub id = 3 ]! !

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 6/24/2015 15:31'!
testCopySlotTag
	| foo bla |
	foo := (ThingVO newOfType: #foo)
		define: #zag val: #priv tag: #blabla;
		yourself.
	bla := foo copy.
	self should: [ bla slots associations anyOne viewTag = #blabla ]! !

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 6/24/2015 15:38'!
testCopySlots
	| foo foo2 email |
	email := (ThingVO newOfType: 'GCemail') define: #val val: #bla.
	foo := (ThingVO newOfType: #foo)
		define: #zag val: 'private';
		define: #enum enum: #(#a #b);
		define: #email thing: email;
		define: #range
			val: 9
			from: 7
			to: 11;
		define: #date date: '7.4.15';
		yourself.
	foo2 := foo copy.
	self should: [ (foo2 slots associationAt: #email) isKindOf: SlotAssociation ].
	self shouldnt: [ foo2 email = foo email ].
	self should: [ foo2 email val == foo email val ].	"Symbols are ="
	self should: [ foo2 zag = foo zag ].
	self shouldnt: [ (foo2 slotAt: #zag) = (foo slotAt: #zag) ].
	self should: [ foo2 date = foo date ].
	self shouldnt: [ foo2 date == foo date ].
	self shouldnt: [ (foo2 slotAt: #range) = (foo slotAt: #range) ].
	self should: [ foo2 range = foo range ].
	self shouldnt: [ (foo2 slotAt: #enum) = (foo slotAt: #enum) ].
	self should: [ foo2 enum = foo enum ]! !

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 5/27/2015 11:42'!
testVOCopyAccess1
	"a copy is only stored in copies"
	| bla1 bla2 bla3 |
	
	bla2 := (bla1 := ThingVO newOfType: #bla) copy .
	bla2 save .
	self should: [ (bla1 copyNamed: #bla2) = bla2] .
	self should: [ bla2 longName = 'bla2 v1 ThingVO2' ] .
	self should: [ bla2 version = 1 ] .
	
	bla3 := bla2 copy.
	bla3 save .
	self should: [ (bla2 copyNamed:  #bla23) = bla3 ] .
	self should: [ bla3 longName = 'bla23 v1 ThingVO3' ] .
	self should: [ bla3 version = 1 ] .
! !

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 5/27/2015 11:43'!
testVOCopyWithName1
	| bla1 bla2 bla3 |
	bla1 := ThingVO newOfType: #bla .		"no 1 & 2"
	bla2 := 	bla1 copy: #new .
	self should: [ bla1 longName = 'bla v0 ThingVO1' ] .								
	self should: [ (bla1 copyNamed: #new) = bla2 ] .
	self should: [ bla2 longName = 'new v0 ThingVO2' ] .								
	bla2 save .
	self should: [ bla2  version = 1 ] .	
	bla3 := bla2 copy.								" no 3 "
	self should: [ (bla2 copyNamed: #new3) = bla3 ] .
	self should: [ bla3 longName = 'new3 v0 ThingVO3' ] .								
	self should: [ bla3 version = 0 ] .
! !

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 5/27/2015 11:43'!
testVONameAccess1
	| bla |
	bla := ThingVO newOfType: #bla .
	self should: [ (ThingVO named: #bla) longName = 'bla v0 ThingVO1' ] .
	self should: [ (ThingVO named: #bla) version = 0 ] .
	bla save .
	bla save .
	bla save .
	bla save .
	self should: [ (ThingVO named: #bla) longName = 'bla v4 ThingVO1' ] .
	self should: [ (ThingVO named: #bla) version = 4 ] .
! !

!SVOTestSuiteCopying methodsFor: 'tests' stamp: 'hk 5/27/2015 11:43'!
testVONameAccess2
	| bla |
	bla := ContextFolderVO newOfType: #bla .
	self should: [ (ContextFolderVO named: #bla) longName = 'bla v0 ContextFolderVO1' ] .
	self should: [ (ContextFolderVO named: #bla) version = 0 ] .
	bla save .
	bla save .
	bla save .
	bla save .
	self should: [ (ContextFolderVO named: #bla) longName = 'bla v4 ContextFolderVO1' ] .
	self should: [ (ContextFolderVO named: #bla) version = 4 ] .
! !

SVOTests subclass: #SVOTestSuiteScripting
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Tests'!

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 6/24/2015 15:34'!
testAddChoice
	| bla |
	bla := (ThingVO newOfType: #bla)
		define: #street enum: #(#private #biz);
		define: #zip enumSorted: #(#telPrivate #telBiz #faxPrivate #faxBiz #mobile);
		yourself.
	self should: [ (bla slotAt: #zip) val = #telPrivate ].
	self should: [ (bla slotAt: #street) val = #private ].
	self should: [ (bla slotAt: #zip) list first = #faxBiz ].
	self should: [ (bla slotAt: #street) list first = #private ]! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 6/24/2015 15:31'!
testAddMultipleValues
	| address |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city';
		yourself.
	self should: [ address slots keys first = #street ].
	self should: [ address slots keys last = #city ].
	self should: [ (address slotAt: #zip) val = '9999' ]! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 6/24/2015 15:38'!
testAddVO
	| bla blabla |
	bla := (ThingVO newOfType: #bla)
		define: #street enum: #(#private #biz);
		define: #zip enumSorted: #(#telPrivate #telBiz #faxPrivate #faxBiz #mobile);
		yourself.
	blabla := (ThingVO newOfType: #blabla)
		define: #bla thing: bla;
		yourself.
	self should: [ ((blabla slotAt: #bla) slotAt: #street) val = #private ].
	self should: [ ((blabla slotAt: #bla) slotAt: #zip) val = #telPrivate ]! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 4/15/2015 15:54'!
testNamed
	| vo1  |
	ValueVO new.
	vo1 := ValueVO named: #' v0 ValueVO1' .
	self should: [ vo1 longName = ' v0 ValueVO1' ] .
	self should: [ (SVO named: ' v0 ListVO1') isNil ]! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 6/25/2015 12:42'!
testNew
	| vo1 vo2 |
	vo1 := UlistVO new.
	vo2 := UlistVO new.
	self should: [ vo1 longName = ' v0 UlistVO1' ] .
	self should: [ vo2 longName = ' v0 UlistVO2' ]! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 6/24/2015 15:38'!
testPathScripting
	| bla blabla end |
	end := (ThingVO newOfType: #end)
		define: #street val: #private;
		define: #zip val: #mobile;
		yourself.
	bla := (ThingVO newOfType: #bla)
		define: #end thing: end;
		yourself.
	blabla := (ThingVO newOfType: #blabla)
		define: #bla thing: bla;
		yourself.
	self should: [ bla end street = #private ].
	self should: [ blabla bla end zip = #mobile ].
	bla end street: #irgend.
	blabla bla end zip: 'was'.
	self should: [ bla end street = #irgend ].
	self should: [ blabla bla end zip = #was ]! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 4/15/2015 15:54'!
testVONew
	self should: [ SVO new] raise: Error! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 6/9/2015 10:26'!
xtestNamedVO1
	| vo1  |
	self should: [ ThingVO new] raise: Error .
	vo1 := ThingVO newOfType: #mySVO .
	self should: [ vo1 longName =  'mySVO v0 ThingVO1' ] .
	self should: [ vo1 autoName =  ' v0 ThingVO1' ] .
	self should: [ vo1 = (ThingVO named: 'mySVO') ]
! !

!SVOTestSuiteScripting methodsFor: 'tests' stamp: 'hk 6/9/2015 10:26'!
xtestNamedVO2
	| vo1  |
	self should: [ ContextFolderVO new] raise: Error .
	vo1 := ContextFolderVO newOfType: #mySVO .
	self should: [ vo1 longName =  'mySVO v0 ContextFolderVO1' ] .
	self should: [ vo1 autoName =  ' v0 ContextFolderVO1' ] .
	self should: [ vo1 = (ContextFolderVO named: 'mySVO') ]
! !

SVOTests subclass: #SVOTestSuiteSton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Tests'!

!SVOTestSuiteSton methodsFor: 'tests' stamp: 'hk 6/25/2015 14:12'!
testStonAttributes
	| a0 a1 a2 |
	a0 := ThingVO newOfType: 'GCSList'.
	a0 copy.
	a1 := (ThingVO newOfType: 'GCActivity')
		define: #type enum: #(#elementary #basic #composite);
		define: #tariff val: 'standard tariff';
		define: #quantity
			val: 0
			from: 0
			to: 1000;
		define: #deadline date: '1.1.00';
		yourself.
	a2 := STON fromString: (STON toStringPretty: a1).
	self should: [ a2 type = a1 type ].
	self should: [ a2 tariff = a1 tariff ].
	self should: [ a2 quantity = a1 quantity ].
	self should: [ a2 deadline = a1 deadline ]! !

!SVOTestSuiteSton methodsFor: 'tests' stamp: 'hk 6/25/2015 14:17'!
testStonSlotAssociations
	| a0 a1 a2 |
	a0 := ThingVO newOfType: 'Type1'.
	a1 := (ThingVO newOfType: 'Type2')
		define: #node thing: a0.
		
	a2 := (STON fromString: (STON toStringPretty: a1)) convertToSlotAssociations.
	
	self should: [ a2 slots size = 1 ].
	self should: [ a2 slots associations anyOne isKindOf: SlotAssociation ].! !

!SVOTestSuiteSton methodsFor: 'tests' stamp: 'hk 6/25/2015 18:28'!
testStonUlist
	| a0 a1 a2 |
	a0 := ThingVO newOfType: 'GCSList'.
	a1 := (ThingVO newOfType: 'GCActivity')
		define: #list listOf: a0.
	a1 list newExemplar.
		
	a2 := (STON fromString: (STON toStringPretty: a1)) convertToSlotAssociations .
	
	self should: [ a2 list listSize = 1 ]! !

SVOTests subclass: #SVOTestSuiteUlist
	instanceVariableNames: 'view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Tests'!

!SVOTestSuiteUlist methodsFor: 'tests' stamp: 'hk 6/25/2015 18:29'!
testCreation
	| list node1 node2 |
	(node1 := ThingVO newOfType: 'GCUnode1') define: #n1 val: #bla1 .
	(node2 := ThingVO newOfType: 'GCUnode2') define: #n2 val: #bla2 .
	list := (UlistVO newOfType: 'GCUlist')
		addElement: node1;
		addElement: node2.
		
	self should: [ list listSize  = 2 ].
	self should: [ ((list slotAt: #'1') slotAt: #n1) val = #bla1 ].
	self should: [ (list slotAt: #'2') n2 = #bla2 ].
! !

!SVOTestSuiteUlist methodsFor: 'tests' stamp: 'hk 6/25/2015 18:29'!
testView
	| list node1 node2 |
	(node1 := ThingVO newOfType: 'GCUnode1') define: #n1 val: #bla1.
	(node2 := ThingVO newOfType: 'GCUnode2') define: #n2 val: #bla2.
	list := (UlistVO newOfType: 'GCUlist')
		addElement: node1;
		addElement: node2.
	(view := ViewVO new)
		slot: #eins tag: #n1;
		slot: #zwei tag: #n2;
		target: list.
	self should: [ view copyTarget isKindOf: ThingVO ].
	self should: [ view eins = #bla1 ].
	self should: [ view zwei = #bla2 ]! !

!SVOTestSuiteUlist methodsFor: 'tests' stamp: 'hk 6/25/2015 18:29'!
testViewDisplay
	"Test loopt bei slot-Zugriff in displayStringFor:"
	| list node1  |
	(node1 := ThingVO newOfType: 'GCUnode1') 
		define: #n1 val: #bla1;
	 	define: #n2 val: #bla2.
	(view := ViewVO new)
		slot: #eins tag: #n1;
		slot: #zwei tag: #n2.

	list := (UlistVO new)
		view: view ;
		addElement: node1.
		
	self should: [ (list displayStringFor: list list anyOne) = '1  GCUnode1	bla1	bla2' ].
! !

SVOTests subclass: #SVOTestSuiteViewCopying
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Tests'!

!SVOTestSuiteViewCopying methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testMultiTargetViewCopy
	| contact address newView copyView |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	newView := ViewVO new
		slot: #strasse tag: #foo1;
		slot: #myName tag: #foo8;
		slot: #stadt tag: #foo3;
		slot: #nachname tag: #foo9.
	newView
		addTarget: contact;
		addTarget: address.
	copyView := newView copyTarget.
	self should: [ copyView isKindOf: ThingVO ].
	self should: [ copyView nachname = 'Krase' ].	"do the references work?"
	address define: #city val: 'Hamburg'.	"original slots changed"
	contact define: #nname val: 'Hamburg'.	"to prove the copy"
	self should: [ copyView stadt = 'city' ].	"copied slots are not changed"
	self should: [ copyView nachname = 'Krase' ]! !

!SVOTestSuiteViewCopying methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testNestedViewCopy
	| contact address myAddr myCont copCont |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := ViewVO new
		slot: #strasse tag: #foo1;
		slot: #myName tag: #foo8.
	myCont := ViewVO new
		slot: #stadt tag: #foo3;
		slot: #nachname tag: #foo9;
		slot: #myAdresse tag: myAddr.
	myCont target: contact.
	copCont := myCont copyTarget.
	self should: [ copCont isKindOf: ThingVO ].
	self should: [ copCont nachname = 'Krase' ].	"do the references work?"
	self should: [ copCont myAdresse strasse = 'street' ].
	address define: #city val: 'Hamburg'.	"original slots changed"
	contact define: #vname val: 'Hamburg'.	"to prove the copy"
	self should: [ copCont stadt = 'city' ].	"copied slots are not changed"
	self should: [ copCont myAdresse myName = 'Hartmut' ]! !

!SVOTestSuiteViewCopying methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testSimpleViewCopy
	| address myAddr copAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	myAddr := (ViewVO newOfType: 'MyAddr')
		target: address;
		slot: #Stadt tag: #foo3;
		slot: #Straße tag: #foo1.
	copAddr := myAddr copyTarget.
	self should: [ copAddr isKindOf: ThingVO ].
	self should: [ (copAddr slots associationAt: #Stadt) isKindOf: SlotAssociation ].
	self should: [ copAddr userName = 'MyAddr_2' ].
	self should: [ (copAddr slotAt: #Straße) val = 'street' ].
	address define: #city val: 'Hamburg'.	"original slot changed to prove the copy"
	self should: [ (copAddr slotAt: #Stadt) val = 'city' ]	"copy remains unchanged"! !

SVOTests subclass: #SVOTestSuiteViews
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Tests'!

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testDefaultMultiTargetViewAccess
	| contact address newView |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city'.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase';
		define: #adr thing: address;
		define: #vname val: 'Hartmut'.
	newView := ViewVO new
		slot: #stadt tag: #city;
		slot: #nachname tag: #nname.
	newView
		addTarget: contact;
		addTarget: address.
	self should: [ newView nachname = 'Krase' ].	"do the references work?"
	self should: [ newView stadt = 'city' ].	"do the references work?"
	newView stadt: 'Hamburg'.	"original slots changed"
	newView nachname: 'Bonn'.	"to prove the copy"
	self should: [ newView stadt = 'Hamburg' ].	"copied slots are not changed"
	self should: [ newView nachname = 'Bonn' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testDefaultTagsSimple
	| address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city'.
	myAddr := (ViewVO on: address)
		slot: #Postleitzahl tag: #zip;
		slot: #Stadt tag: #city;
		slot: #Straße tag: #street.
	self should: [ (myAddr slotAt: #Postleitzahl) val = '9999' ].
	self should: [ (myAddr slotAt: #Stadt) val = 'city' ].
	self should: [ (myAddr slotAt: #Straße) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testDefaultTagsVOstructure
	| contact address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city'.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase';
		define: #adr thing: address;
		define: #vname val: 'Hartmut'.
	myAddr := (ViewVO on: contact)
		slot: #vorname tag: #vname;
		slot: #nachname tag: #nname;
		slot: #Straße tag: #street;
		slot: #Stadt tag: #city.
	self should: [ (myAddr slotAt: #vorname) val = 'Hartmut' ].
	self should: [ (myAddr slotAt: #nachname) val = 'Krase' ].
	self should: [ (myAddr slotAt: #Stadt) val = 'city' ].
	self should: [ (myAddr slotAt: #Straße) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testMultiTargetViewAccess
	| contact address newView |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	newView := ViewVO new
		slot: #stadt tag: #foo3;
		slot: #nachname tag: #foo9.
	newView
		addTarget: contact;
		addTarget: address.
	self should: [ newView nachname = 'Krase' ].	"do the references work?"
	self should: [ newView stadt = 'city' ].	"do the references work?"
	newView stadt: 'Hamburg'.	"original slots changed"
	newView nachname: 'Bonn'.	"to prove the copy"
	self should: [ newView stadt = 'Hamburg' ].	"copied slots are not changed"
	self should: [ newView nachname = 'Bonn' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testNilThingVOnode
	| contact myAddr |
	contact := (ThingVO newOfType: 'GCContact') define: #adr tag: #foo7.
	myAddr := (ViewVO on: contact) slot: #adresse tag: #foo7.
	self should: [ (myAddr slotAt: #adresse) isKindOf: NilThingVO ].
	self should: [ myAddr adresse isKindOf: NilThingVO ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:33'!
testStructuredViewOnParts
	| contact address myAddr myCont |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: address)
		slot: #strasse tag: #foo1;
		slot: #stadt tag: #foo3.
	myCont := (ViewVO on: contact)
		slot: #vorname tag: #foo8;
		slot: #nachname tag: #foo9;
		slot: #myAdresse view: myAddr.
	self should: [ myCont vorname = 'Hartmut' ].
	self should: [ myCont nachname = 'Krase' ].
	self should: [ myCont myAdresse stadt = 'city' ].
	self should: [ myCont myAdresse strasse = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testStructuredViewOnSame
	| contact address myAddr myCont |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := ViewVO new
		slot: #strasse tag: #foo1;
		slot: #stadt tag: #foo3.
	myCont := ViewVO new
		slot: #vorname tag: #foo8;
		slot: #nachname tag: #foo9;
		slot: #myAdresse tag: myAddr.
	myCont target: contact.
	self should: [ myCont vorname = 'Hartmut' ].
	self should: [ myCont nachname = 'Krase' ].
	self should: [ myCont myAdresse stadt = 'city' ].
	self should: [ myCont myAdresse strasse = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testStructuredViewSet
	| contact address myAddr myCont |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := ViewVO new
		slot: #strasse tag: #foo1;
		slot: #stadt tag: #foo3.
	myCont := ViewVO new
		slot: #vorname tag: #foo8;
		slot: #nachname tag: #foo9;
		slot: #myAdresse tag: myAddr.
	myCont target: contact.
	myCont vorname: 'Fritz'.
	myCont myAdresse stadt: 'Bonn'.
	self should: [ myCont vorname = 'Fritz' ].
	self should: [ myCont myAdresse stadt = 'Bonn' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testTagsSimple
	| address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999' tag: #foo2;
		define: #city val: 'city' tag: #foo3.
	myAddr := (ViewVO on: address)
		slot: #Postleitzahl tag: #foo2;
		slot: #Stadt tag: #foo3;
		slot: #Straße tag: #foo1.
	self should: [ (myAddr slotAt: #Postleitzahl) val = '9999' ].
	self should: [ (myAddr slotAt: #Stadt) val = 'city' ].
	self should: [ (myAddr slotAt: #Straße) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testTagsVOnodes
	| contact address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city'.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: contact)
		slot: #vorname tag: #foo8;
		slot: #name tag: #foo9;
		slot: #adresse tag: #foo7.
	self should: [ (myAddr slotAt: #vorname) val = 'Hartmut' ].
	self should: [ (myAddr slotAt: #name) val = 'Krase' ].
	self should: [ ((myAddr slotAt: #adresse) slotAt: #city) val = 'city' ].
	self should: [ ((myAddr slotAt: #adresse) slotAt: #street) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testTagsVOstructure
	| contact address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999';
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: contact)
		slot: #vorname tag: #foo8;
		slot: #nachname tag: #foo9;
		slot: #Straße tag: #foo1;
		slot: #Stadt tag: #foo3.
	self should: [ (myAddr slotAt: #vorname) val = 'Hartmut' ].
	self should: [ (myAddr slotAt: #nachname) val = 'Krase' ].
	self should: [ (myAddr slotAt: #Stadt) val = 'city' ].
	self should: [ (myAddr slotAt: #Straße) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testTagsVOstructureError
	| contact address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999';
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: address)
		slot: #vorname tag: #foo8;
		slot: #nachname tag: #foo9;
		slot: #Straße tag: #foo1;
		slot: #Stadt tag: #foo3.
	self should: [ (myAddr slotAt: #vorname) = nil ].
	self should: [ (myAddr slotAt: #nachname) = nil ].
	self should: [ (myAddr slotAt: #Stadt) val = 'city' ].
	self should: [ (myAddr slotAt: #Straße) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testTagsVOstructureNodes
	| contact address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city'.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: contact)
		slot: #vorname tag: #foo8;
		slot: #name tag: #foo9;
		slot: #adresse tag: #foo7.
	self should: [ (myAddr slotAt: #vorname) val = 'Hartmut' ].
	self should: [ (myAddr slotAt: #name) val = 'Krase' ].
	self should: [ ((myAddr slotAt: #adresse) slotAt: #city) val = 'city' ].
	self should: [ ((myAddr slotAt: #adresse) slotAt: #street) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:35'!
testTagsViewStructure
	| contact address myAddr myContact |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999';
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: address)
		slot: #stadt tag: #foo3;
		slot: #strasse tag: #foo1.
	myContact := (ViewVO on: contact)
		slot: #nachname tag: #foo9;
		slot: #adr view: myAddr.
	self should: [ (myContact slotAt: #nachname) val = 'Krase' ].
	self should: [ ((myContact slotAt: #adr) slotAt: #stadt) val = 'city' ].
	self should: [ ((myContact slotAt: #adr) slotAt: #strasse) val = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:35'!
testTagsViewStructureWithPathAccess
	| contact address myAddr myContact |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street' tag: #foo1;
		define: #zip val: '9999';
		define: #city val: 'city' tag: #foo3.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: address)
		slot: #stadt tag: #foo3;
		slot: #strasse tag: #foo1.
	myContact := (ViewVO on: contact)
		slot: #nachname tag: #foo9;
		slot: #adr view: myAddr.
	self should: [ myContact nachname = 'Krase' ].
	self should: [ myContact adr stadt = 'city' ].
	self should: [ myContact adr strasse = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testTagsViewWithPathAccessGet
	| contact address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city'.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: contact)
		slot: #vorname tag: #foo8;
		slot: #nachname tag: #foo9;
		slot: #adresse tag: #foo7.
	self should: [ myAddr vorname = 'Hartmut' ].
	self should: [ myAddr nachname = 'Krase' ].
	self should: [ myAddr adresse city = 'city' ].
	self should: [ myAddr adresse street = 'street' ]! !

!SVOTestSuiteViews methodsFor: 'tests' stamp: 'hk 6/25/2015 14:32'!
testTagsViewWithPathAccessSet
	| contact address myAddr |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: 'street';
		define: #zip val: '9999';
		define: #city val: 'city'.
	contact := (ThingVO newOfType: 'GCContact')
		define: #nname val: 'Krase' tag: #foo9;
		define: #adr thing: address tag: #foo7;
		define: #vname val: 'Hartmut' tag: #foo8.
	myAddr := (ViewVO on: contact)
		slot: #vorname tag: #foo8;
		slot: #nachname tag: #foo9;
		slot: #adresse tag: #foo7.
	myAddr vorname: 'Karl'.
	myAddr nachname: 'Weller'.
	myAddr adresse city: 'Hamburg'.
	myAddr adresse street: 'ABCstrasse'.
	self should: [ myAddr vorname = 'Karl' ].
	self should: [ myAddr nachname = 'Weller' ].
	self should: [ myAddr adresse city = 'Hamburg' ].
	self should: [ myAddr adresse street = 'ABCstrasse' ]! !

!SVOTests methodsFor: 'running' stamp: 'hk 4/3/2015 17:03'!
setUp
	self tearDown ! !

!SVOTests methodsFor: 'running' stamp: 'hk 4/15/2015 15:54'!
tearDown 
	SVO initialize .
	SVO allSubclassesDo: [ :cls | cls initialize ]! !

PPCompositeParser subclass: #DslMenuGrammar
	instanceVariableNames: 'action id menuAction argument target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Parser'!

!DslMenuGrammar commentStamp: 'hk 5/9/2015 12:33' prior: 0!
I am the Parser for menu actions.

Instance variables: 
	the single parsers
	

PPBrowser open!

DslMenuGrammar subclass: #DslMenuEvaluator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Parser'!

!DslMenuEvaluator commentStamp: 'hk 5/9/2015 12:33' prior: 0!
I execute a menu command 
!

!DslMenuEvaluator class methodsFor: 'instance-creation' stamp: 'hk 5/8/2015 17:05'!
withProperties: anArray
	"the property keys start unambiguously with gcProp - must not interfere with parser properties"
	| inst  |
	inst := self new. 
	1 to: anArray size do: [ :i | inst propertyAt: ('gcProp', i asString) asSymbol put: (anArray at: i) ] .
	^inst! !

!DslMenuEvaluator methodsFor: 'rules' stamp: 'hk 5/8/2015 18:40'!
action
	^ super action ==> [ :n | n inputValue ]
	
	! !

!DslMenuEvaluator methodsFor: 'rules' stamp: 'hk 6/4/2015 13:34'!
argument
	^ super argument ==> [ :n |  n ifNotNil: [ (n at: 2) inputValue ] ifNil: [ 'no arguments' ] ] ! !

!DslMenuEvaluator methodsFor: 'dsl' stamp: 'hk 5/8/2015 16:29'!
compile:  aSpec
	"answer the parse result or an error string"
	^self parse: aSpec onError: [:msg :pos | (msg, ' at: ', pos printString)] ! !

!DslMenuEvaluator methodsFor: 'rules' stamp: 'hk 6/7/2015 13:26'!
menuAction
	"collect all arguments from the dsl text and the properties provided
	n at: 1     	the command
	n at: 2 		the target  or nil
	n at: 3 		the argument or 'no arguments'  "
	
	^ super menuAction ==> [ :n |  | args |  
		args := (n at: 2) ifNil: [ { (n at: 3) } ] ifNotNil: [ { (n at: 2) . (n at: 3) } ] .	"args = {target}, template"
		properties ifNotNil: [ args := args, properties values] .					 "args = {target}, template, context, UI "
		{ (n at: 1) . args }] ! !

!DslMenuEvaluator methodsFor: 'rules' stamp: 'hk 6/4/2015 13:34'!
start
	"a production in this place will only be executed on success.
	call the SVO dsl menu service with the collected arguments"
	^super start ==> 
		[ :n |  SVO perform: (SVO perform: (n at: 1) asSymbol) withArguments: (n at: 2)  ]! !

!DslMenuEvaluator methodsFor: 'rules' stamp: 'hk 6/4/2015 12:33'!
target
	^ super target ==> [ :n | n ifNotNil: [(n at: 2) inputValue] ]! !

!DslMenuGrammar methodsFor: 'grammar' stamp: 'hk 5/8/2015 18:35'!
action
	^id ! !

!DslMenuGrammar methodsFor: 'grammar' stamp: 'hk 6/4/2015 13:33'!
argument
	^('to ' asParser, id) optional! !

!DslMenuGrammar methodsFor: 'grammar' stamp: 'hk 10/27/2014 13:18'!
id
	^ (#letter asParser , #word asParser star) token trim ! !

!DslMenuGrammar methodsFor: 'grammar' stamp: 'hk 6/4/2015 13:21'!
menuAction
	^action , target, argument! !

!DslMenuGrammar methodsFor: 'grammar' stamp: 'hk 5/8/2015 18:34'!
start
	^menuAction end ! !

!DslMenuGrammar methodsFor: 'grammar' stamp: 'hk 6/4/2015 13:33'!
target
	^ ('send ' asParser, id) optional! !

PPCompositeParser subclass: #DslPushGrammar
	instanceVariableNames: 'type action id pushAction target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Parser'!

!DslPushGrammar commentStamp: 'hk 5/9/2015 12:32' prior: 0!
I am the Parser for push actions.

Instance variables: 
	the single parsers
	

PPBrowser open!

DslPushGrammar subclass: #DslPushEvaluator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Parser'!

!DslPushEvaluator commentStamp: 'hk 5/9/2015 12:33' prior: 0!
I execute a push command!

!DslPushEvaluator class methodsFor: 'instance-creation' stamp: 'hk 5/9/2015 15:00'!
withProperties: anArray
	"the property keys start unambiguously with gcProp - must not interfere with parser properties"
	| inst  |
	inst := self new. 
	1 to: anArray size do: [ :i | inst propertyAt: ('gcProp', i asString) asSymbol put: (anArray at: i) ] .
	^inst! !

!DslPushEvaluator methodsFor: 'dsl' stamp: 'hk 5/9/2015 15:01'!
compile:  aSpec
	"answer the parse result or an error string"
	^self parse: aSpec onError: [:msg :pos | (msg, ' at: ', pos printString)] ! !

!DslPushEvaluator methodsFor: 'rules' stamp: 'hk 6/7/2015 13:27'!
pushAction
	"call the SVO dsl service with the first arg provided by the dsl text and all other args as found in properties"
	
	^ super pushAction ==> [ :n |  | args |  				" (n at: 2) is the action "
		args := ( {(n at: 1) . (n at: 3)} ) .					"args are type, target "
		properties ifNotNil: [ args := args, properties values] .	"property = context"
		{ (n at: 2) . args} ] ! !

!DslPushEvaluator methodsFor: 'rules' stamp: 'hk 5/9/2015 13:46'!
start
	^super start ==>
		[ :n | SVO perform: (SVO perform: (n at: 1) asSymbol) withArguments: (n at: 2) ]! !

!DslPushEvaluator methodsFor: 'rules' stamp: 'hk 5/9/2015 12:39'!
target
	^ super target ==> [ :n | n ifNotNil: [n inputValue] ifNil: [ 'noTemplate' ] ] ! !

!DslPushEvaluator methodsFor: 'rules' stamp: 'hk 5/9/2015 12:34'!
type
	^super type ==> [ :n | n inputValue ]! !

!DslPushGrammar methodsFor: 'accessing' stamp: 'hk 5/9/2015 12:36'!
action
	^ 'pushTo' asParser | 'attach' asParser! !

!DslPushGrammar methodsFor: 'accessing' stamp: 'hk 5/9/2015 12:25'!
id
	^ (#letter asParser , #word asParser star) token trim ! !

!DslPushGrammar methodsFor: 'accessing' stamp: 'hk 5/9/2015 12:24'!
pushAction
	^type, action, target  ! !

!DslPushGrammar methodsFor: 'accessing' stamp: 'hk 5/9/2015 12:23'!
start
	^pushAction end ! !

!DslPushGrammar methodsFor: 'accessing' stamp: 'hk 5/9/2015 12:26'!
target
	^id optional! !

!DslPushGrammar methodsFor: 'accessing' stamp: 'hk 5/9/2015 12:30'!
type
	^ id! !

Object subclass: #BizNet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Utilities'!

BizNet class
	instanceVariableNames: 'companies pollProcess'!

BizNet class
	instanceVariableNames: 'companies pollProcess'!

!BizNet class methodsFor: 'initializing' stamp: 'hk 6/9/2015 10:10'!
clearRegistry
	"BizNet clearRegistry"
	self messageDir ensureDeleteAllChildren.
	pollProcess terminate! !

!BizNet class methodsFor: 'accessing' stamp: 'hk 6/7/2015 13:58'!
messageDir
	^(FileSystem workingDirectory / 'messageDir') ensureCreateDirectory ! !

!BizNet class methodsFor: 'receiving' stamp: 'hk 6/8/2015 18:51'!
poll: aCompany to: aContext
	(self messageDir / aCompany ) children do: 
		[ :c |   |stonString |
		stonString := c readStreamDo: [ :stream | stream contents ] .
		c delete .
		aContext addContextVO: (self read: stonString ) ]! !

!BizNet class methodsFor: 'receiving' stamp: 'hk 6/12/2015 12:26'!
read: aString
	"deserialize and answer aThing"
	^(STON fromString: aString) convertToSlotAssociations 
! !

!BizNet class methodsFor: 'protocol' stamp: 'hk 6/7/2015 14:13'!
receive: aThing for: aReceiver
	| rec dir |
	[rec := aReceiver company] on: Error do: [ ^self inform: 'receiver has no company' ].
	rec isString ifFalse: [ ^self inform: 'receiver company not a String' ] .
	(dir := self messageDir / rec ) exists ifTrue: [ self write: aThing to: dir ] ifFalse: [ ^self inform: rec, ' is not registered'  ]
! !

!BizNet class methodsFor: 'protocol' stamp: 'hk 6/15/2015 14:15'!
register: aCompany with: aContext
	"self register: 'ABCcompany' 
	received things are put in the contextVOs of aContext"
	
	(self messageDir / aCompany) ensureCreateDirectory .	
	pollProcess := [[true] whileTrue: 
		[(Delay forSeconds: 1) wait. self poll: aCompany to: aContext]
			] fork

! !

!BizNet class methodsFor: 'sending' stamp: 'hk 6/8/2015 15:42'!
write: aThing to: aFileReference
	"serialize aThing and write it to the directory aFileReference"   
	(aFileReference / aThing userName)  writeStreamDo: [ :stream | stream nextPutAll: (STON toStringPretty: (aThing) ) ].! !

Object subclass: #Command
	instanceVariableNames: 'reverseBlock actionBlock receiver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Commands'!

!Command commentStamp: 'hk 4/13/2015 17:50' prior: 0!
A Command is used in between saves editing a SPO.
The command stack of a SPO is used for undo.
I can hold a SPO since I am not stored - no danger of cycles

Instance Variables
	actionBlock:	<Block>
	reverseBlock:	< Block>		for undo
	receiver:		<VO>			that executes the command
!

!Command methodsFor: 'executing' stamp: 'hk 11/4/2014 09:13'!
do
	"prepare my blocks,
	execute the command"! !

!Command methodsFor: 'accessing' stamp: 'hk 11/3/2014 15:58'!
receiver: aSPO
	receiver := aSPO ! !

!Command methodsFor: 'executing' stamp: 'hk 11/4/2014 08:58'!
redo
	actionBlock value! !

!Command methodsFor: 'executing' stamp: 'hk 11/4/2014 08:58'!
undo
	reverseBlock value! !

Object subclass: #GcExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-App'!

GcExample class
	instanceVariableNames: 'gcSupplierProcessesContext gcProcessCustView gcProcessView gcMakeNewProcess gcBizNetOrder gcBizNetReq gcEmail address process contact gcMainContext gcProcessesContext gcContactsContext gcSendEmail gcNewItem gcEdit gcDelete gcDashContext gcCalcContext gcTaskOrdersContext taskOrder serviceData activity transportRoad serviceRequest gcTransportView gcPushMain gcPushProcesses gcAnfrageContext serviceRequestTemplate1 serviceRequestTemplate2 supplierProcess transportRoad1 transportRoad2 serviceData1 serviceData2 projectService serviceData3'!

!GcExample commentStamp: 'hk 4/24/2015 15:54' prior: 0!
I hold the authored application that helped develop the semantic angine.

The SVO texts are all found on the class side.

dependencies between single parts of the SVO-text act via class instance variables
!

GcExample class
	instanceVariableNames: 'gcSupplierProcessesContext gcProcessCustView gcProcessView gcMakeNewProcess gcBizNetOrder gcBizNetReq gcEmail address process contact gcMainContext gcProcessesContext gcContactsContext gcSendEmail gcNewItem gcEdit gcDelete gcDashContext gcCalcContext gcTaskOrdersContext taskOrder serviceData activity transportRoad serviceRequest gcTransportView gcPushMain gcPushProcesses gcAnfrageContext serviceRequestTemplate1 serviceRequestTemplate2 supplierProcess transportRoad1 transportRoad2 serviceData1 serviceData2 projectService serviceData3'!

GcExample subclass: #Company1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-App'!

!Company1 class methodsFor: 'examples' stamp: 'hk 6/4/2015 19:20'!
exampleUC1
	self createUC1.
	gcMainContext  open: 'Gustav Gans GmbH'
! !

GcExample subclass: #Company2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-App'!

!Company2 class methodsFor: 'examples' stamp: 'hk 6/4/2015 19:20'!
exampleUC1
	self createUC1.
	gcMainContext  open: 'Tony Transport GmbH'
! !

!GcExample class methodsFor: 'things' stamp: 'hk 6/24/2015 17:31'!
activity
	"things are defined by ASVO. ASVO reference other ASVO, their nodes"

	^ activity := (ThingVO newOfType: 'GCActivity')
		define: #type enum: #(#emementary #basic #composite);
		define: #calculationType enum: #(#totals #tariff);
		define: #currency enum: #(#euro #dollars #pound);
		define: #deadline date: (Date today addDays: 7);
		yourself	! !

!GcExample class methodsFor: 'things' stamp: 'hk 6/25/2015 18:28'!
contact
	"things are defined by ASVO. ASVO reference other ASVO, their nodes"

	| email tel url communication contactIdentity personIdentity person |
	address := (ThingVO newOfType: 'GCAddress')
		define: #street val: '';
		define: #zip val: '';
		define: #city val: '';
		yourself.
	personIdentity := (ThingVO newOfType: 'GCPersonIdentity')
		define: #contactId val: '';
		define: #lastName val: '' tag: #surname;
		define: #firstName val: '' tag: #prename;
		yourself.
	email := (ThingVO newOfType: 'GCemail')
		define: #usage enum: #(#private #biz);
		define: #address val: '' tag: #emailAddress;
		yourself.
	tel := (ThingVO newOfType: 'GCtel')
		define: #usage enum: #(#telPrivate #telBiz #faxPrivate #faxBiz #mobile);
		define: #no val: '';
		yourself.
	url := (ThingVO newOfType: 'GCurl')
		define: #usage enum: #(#private #biz);
		define: #url val: '';
		yourself.
	communication := (ThingVO newOfType: 'GCCommunication')
		define: #email thing: email;
		define: #tel thing: tel;
		define: #url thing: url;
		yourself.
	person := (ThingVO newOfType: 'GCPerson')
		define: #personIdentity thing: personIdentity;
		define: #salutation enum: #('Mr.' 'Mrs.' 'Ms.');
		define: #xtitle enum: #('' 'Prof.' 'Dr.');
		define: #communication thing: communication;
		define: #address thing: address;
		define: #role val: '';
		define: #birthday date: '';
		yourself.
	contactIdentity := (ThingVO newOfType: 'GCContactIdentity')
		define: #companyName val: '' tag: #company;
		define: #mainContactPerson thing: person;
		define: #bizContactRoles enum: #('supplier' 'customer' 'endCustomer' 'bizContact');
		yourself.
	contact := (ThingVO newOfType: 'GCContact')
		define: #contactIdentity thing: contactIdentity;
		define: #contactStaff listOf: person;
		yourself.
	^ contact! !

!GcExample class methodsFor: 'program' stamp: 'hk 6/11/2015 11:19'!
contactContexts
	
	| gcContactContext |
		
	(gcContactContext := CfPlainVO newOfType: 'GCContactContext') 
		displayAs:  'Kontakt'  ;
		uiSpec: #GcItemUI at: #contact;
		menuBehavior: (MenuVO new dsl: 'email' menu: #email)  ;
		yourself .

	(gcContactsContext := CfListVO newOfType: 'GCContactsContext' ) 
		displayAs: 'Kontakte'  ;
		uiSpec: #GcListUI at: #contactsPane;
		asvo: contact  ;
		subcontext:  gcContactContext ;
		view: gcProcessCustView ;
		yourself .
! !

!GcExample class methodsFor: 'views' stamp: 'hk 6/25/2015 14:32'!
contactViews
	^ gcProcessCustView := (ViewVO newOfType: 'GCCustomer')
		slot: #vorname tag: #prename;
		slot: #nachname tag: #surname;
		slot: #company tag: #company;
		slot: #email tag: #emailAddress;
		yourself! !

!GcExample class methodsFor: 'program' stamp: 'hk 6/18/2015 14:42'!
createContexts
	"sequence is critical. 
	- menuBehaviors are used everywhere. 
	- gcContactsContext used in processContexts"
	self menuBehavior .
	self contactContexts .			
	self processContexts .		
	self taskOrderContexts .
	self supplierContexts .
	self otherContexts .
	
	gcMainContext := (CfPlainVO newOfType: 'GCMainContext' )
		aspect: gcProcessesContext on: nil ;		"es gibt keinen Slot in einem übergeordneten Objekt"
		aspect: gcContactsContext on: nil  ;
		aspect: gcTaskOrdersContext on: nil  ;
		aspect: gcSupplierProcessesContext on: nil  ;
		aspect: gcDashContext on: nil ;
		aspect: gcCalcContext on: nil  ;
		uiSpec: #GcMainUI at: #main;
		yourself.
	
	self pushBehavior .
! !

!GcExample class methodsFor: 'templates' stamp: 'hk 6/25/2015 18:42'!
createTemplates
	self transportRoad .
	self projectService .
	self serviceRequests .
	self	serviceData ! !

!GcExample class methodsFor: 'things' stamp: 'hk 6/22/2015 17:48'!
createThings
	"sequence is critical: "
	self email .
	self contact .
	"sequence is uncritical: "
	self activity .
	self supplierProcess .
	self process .
	self taskOrder .
! !

!GcExample class methodsFor: 'examples' stamp: 'hk 6/11/2015 12:27'!
createUC1
	"GcExample exampleUC1"

	self initInstanceVars.			
	self initSVO .					"sequence is critical!!"
	self createThings.				"things first"
	self createViews  .				"then views"
	self createTemplates  .		"& templates"
	self createContexts.			"then program"
	self populateUC1 				" & data"
! !

!GcExample class methodsFor: 'views' stamp: 'hk 6/10/2015 11:52'!
createViews
	"sequence is critical: "
	self processView.
	self contactViews .
	self serviceViews ! !

!GcExample class methodsFor: 'things' stamp: 'hk 6/24/2015 17:32'!
email
	"things are defined by ASVO. ASVO reference other ASVO, their nodes"

	^ (ThingVO newOfType: 'Email')
		define: #sender val: '';
		define: #receiver val: '';
		define: #subject val: '';
		define: #content val: ''! !

!GcExample class methodsFor: 'examples' stamp: 'hk 6/9/2015 14:40'!
exampleClearBizNet
	BizNet clearRegistry ! !

!GcExample class methodsFor: 'examples' stamp: 'hk 6/9/2015 14:39'!
exampleSendRequest
	"mimick sending a request to the running instance"
		
	gcMainContext addContextVO: serviceRequestTemplate1 copy .
		
	gcMainContext addContextVO: serviceRequestTemplate2 copy 

! !

!GcExample class methodsFor: 'analysis' stamp: 'hk 5/28/2015 21:57'!
exampleShowContexts
	"self exampleShowContexts"

	| builder view |
	(builder := RTUMLClassBuilder new)
		addObjects: CfListVO asvos values;
		addObjects: CfPlainVO asvos values;
		classname: #userName ;
		instanceVariables: #rtAttributes ;
		methodsNames: #rtBehaviors ;
		methodselector: #yourself ;
		superclass: #container ;
		treeLayout.
	view := builder view setDefaultMenu.
	builder build.
	view openInWindowSized: 1600 @ 800 titled: 'Context Hierarchy'! !

!GcExample class methodsFor: 'analysis' stamp: 'hk 6/3/2015 11:40'!
exampleShowSVOgui
	"self exampleShowSVOgui"

	| builder view |
	(builder := RTUMLClassBuilder new)
		addObjects: GcUI withAllSubclasses;
		addObjects: (Array with: DynamicComposableModel) ;
		addObjects: TableListUI withAllSubclasses;
		addObjects: GcSlotWidget withAllSubclasses;
		addObjects: GcVOEditor withAllSubclasses;
		treeLayout.
	view := builder view setDefaultMenu.
	builder lineShape arrowedLine inverted.
	builder build.
	view openInWindowSized: 1200 @ 800 titled: 'GUI classes'! !

!GcExample class methodsFor: 'analysis' stamp: 'hk 6/3/2015 11:39'!
exampleShowSVOmachine
	"self exampleShowSVOmachine"

	| builder view |
	(builder := RTSVOBuilder new)
		addObjects: SVO withAllSubclasses;
		treeLayout.
	view := builder view setDefaultMenu.
	builder lineShape arrowedLine inverted.
	builder build.
	view openInWindowSized: 1200 @ 800 titled: 'SVO classes - authoring protocol'! !

!GcExample class methodsFor: 'analysis' stamp: 'hk 5/29/2015 11:43'!
exampleShowThings
	"self exampleShowThings"

	| builder view |
	(builder := RTUMLClassBuilder new)
	 	addObjects: ThingVO asvos values ;
		classname: #userName ;
		instanceVariables: #rtAttributes ;
		methodsNames: #rtSlots ;
		superclass: #container ;
		methodselector: #yourself ;
		treeLayout .
"	builder lineShape arrowedLine.
"	view := builder view setDefaultMenu.
	RTZoomOutMove on: view . 
	builder build.
	view openInWindowSized: 1600 @ 800 titled: 'Things contain Things'! !

!GcExample class methodsFor: 'examples' stamp: 'hk 6/2/2015 12:52'!
exampleUC1
	"GcExample exampleUC1"

	self createUC1.				"populate with data"
	gcMainContext  open: 'myCompany'! !

!GcExample class methodsFor: 'examples-helpers' stamp: 'hk 6/25/2015 19:15'!
initInstanceVars 
	"
	| str |
	str := WriteStream on: '' .
	GcExample class instanceVariables do: [ :e | str tab; nextPutAll: e; nextPutAll: ':= nil .'; cr  ].
	str contents
	"
	
 	gcSupplierProcessesContext:= nil .
	gcProcessCustView:= nil .
	gcProcessView:= nil .
	gcMakeNewProcess:= nil .
	gcBizNetOrder:= nil .
	gcBizNetReq:= nil .
	gcEmail:= nil .
	address:= nil .
	process:= nil .
	contact:= nil .
	gcMainContext:= nil .
	gcProcessesContext:= nil .
	gcContactsContext:= nil .
	gcSendEmail:= nil .
	gcNewItem:= nil .
	gcEdit:= nil .
	gcDelete:= nil .
	gcDashContext:= nil .
	gcCalcContext:= nil .
	gcTaskOrdersContext:= nil .
	taskOrder:= nil .
	serviceData:= nil .
	activity:= nil .
	transportRoad:= nil .
	serviceRequest:= nil .
	gcTransportView:= nil .
	gcPushMain:= nil .
	gcPushProcesses:= nil .
	gcAnfrageContext:= nil .
	serviceRequestTemplate1:= nil .
	serviceRequestTemplate2:= nil .
	supplierProcess:= nil .
	transportRoad1:= nil .
	transportRoad2:= nil .
	serviceData1:= nil .
	serviceData2:= nil .
	serviceData3 := nil.
	projectService:= nil .
! !

!GcExample class methodsFor: 'examples-helpers' stamp: 'hk 6/9/2015 10:38'!
initSVO 
	SVO initialize .
	SVO allSubclassesDo: [ :cls | cls initialize ]! !

!GcExample class methodsFor: 'class initialization' stamp: 'hk 6/9/2015 10:33'!
initialize
	"replace the Pharo image by GloConn"
	(World submorphs detect: [ :sm | sm isKindOf: ImageMorph ]) image: GcIcons default headerIcon! !

!GcExample class methodsFor: 'program' stamp: 'hk 6/24/2015 17:48'!
menuBehavior
		
	gcEmail := (MenuVO new) 
		dsl: 'email to customer' menu: #email;
		yourself.
		
	gcMakeNewProcess := (MenuVO new )
		menuName: #neuerProzess ;
		dsl: 'neu to ignored' at: 'neuer Transport Land';
		dsl: 'neu bla ignored' at: #neueAnfrageFehler;
		dsl: 'neu to ignored' at: 'neuer Transport See' ;
		dsl: 'neu to ignored' at: #intern ;
		yourself.
	
	gcBizNetReq := (MenuVO new)  "request and customer are slots in gcSubProcess"
		dsl: 'bizNet send request to supplier' menu: #Request;
		yourself.

	gcBizNetOrder := (MenuVO new) 
		dsl: 'bizNet send order to supplier' menu: #Order;
		yourself.
	
	
	
! !

!GcExample class methodsFor: 'program' stamp: 'hk 5/27/2015 11:50'!
otherContexts
	
	(gcDashContext := CfListVO newOfType: 'GCDashContext' ) 
		displayAs: 'Dash Board'  ;
		uiSpec: #GcDashUI at: #dashPane; 
		yourself .

	(gcCalcContext := CfListVO newOfType: 'GCCalcContext' ) 
		displayAs: 'Calculator'  ;
		uiSpec: #GcCalcUI at: #calcPane; 
		yourself .
		
! !

!GcExample class methodsFor: 'examples' stamp: 'hk 6/25/2015 19:14'!
populateUC1
	| p1 serviceReq c1 c2 c3 p2 p3 |
	"fill the list"
	contact contactStaff newExemplar.	"make 3 contacts and fill data in using the View"
	c1 := contact copy.
	(gcProcessCustView target: c1)
		nachname: 'Crasemann';
		vorname: 'Chris W. T. ';
		email: 'Chris@Crasemann.de';
		company: 'myCompany'.
	c2 := contact copy.
	(gcProcessCustView target: c2)
		nachname: 'Meier';
		vorname: 'Hans';
		email: 'Hans@Meier.de';
		company: 'Tony Transport GmbH'.
	c3 := contact copy.
	(gcProcessCustView target: c3)
		nachname: 'Montabur';
		vorname: 'Carl G. ';
		email: 'cg@montabur.de';
		company: 'Gustav Gans GmbH'.	"make 3 view-copies of those contacts"
	c1 := gcProcessCustView copyFrom: c1.
	c2 := gcProcessCustView copyFrom: c2.
	c3 := gcProcessCustView copyFrom: c3.	"make 3 processes"
	(p1 := process copy: 'intern')
		type: 'internal';
		define: #customer thing: c1;
		define: #serviceData thing: serviceData3;
		ownership: 'hartmut';
		dateDue: '3.3.2015'.
	(p2 := process copy: 'transportLand')
		type: 'transport land';
		define: #customer thing: c2;
		define: #serviceData thing: serviceData1;
		ownership: 'chris';
		dateDue: '9.3.2015'.
	(p3 := process copy: 'TransportSee')
		type: 'transport sea';
		define: #customer thing: c3;
		define: #serviceData thing: serviceData2;
		ownership: 'chris';
		dateDue: '9.2.2015'.	"make a serviceRequest"
	(serviceReq := serviceRequest copy: '5 Colli HH-HB')
		define: #customer thing: c1 copy;
		define: #supplier thing: c2 copy;
		define: #transport thing: transportRoad copy;
		yourself.	"new thing"	"new thing"	"make a supplierProcess with the serviceRequest"
	(supplierProcess copy: 'HH-HB') define: #request thing: serviceReq! !

!GcExample class methodsFor: 'things' stamp: 'hk 6/25/2015 18:16'!
process
	"things are defined by ASVO. ASVO reference other ASVO, their nodes"

	serviceRequest := (ThingVO newOfType: 'GCBizServiceRequest')
		define: #customerRequestNo val: '';
		define: #supplierRequestNo val: '';
		define: #customer;
		define: #supplier ;
		define: #processID val: '';
		define: #aspect val: #request;
		define: #status enum: #(#inProgress #completed);
		yourself.

	^ process := (ThingVO newOfType: 'GCProcess')
		define: #type val: '';
		define: #status enum: #(#bizNet #open #delivered #billed);
		define: #ownership val: '' tag: #owner;
		define: #dateDue date: (Date today addDays: 7) asString tag: #dateDue;
		define: #customer;
		define: #serviceData;
		define: #activity;
		define: #request;
		define: #offer;
		define: #order;
		yourself! !

!GcExample class methodsFor: 'program' stamp: 'hk 6/24/2015 17:59'!
processContexts
	"exports:
	- gcCustContext
	- GCProcessesContext"
	
	|  gcAngebotsContext gcAuftragsContext  gcActivitiesContext gcBizProcessContext  gcCustContext gcServiceContext |
		
	gcCustContext := (CfPlainVO newOfType: 'GCCustContext' ) 
		displayAs: 'Customer' ;
		uiSpec: #GcReferenceAspectUI at: #customer;
		view: gcProcessCustView ;
		chooseWith: gcContactsContext ;			"calls the gcContactsContext dialog for adding"
		yourself.		
 
	gcServiceContext := (CfPlainVO newOfType: 'GCServiceContext' ) 
		displayAs:  'Service Data'  ;
		uiSpec: #GcTemplateAspectUI at: #transportRoad;
		template: serviceData1 ;						"offers the templates for adding"
		template: serviceData2 ;
		yourself.
		
	gcActivitiesContext := (CfListVO newOfType: 'GCActivitiesContext' ) 
		displayAs:  'Activities' ;
		uiSpec: #GcListUI at: #activity;
		asvo: activity ;
		yourself .
		
	gcAnfrageContext := (CfPlainVO newOfType: 'GCAnfrageContext' ) 
		displayAs:  'Anfrage'  ;
		uiSpec: #GcTemplateAspectUI at: #request;
		template: serviceRequestTemplate1  ;
		template: serviceRequestTemplate2  ;
		yourself .
		
	gcAngebotsContext := (CfPlainVO newOfType: 'GCAngebotsContext' ) 
		displayAs:  'Angebot'  ;
		yourself .
	gcAuftragsContext := (CfPlainVO newOfType: 'GCAuftragsContext' )
		displayAs:  'Auftrag' ;
		yourself .
	
	(gcBizProcessContext := CfPlainVO newOfType: 'GCBizProcessContext') 
		displayAs:  'Biz Vorgang'  ;
		uiSpec: #GcItemWithTabsUI at: #bizProcess;
		menuBehavior: gcEmail ;
		aspect: gcServiceContext on: #serviceData ;
		aspect: gcCustContext on: #customer ;
		aspect: gcAnfrageContext on: #request ;
		aspect: gcAngebotsContext on: #offer ;
		aspect: gcAuftragsContext on: #order ;
		aspect: gcActivitiesContext on: #activities ;
		view: gcProcessView ;
		yourself .

	(gcProcessesContext := CfListVO newOfType: 'GCProcessesContext' ) 
		displayAs: 'Biz Vorgänge'  ;
		uiSpec: #GcListUI at: #bizProcessesPane;
		menuBehavior: gcMakeNewProcess ;
		asvo: process ;
		view: gcProcessView ;
		subcontext: gcBizProcessContext ;
		yourself .

! !

!GcExample class methodsFor: 'views' stamp: 'hk 6/25/2015 14:32'!
processView
	^ gcProcessView := (ViewVO newOfType: 'GCProcessView')
		slot: #vorname tag: #vorname;
		slot: #nachname tag: #nachname;
		slot: #company tag: #company;
		slot: #email tag: #email;
		slot: #owner tag: #owner;
		slot: #due tag: #dateDue;
		yourself	"the next 4 lines"	"use the default tag"! !

!GcExample class methodsFor: 'templates' stamp: 'hk 6/25/2015 18:42'!
projectService
		
	projectService := (ThingVO newOfType: 'GCProjectService')
		define: #type val: #projectService;
		define: #cost val: '17';
		define: #dateDue date: '1.1.2016';
		yourself.
! !

!GcExample class methodsFor: 'program' stamp: 'hk 5/9/2015 16:36'!
pushBehavior
		
	gcPushMain := (PushVO new)
		dsl: 'request pushTo GCProcessesContext' at: #pushRequest;
		dsl: 'gcOffer pushTo GCTaskOrderContext' at: #pushOffer;
		dsl: 'gcOrder pushTo GCProcessesContext' at: #pushOrder;
		yourself.
		
	gcMainContext pushBehavior: gcPushMain .

	gcPushProcesses := (PushVO new)
		dsl: 'request attach' at: #pushRequest;
		dsl: 'gcOrder attach' at: #pushOrder;
		yourself.

	gcProcessesContext pushBehavior: gcPushProcesses  .! !

!GcExample class methodsFor: 'templates' stamp: 'hk 6/25/2015 19:14'!
serviceData
	"templates are ASVO with data. They reference other ASVO copies with data filled in"
		
	serviceData := (ThingVO newOfType: 'GCServiceDescription')
		define: #serviceDataItems thing: UlistVO new ;
		yourself.
		
	(serviceData1 := serviceData copy: 'transport Road') serviceDataItems 
		addElement: transportRoad1;
		addElement: projectService copy .
		
	(serviceData2 := serviceData copy: 'ServiceData2') serviceDataItems 
		addElement: transportRoad2 .
		
	(serviceData3 := serviceData copy: 'projects') serviceDataItems 
		addElement: projectService copy ;
		addElement: projectService copy .
! !

!GcExample class methodsFor: 'templates' stamp: 'hk 6/24/2015 19:01'!
serviceRequests
	"templates are ASVO with data. They reference other ASVO copies with data filled in"

	| tr1 tr2 c2 c1 |
		
	(c1 := gcProcessCustView copyFrom: contact)
		nachname: 'Montabur';
		vorname: 'Carl G. ';
		email: 'cg@montabur.de';
		company: 'Gustav Gans GmbH';
		yourself.

	(c2 := gcProcessCustView copyFrom: contact)
		nachname: 'Meier';
		vorname: 'Hans';
		email: 'Hans@Meier.de';
		company: 'Tony Transport GmbH';
		yourself.
		
	tr1 := transportRoad copy.
	gcTransportView			"edit helper"
		target: tr1;
		gewicht: '2 tons';
		volumen: '1 qm';
		ladung: 'diverses';
		yourself.
	gcTransportView origin city: 'Hamburg'.
	gcTransportView dest city: 'Bremen'.
	
	tr2 := transportRoad copy.
	gcTransportView			"edit helper"
		target: tr2;
		gewicht: '1 tons';
		volumen: '5 qm';
		ladung: 'Dämmplatten';
		yourself.	
	gcTransportView origin city: 'München'.
	gcTransportView dest city: 'Oldenburg'.
	
	serviceRequestTemplate1 := (serviceRequest copy: 'Montabur HH-B')
		define: #customer thing: c1;
		define: #transport thing: tr1;
		yourself.
		
	serviceRequestTemplate2 := (serviceRequest copy: 'Meier M-OL')
		define: #customer thing: c2;
		define: #transport thing: tr2;
		yourself! !

!GcExample class methodsFor: 'views' stamp: 'hk 6/25/2015 14:32'!
serviceViews
	^ gcTransportView := ViewVO new
		slot: #gewicht tag: #weight;
		slot: #volumen tag: #volume;
		slot: #ladung tag: #goods;
		slot: #origin tag: #origin;
		slot: #dest tag: #dest;
		yourself	"anonymous edit helper"! !

!GcExample class methodsFor: 'program' stamp: 'hk 6/24/2015 18:01'!
supplierContexts
	
	| gcSupplierProcessContext gcSubRequestContext gcSubAngebotsContext gcSubAuftragsContext    gcSupplierContext gcSupServiceContext |


	gcSupplierContext := (CfPlainVO newOfType: 'GCSupplierContext' ) 
		displayAs: 'Supplier' ;
		uiSpec: #GcReferenceAspectUI at: #supplier;
		view: gcProcessCustView ;
		chooseWith: gcContactsContext ;		"calls the gcContactsContext dialog for adding"
		yourself.				
 
	gcSupServiceContext := (CfPlainVO newOfType: 'GCSupServiceContext' ) 
		displayAs:  'Service Data'  ;
		uiSpec: #GcTemplateAspectUI at: #transportRoad;
		template: serviceData ;					"offers the templates for adding"
		yourself.

	gcSubRequestContext := (CfPlainVO newOfType: 'GCSubAnfrageContext' ) 
		displayAs:  'Anfrage'  ;
		uiSpec: #GcTemplateAspectUI at: #request;
		template: serviceRequestTemplate1  ;
		template: serviceRequestTemplate2  ;
		yourself .
		
	gcSubAngebotsContext := (CfPlainVO newOfType: 'GCSubAngebotsContext' )
		displayAs:  'Angebot'  ;
		yourself .
		
	gcSubAuftragsContext := (CfPlainVO newOfType: 'GCSubAuftragsContext' )
		displayAs:  'Auftrag' ;
		yourself .

	(gcSupplierProcessContext := CfPlainVO newOfType: 'GCSupplierProcessContext') 
		displayAs:  'Anbieter Vorgang'  ;
		uiSpec: #GcItemWithTabsUI at: #tourSub;
		menuBehavior: gcBizNetReq ; 
		menuBehavior: gcBizNetOrder  ; 
		aspect: gcSupServiceContext on: #serviceData ;
		aspect: gcSupplierContext on: #supplier ;
		aspect: gcSubRequestContext on: #request ;
		aspect: gcSubAngebotsContext on: #offer ;
		aspect: gcSubAuftragsContext on: #order ;
		yourself .
		
	(gcSupplierProcessesContext := CfListVO newOfType: 'GCSupplierProcessesContext' ) 
		displayAs: 'Anbieter Vorgänge'  ;
		uiSpec: #GcListUI at: #tourSub; 
		asvo:  supplierProcess ;
		subcontext: gcSupplierProcessContext ;
		yourself .! !

!GcExample class methodsFor: 'things' stamp: 'hk 6/25/2015 17:39'!
supplierProcess
	"things are defined by ASVO. ASVO reference other ASVO, their nodes"

	^ supplierProcess := (ThingVO newOfType: 'gcSubProcess')
		define: #dateDue date: (Date today addDays: 7) asString;
		define: #type enum: #(#supplier #tour #project #empty);
		define: #status enum: #(#preparing #request #offer #done);
		define: #owner val: '';
		define: #angebotspreis val: '';
		define: #serviceData ;
		define: #supplier ;
		define: #request ;
		define: #offer ;
		define: #order ;
		yourself! !

!GcExample class methodsFor: 'things' stamp: 'hk 6/24/2015 17:32'!
taskOrder
	"things are defined by ASVO. ASVO reference other ASVO, their nodes"

	|  supplierOrder tourOrder projectOrder |
	supplierOrder := (ThingVO newOfType: 'GCSupplierTaskOrder')
		define: #type val: 'supplier';
		yourself.
	tourOrder := (ThingVO newOfType: 'GCTourTaskOrder')
		define: #type val: 'tour';
		yourself.
	projectOrder := (ThingVO newOfType: 'GCProjectTaskOrder')
		define: #type val: 'project';
		yourself.
	^ taskOrder := (ThingVO newOfType: 'GCTaskOrder')
		define: #type enum: #(#supplier #tour #project #empty);
		define: #deliveryStatus enum: #(#planned #request #offer #complete );
		define: #bizProcessNo val: '';
		define: #activityNo val: '';
		define: #item ;   "hier kommt die Liste der taskOrders hin"
		yourself! !

!GcExample class methodsFor: 'program' stamp: 'hk 6/3/2015 12:18'!
taskOrderContexts
	
	| gcTaskOrderContext |
	(gcTaskOrderContext := CfPlainVO newOfType: 'GCTaskOrderContext') 
		displayAs:  'Task Order'  ;
		uiSpec: #GcItemUI at: #taskOrder;
		yourself .
		
	(gcTaskOrdersContext := CfListVO newOfType: 'GCTaskOrdersContext' ) 
		displayAs: 'Task Orders'  ;
		uiSpec: #GcListUI at: #taskOrdersPane; 
		asvo: taskOrder ;
		subcontext: gcTaskOrderContext ;
		yourself .
 

! !

!GcExample class methodsFor: 'templates' stamp: 'hk 6/25/2015 18:42'!
transportRoad
	"templates are ASVO with data. They reference other ASVO copies with data filled in"

	| route item route1 route2 item1 item2 |
	route := (ThingVO newOfType: 'GCRoute')
		define: #origin thing: address copy;
		define: #destination thing: address copy tag: #dest;
		define: #via thing: address copy;
		define: #distance val: 0	from: 0 to: 1000;
		yourself.
		
	(route1 := route copy) distance: 100.
	route1 origin city: 'Hamburg'.
	route1 destination city: #Bremen.
	
	(route2 := route copy) distance: 1000.
	route2 origin city: 'WHV'.
	route2 destination city: #Marseille.
	
	item := (ThingVO newOfType: 'GCTransportItem')
		define: #means val: '';
		define: #amount val: '';
		define: #weight val: '' tag: #weight;
		define: #volume val: '' tag: #volume;
		define: #goods val: '' tag: #goods;
		yourself.
		
	(item1 := item copy) means: #LKW ; weight: '5 t'.
	
	(item2 := item copy) means: #Schiff; weight: '50 t'.
	
	transportRoad := (ThingVO newOfType: 'GCTransportRoad')
		define: #type val: #transportRoad;
		define: #route thing: route;
		define: #cargo thing: item;
		yourself.
		
	transportRoad1 := transportRoad copy
		define: #route thing: route1;
		define: #cargo thing: item1;
		yourself.
		
	transportRoad2 := transportRoad copy
		define: #route thing: route2;
		define: #cargo thing: item2;
		yourself .
		
! !

Object subclass: #SVO
	instanceVariableNames: 'version isAsvo id master container dataPending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-core'!

SVO class
	instanceVariableNames: 'lastID asvos'!

!SVO commentStamp: 'hk 6/9/2015 10:29' prior: 0!
I am an abstract Smart Value Object, domain as well as program objects. 

My name consists of 2 resp. 3 words (use substrings): 
1.  userName (only present in objects with userName) . 
The autoName follows:
2. version (use asInteger)
3. className (=type name) followed by id (running no of the instances)
This way it is easy to select: 
- a specific VO (instance) 
- all VO with a given name (= the version history)
- the actual VO of a given name with the highest version

class instance variables: 
	lastID: 			<SmallInteger>			initialize to 0 
	asvos: 			<Dictionary> 			of my ASVO instances 

Instance Variables
	container:		<VO | nil>				used in hierarchies
	id:				<Integer>				unique for instances of class
	isAsvo:			<Boolean>
	master:			<VO>					where I am copied from
	version:		<String>				starting at '0'
	dataPending: 	<NewValueHolder> 	on true or false					!

SVO class
	instanceVariableNames: 'lastID asvos'!

SVO subclass: #BehaviorVO
	instanceVariableNames: 'sourceDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!BehaviorVO commentStamp: 'hk 5/8/2015 12:48' prior: 0!
I hold one or more DSL texts that execute an action or side effect.

Instance Variables
	sourceDict:		<OrderedIdentityDictionary of String>	the DSL source
!

!BehaviorVO methodsFor: 'analyzing' stamp: 'hk 5/28/2015 21:47'!
behaviors
	^sourceDict keys! !

!BehaviorVO methodsFor: 'authoring' stamp: 'hk 4/6/2015 20:57'!
dsl: aSourceString at: slotName
	sourceDict at: slotName put: aSourceString ! !

!BehaviorVO methodsFor: 'initialization' stamp: 'hk 4/6/2015 11:16'!
initialize
	sourceDict := OrderedIdentityDictionary  new! !

BehaviorVO subclass: #MenuVO
	instanceVariableNames: 'menuName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!MenuVO commentStamp: 'hk 5/8/2015 12:48' prior: 0!
I hold one or more menu actions.

Instance Variables:
	menuName: 	<String>	for a group menu
!

!MenuVO methodsFor: 'authoring' stamp: 'hk 6/9/2015 09:47'!
dsl: aSourceString menu: slotName
	self dsl: aSourceString at: slotName! !

!MenuVO methodsFor: 'parser' stamp: 'hk 5/8/2015 18:00'!
evaluator: anArray
	"answer the parser"
	^DslMenuEvaluator withProperties: anArray! !

!MenuVO methodsFor: 'executing' stamp: 'hk 5/9/2015 14:55'!
exec: aKey withProperties: anArray
	"execute the dsl source at aKey.
	return an empty string if the action is successful, else the error string"
	| res parser |
	parser := self evaluator: anArray.
	res :=  parser compile:  (sourceDict at: aKey) .
	^res isString ifTrue: [ res ] ifFalse: [ '' ]
! !

!MenuVO methodsFor: 'testing' stamp: 'hk 5/8/2015 11:34'!
hasMenu
	^sourceDict isNotEmpty ! !

!MenuVO methodsFor: 'testing' stamp: 'hk 5/8/2015 11:34'!
hasSingleMenu
	^sourceDict size = 1! !

!MenuVO methodsFor: 'protocol' stamp: 'hk 5/8/2015 15:49'!
menuFor: argArray wrapWith: aBlock
	"answer a menuItem for a Spec Widget. 
	The argArray normally is #(context UI) .
	aBlock is normally used to notify the GUI with a failure of the execution"
	
	|   menuItem |
	self hasMenu ifFalse: [ ^nil ] .
	menuItem := MenuItemModel new .
	self hasSingleMenu 
		ifTrue: [ | key | menuItem	
			name: ( key := sourceDict keys anyOne) ;
			action: [aBlock value: (self exec: key withProperties: argArray)]] 
		ifFalse: [ menuItem	
			name: self menuName ; 
			subMenu: (MenuModel new addGroup: [ :g | 
				sourceDict keysDo: [ :k | 
					g addItem: [ :item | item
						name: k ; 
						action: [aBlock value: (self exec: k withProperties: argArray)  ] ] ] ] ) ] .
	^menuItem! !

!MenuVO methodsFor: 'accessing' stamp: 'hk 6/4/2015 15:21'!
menuName
	^menuName ifNil: [ sourceDict  keys at: 1 ]! !

!MenuVO methodsFor: 'authoring' stamp: 'hk 5/8/2015 11:30'!
menuName: aString
	menuName := aString! !

BehaviorVO subclass: #PushVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!PushVO methodsFor: 'accessing' stamp: 'hk 5/9/2015 14:46'!
evaluator: anArray
	"answer the parser"
	^DslPushEvaluator withProperties: anArray! !

!PushVO methodsFor: 'executing' stamp: 'hk 6/7/2015 13:33'!
execAllWith: aContext
	| res parser |
	parser := self evaluator: { aContext} .
	sourceDict keysDo: 
		[ :k | res := parser compile:  (sourceDict at: k) .
			 res isString ifTrue: [ self inform: res ] ] .
! !

SVO subclass: #ConstraintVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

SVO subclass: #ContextFolderVO
	uses: TSVOUserNameAccess
	instanceVariableNames: 'contextVOs uiSpec displayName filter view personalisation uiKey userName targetSlotname pushBehaviors menuBehaviors choiceContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!ContextFolderVO commentStamp: 'hk 6/4/2015 15:29' prior: 0!
I am a ACM Case Folder or DCI Context.
My contextVOs constitute dynamic context data.
My roles constitute my behaviour.
I am not persistent !! I am instantiated from my vo or asvo, views or filters.

My subclasses represent 
- a plain context on a domain VO and
- a list contect on a ASVO. 

Instance Variables
	username:			<String>				user defined part of name
	contextVOs:		<Collection of DomainVO>
	menuBehaviors: 	<OrderedCollection of MenuVO>
	pushBehaviors:		<OrderedCollection of PushVO> 
	uiSpec:				<Symbol>				my Spec class name
	uiKey:				<Symbol>				key for Spec & Icon lookup
	displayName: 		<String>				for the GUI
	filter:		 		<FilterVO>				for the DomainVO(s)
	view:		 		<ViewVO>				for the DomainVO(s)
	targetSlotname: 	<Symbol>				slot of the container vo for view 
	targetContext: 		<DContextFolderVO>	edit context for view target			
	personalisation:  	<PersonalisationVO>	for the GUI
	dirty 				<NewValueHolder>		true when changed	

the inherited elder slot is used:
	container:									embedding context!

ContextFolderVO class
	uses: TSVOUserNameAccess classTrait
	instanceVariableNames: ''!

ContextFolderVO subclass: #CfListVO
	instanceVariableNames: 'asvo subcontext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!CfListVO commentStamp: 'hk 6/9/2015 13:01' prior: 0!
I am a list context,
bound to a template (asvo).
Thus indirectly I know all its copies, i.e. my list.
cfPlain is the subContexts for any copy.

I use my view to arrange the list display.


Instance Variables
	asvo:		<DomainVO>		isAsvo, my template & list
	subcontext:	<CfPlainVO>		context of a list item!

!CfListVO methodsFor: 'accessing' stamp: 'hk 4/6/2015 16:29'!
asvo
	^asvo! !

!CfListVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 17:03'!
asvo: anASVO
	"tell the ASVO that holds my list"
	
	asvo := anASVO! !

!CfListVO methodsFor: 'pending data' stamp: 'hk 5/31/2015 18:10'!
checkDirty
	self list detect: [ :sc | sc hasDataPending ] ifNone: [ self setDataNotPending ].
	 container ifNotNil: [container checkDirty]
! !

!CfListVO methodsFor: 'context things' stamp: 'hk 6/24/2015 15:38'!
findOrMakeProcessFor: aVO
	| pID process aspect |
	pID := (aVO slotAt: #processID) val.
	aspect := (aVO slotAt: #aspect) val.
	process := self list detect: [ :e | e id = pID ] ifNone: [ self asvo copy ].
	process define: aspect asSymbol thing: aVO.
	process save.
	process setDataPending.
	self setDataPending! !

!CfListVO methodsFor: 'protocol' stamp: 'hk 4/6/2015 12:55'!
list
	^asvo copies! !

!CfListVO methodsFor: 'analysing' stamp: 'hk 5/28/2015 21:33'!
rtAttributes
	^Array with: 'asvo: ', (self asvo  ifNil: #nil ifNotNil: [self asvo userName]) 
		withAll: super rtAttributes   ! !

!CfListVO methodsFor: 'accessing' stamp: 'hk 4/13/2015 12:19'!
subcontext
	^subcontext! !

!CfListVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 17:04'!
subcontext:	aCfPlainVO
	"tell the context for my list elements"
	
	aCfPlainVO  container: self .
	subcontext :=	aCfPlainVO! !

ContextFolderVO subclass: #CfPlainVO
	instanceVariableNames: 'vo subcontexts templates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!CfPlainVO commentStamp: 'hk 5/28/2015 19:12' prior: 0!
I am a normal (plain) context,
bound to one VO.
Since there may be many different views on my VO, 
I hold a subcontext for each in my subcontexts

Instance Variables
	vo:				<VO>							for which I define the context
	templates:		<OrderedCollection of VO>		from which to draw a copy
	subcontexts: 	<OrderedCollection of CfListVO>	all my possible subContexts
!

!CfPlainVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 17:08'!
aspect: aCf on: slot
	"tell a subcontext of one of my aspects (= subnodes).
	and to which slot of my VO it belongs "
	
	aCf container: self .
	aCf targetSlotname: slot .
	subcontexts add: aCf! !

!CfPlainVO methodsFor: 'protocol' stamp: 'hk 5/28/2015 14:52'!
asvo
	"when my elder is a list context ..."
	^container asvo! !

!CfPlainVO methodsFor: 'changes' stamp: 'hk 5/31/2015 18:11'!
checkDirty

	|  scDirty  eqDirty |		
	scDirty := eqDirty := true .
	subcontexts detect: [ :sc | sc hasDataPending ] ifNone: [ scDirty := false ]. 
	vo ifNotNil: [ eqDirty := scDirty = vo hasDataPending ] .
	
	self hasDataPending 
		ifTrue: [ (eqDirty & scDirty not ) ifTrue: [ ^self setDataNotPending ] ] 
		ifFalse: [ eqDirty ifFalse: [ ^self setDataPending ] .
				 (eqDirty & scDirty ) ifTrue: [ ^self setDataPending ] ].
	 container ifNotNil: [container checkDirty]
! !

!CfPlainVO methodsFor: 'initialization' stamp: 'hk 5/10/2015 15:18'!
initialize
	super initialize .
	templates := OrderedCollection new .
	subcontexts := OrderedCollection new! !

!CfPlainVO methodsFor: 'analysing' stamp: 'hk 5/28/2015 21:33'!
rtAttributes
	^Array with: 'vo: ', (self vo  ifNil: #nil ifNotNil: [self vo userName]) 
		withAll: super rtAttributes   ! !

!CfPlainVO methodsFor: 'accessing' stamp: 'hk 4/13/2015 10:37'!
subcontexts
	^subcontexts! !

!CfPlainVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 17:10'!
template: aThingVO
	"the thing template from which to draw a copy"
	
	templates add: aThingVO! !

!CfPlainVO methodsFor: 'accessing' stamp: 'hk 5/10/2015 15:17'!
templates
	"the DomainVO template from which to draw a copy"
	^templates! !

!CfPlainVO methodsFor: 'accessing' stamp: 'hk 4/6/2015 16:34'!
vo
	^vo! !

!CfPlainVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 17:04'!
vo: aVO
	"tell my thing"
	
	vo := aVO! !

!ContextFolderVO methodsFor: 'context things' stamp: 'hk 5/31/2015 18:11'!
addContextVO: aVO
	contextVOs add: aVO .
	self setDataPending .
	pushBehaviors do: [ :e | e execAllWith: self ]! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 6/9/2015 12:52'!
choiceContext
	^choiceContext ! !

!ContextFolderVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 17:03'!
chooseWith: aChoiceContext
	"tell the contexct (editor) from which to choose new content
	? maybe only useful for CfPlainVO ? cf. view: "
	
	choiceContext := aChoiceContext! !

!ContextFolderVO methodsFor: 'context things' stamp: 'hk 5/9/2015 15:14'!
contextVOs
	^contextVOs! !

!ContextFolderVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 16:55'!
displayAs: aString
	"tell the GUI display string"
	
	displayName := aString! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/13/2015 11:28'!
displayName
	^displayName! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/6/2015 16:27'!
filter
	^ filter! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/6/2015 16:27'!
filter: anObject
	filter := anObject! !

!ContextFolderVO methodsFor: 'initialization' stamp: 'hk 6/4/2015 15:27'!
initialize
	super initialize.
	menuBehaviors := OrderedCollection new.
	pushBehaviors := OrderedCollection new.
	contextVOs :=	OrderedCollection new .
! !

!ContextFolderVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 16:56'!
menuBehavior: aBehaviorVO 
	"tell a menu bound behavior"
	
	menuBehaviors add: aBehaviorVO ! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 6/4/2015 15:30'!
menuBehaviors
	^menuBehaviors ! !

!ContextFolderVO methodsFor: 'context things' stamp: 'hk 5/9/2015 15:22'!
moveContextVO: aVO to: targetContxt
	targetContxt addContextVO:  aVO .
	self removeContextVO: aVO! !

!ContextFolderVO methodsFor: 'helpers' stamp: 'hk 4/13/2015 10:39'!
newUI
	^(Smalltalk at: uiSpec) on: self! !

!ContextFolderVO methodsFor: 'helpers' stamp: 'hk 6/1/2015 16:44'!
open
	self open: nil! !

!ContextFolderVO methodsFor: 'helpers' stamp: 'hk 6/8/2015 15:53'!
open: aString
	| ui |
	ui := ((Smalltalk at: uiSpec) on: self ).
	ui  companyLabel label: aString, ' ', Date today asString .
	ui openWithSpec .
	BizNet register: aString with: self! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/6/2015 16:27'!
personalisation
	^ personalisation! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/6/2015 16:27'!
personalisation: anObject
	personalisation := anObject! !

!ContextFolderVO methodsFor: 'copying' stamp: 'hk 5/10/2015 14:15'!
postCopy
	super postCopy .
	self inform: 'contexts should not be copied'! !

!ContextFolderVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 16:57'!
pushBehavior: aBehaviorVO 
	"tell a automatic (push) behavior"

	pushBehaviors add: aBehaviorVO .
! !

!ContextFolderVO methodsFor: 'context things' stamp: 'hk 5/9/2015 15:23'!
removeContextVO: aVO
	contextVOs remove: aVO .
! !

!ContextFolderVO methodsFor: 'analyzing' stamp: 'hk 6/7/2015 13:32'!
rtAttributes
	^{ 'uiSpec: ', (self uiSpec ifNil: #nil) }! !

!ContextFolderVO methodsFor: 'analyzing' stamp: 'hk 5/28/2015 21:56'!
rtBehaviors
	| Bs |
	Bs := OrderedCollection new.
	pushBehaviors do: [ :pB | Bs addAll: pB behaviors ].
	menuBehaviors do: [ :mB | Bs addAll: mB behaviors ].
	^Bs! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 5/6/2015 14:50'!
targetSlotname
	^ targetSlotname! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 5/6/2015 14:50'!
targetSlotname: aSymbol
	targetSlotname := aSymbol! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/13/2015 10:31'!
uiKey
	^ uiKey! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/13/2015 16:37'!
uiSpec
	^uiSpec ! !

!ContextFolderVO methodsFor: 'authoring' stamp: 'hk 6/23/2015 10:30'!
uiSpec: aClassName at: aUikey
	"tell the Spec model of this context. 
	aUikey identifies the icon shown"
	
	uiSpec := aClassName .
	uiKey := aUikey asSymbol! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/28/2015 13:06'!
userName
	^ userName ! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/28/2015 13:07'!
userName: aString
	userName := aString ! !

!ContextFolderVO methodsFor: 'accessing' stamp: 'hk 4/6/2015 16:27'!
view
	^ view! !

!ContextFolderVO methodsFor: 'authoring' stamp: 'hk 6/22/2015 17:02'!
view: anObject 
	"CfListVO: tell the view defining my list columns 
	CfPlainVO: tell the view defining my content, cf. chooseWith: "
	
	view := anObject ! !

SVO subclass: #DefaultNoPolicyVO
	instanceVariableNames: 'number'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!DefaultNoPolicyVO methodsFor: 'comparing' stamp: 'hk 6/25/2015 10:47'!
< aPolicyNo! !

!DefaultNoPolicyVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 12:12'!
first
	^1 ! !

!DefaultNoPolicyVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 11:04'!
initialize 
	number := self first -1! !

!DefaultNoPolicyVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 10:49'!
last
 	^99! !

!DefaultNoPolicyVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 10:52'!
next
	number := number + 1.
	number > self last ifTrue: [number := self first].
	^ number! !

!DefaultNoPolicyVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 12:15'!
nextString
	^ self next asString! !

!DefaultNoPolicyVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 10:47'!
validFrom: aDate! !

!DefaultNoPolicyVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 10:47'!
validUntil: aDate! !

SVO subclass: #FilterVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

SVO subclass: #FullViewVO
	instanceVariableNames: 'targets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!FullViewVO commentStamp: 'hk 5/27/2015 11:35' prior: 0!
I am a trivial 1:1 view on my target, a DomainVO. 

Instance variables:
	targets: 			<OrderedCollection of DomainVO>

!

!FullViewVO class methodsFor: 'instance creation' stamp: 'hk 5/27/2015 11:34'!
new
	"for an anonymous view: don't use the new of superclass"
	^self basicNew initialize! !

!FullViewVO class methodsFor: 'instance creation' stamp: 'hk 5/25/2015 16:09'!
on: aDomainVO
	"convenience - normally the target is added later"
	^self new 
		target: aDomainVO; 
		yourself! !

!FullViewVO methodsFor: 'authoring' stamp: 'hk 5/25/2015 17:48'!
addTarget: aDomainVO
	^targets add: aDomainVO 
! !

!FullViewVO methodsFor: 'authoring' stamp: 'hk 6/11/2015 16:24'!
copyFrom: aTarget
	"we do not copy the tags"
	self target: aTarget.
	^self copyTarget! !

!FullViewVO methodsFor: 'copying' stamp: 'hk 6/9/2015 14:20'!
copyTarget
	"just answer the single target copy"
	^self target copy! !

!FullViewVO methodsFor: 'printing' stamp: 'hk 6/9/2015 16:04'!
displayArray
	"adapted from ThingVO.
	a Views target may be locally empty, then the slot value will be nil"
	
	| ar values idString |  
	idString := self target hasDataPending ifTrue: [ self target id asString , '*' ] ifFalse: [self target id asString ] .  
	ar := OrderedCollection with: idString with: self target userName .
	values := self slots select: [ :s | s isNil or: [s isValue] ]  .
	values keysAndValuesDo: [ :k :v | ar add: v asString ].  
	 ^ar
	
! !

!FullViewVO methodsFor: 'printing' stamp: 'hk 6/9/2015 16:04'!
displayLabelArray
	"code adapted from ThingVO.
	a Views target may be locally empty, then the slot value will be nil"
	
	| ar values |     
	ar := OrderedCollection with: 'id'  with: 'name' .
	values := self slots select: [ :s | s isNil or: [s isValue] ]  .
	values keysAndValuesDo: [ :k :v | ar add: k ].
	 ^ar! !

!FullViewVO methodsFor: 'private' stamp: 'hk 5/25/2015 17:34'!
doesNotUnderstand: aMessage 
	"answer the slot named by aMessage or change a valueSlot"

	 ^targets detect: [:t | t respondsTo: aMessage ] 
				ifFound: [:t | t perform: aMessage ]
				ifNone:  [super doesNotUnderstand: aMessage ] 
				
	! !

!FullViewVO methodsFor: 'initialization' stamp: 'hk 5/25/2015 16:09'!
initialize
	super initialize.
	targets := OrderedCollection new! !

!FullViewVO methodsFor: 'testing' stamp: 'hk 4/28/2015 19:39'!
isView
	^true! !

!FullViewVO methodsFor: 'protocol' stamp: 'hk 6/23/2015 10:33'!
slotAt: aSlotName
	"for compatibility with Things.
	answers the slot VO, i.e. target node if not own node"

	^ self slotAt: aSlotName ifAbsent: [ ]! !

!FullViewVO methodsFor: 'protocol' stamp: 'hk 5/30/2015 18:34'!
slotAt: aSlotName ifAbsent: aBlock 
	"for compatibility with Things.
	answers thetarget slot - note that a further slot with same name is masked"
	
	^ targets detect: [ :t | t slotAt: aSlotName isNotNil ] ifNone: aBlock! !

!FullViewVO methodsFor: 'protocol' stamp: 'hk 6/9/2015 14:26'!
slots   
	"answer a slot dictionary like a DomainVO (used for editors)"
	
	| slots |
	slots := OrderedIdentityDictionary new.
	targets do: [ :t | slots addAll: t slots ].
	^slots
	! !

!FullViewVO methodsFor: 'accessing' stamp: 'hk 6/9/2015 14:20'!
target
	"just answer the single target "
	^targets first ! !

!FullViewVO methodsFor: 'authoring' stamp: 'hk 6/5/2015 11:45'!
target: aDomainVO
	targets := OrderedCollection with: aDomainVO! !

FullViewVO subclass: #ViewVO
	uses: TSVOUserNameAccess
	instanceVariableNames: 'slots pathCache copies userName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!ViewVO commentStamp: 'hk 5/28/2015 19:00' prior: 0!
I am a view on my target, a DomainVO. 

I represent a tree, 
- nodes made up of my kind (from possibly different targets), 
- leaves made up of tags found in the targets, denoting the corresponding subtree or leaf
the tags must be unique.

I can be used as a projector from the targets structure to the view structure
- usually in GUIs
- eventually for projected copies

My slots are either other views or they contain tag symbols to be found in the targets.

Instance variables:
	slots:		<OrderedIdentityDictionary of ViewVO | Symbol>	subview | tag
	pathCache: 	<OrderedIdentityDictionary of Path>
	copies: 		<Dictionary>	not clear if this is needed			of my copies
	userName: 	<String>			of the thing created by targetCopy
!

ViewVO class
	uses: TSVOUserNameAccess classTrait
	instanceVariableNames: ''!

!ViewVO class methodsFor: 'instance creation' stamp: 'hk 6/25/2015 11:37'!
new
	"for an anonymous view: don't use the new of superclass"
	^self basicNew id: self nextID; initialize! !

!ViewVO methodsFor: 'accessing' stamp: 'hk 5/11/2015 17:06'!
addCopy: aVO
	copies add: aVO longName -> aVO. 
	^aVO! !

!ViewVO methodsFor: 'authoring' stamp: 'hk 6/1/2015 14:30'!
addTarget: aThing
	super addTarget: aThing .
	pathCache := OrderedIdentityDictionary new .		"empty cache"
	slots ifNil: [ ^self ] .		"may be used by the convenience instantiation with on: "
	slots do: [ :s | (s isKindOf: ViewVO) ifTrue: [ s addTarget: aThing ] ]! !

!ViewVO methodsFor: 'authoring' stamp: 'hk 6/11/2015 12:40'!
copyTarget
	"we do not copy the tags"
	| cp | 
	cp := ThingVO newOfType: self userName .
	cp userName: cp userName, '_', cp id asString .
	self addCopy: cp .
	self slots associationsDo: 
		[ :a | |t|
		t := (self slotAt: a key) ifNil: [ ^self error: 'projection error' ] .
		cp slots add: (SlotAssociation key: a key value: t value copyTarget )] .
	^cp! !

!ViewVO methodsFor: 'private' stamp: 'hk 5/30/2015 18:43'!
doesNotUnderstand: aMessage 
	"answer the slot res the value named by aMessage 
	or change a valueSlot"
	
	| sel getter result |
	sel := aMessage selector.
	getter := (sel trimRight: [ :c | c = $: ]) asSymbol .
	result := self slotAt: getter ifAbsent: "not my node"
			[^super doesNotUnderstand: aMessage] .	
	sel isKeyword ifTrue:		"is setter"
		[ ^result isValue 		"EnumVO or ValueVO"
			ifTrue: [ result val:  aMessage argument ] 
			ifFalse: [super doesNotUnderstand: aMessage]  ].

	^result isValue ifTrue: [ result val ] ifFalse: [ result]
	! !

!ViewVO methodsFor: 'initialization' stamp: 'hk 5/11/2015 17:06'!
initialize
	super initialize.
	slots := OrderedIdentityDictionary new.
	copies := Dictionary new! !

!ViewVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 14:33'!
slot: aSlotName tag: aTag
	slots add: (SlotAssociation key: aSlotName asSymbol value: aTag)! !

!ViewVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 14:32'!
slot: aSlotName view: aViewVO
	slots add: (SlotAssociation key: aSlotName asSymbol value: aViewVO)! !

!ViewVO methodsFor: 'protocol' stamp: 'hk 6/25/2015 17:12'!
slotAt: aSlotName ifAbsent: aBlock 
	"for compatibility with Things.
	answer the slot VO, i.e. own node or target node.
	note that a further target node with same name will be masked"
	
	| interim path value |  
	value := slots at: aSlotName .
	(value isKindOf: ViewVO) ifTrue: [ ^value ] .		"else a tag"
	targets do: 
		[ :t | 
		interim := t. 
		"use pathCache for optimization"
		path := pathCache at: (t userName, '.', aSlotName) asSymbol
								ifAbsentPut: [t nodeOrLeafAtKey: value] . 
		path ifNotNil: 
			[path substrings do: 
				[ :slot | interim := (interim slotAt: slot asSymbol)].
			^interim]].
	^aBlock value
! !

!ViewVO methodsFor: 'protocol' stamp: 'hk 6/11/2015 12:49'!
slots
	"answer a slot dictionary like a ThingVO "
	| res| 
	res := OrderedIdentityDictionary new.
	slots associationsDo:  [ :a | res at: a key put: (self slotAt: a key) ] . 
	^res
	! !

!ViewVO methodsFor: 'authoring' stamp: 'hk 4/28/2015 19:44'!
target: aDomainVO
	super target: aDomainVO .
	pathCache := OrderedIdentityDictionary new .		"empty cache"
	slots ifNil: [ ^self ] .		"may be used by the convenience instantiation with on: "
	slots do: [ :s | (s isKindOf: ViewVO) ifTrue: [ s target: aDomainVO ] ]! !

!ViewVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 11:35'!
userName
	^userName ifNil: [ '' ]! !

!ViewVO methodsFor: 'accessing' stamp: 'hk 5/27/2015 12:19'!
userName: aString
	userName := aString! !

SVO subclass: #LeafVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!LeafVO commentStamp: 'hk 4/10/2015 12:44' prior: 0!
I am an abstract leaf in a VO tree.

!

LeafVO subclass: #EnumVO
	instanceVariableNames: 'list selection sorted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!EnumVO commentStamp: 'hk 4/4/2015 13:08' prior: 0!
I represent an enumeration with a selection.
The first item of the enumeration becomes the default selection.
My collection may be sorted alphabetically.

Instance Variables
	list:				<Collection>
	selection:		<String | Number>
	sorted: 		<Boolean  nil>!

!EnumVO class methodsFor: 'ui' stamp: 'hk 4/10/2015 12:37'!
uiClass
	^GcChoiceWidget! !

!EnumVO methodsFor: 'displaying' stamp: 'hk 6/9/2015 16:01'!
asString
	^selection asString! !

!EnumVO methodsFor: 'testing' stamp: 'hk 4/29/2015 17:33'!
isValue
	^true! !

!EnumVO methodsFor: 'accessing' stamp: 'hk 4/4/2015 13:33'!
list
	^list! !

!EnumVO methodsFor: 'accessing' stamp: 'hk 4/4/2015 13:06'!
list: arrayOfString
	list := arrayOfString asOrderedCollection .
	selection := list first .
	self sorted ifTrue: [list sort: [:m1 :m2 | m1 < m2 ]]
	! !

!EnumVO methodsFor: 'printing' stamp: 'hk 5/7/2015 22:01'!
printOn: aStream
	super printOn: aStream .
	aStream nextPutAll: ': ',self val! !

!EnumVO methodsFor: 'accessing' stamp: 'hk 4/10/2015 18:05'!
selection: aListItem
	^ selection := aListItem ! !

!EnumVO methodsFor: 'private' stamp: 'hk 4/4/2015 13:35'!
sorted 
	^sorted isNotNil
	! !

!EnumVO methodsFor: 'accessing' stamp: 'hk 4/4/2015 13:36'!
sorted: aBoolean 
	sorted := aBoolean
	! !

!EnumVO methodsFor: 'accessing' stamp: 'hk 4/29/2015 11:51'!
val
	^ selection! !

!LeafVO class methodsFor: 'ui' stamp: 'hk 4/10/2015 12:39'!
uiClass
	^self subclassResponsibility ! !

!LeafVO methodsFor: 'copying' stamp: 'hk 5/7/2015 13:28'!
copyTarget
	"synonym in view copy functions for compatibility"
	^self copy! !

!LeafVO methodsFor: 'instance creation' stamp: 'hk 4/10/2015 12:50'!
widgetLabelled: aKey
	^ self class uiClass on: self key: aKey! !

LeafVO subclass: #ValueVO
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!ValueVO commentStamp: 'hk 4/9/2015 12:28' prior: 0!
I am a value - leaf in a VO tree

Instance Variables
	value:		<String>

!

ValueVO subclass: #DateVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!DateVO commentStamp: 'hk 5/6/2015 11:58' prior: 0!
I hold a date as my value


====== Tests ======  see Date class

DateVO new date: '2012.06.15'
DateVO new date: '6.15.2012'
DateVO new date: '6/15/2012'
DateVO new date: '15 June 2012'
DateVO new date: 'June 15 2012'
DateVO new date: ' 6 15 2012'
DateVO new date: '2012-06-15'
DateVO new date: '2012 06 15'

DateVO new date: Date today
!

!DateVO class methodsFor: 'ui' stamp: 'hk 4/11/2015 16:29'!
uiClass
	^ GcDateWidget ! !

!DateVO methodsFor: 'accessing' stamp: 'hk 6/24/2015 17:11'!
date
	^ [Date fromString: value] ifError: [ Date fromString:  '1000-01-01'  ]! !

!DateVO methodsFor: 'accessing' stamp: 'hk 5/6/2015 11:57'!
date: aDateOrString
	value := aDateOrString asString 

	 ! !

ValueVO subclass: #RangeVO
	instanceVariableNames: 'min max'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!RangeVO commentStamp: 'hk 4/29/2015 11:53' prior: 0!
I am a range - leaf in a VO tree

Instance Variables
	max:		<Number>
	min:		<Number>
!

!RangeVO class methodsFor: 'ui' stamp: 'hk 4/10/2015 12:37'!
uiClass
	^ GcRangeWidget ! !

!RangeVO methodsFor: 'accessing' stamp: 'hk 4/9/2015 12:32'!
max: aNumber
	max := aNumber! !

!RangeVO methodsFor: 'accessing' stamp: 'hk 4/9/2015 12:32'!
min: aNumber
	min := aNumber! !

!RangeVO methodsFor: 'copying' stamp: 'hk 4/29/2015 11:53'!
postCopy
	super postCopy  .
	min := min copy .
	max := max copy! !

!ValueVO class methodsFor: 'ui' stamp: 'hk 4/10/2015 12:35'!
uiClass
	^ GcValueWidget ! !

!ValueVO methodsFor: 'displaying' stamp: 'hk 6/25/2015 17:51'!
asString
	^value asString! !

!ValueVO methodsFor: 'testing' stamp: 'hk 4/4/2015 14:11'!
isValue
	^true! !

!ValueVO methodsFor: 'copying' stamp: 'hk 4/4/2015 19:28'!
postCopy
	super postCopy  .
	value := value copy! !

!ValueVO methodsFor: 'printing' stamp: 'hk 6/25/2015 15:10'!
printOn: aStream
	super printOn: aStream .
	aStream nextPutAll: ': ' , self value asString! !

!ValueVO methodsFor: 'accessing' stamp: 'hk 4/29/2015 11:27'!
val
	^value ! !

!ValueVO methodsFor: 'accessing' stamp: 'hk 4/29/2015 11:57'!
val: aString
	value := aString! !

SVO subclass: #NilThingVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!NilThingVO commentStamp: 'hk 6/23/2015 09:49' prior: 0!
I am a place holder for a thing. 
!

!NilThingVO methodsFor: 'as yet unclassified' stamp: 'hk 6/23/2015 10:50'!
asText
	^ 'not yet assigned'! !

!NilThingVO methodsFor: 'projection' stamp: 'hk 6/23/2015 09:52'!
nodeOrLeafAtKey: aTagSymbol
	"answer the path of the node or leaf with aTagSymbol or nil"

	| res |
	res := self nodeOrLeafAtKey: aTagSymbol onPath: String new.
	^ (res at: 1) = #success
		ifTrue: [ res at: 2]
		ifFalse: [ nil ]! !

!NilThingVO methodsFor: 'projection' stamp: 'hk 6/23/2015 09:59'!
nodeOrLeafAtKey: aTagSymbol onPath: aPath
	"answer the path of the node or leaf with the unique key aKey"

	^{ #failure . aPath}! !

NilThingVO subclass: #ThingVO
	uses: TSVOUserNameAccess
	instanceVariableNames: 'slots userName copies sortKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!ThingVO commentStamp: 'hk 6/25/2015 12:06' prior: 0!
I am a thing (Domain Value Object) or a node in a thing. 
My slots are ThingVO or LeafVO


Instance variables:
	userName:	<String>							user defined part of name
	copies: 		<OrderedCollection>				of my copies
	slots:		<OrderedIdentityDictionary of VO>	see SlotAssociation for annotations
													(projection tag , mandatory, ... )
	sortKey: 		<String>							key in lists
!

ThingVO class
	uses: TSVOUserNameAccess classTrait
	instanceVariableNames: ''!

!ThingVO class methodsFor: 'instance creation' stamp: 'hk 6/9/2015 10:29'!
new
	"instantiationh without userName from permStore or bizNet"
	 ^super new! !

!ThingVO class methodsFor: 'ui' stamp: 'hk 4/10/2015 12:36'!
uiClass
	^ GcVoWidget ! !

!ThingVO methodsFor: 'copying' stamp: 'hk 5/12/2015 13:14'!
addCopy: aVO
	^copies add: aVO 
! !

!ThingVO methodsFor: 'printing' stamp: 'hk 6/16/2015 20:42'!
asText
	"((ThingVO newOfType: #bla) val: #bla1 at: #bla2; yourself )  asText"
	"GcExample initSVO. GcExample email asText"
	^self asTextIndented: 0 aspect: ''! !

!ThingVO methodsFor: 'printing' stamp: 'hk 6/23/2015 20:03'!
asTextIndented: int aspect: key

	| txt |
	txt := TextStream on: (Text new: 10) .
	txt tab: int ; nextPutAll: ((key, '  ', self userName) asText addAttribute: TextEmphasis italic) .
	self valueCollBy3 do:  [ :ar3 | 
			txt crtab: (int + 2) . 
			1 to: ar3 size do: 
			[ :i | txt nextPutAll: ((ar3 at: i) at: 1) ; nextPut: $: ; space:2 ; nextPutAll: ((ar3 at: i) at: 2) ; tab; tab  ]  ] .
	txt cr.
	self subnodes associationsDo: [ :s | txt nextPutAll:  (s value asTextIndented: int + 1 aspect: s key) ].
	^txt contents
	! !

!ThingVO methodsFor: 'STON helper' stamp: 'hk 6/12/2015 12:52'!
convertToSlotAssociations
	"STON writes any dictionary with Associations"

	| newSlots |
	newSlots := OrderedIdentityDictionary new .
	slots associationsDo: 
		[ :a |   	"need not change the value container" 
		newSlots add:  (SlotAssociation new 
							key: a key ;
							value: a value ) .
		a value isNode ifTrue: [ a value convertToSlotAssociations  ] ] .
	slots := newSlots! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 5/10/2015 12:12'!
copies
	^ copies! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 5/30/2015 19:01'!
copy
	^super copy! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 5/10/2015 13:58'!
copy: newName
	| cp |
	cp := self copy . 
	self copyNamed: newName ifAbsent: 
		[cp userName: newName .  ^cp ] .
	self inform: 'could not name the copy ', newName .
	^cp! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 5/10/2015 12:12'!
copyNamed: aName 
	^ self copyNamed: aName ifAbsent:  [  ] ! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 5/12/2015 13:15'!
copyNamed:  aUserName ifAbsent: absentBlock
	"answer the vo with userName and the highest version"
	| res |
	res := copies select: [ :k | k userName = aUserName ] .
	^res 
		ifEmpty: [absentBlock value] 
		ifNotEmpty: [ (res sorted: [ :a :b | a longName <= b longName ] ) first ] .
	! !

!ThingVO methodsFor: 'copying' stamp: 'hk 5/7/2015 13:29'!
copyTarget
	"synonym in view copy functions for compatibility"
	^self copy! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/23/2015 10:07'!
define: slotName 
	self define: slotName tag: nil ! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:32'!
define: slotName date: aStringOrDate
	self define: slotName date: aStringOrDate tag: nil! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:32'!
define: slotName  date: aStringOrDate tag: aSymbol
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value: (DateVO new date: aStringOrDate ) )
					viewTag:  aSymbol )! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:34'!
define: slotName enum: arrayOfString
	self define: slotName enum: arrayOfString tag: nil! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:34'!
define: slotName  enum: arrayOfString tag: aSymbol 
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value: (EnumVO new list: arrayOfString ) )
					viewTag:  aSymbol )! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:35'!
define: slotName enumSorted: arrayOfString
	self define: slotName enumSorted: arrayOfString tag: nil! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:35'!
define: slotName  enumSorted: arrayOfString tag: aSymbol 
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value: (EnumVO new sorted: true; list: arrayOfString ) )
					viewTag:  aSymbol )! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 18:28'!
define: slotName listOf: aASVO
	self define: slotName listOf: aASVO tag: nil! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 18:27'!
define: slotName  listOf: aASVO tag: aSymbol 
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value:  ( UlistVO new asvo: aASVO) )
					viewTag:  aSymbol  )! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/23/2015 10:06'!
define: slotName tag: aSymbol 
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value: (NilThingVO new) )
					viewTag:  aSymbol )! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:38'!
define: slotName thing: aThingVO
	self define: slotName thing: aThingVO tag: nil! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:38'!
define: slotName  thing: aThingVO tag: aSymbol 
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value: aThingVO )
					viewTag:  aSymbol ) .
	aThingVO container: self! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:31'!
define: slotName val: aString
	self define: slotName val: aString tag: nil! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:37'!
define: slotName val: aNumber from: floor to: ceiling
	self
		define: slotName
		val: aNumber
		from: floor
		to: ceiling
		tag: nil! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:37'!
define: slotName val: aNumber from: floor  to: ceiling tag: aSymbol 
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value: (RangeVO new min: floor; max: ceiling; val: aNumber ) )
					viewTag:  aSymbol  )! !

!ThingVO methodsFor: 'authoring' stamp: 'hk 6/24/2015 15:31'!
define: slotName  val: aString tag: aSymbol 
	slots add: ((SlotAssociation 
					key: slotName asSymbol 
					value: (ValueVO new val: aString ) )
					viewTag:  aSymbol )! !

!ThingVO methodsFor: 'printing' stamp: 'hk 6/16/2015 11:10'!
displayArray
	| ar  idString |  
	idString := self hasDataPending ifTrue: [ id asString , '*' ] ifFalse: [ id asString ] .  
	ar := OrderedCollection with: idString with: userName .
	self valueSlots keysAndValuesDo: [ :k :v | ar add: v asString ].  
	 ^ar! !

!ThingVO methodsFor: 'printing' stamp: 'hk 4/29/2015 17:44'!
displayLabelArray
	| ar values |     
	ar := OrderedCollection with: 'id'  with: 'name' .
	values := self slots select: [ :s | s isValue ]  .
	values keysAndValuesDo: [ :k :v | ar add: k ].
	 ^ar! !

!ThingVO methodsFor: 'private' stamp: 'hk 6/10/2015 10:55'!
doesNotUnderstand: aMessage
	"answer the slot named by aMessage or change a valueSlot"

	| sel getter res |
	sel := aMessage selector.
	getter := (sel trimRight: [ :c | c = $: ]) asSymbol.
	res := self slotAt: getter ifAbsent: [ ^ super doesNotUnderstand: aMessage ].
	sel isKeyword ifTrue: [  "= setter"
		^ res isValue
			ifTrue: [ res val: aMessage argument ]
			ifFalse: [ super doesNotUnderstand: aMessage ] ].
	^ res isValue
		ifTrue: [ res val ]
		ifFalse: [ res ]! !

!ThingVO methodsFor: 'initialization' stamp: 'hk 5/12/2015 13:15'!
initialize
	super initialize.
	copies := OrderedCollection new.
	self initializeSlots! !

!ThingVO methodsFor: 'initialization' stamp: 'hk 5/7/2015 13:07'!
initializeSlots
	slots := OrderedIdentityDictionary new! !

!ThingVO methodsFor: 'testing' stamp: 'hk 6/23/2015 12:08'!
isNode
	^true! !

!ThingVO methodsFor: 'projection' stamp: 'hk 6/11/2015 12:54'!
nodeOrLeafAtKey: aTagSymbol onPath: aPath
	"answer the path of the node or leaf with the unique key aKey"

	| currentPath res |
	currentPath := aPath.
	slots associationsDo: [ :a | 
		a key = aTagSymbol  		"use the key=slotName as default-tag"
			ifTrue: [ ^ { #success . currentPath , ' ' , a key} ] .
		a viewTag = aTagSymbol
			ifTrue: [ ^ { #success . currentPath , ' ' , a key} ]
			ifFalse: [a value isNode ifTrue: 
				[res := a value nodeOrLeafAtKey: aTagSymbol onPath: currentPath , ' ' , a key.
				(res at: 1) = #success ifTrue: [ ^ res ] ] ] ].
	^{ #failure . currentPath}! !

!ThingVO methodsFor: 'copying' stamp: 'hk 6/25/2015 12:40'!
postCopy
	| newSlots |
	super postCopy .
	userName  := self userName, id asString  .
	copies := OrderedCollection new .
	isAsvo := false.		"copies are not ASVO"	
	newSlots := OrderedIdentityDictionary new.
	slots associationsDo: 
		[ :a | | assoc cp |
		assoc := a copy.
		cp := assoc value .
		newSlots add: assoc .
		cp container: self] .
	slots := newSlots! !

!ThingVO methodsFor: 'copying' stamp: 'hk 5/12/2015 13:11'!
removeCopy: aVO
	"used to shift a copy to its View"
	copies remove: aVO  . 
	^aVO! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/23/2015 20:00'!
removeNode: aSlotName   
	| slotA |
	slotA := self slots associationAt: aSlotName.
	slotA value isNode ifTrue: [slotA value: NilThingVO new]! !

!ThingVO methodsFor: 'analyzing' stamp: 'hk 5/28/2015 16:58'!
rtSlots
	^self slots keys ! !

!ThingVO methodsFor: 'copying' stamp: 'hk 6/8/2015 12:39'!
shallowCopy
	| cp |
	cp := super shallowCopy .
	self addCopy: cp .
	cp master: self.
	^cp! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/10/2015 10:29'!
slotAt: aSlotName 
	"answer the slot content, i.e. the VO, not a vlaue"
	^self slotAt: aSlotName ifAbsent: []
! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/10/2015 10:37'!
slotAt: aSlotName ifAbsent: aBlock
	"answer the slot content, i.e. the VO, not a vlaue"
	| res |
	res :=  slots associationAt: aSlotName ifAbsent: [ ^aBlock value ] .
	^res value! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 4/3/2015 17:39'!
slots
	^ slots! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 12:05'!
sortKey
	^sortKey ifNil: [ '' ]! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 12:05'!
sortKey: aSortKeyString
	sortKey := aSortKeyString ! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/8/2015 13:49'!
subnodes
	^ slots select: [ :s | s isNode ]! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 5/31/2015 19:03'!
tag: aSymbol at: aSlotName
	^ (slots associationAt: aSlotName ifAbsent: [  ]) viewTag: aSymbol! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/25/2015 19:06'!
userName
	^ userName ifNil: [ 'default' ]! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 4/28/2015 13:07'!
userName: aString
	userName := aString ! !

!ThingVO methodsFor: 'printing' stamp: 'hk 6/16/2015 11:33'!
valueColl
	| ar |
	ar := OrderedCollection new .
	self valueSlots  keysAndValuesDo: [ :k :v | ar add: { k . v asString} ].  
	 ^ar ! !

!ThingVO methodsFor: 'printing' stamp: 'hk 6/16/2015 11:37'!
valueCollBy3
"GcExample initSVO. GcExample email valueCollBy3"
	| res size ar |  
	res := OrderedCollection new.
	ar := self valueColl asArray.
	size := ar size . 
	1 to: size by: 3 do: 
		[ :a |  | o arr | 
		o := (a+3) min: (size+1) .
		arr := Array new: (o - a) .
		a to: o-1 do: [ :i | arr at: (i-1 \\ 3 +1) put: (ar at: i) ] .
		res add: arr ] .
	 ^res! !

!ThingVO methodsFor: 'accessing' stamp: 'hk 6/16/2015 11:09'!
valueSlots
	^ self slots select: [ :s | s isValue ]  .

! !

!ThingVO methodsFor: 'ui helpers' stamp: 'hk 4/10/2015 13:16'!
widgetLabelled: aKey
	^ self class uiClass on: self key: aKey! !

ThingVO subclass: #UlistVO
	instanceVariableNames: 'asvo numberPolicy view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Things'!

!UlistVO commentStamp: 'hk 6/25/2015 14:23' prior: 0!
I am a node thing that treats its subnodes as a list.
The slotNames are defined by my numberPolicy.
I am transparent for Views.

    Instance Variables
	asvo:			<SVO>					master for default exemplars
	numberPolicy:	<DefaultNoPolicyVO >
	view:			<ViewVO >

newExemplar only works with an asvo!

!UlistVO class methodsFor: 'as yet unclassified' stamp: 'hk 6/25/2015 12:27'!
uiClass
	^ GcUListWidget ! !

!UlistVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 18:29'!
addElement: aThing
	| slotName |
	slotName := numberPolicy nextString asSymbol.
	self define: slotName thing: aThing.
	aThing sortKey: slotName ! !

!UlistVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 10:40'!
asvo: aVO
	asvo := aVO ! !

!UlistVO methodsFor: 'displaying' stamp: 'hk 6/25/2015 15:02'!
displayStringFor: item 
	^ item sortKey, '  ', item userName, (self viewDisplayFor: item)! !

!UlistVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 11:15'!
initialize
	super initialize.
	self numberPolicy: DefaultNoPolicyVO new .
 "	self view: (NoViewVO new)"! !

!UlistVO methodsFor: 'protocol' stamp: 'hk 6/25/2015 10:38'!
list
	^self subnodes! !

!UlistVO methodsFor: 'protocol' stamp: 'hk 6/25/2015 10:38'!
listSize
	^self list size! !

!UlistVO methodsFor: 'protocol' stamp: 'hk 6/25/2015 18:29'!
newExemplar
	asvo ifNotNil: 	[self addElement: asvo copy]! !

!UlistVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 10:43'!
numberPolicy: aPolicyVO
	numberPolicy := aPolicyVO ! !

!UlistVO methodsFor: 'authoring' stamp: 'hk 6/25/2015 10:45'!
view: aViewVO
	view := aViewVO ! !

!UlistVO methodsFor: 'displaying' stamp: 'hk 6/25/2015 17:53'!
viewDisplayFor: item
	| stream |
	^ view ifNil: [''] ifNotNil: 
		[stream := WriteStream on: ''.
		view target: item . 
		view slots associationsDo:[ :slotA | stream tab; nextPutAll: slotA value asString ] .
		stream contents]! !

SVO subclass: #PersonalizationVO
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-ProgramVO'!

!SVO class methodsFor: 'accessing' stamp: 'hk 4/28/2015 12:39'!
addASVO: aASVO			
	self checkIfPresent: aASVO .
	asvos add: aASVO longName -> aASVO. 
	^aASVO 
	
! !

!SVO class methodsFor: 'accessing' stamp: 'hk 4/28/2015 12:39'!
asvos
	^asvos 
	
! !

!SVO class methodsFor: 'dsl push commands' stamp: 'hk 5/9/2015 15:41'!
attach
 ^#attach:target:in:

! !

!SVO class methodsFor: 'dsl push commands' stamp: 'hk 5/30/2015 18:44'!
attach: type target: irrelevant in: context
	"find or create the list element and attach the thing type to it"
	
	(context isKindOf: CfListVO) ifFalse: [ ^self ].
	
	context contextVOs copy do: 
		[ :vo | ((vo slotAt: #aspect) val = type) ifTrue: 
			[ context findOrMakeProcessFor: vo.
			context removeContextVO: vo ] ]    ! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 6/4/2015 15:00'!
bizNet
 ^#bizNet:to:with:with:! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 6/12/2015 12:07'!
bizNet:  parsedTarget to:  parsedArg with: context with: aUI
	"sent parsedTarget to parsedArg. answer a string with the actions result"

	| target rec supProcess |

	supProcess := context vo.  
	[rec :=  supProcess perform: parsedArg asSymbol] 
		on: Error do: [ ^ 'no receiver found' ] .
	[target := supProcess perform: parsedTarget asSymbol] 
		on: Error do: [ ^ 'no target found' ] .
	BizNet receive: target copy for: rec. 
	^ 'done'
! !

!SVO class methodsFor: 'private' stamp: 'hk 4/28/2015 12:39'!
checkIfPresent: aVo		
	| uname |
	uname := aVo longName .	
	asvos at: uname ifPresent: [ self inform: uname, ' overwritten']! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 4/13/2015 19:46'!
delete
 ^#delete:inContext:onUI: 

! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 5/8/2015 17:37'!
delete: parsedTarget inContext: aContext onUI: aUI
	"callback to UI"

	^aUI deleteSelection
 ! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 4/13/2015 19:46'!
edit
 ^#edit:inContext:onUI: 

! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 5/9/2015 14:02'!
edit: parsedTarget inContext: aContext onUI: aUI
	"callback to UI"

	^aUI editSelection
 ! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 6/4/2015 15:01'!
email
 ^#email:with:with:
! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 6/24/2015 15:31'!
email: parsedArgument with: context with: aUI
	"do the action. answer a string with the actions result
	depends on the existence of an Email ASPO"

	| email contact adr |
	email := (ThingVO named: #Email ifAbsent: [ ^ 'Email not found' ]) copy.
	[ 
	contact := context vo.
	parsedArgument = 'no arguments'
		ifFalse: [ contact := contact perform: parsedArgument asSymbol ] ]
		on: Error
		do: [ ^ 'no contact found' ].
	[ 
	[ adr := contact email ]
		on: Error
		do: [ adr := contact contactIdentity mainContactPerson communication email address ] ]
		on: Error
		do: [ ^ 'no email found' ].
	email define: #receiver val: adr.
	email define: #content val: 'irgendwas'.
	email edit.
	^ 'done'! !

!SVO class methodsFor: 'class initialization' stamp: 'hk 4/28/2015 12:39'!
initialize
	lastID := 0 .
	asvos := Dictionary new .! !

!SVO class methodsFor: 'accessing' stamp: 'hk 4/5/2015 13:13'!
named: aName
	^ self named: aName ifAbsent:  nil ! !

!SVO class methodsFor: 'accessing' stamp: 'hk 4/28/2015 12:39'!
named: aName ifAbsent: absentBlock
	^ asvos at: aName ifAbsent:  absentBlock ! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 4/13/2015 19:48'!
neu
	^#newContext:inContext:onUI: ! !

!SVO class methodsFor: 'authoring' stamp: 'hk 4/28/2015 12:26'!
new
	(self = SVO) ifTrue: [  ^self error: 'SVO is an abstract class'  ] .
	^ self addASVO: (super new id: self nextID)! !

!SVO class methodsFor: 'dsl menu commands' stamp: 'hk 5/11/2015 14:55'!
newContext: parsedTarget inContext: aContext onUI: aUI
	"callback to UI"

	aUI activateSubContextForNew! !

!SVO class methodsFor: 'accessing' stamp: 'hk 4/2/2015 20:24'!
nextID
	"answer a smallInteger - modulo largest integer"
	^ lastID := (lastID + 1) \\ SmallInteger maxVal! !

!SVO class methodsFor: 'as yet unclassified' stamp: ''!
o4.0.app/Contents/Resources/Pharo4.0PP-SVO.image! !

!SVO class methodsFor: 'dsl push commands' stamp: 'hk 6/16/2015 12:46'!
push: type to: tgtName from: context
 	"move the thing of type type to the context tgtName. Ignore context "
	
	| target |
	target := CfPlainVO named: tgtName ifAbsent: [ CfListVO named: tgtName ].
	target ifNil: [ ^self ] .
	context contextVOs copy do: 
		[ :vo| ((vo slotAt: #aspect) val = type) ifTrue: 
			[ context moveContextVO: vo to: target ] ]
! !

!SVO class methodsFor: 'dsl push commands' stamp: 'hk 5/9/2015 12:52'!
pushTo
 ^#push:to:from:

! !

!SVO methodsFor: 'printing' stamp: 'hk 4/5/2015 13:20'!
autoName
	^' v', version asString, ' ', self className , id asString! !

!SVO methodsFor: 'authoring' stamp: 'hk 4/2/2015 19:58'!
beAsvo
	isAsvo := true
! !

!SVO methodsFor: 'accessing' stamp: 'hk 5/28/2015 14:53'!
container
	^ container! !

!SVO methodsFor: 'accessing' stamp: 'hk 5/30/2015 18:18'!
container: anObject
	container ifNotNil: [ ^self error: 'cant be double used in ', anObject userName ] .
	container := anObject! !

!SVO methodsFor: 'pending data' stamp: 'hk 6/8/2015 15:38'!
dataPending
	^dataPending ifNil: [ dataPending := false asValueHolder  ]! !

!SVO methodsFor: 'utility' stamp: 'hk 5/6/2015 12:19'!
edit
	(GcVOEditor on: self) openWithSpec! !

!SVO methodsFor: 'pending data' stamp: 'hk 6/8/2015 15:39'!
hasDataPending
	^self dataPending value = true! !

!SVO methodsFor: 'accessing' stamp: 'hk 4/4/2015 19:20'!
id
	^id ! !

!SVO methodsFor: 'accessing' stamp: 'hk 4/4/2015 19:19'!
id: anInteger
	id := anInteger! !

!SVO methodsFor: 'initialization' stamp: 'hk 6/8/2015 15:39'!
initialize
	super initialize.
	isAsvo := true.		"ASVO are created by script, DSVO by copy"
	self initializeFields! !

!SVO methodsFor: 'initialization' stamp: 'hk 6/8/2015 13:39'!
initializeFields
	container := nil .
	self resetVersion .! !

!SVO methodsFor: 'testing' stamp: 'hk 4/4/2015 19:59'!
isAsvo
	^isAsvo! !

!SVO methodsFor: 'testing' stamp: 'hk 4/15/2015 17:15'!
isNode
	"used for slot navigation in DomainVO"
	^false! !

!SVO methodsFor: 'testing' stamp: 'hk 4/15/2015 17:14'!
isValue
	"used for slot navigation in DomainVO"
	^false! !

!SVO methodsFor: 'testing' stamp: 'hk 4/16/2015 22:02'!
isView
	^false! !

!SVO methodsFor: 'printing' stamp: 'hk 4/5/2015 20:07'!
longName
	^self autoName ! !

!SVO methodsFor: 'accessing' stamp: 'hk 5/10/2015 15:09'!
master
	^ master! !

!SVO methodsFor: 'accessing' stamp: 'hk 6/8/2015 12:21'!
master: anObject
	master := anObject longName! !

!SVO methodsFor: 'saving' stamp: 'hk 4/5/2015 13:20'!
nextVersion
	^version := version  +1! !

!SVO methodsFor: 'copying' stamp: 'hk 6/8/2015 15:39'!
postCopy
	super postCopy .
	isAsvo := true .		"VO subclass will change that"
	id := self class nextID .
	self initializeFields .! !

!SVO methodsFor: 'private' stamp: 'hk 4/5/2015 13:20'!
resetVersion
	version := 0! !

!SVO methodsFor: 'analysing' stamp: 'hk 6/7/2015 13:31'!
rtAttributes
	^{ ( 'id: ', self id asString) . ( 'version: ', self version asString)}! !

!SVO methodsFor: 'analysing' stamp: 'hk 5/28/2015 13:13'!
rtSlots
	^#()! !

!SVO methodsFor: 'saving' stamp: 'hk 4/5/2015 13:01'!
save
	"save the changed VO to permanent store"
	self nextVersion .
	self storeFile .
! !

!SVO methodsFor: 'pending data' stamp: 'hk 6/8/2015 15:39'!
setDataNotPending
	self dataPending value: false! !

!SVO methodsFor: 'pending data' stamp: 'hk 6/8/2015 15:39'!
setDataPending
	self dataPending value: true! !

!SVO methodsFor: 'saving' stamp: 'hk 4/5/2015 13:02'!
storeFile 
! !

!SVO methodsFor: 'printing' stamp: 'hk 6/8/2015 12:23'!
userName
	^ ''! !

!SVO methodsFor: 'accessing' stamp: 'hk 4/3/2015 18:11'!
version
	^version! !

Object subclass: #SVOCodeSize
	instanceVariableNames: 'vars methods loc classes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SVO2-Utilities'!

!SVOCodeSize commentStamp: 'hk 5/6/2015 11:22' prior: 0!
I help counting classes, LOC, methods and variables.
I can be added and subtracted.  

Instance Variables
	classes:		<Integer>
	loc:				<Integer>
	methods:		<Integer>
	vars:			<Integer>



(SVOCodeSize  newOnPackageOf:  VO) - (SVOCodeSize newOn: GcIcons)
 a SVOCodeSize: 50 Klassen; 3440 LOC; 113 Variablen; 665 Methoden

SVOCodeSize newOnCategoryOf: DomainVO
 a SVOCodeSize: 7 Klassen; 215 LOC; 9 Variablen; 87 Methoden

SVOCodeSize  newOnCategoryOf:  CfListVO
 a SVOCodeSize: 9 Klassen; 162 LOC; 19 Variablen; 69 Methoden

SVOCodeSize  newOnCategoryOf:  GcMainUI
 a SVOCodeSize: 17 Klassen; 753 LOC; 25 Variablen; 150 Methoden

SVOCodeSize  newOnCategoryOf:  SVOTests
 a SVOCodeSize: 4 Klassen; 470 LOC; 0 Variablen; 32 Methoden







!

!SVOCodeSize class methodsFor: 'instance creation' stamp: 'hk 5/6/2015 11:22'!
newOn: aClassOrCollectionOfClasses

	| size |
	(aClassOrCollectionOfClasses isCollection) 
		ifFalse: [^self new sizeOfClass: aClassOrCollectionOfClasses].
	size := self newOn: (aClassOrCollectionOfClasses first).
	aClassOrCollectionOfClasses allButFirst do: [:cls| size + (self newOn: cls)].
	^size! !

!SVOCodeSize class methodsFor: 'instance creation' stamp: 'hk 5/13/2010 14:48'!
newOnCategoryOf: aClass

	^self new sizeOfCategory: aClass! !

!SVOCodeSize class methodsFor: 'instance creation' stamp: 'hk 3/23/2015 19:34'!
newOnPackageOf: aClass

	^self new sizeOfPackage: aClass! !

!SVOCodeSize class methodsFor: 'instance creation' stamp: 'hk 5/13/2010 14:51'!
newOnSubclassesOf: aClass

	^self new sizeOfSuclassesOf: aClass! !

!SVOCodeSize class methodsFor: 'instance creation' stamp: 'hk 5/19/2010 20:31'!
newOnWithSubclasses: aClass

	| size |
	(aClass isKindOf: Class) ifFalse: [^self error: 'argument is no single class'].
	^(self newOn: aClass) + (self newOnSubclassesOf: aClass)! !

!SVOCodeSize methodsFor: 'arithmetic' stamp: 'hk 2/10/2015 13:25'!
+ anSvoCodeSize

	vars := vars + (anSvoCodeSize vars).
	methods  := methods + (anSvoCodeSize methods).
	loc := loc + (anSvoCodeSize loc).
	classes := classes + (anSvoCodeSize classes).! !

!SVOCodeSize methodsFor: 'arithmetic' stamp: 'hk 2/10/2015 13:25'!
- anSvoCodeSize

	vars := vars - (anSvoCodeSize vars).
	methods  := methods - (anSvoCodeSize methods).
	loc := loc - (anSvoCodeSize loc).
	classes := classes - (anSvoCodeSize classes).! !

!SVOCodeSize methodsFor: 'accessing' stamp: 'hk 5/10/2010 23:42'!
classes
	^classes! !

!SVOCodeSize methodsFor: 'accessing' stamp: 'hk 5/10/2010 19:18'!
loc
	^loc! !

!SVOCodeSize methodsFor: 'accessing' stamp: 'hk 5/10/2010 19:17'!
methods
	^methods! !

!SVOCodeSize methodsFor: 'printing' stamp: 'hk 5/10/2010 23:47'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ': ',classes printString, ' Klassen; ', loc printString, ' LOC; ', vars printString, ' Variablen; ', methods printString, ' Methoden'
	! !

!SVOCodeSize methodsFor: 'initialize' stamp: 'hk 4/28/2015 11:38'!
sizeOfCategory: aClass

	loc := vars := methods := classes := 0.
	(SystemOrganization listAtCategoryNamed: aClass category)
		do: [:cls | self + (SVOCodeSize newOn:(Smalltalk at: cls)) ].
! !

!SVOCodeSize methodsFor: 'initialize' stamp: 'hk 5/13/2010 14:46'!
sizeOfClass: aClass

	loc := 0.
	aClass methodsDo: [:m | loc := loc +  (m getSource  asArray select: [:c | c = Character cr]) size].
	aClass class methodsDo: [:m | loc := loc +  (m getSource  asArray select: [:c | c = Character cr]) size].
	vars := ((aClass instVarNames size) + (aClass classVarNames size)).
	methods := ((aClass methodDict size) + (aClass class methodDict size)).
	classes := 1.! !

!SVOCodeSize methodsFor: 'initialize' stamp: 'hk 4/28/2015 11:38'!
sizeOfPackage: aClass

	loc := vars := methods := classes := 0.
	aClass package orderedClasses do: [:cls | self + (SVOCodeSize newOn: cls) ].
! !

!SVOCodeSize methodsFor: 'initialize' stamp: 'hk 4/28/2015 11:38'!
sizeOfSuclassesOf: aClass

	loc := vars := methods := classes := 0.
	aClass subclassesDo: [:cls |self + (SVOCodeSize newOn: cls)]! !

!SVOCodeSize methodsFor: 'accessing' stamp: 'hk 5/10/2010 19:17'!
vars
	^vars ! !
GcExample initialize!
SVO initialize!
